/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.configgen.loaders.serial;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Logger;

import com.iona.soa.repository.util.CRLFFilterReader;
import com.iona.soa.repository.util.InputStreamHelper;
import com.iona.soa.serialization.AttributeHandler;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;

public class ClasspathURLAttributeHandler implements AttributeHandler {
    protected static final Logger LOG = Logger.getLogger(ClasspathURLAttributeHandler.class.getName());
    
    private static final String BASE_URL_PLACEHOLDER = "file:[CLASSPATH]";
    private EAttribute attribute; 

    public ClasspathURLAttributeHandler(EAttribute eAttr) {
        attribute = eAttr;
    }

    public Object deserialize(EObject eObj, URI uri) {
        URL url;
        if (uri.toString().startsWith(BASE_URL_PLACEHOLDER)) {
            String path = uri.toString().substring(16);
            try {
                url = getClass().getClassLoader().getResource(path);
            } catch (Exception ex) {
                throw new RuntimeException(
                    new SerialModelConfigurationException("Error resolving classpath URL defined as "
                                                          + "\"" + path + "\"", ex));
            }
        } else {
            try {
                url = new URL(uri.toString());
            } catch (MalformedURLException ex) {
                throw new RuntimeException(
                        new SerialModelConfigurationException("Error resolving URL " + uri.toString()));
            }
        }

        
        String val = null;
        InputStream is = null;
        try {
            is = new BufferedInputStream(url.openStream());
            Reader isr = new CRLFFilterReader(new InputStreamReader(is, InputStreamHelper.UTF8));
            Writer sw = new StringWriter();
            InputStreamHelper.drain(isr, sw);
            InputStreamHelper.close(isr);
            val = sw.toString();
        } catch (Exception ex) {
            throw new RuntimeException(new SerialModelConfigurationException(ex));
        } finally {
            InputStreamHelper.close(is);
        }
        return val;
    }

    public EAttribute getAttribute() {
        return attribute;
    }

    public URI serialize(EObject eObj, Object data) {
        throw new UnsupportedOperationException();
    }
}
