/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.configgen.loaders.serial;

import java.net.URISyntaxException;

import com.iona.soa.model.SOA_NetworkPackage;
import com.iona.soa.repository.configuration.HttpConfiguration;
import com.iona.soa.serialization.AttributeHandler;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;

public class HostnameAttributeHandler implements AttributeHandler {
    
    
    public static final String DEPOT_HOSTNAME_PLACEHOLDER = "@DEPOT_HOSTNAME@";
    public static final String DEPOT_INSECURE_PORT_PLACEHOLDER = "@DEPOT_INSECURE_PORT@";
    private HttpConfiguration httpConfig;
    
    public HostnameAttributeHandler(HttpConfiguration httpConfig) {
        this.httpConfig = httpConfig;
        
    }

    public Object deserialize(EObject obj, URI uri) {
        if (uri == null) {
            return null;
        }
        
        String attrStr = uri.toString();
        if (httpConfig != null) {
            String insecurePort = Integer.toString(httpConfig.getHttpPort());
            attrStr = attrStr.replace(DEPOT_HOSTNAME_PLACEHOLDER, httpConfig.getHostName())
                             .replace(DEPOT_INSECURE_PORT_PLACEHOLDER, insecurePort);
        }
        
        try {
            return new java.net.URI(attrStr);
        } catch (URISyntaxException e) {
            return null;
        }           
    }
        
    public EAttribute getAttribute() {
        EClass mavenRepoClass = SOA_NetworkPackage.eINSTANCE.getMavenRepository();
        EStructuralFeature feature = mavenRepoClass.getEStructuralFeature("url");
        if (feature instanceof EAttribute) {
            return (EAttribute)feature;
        }
        throw new RuntimeException("Unsupported model change detected!");
    }

    public URI serialize(EObject obj, Object data) {
        throw new UnsupportedOperationException();   
    }

}
