/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.configgen.loaders.serial;

import java.sql.Timestamp;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;

import com.iona.soa.model.repository.IPersistableObject;
import com.iona.soa.serialization.ResourceEObjectMapper;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.Resource;

public class KnownObjectsResolver extends ResourceEObjectMapper {
    private Collection<EObject> newObjects;

    KnownObjectsResolver(Resource resource) {
        super(resource);
        this.newObjects = new LinkedList<EObject>();
    }

    @SuppressWarnings("unchecked")
    @Override
    public EObject getEObject(EClass eClass, URI uri, boolean resolve) {
        EObject ret = null;
        if (uri.hasFragment()) {
            ret = getResource().getEObject(uri.fragment());
        }
        if (ret == null && uri.hasFragment()) {
            // find object by namespace & name
            String[] parts = uri.toString().split("#");
            if (parts.length == 2 && parts[0] != null && parts[1] != null) {
                ret = findRepoObject((Collection<EObject>)getResource().getContents(), parts[0], parts[1]);
            }
        } 
        if (ret == null) {
            ret = getResource().getEObject(uri.toString());
        }
        return ret;
    }
    
    @Override 
    public EObject createEObject(EClass eClass, Object id) {
        
        EObject eobject = super.createEObject(eClass, id);
        
        // Make sure timestamp and lastEditedBy are set to default values.
        if (eobject instanceof IPersistableObject) {
            IPersistableObject po = (IPersistableObject) eobject;
            po.setTimestamp(new Timestamp(System.currentTimeMillis()));
            // TODO: Externalize this String
            po.setLastEditedBy("Admin");
        }
        return eobject;
    }



    @Override
    public boolean addToResource(EObject eObj) {
        boolean ret = super.addToResource(eObj);
        if (ret) {
            newObjects.add(eObj);
        }
        return ret;
    }

    public Collection<EObject> getNewObjects() {
        Collection<EObject> ret = new HashSet<EObject>();
        for (EObject newObj : newObjects) {
            EObject root = newObj;
            while (root.eContainer() != null) {
                root = root.eContainer();
            }
            ret.add(root);
        }
        return ret;
    }

    @SuppressWarnings("unchecked")
    private EObject findRepoObject(Collection<EObject> eObjs, String ns, String name) {
        for (EObject eObj : eObjs) {
            EStructuralFeature nsAttr = eObj.eClass().getEStructuralFeature("namespace");
            EStructuralFeature nameAttr = eObj.eClass().getEStructuralFeature("name");
            if (nsAttr instanceof EAttribute && nameAttr instanceof EAttribute) {
                if (ns.equals(eObj.eGet(nsAttr)) && name.equals(eObj.eGet(nameAttr))) {
                    return eObj;
                }
            }
            EObject ret = findRepoObject((Collection<EObject>)eObj.eContents(), ns, name);
            if (ret != null) {
                return ret;
            }
        }
        return null;
    }
}