/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.configgen.loaders.serial;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.List;
import java.util.logging.Logger;

import com.iona.soa.repository.ModelRegistry;
import com.iona.soa.repository.configuration.HttpConfiguration;
import com.iona.soa.repository.model.util.ModelHelper;
import com.iona.soa.serialization.Deserializer;
import com.iona.soa.serialization.SerializationFactory;
import com.iona.soa.serialization.XMLParseException;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceImpl;

public class SerialModelConfigurationLoader {
    
    protected static final Logger LOG = Logger.getLogger(SerialModelConfigurationLoader.class.getName());
    
    private static final String SERIAL_MODEL_FILE_NAME = "data.xml";

    private String configResourceBase;
    
    public SerialModelConfigurationLoader(String configResourceBase) {
        this.configResourceBase = configResourceBase;
    }
    
    @SuppressWarnings("unchecked")
    public void createProductsFromConfiguration(List<EObject> objects, HttpConfiguration httpConfig) 
        throws SerialModelConfigurationException {

        ClassLoader classLoader = getClass().getClassLoader();
        String configSource = configResourceBase + "/" + SERIAL_MODEL_FILE_NAME;
        InputStream is = classLoader.getResourceAsStream(configSource);
        
        Deserializer deserializer = null;
        Resource resource = new ResourceImpl();
        resource.getContents().addAll(objects);
        
        // Setting the object resolver (to get access to objects already loaded up)
        KnownObjectsResolver crossBundleResolver = new KnownObjectsResolver(resource);

        try {
            deserializer = SerializationFactory.getDeserializer(is, crossBundleResolver);
        } catch (XMLParseException ex) {
            throw new SerialModelConfigurationException("Error when parsing configuration at \""
                                                        + configSource + "\"", ex);
        } catch (IOException ex) { 
            throw new SerialModelConfigurationException("Error when accessing configuration at \""
                                                        + configSource + "\"", ex);
        }

        ModelRegistry soanRegistry = ModelHelper.getSOAModelRegistry();
        deserializer.setModelRegistry(soanRegistry);
        
        deserializer.registerAttributeHandler(new HostnameAttributeHandler(httpConfig));
        
        try {
            deserializer.deserialize();
        } catch (XMLParseException ex) {
            throw new SerialModelConfigurationException("Error when parsing configuration at \""
                                                        + configSource + "\"", ex);
        } catch (RuntimeException ex) {
            Throwable cause = ex.getCause();
            if (cause instanceof SerialModelConfigurationException) {
                throw (SerialModelConfigurationException)ex.getCause();
            } else if (cause != null) {
                throw new SerialModelConfigurationException(cause.getMessage(), cause);
            } else {
                throw new SerialModelConfigurationException(ex.getMessage(), ex);
            }
        }

        Collection<EObject> newObjects = crossBundleResolver.getNewObjects();
        
        objects.addAll(newObjects);
    }
}
