/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.configgen.util;

import java.util.LinkedList;
import java.util.List;

import com.iona.soa.configgen.loaders.serial.SerialModelConfigurationLoader;
import com.iona.soa.model.repository.ModelMetadata;
import com.iona.soa.model.repository.RepositoryFactory;
import com.iona.soa.repository.configuration.HttpConfiguration;
import com.iona.soa.repository.util.VersionType;

import org.eclipse.emf.ecore.EObject;

/**
 * @author ohurley
 *
 */
public final class GenerateModel {
    private GenerateModel() {
    }
    
    public static List<EObject> generate() throws Exception {
        return generate(null, null);
    }

    public static List<EObject> generate(VersionType schemaVersion, 
                                         HttpConfiguration httpConfig) 
        throws Exception {
        
        List<EObject> objects = new LinkedList<EObject>();
        
        objects.addAll(loadSecurityData());
        objects.addAll(loadAppTypeData());
        objects.addAll(loadMavenRepoData(httpConfig));
        
        if (schemaVersion != null) {
            ModelMetadata md = RepositoryFactory.eINSTANCE.createModelMetadata();
            md.setInternal(true);
            md.setModifiable(false);
            md.setSchemaVersion(schemaVersion.toString());
            objects.add(md);
        }
        
        return objects;
    }

    public static List<EObject> loadSecurityData() throws Exception {
        
        SerialModelConfigurationLoader configurationLoader = new SerialModelConfigurationLoader(
            "com/iona/soa/configgen/security/serial");

        List<EObject> objects = new LinkedList<EObject>();
        configurationLoader.createProductsFromConfiguration(objects, null);
        
        return objects;
    }
    
    public static List<EObject> loadAppTypeData() throws Exception {
        
        SerialModelConfigurationLoader configurationLoader = new SerialModelConfigurationLoader(
            "com/iona/soa/configgen/types/apptypes");

        List<EObject> objects = new LinkedList<EObject>();
        configurationLoader.createProductsFromConfiguration(objects, null);
        
        return objects;
    }
    
    public static List<EObject> loadMavenRepoData(HttpConfiguration httpConfig) throws Exception {
        
        SerialModelConfigurationLoader configurationLoader = new SerialModelConfigurationLoader(
            "com/iona/soa/configgen/mavenrepos");

        List<EObject> objects = new LinkedList<EObject>();
        configurationLoader.createProductsFromConfiguration(objects, httpConfig);
        
        return objects;
    } 
}
    
