/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.pac;

import java.util.logging.Logger;

import javax.servlet.http.HttpServletRequest;

import com.iona.soa.repository.SOARepositoryService;
import com.iona.soa.security.AuthenticationException;
import com.iona.soa.security.SecurityContext;
import com.iona.soa.security.SecurityService;

import org.fusesource.cloudmix.common.HttpAuthenticator;
import org.springframework.beans.factory.InitializingBean;

public class ProvisioningAuthenticator implements HttpAuthenticator, InitializingBean {

    private static final Logger LOG = Logger.getLogger(ProvisioningAuthenticator.class.getName());

    private static ThreadLocal<SecurityContext> threadLocal = new ThreadLocal<SecurityContext>();
    
    private SecurityService securityService;
    private SOARepositoryService repositoryService;


    /**
     * Spring Injected
     */    
    public final void setSecurityService(SecurityService ss) {
        LOG.fine("Injected SecurityService");
        securityService = ss;
    }    

    /**
     * Spring Injected
     * TODO: This is needed in place of setSecurityService as there is currently a
     * a problem injecting the SecurityService directory.
     */    
    public final void setRepositoryService(SOARepositoryService rs) {
        LOG.fine("Injected RepositoryService");
        repositoryService = rs;
    }    

    public void afterPropertiesSet() throws Exception {
        init();        
    }


    public boolean authenticate(HttpServletRequest request) {
        
        if (securityService == null) {
            LOG.severe("cannot authenticate request as there is no security service");
            return false;
        }
        
        try {
            SecurityContext ctx = securityService.authenticate(request.getHeader("Authorization"));
            setCurrentSecurityContext(ctx);
            return true;
        } catch (AuthenticationException e) {
            return false;
        }
    }

    public static void setCurrentSecurityContext(SecurityContext ctx) {
        threadLocal.set(ctx);            
    }
    
    public static SecurityContext getCurrentSecurityContext() {
        return threadLocal.get();
    }
    
    private void init() {
        if (securityService == null) {
            if (repositoryService != null) {
                securityService = repositoryService.getSecurityService();
            }
        }
    }

}
