/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.pac;

import java.util.logging.Level;
import java.util.logging.Logger;

import com.iona.repository.event.DataChangeAdapter;
import com.iona.repository.event.DataChangeEventObserver;
import com.iona.soa.model.SOA_NetworkPackage;
import com.iona.soa.model.events.DataChangeEvent;
import com.iona.soa.model.events.DataChangeKind;

public class RepositoryChangeListener implements DataChangeEventObserver {
    
    private static final Logger LOG = Logger.getLogger(RepositoryChangeListener.class.getName());
    private final ProvisioningDataProvider provisioningDataProvider;
    
    public RepositoryChangeListener(ProvisioningDataProvider pdp) {
        getEventPublisher().registerObserver(this);
        provisioningDataProvider = pdp;
        
        LOG.fine("Registered " + DataChangeEventObserver.class.getName());
    }

    public void dataChanged(DataChangeEvent dce) {
        LOG.fine("Data change event received: " + dce);

        try {
            if (SOA_NetworkPackage.eINSTANCE.getDeploymentProfile().equals(dce.getTargetType())) {
                provisioningDataProvider.refreshProfile(dce.getTargetGuid(),
                                                        dce.getChangeKind() == DataChangeKind.REMOVE_LITERAL);
            } else if (SOA_NetworkPackage.eINSTANCE.getApplication().equals(dce.getTargetType())) {
                provisioningDataProvider.refreshFeature(dce.getTargetGuid(),
                                                        dce.getChangeKind() == DataChangeKind.REMOVE_LITERAL);
            } else if (SOA_NetworkPackage.eINSTANCE.getContainer().equals(dce.getTargetType())) {
                provisioningDataProvider.refreshAgent(dce.getTargetGuid(),
                                                      dce.getChangeKind() == DataChangeKind.REMOVE_LITERAL);
            } else if (SOA_NetworkPackage.eINSTANCE.getArtifact().equals(dce.getTargetType())) {
                provisioningDataProvider.refreshArtifact(dce.getTargetGuid(),
                                                        dce.getChangeKind() == DataChangeKind.REMOVE_LITERAL);
            }
        } catch (Exception e) {
            LOG.log(Level.WARNING, "Problem applying repository data update ", e);
        }
    }

    public void destroy() {
        getEventPublisher().unregisterObserver(this);
        LOG.fine("Unregistered " + DataChangeEventObserver.class.getName());
    }
    
    DataChangeAdapter getEventPublisher() {
        return DataChangeAdapter.getInstance();
    }    
}
