/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.agentprovider.osgi;

import java.util.Dictionary;
import java.util.Hashtable;
import java.util.logging.Logger;

import com.iona.soa.repository.configuration.ConfigurationService;
import com.iona.soa.security.SecurityService;
import com.iona.soa.web.repository.agentprovider.providers.IAgentFileProviderFactory;
import com.iona.soa.web.repository.agentprovider.providers.SMX3FileProvider;
import com.iona.soa.web.repository.agentprovider.providers.SMX4FileProvider;
import com.iona.soa.web.repository.agentprovider.providers.TomcatFileProvider;
import com.iona.soa.web.repository.agentprovider.servlet.AgentProviderServlet;

import org.osgi.service.http.HttpService;

public class AgentProviderServletBean {

    private static final Logger LOG = Logger.getLogger(AgentProviderServletBean.class.getName());

    private HttpService httpService;
    private ConfigurationService configurationService;
    private SecurityService securityService;
    private IAgentFileProviderFactory agentFileProviderFactory;
    
    public SecurityService getSecurityService() {
        return securityService;
    }

    public void setSecurityService(SecurityService securityService) {
        this.securityService = securityService;
    }


    public IAgentFileProviderFactory getAgentFileProviderFactory() {
        return agentFileProviderFactory;
    }

    public void setAgentFileProviderFactory(IAgentFileProviderFactory agentFileProviderFactory) {
        this.agentFileProviderFactory = agentFileProviderFactory;
    }

    public void start() {
        try {
            registerInternalProviders();
            AgentProviderServlet agentServlet = 
                new AgentProviderServlet(agentFileProviderFactory,
                                         securityService,
                                         configurationService);
            Dictionary<String, Object> initParams = new Hashtable<String, Object>();
            httpService.registerServlet("/agents/*", agentServlet, initParams, null);
        } catch (Exception e) {
            LOG.warning("Error registering agent provider servlet: " + e.toString());
        }
    }
    
    private void registerInternalProviders() {
        agentFileProviderFactory.registerAgentFileProvider("SMX4",
                                                           new SMX4FileProvider(configurationService));
        agentFileProviderFactory.registerAgentFileProvider("SMX3",
                                                           new SMX3FileProvider(configurationService));
        agentFileProviderFactory.registerAgentFileProvider("TOMCAT",
                                                           new TomcatFileProvider(configurationService));
    }

    public void stop() {
        try {
            httpService.unregister("/agents/*");
        } catch (Exception e) {
            LOG.warning("Error unregistering agent provider servlet: " + e.toString());
        }
    }

    public HttpService getHttpService() {
        return httpService;
    }

    public void setHttpService(HttpService httpService) {
        this.httpService = httpService;
    }

    public ConfigurationService getConfigurationService() {
        return configurationService;
    }

    public void setConfigurationService(ConfigurationService configService) {
        this.configurationService = configService;
    }

}
