/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression;

public class Operation implements Term {
    private final OperationType name; 
    private final Operand[] arguments;
    
    public Operation(OperationType type, Object ... args) {
        this.name = type;
        arguments = new Operand[args.length];
        for (int i = 0; i < args.length; i++) {
            if (args[i] instanceof Operand) {
                arguments[i] = (Operand)args[i];
            } else if (args[i] instanceof Term) {
                throw new IllegalArgumentException("Only Operands allowed as arguments: " + args[i]);
            } else {
                arguments[i] = new Operand(args[i]);
            }
        }

        if (name.numArgs != -1 && arguments.length != name.numArgs) {
            throw new IllegalArgumentException(getAsString() 
                + " - Incorrect number of arguments passed to Operation " + name + ": " + arguments.length);  
        }        
    }
    
    public OperationType getName() {
        return name;
    }
    
    public Operand[] getArguments() {
        return arguments;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;            
        }
        
        if (!(obj instanceof Operation)) {
            return false;
        }
        
        Operation o = (Operation) obj;
        
        if (arguments.length != o.arguments.length) {
            return false;
        }
        for (int i = 0; i < arguments.length; i++) {
            if (!(arguments[i].toString().equals(o.arguments[i].toString()))) {
                return false;
            }
        }
        
        return name == o.name; 
    }

    @Override
    public int hashCode() {
        int hc = name.hashCode();
        for (Operand arg : arguments) {
            hc += 17 * arg.hashCode();
        }
        return hc;
    }

    public String toString() {
        return getAsString();
    }
    private String getAsString() {
        StringBuffer buf = new StringBuffer(name.toString());
        buf.append(" [");
        boolean first = true;
        for (Operand op : arguments) {
            if (!first) {
                buf.append(", ");
            }
            first = false;
            buf.append(op.getValue().toString());
        }
        buf.append("]");
        return buf.toString();
    }
}
