/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.runtime;

import java.lang.reflect.Constructor;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.iona.soa.expression.Operator;
import com.iona.soa.repository.util.ConstructorResolver;

public final class DelegatingFactory {
    private static Map<String, String> factoriesRegistry = 
        new HashMap<String, String>();
    private static Map<String, Factory> factories = 
        new HashMap<String, Factory>();
    private static String defaultNs = "model";
    private static String ns2factory = "ns2factory.properties";
    private static final Logger LOG = Logger.getLogger(DelegatingFactory.class.getName());
    
    private DelegatingFactory() {
        // complete
    }
    
    static {
        try {
            URL url = DelegatingFactory.class.getResource(ns2factory);
            if (url != null) {
                Properties mapping = new Properties();
                mapping.load(url.openStream());
                for (Map.Entry<Object, Object> entry : mapping.entrySet()) {
                    String ns = (String)entry.getKey();
                    String factory = (String)entry.getValue();
                    register(ns, factory);
                }
            }
        } catch (Exception e) {
            LOG.log(Level.SEVERE, "Failed to resolve expression factories mapping file", e);
        }
    }
    
    public static ExpressionRuntime create(Operator op) {
        return create(defaultNs, op);
    }
    
    @SuppressWarnings("unchecked")
    public static ExpressionRuntime create(String ns, Operator op) {
        Factory factory = factories.get(ns);
        if (factory == null) {
            String name = factoriesRegistry.get(ns);
            Class<? extends Factory> c = null;
            try {
                c = (Class<? extends Factory>)Class.forName(name);
            } catch (Exception e) {
                // log and ignore
            }
            if (c == null) {
                throw new IllegalArgumentException("Failed to create an function" 
                                                   + " implementation with name '" 
                                                   + op.getLiteral()
                                                   + "' and namesapce '" + ns + "'");
            }
            Constructor<? extends Factory> ctor = ConstructorResolver.resolve(c, new Class[]{});
            if (ctor == null) {
                throw new IllegalArgumentException("Failed to create an function" 
                                                   + " implementation with name '" 
                                                   + op.getLiteral()
                                                   + "' and namesapce '" + ns + "'");
            }
            try {
                factory = (Factory)ctor.newInstance(new Object[]{});
                factories.put(ns, factory);
            } catch (Exception e) {
                throw new IllegalArgumentException("Failed to create an function" 
                                                   + " implementation with name '" 
                                                   + op.getLiteral()
                                                   + "' and namesapce '" + ns + "'");
            }
        }
        return factory.create(op);
    }
    
    public static void register(String namespace, String name) {
        factoriesRegistry.put(namespace, name);
    }
    
    public static void unregister(String namespace) {
        factoriesRegistry.remove(namespace);
        factories.remove(namespace);
    }
}
