/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.runtime.model;

import java.util.Map;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operand;
import com.iona.soa.expression.Operator;
import com.iona.soa.expression.Term;
import com.iona.soa.expression.runtime.DelegatingFactory;
import com.iona.soa.expression.runtime.ExpressionRuntime;

public class And implements ExpressionRuntime {

    @SuppressWarnings("unchecked")
    public void run(Expression exp, Map context) {
        for (Term term : exp.getOperands()) {
            if (term instanceof Operand) {
                Operand o = (Operand)term;
                String value = o.getValue().toString();
                if (value.equalsIgnoreCase(Boolean.TRUE.toString()) 
                    || value.equalsIgnoreCase(Boolean.FALSE.toString())) {
                    if (Boolean.parseBoolean(value)) {
                        continue;
                    } else {
                        context.put(ContextKey.RESULT, Boolean.FALSE);
                        return;
                    }
                } else {
                    throw new IllegalArgumentException("Illegal argument passed into function " 
                                                       + Operator.AND.getLiteral()
                                                       + "; argument must have value of "
                                                       + "'true' or 'false' but was: " 
                                                       + term.toString());
                }
            } else if (term instanceof Expression) {
                Expression e = (Expression)term;
                ExpressionRuntime runtime = DelegatingFactory.create(e.getOperator());
                runtime.run(e, context);
                Object result = context.get(ContextKey.RESULT);
                if (!(result instanceof Boolean)) {
                    throw new IllegalArgumentException("Illegal return type from an" 
                                                       + " argument of function " 
                                                       + Operator.AND.getLiteral()
                                                       + "; return value type must be of type "
                                                       + " of type boolean but was " 
                                                       + result.toString());  
                }
                if (result.equals(Boolean.FALSE)) {
                    context.put(ContextKey.RESULT, Boolean.FALSE);
                    return;
                }
            } 
        }
        context.put(ContextKey.RESULT, Boolean.TRUE);
    }
}
