/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.runtime.model;

import java.util.Collection;
import java.util.LinkedList;
import java.util.Map;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operand;
import com.iona.soa.expression.Operator;
import com.iona.soa.expression.Term;
import com.iona.soa.expression.runtime.DelegatingFactory;
import com.iona.soa.expression.runtime.ExpressionRuntime;

import org.eclipse.emf.ecore.EObject;

public class DanglingReferences implements ExpressionRuntime {
    
    @SuppressWarnings("unchecked")
    public void run(Expression exp, Map context) {
        Collection<EObject> results = new LinkedList<EObject>(); 
        for (Term term : exp.getOperands()) {
            String value;
            if (term instanceof Expression) {
                Expression e = (Expression)term;
                ExpressionRuntime runtime = DelegatingFactory.create(e.getOperator());
                runtime.run(e, context);
                Object result = context.get(ContextKey.RESULT);
                if (!(result instanceof String) && !(result instanceof Operand)) {
                    throw new IllegalArgumentException("Illegal argument passed into function '" 
                                               + Operator.DANGLING_REFERENCES.getLiteral()
                                               + "'; argument must resolve to a string");
                }
                value = result.toString();
            } else {
                value = term.toString();
            }
            if (value.split("\\.").length < 2) {
                throw new IllegalArgumentException("Illegal argument passed into function '" 
                                                   + Operator.DANGLING_REFERENCES.getLiteral()
                                                   + "'; argument must resolve to a string of form: "
                                                   + " 'reference_name{.reference_name}+'" 
                                                   + " but was " + value);
            }

            Expression e = 
                buildExpression(new Operand(value), 
                                new Operand(value.substring(0, value.lastIndexOf('.'))), 
                                context);
            ExpressionRuntime runtime = DelegatingFactory.create(e.getOperator());
            runtime.run(e, context);
            Object result = context.get(ContextKey.STACKSET);
            if (result instanceof Collection) {
                results.addAll((Collection<EObject>)result);
            }
        }
        context.put(ContextKey.RESULT, results);
    }
    
    protected static Expression buildExpression(Term inTerm, Term refTerm, Map context) {
        Expression envExp = new Expression(Operator.ENV, ContextKey.SET);
        Expression inExp = new Expression(Operator.IN, envExp, inTerm);
        Expression refExp = new Expression(Operator.REFERENCES, refTerm);
        Expression ifExp = new Expression(Operator.IF, inExp, refExp);
        return ifExp;
    }
}
