/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.runtime.model;

import java.util.Map;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.runtime.ExpressionRuntime;

public class Equals implements ExpressionRuntime {

    @SuppressWarnings("unchecked")
    public void run(Expression op, Map context) {
        Object lhs = Util.evaluate(op.getOperands()[0], context);
        Object rhs = Util.evaluate(op.getOperands()[1], context);
        if (lhs == Constants.NULL || rhs == Constants.NULL) {
            if (lhs == rhs) {
                context.put(ContextKey.RESULT, Boolean.TRUE);
            } else {
                context.put(ContextKey.RESULT, Boolean.FALSE);
            }
        } else if (equals(lhs, rhs)) {
            context.put(ContextKey.RESULT, Boolean.TRUE);
        } else {
            context.put(ContextKey.RESULT, Boolean.FALSE);
        }
    }
    
    protected static boolean equals(Object lhs, Object rhs) {
        if (lhs.getClass() == rhs.getClass()) {
            return lhs.equals(rhs);
        }
        if (rhs.getClass() == String.class) {
            boolean result = false;
            boolean match = false;
            String value = (String)rhs;
            if (lhs.getClass() == Boolean.class) {
                result = lhs.equals(Boolean.parseBoolean(value));
                match = true;
            } else if (lhs.getClass() == Byte.class) {
                result = lhs.equals(Byte.valueOf(value));
                match = true;
            } else if (lhs.getClass() == Double.class) {
                result = lhs.equals(Double.valueOf(value));
                match = true;
            } else if (lhs.getClass() == Float.class) {
                result = lhs.equals(Float.valueOf(value));
                match = true;
            } else if (lhs.getClass() == Integer.class) {
                result = lhs.equals(Integer.valueOf(value));
                match = true;
            } else if (lhs.getClass() == Long.class) {
                result = lhs.equals(Long.valueOf(value));
                match = true;
            } else if (lhs.getClass() == Short.class) {
                result = lhs.equals(Short.valueOf(value));
                match = true;
            }
            if (match) {
                return result;
            }
        }
        return lhs.equals(rhs);
    }
}
