/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.runtime.model;

import java.util.Map;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operand;
import com.iona.soa.expression.Operator;
import com.iona.soa.expression.Term;
import com.iona.soa.expression.runtime.DelegatingFactory;
import com.iona.soa.expression.runtime.ExpressionRuntime;

public class If implements ExpressionRuntime {
    public void run(Expression exp, Map context) {
        Term term = exp.getOperands()[0];
        if (term instanceof Operand) {
            Operand o = (Operand)term;
            String value = o.getValue().toString();
            if (value.equalsIgnoreCase(Boolean.TRUE.toString()) 
                || value.equalsIgnoreCase(Boolean.FALSE.toString())) {
                if (Boolean.parseBoolean(value)) {
                    runBody(exp.getOperands(), context);
                } 
            } else {
                throw new IllegalArgumentException("Illegal argument passed into function " 
                                                   + Operator.IF.getLiteral()
                                                   + "; the argument's value has to be "
                                                   + " 'true' or 'false' but was " 
                                                   + value.toString());
            }
        } else if (term instanceof Expression) {
            Expression e = (Expression)term;
            ExpressionRuntime runtime = DelegatingFactory.create(e.getOperator());
            runtime.run(e, context);
            Object result = context.get(ContextKey.RESULT);
            if (result instanceof Boolean && result.equals(Boolean.TRUE)) {
                runBody(exp.getOperands(), context);
            }
        }
    }
    
    @SuppressWarnings("unchecked")
    protected void runBody(Term[] args, Map context) {
        for (int i = 1; i < args.length; ++i) {
            Term term = args[i];
            if (term instanceof Expression) {
                Expression e = (Expression)term;
                ExpressionRuntime runtime = DelegatingFactory.create(e.getOperator());
                runtime.run(e, context);
            }
        }
    }
}
