/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.runtime.model;

import java.util.Collection;
import java.util.Map;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operator;
import com.iona.soa.expression.runtime.ExpressionRuntime;

/*
 * in(where, what)
 * where - Collection
 * what - EAttribute, EObject, Collection
 */ 
public class In implements ExpressionRuntime {

    @SuppressWarnings("unchecked")
    public void run(Expression op, Map context) {
        Object lhs = Util.evaluate(op.getOperands()[0], context);
        Object rhs = Util.evaluate(op.getOperands()[1], context);
        if (lhs == Constants.NULL) {
            throw new NullPointerException("Argument passed into function " 
                                           + Operator.IN.getLiteral()
                                           + " has a null value; "
                                           + "argument " 
                                           + op.getOperands()[0].toString());
        }
        if (lhs instanceof Collection) {
            if (!(rhs instanceof Collection)) {
                in((Collection)lhs, rhs, context);
            } else if (rhs instanceof Collection) {
                in((Collection)lhs, (Collection)rhs, context);
            } else {
                throw new IllegalArgumentException("Illegal argument(s) passed into function " 
                                                   + Operator.IN.getLiteral()
                                                   + " second's argument type can"
                                                   + " can be List, EObject or EAttribute of arg1's"
                                                   + " Collection enclosed type");
            }
        } else {
            throw new IllegalArgumentException("Illegal argument(s) passed into function " 
                                            + Operator.IN.getLiteral()
                                            + " first argument's type must be List");
        }
    }

    @SuppressWarnings("unchecked")
    protected static void in(Collection where, Object what, Map context) {
        if (where.contains(what)) {
            context.put(ContextKey.RESULT, Boolean.TRUE);
        } else {
            context.put(ContextKey.RESULT, Boolean.FALSE);
        }
    }

    @SuppressWarnings("unchecked")
    protected static void in(Collection where, Collection what, Map context) {
        if (where.containsAll(what)) {
            context.put(ContextKey.RESULT, Boolean.TRUE);
        } else {
            context.put(ContextKey.RESULT, Boolean.FALSE);
        }
    }
}

