/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.runtime.model;

import java.util.Map;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operator;
import com.iona.soa.expression.runtime.ExpressionRuntime;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

public class InstanceOf implements ExpressionRuntime {

    @SuppressWarnings("unchecked")
    public void run(Expression op, Map context) {
        Object lhs = Util.evaluate(op.getOperands()[0], context);
        Object rhs = null;
        if (op.getOperands()[1] instanceof Expression) {
            rhs = Util.evaluate(op.getOperands()[1], context);
        } else {
            rhs = op.getOperands()[1].toString();
        }
        if (!(rhs instanceof String)) {
            throw new IllegalArgumentException("Illegal argument passed into function " 
                                               + Operator.INSTANCEOF.getLiteral()
                                               + "; Second argument must resolve to a string"
                                               + "; Argument : " + rhs.toString());
        }
        String type = (String)rhs;
        if (lhs == Constants.NULL) {
            context.put(ContextKey.RESULT, Boolean.FALSE);
            return;
        }
        if (lhs instanceof EObject) {
            EObject eobj = (EObject)lhs;
            EClass expectedType = Util.getEClass(type, context);
            if (expectedType == eobj.eClass() 
                || eobj.eClass().getEAllSuperTypes().contains(expectedType)) {
                context.put(ContextKey.RESULT, Boolean.TRUE);
            } else {
                context.put(ContextKey.RESULT, Boolean.FALSE);
            }
        } else {
            try {
                Class<?> cls = Class.forName(type);
                if (cls.isAssignableFrom(lhs.getClass())) {
                    context.put(ContextKey.RESULT, Boolean.TRUE);
                } else {
                    context.put(ContextKey.RESULT, Boolean.FALSE);    
                }
            } catch (Exception e) {
                throw new IllegalArgumentException("Failed to resolve type '" + type + "'", e);
            }
        }
    }
}
