/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.runtime.model;

import java.util.Collection;
import java.util.LinkedList;
import java.util.Map;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operand;
import com.iona.soa.expression.Operator;
import com.iona.soa.expression.Term;
import com.iona.soa.expression.runtime.ExpressionRuntime;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.ecore.EObject;

public class References implements ExpressionRuntime {

    @SuppressWarnings("unchecked")
    public void run(Expression op, Map context) {
        EObject eobj = (EObject)context.get(ContextKey.EOBJECT);
        Collection<EObject> set = (Collection<EObject>)context.get(ContextKey.SET);
        if (set == null) {
            set = new LinkedList<EObject>();
            context.put(ContextKey.SET, set);
        }
        Collection<EObject> stackSet = (Collection<EObject>)context.get(ContextKey.STACKSET);
        if (stackSet == null) {
            stackSet = new LinkedList<EObject>();
            context.put(ContextKey.STACKSET, stackSet);
        }
        Collection<EObject> result = new LinkedList<EObject>();
        context.put(ContextKey.RESULT, result);
        for (Term term : op.getOperands()) {
            Object value = Util.evaluate(term, context);
            if (value instanceof String) {
                value = Util.evaluate(new Operand(value), context);
            }
            if (value == Constants.NULL) {
                continue;            
            }
            if (!(value instanceof BasicEList) && !(value instanceof EObject)) {
                throw new IllegalArgumentException("Illegal argument passed into function " 
                                                   + Operator.REFERENCES.getLiteral()
                                                   + "; Argument must resolve to a collection of" 
                                                   + " or a single EObject; Argument: " 
                                                   + term.toString());
            }
            
            if (value instanceof BasicEList) {
                Object[] data = ((BasicEList)value).data();
                if (data != null && data.length > 0 
                    && !EObject.class.isAssignableFrom(data[0].getClass())) {
                    throw new IllegalArgumentException("Illegal argument passed into function " 
                                                       + Operator.REFERENCES.getLiteral()
                                                       + "; Argument must resolve to a collection of" 
                                                       + " EObject; Argument: " 
                                                       + term.toString());
                
                }
                for (EObject o : (Collection<EObject>)value) {
                    result.add(o);
                    if (!set.contains(o)) {
                        set.add(o);
                        stackSet.add(o);
                    }
                }
                
            } else {
                EObject o = (EObject)value;
                result.add(o);
                if (!set.contains(o)) {
                    set.add(o);
                    stackSet.add(o);                
                }
            }
        }
        context.put(ContextKey.RESULT, result);
    }
}
