/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.runtime.model;

import java.util.List;
import java.util.Map;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operand;
import com.iona.soa.expression.Operator;
import com.iona.soa.expression.Term;
import com.iona.soa.expression.runtime.DelegatingFactory;
import com.iona.soa.expression.runtime.ExpressionRuntime;
import com.iona.soa.repository.EClassifierDescriptor;
import com.iona.soa.repository.EMFUtil;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;

public final class Util {
    private Util() {
        // complete
    }

    @SuppressWarnings("unchecked")
    protected static Object getReferent(String refPath, Map context) {
        EObject eobj = (EObject)context.get(ContextKey.EOBJECT);
        Object result = Constants.NULL;
        for (String ref : refPath.trim().split("\\.")) {
            if (result != Constants.NULL && !(result instanceof EObject)) {
                throw new IllegalArgumentException("Illegal argument passed into function" 
                                                   + "; Argument indirection  operator '.'"
                                                   + " follows an argument segment that resolved"
                                                   + " to either a list or attribute;"
                                                   + "; Argument: " + ref);
            }
            if (ref.equals(Constants.ECONTAINER.getLiteral())) {
                eobj = eobj.eContainer();
                result = eobj;
                continue;
            }
            boolean listItem = false;
            int index = -1;
            String reference = ref;
            if (ref.indexOf(Constants.INDEX.getLiteral()) != -1) {
                listItem = true;
                if (ref.length() == ref.indexOf(Constants.INDEX.getLiteral()) + 1) {
                    throw new IllegalArgumentException("Illegal argument passed into function" 
                                                       + "; The argument expressing array "
                                                       + "needs to have a form of "
                                                       + "'name@coordinate' but was "
                                                       + ref);
                }
                String indexLiteral = ref.substring(ref.indexOf(Constants.INDEX.getLiteral()) + 1);
                int end = (indexLiteral.indexOf(".") != -1) 
                            ? indexLiteral.indexOf(".") : indexLiteral.length();
                indexLiteral = indexLiteral.substring(0, end);
                index = Integer.valueOf(indexLiteral).intValue();
                reference = ref.substring(0, ref.indexOf(Constants.INDEX.getLiteral()));
            }
            EStructuralFeature f = eobj.eClass().getEStructuralFeature(reference);
            if (f instanceof EReference) {
                EReference eref = (EReference)f;
                if (eref.isMany()) {
                    List<EObject> list = (List<EObject>)eobj.eGet(eref);
                    if (listItem) {
                        if (index >= list.size()) {
                            throw new IllegalArgumentException("Failed to resolve array element"
                                                               + " as its out list's range. Requested" 
                                                               + " index is " + index + " while list"
                                                               + " lenght is " + list.size() 
                                                               + "; Argument:  " + ref);
                        }
                        eobj = list.get(index);
                        result = eobj;
                    } else {
                        result = list;
                    }
                } else {
                    eobj = (EObject)eobj.eGet(f);
                    result = eobj;
                }
            } else if (f instanceof EAttribute) {
                EAttribute eattr = (EAttribute)f;
                if (eattr.isMany()) {
                    List list = (List)eobj.eGet(eattr);
                    eobj = null;                        
                    if (listItem) {
                        if (index >= list.size()) {
                            throw new IllegalArgumentException("Failed to resolve array element"
                                                               + " as its out list's range. Requested" 
                                                               + " index is " + index + " while list"
                                                               + " lenght is " + list.size() 
                                                               + "; Argument:  " + ref);
                        }
                        result = list.get(index);
                    } else {
                        result = list;
                    }
                } else {
                    result = eobj.eGet(eattr);
                    eobj = null;
                }
            } else {
                throw new IllegalArgumentException("Illegal argument passed into function" 
                                                   + "; The argument didn't resolve"
                                                   + " to a known attribute or reference"
                                                   + "; Argument: " + reference);
            }
        }
        return (result != null) ? result : Constants.NULL;
    }
    
    // 'exp' can be of the form: class_name | {package_namespace}class_name
    @SuppressWarnings("unchecked")
    public static EClass getEClass(String exp, Map context) {
        EObject eobj = (EObject)context.get(ContextKey.EOBJECT);
        EPackage ePackage = eobj.eClass().getEPackage();
        EClass eClass = null;

        EClassifierDescriptor desc = getEClassifierDescriptor(exp, ePackage);
        ePackage = (desc.getPackageNsURI().equals(ePackage.getNsURI())) 
            ? ePackage : EMFUtil.getDefaultModelRegistry().getPackage(desc.getPackageNsURI()); 

        if (ePackage == null) {
            throw new IllegalArgumentException("Function argument '" + exp 
                                               + "' does not resolve to a known package");
        }
        for (EObject eo : (List<EObject>)ePackage.eContents()) {
            if (eo instanceof EClass && ((EClass)eo).getName().equals(desc.getName())) {
                eClass = (EClass)eo;
                break;
            }
        }
        if (eClass == null) {
            throw new IllegalArgumentException("Function argument '" + exp 
                                               + "' does not resolve to a known class");
        }
        return eClass;
    }
    
    public static EClassifierDescriptor getEClassifierDescriptor(String exp, 
                                                                 EPackage ePackage) {
        EClassifierDescriptor desc = null;
        String open = Constants.ECLASS_NS_OPEN_DELIM.getLiteral();
        String close = Constants.ECLASS_NS_CLOSE_DELIM.getLiteral();
        if (!exp.startsWith(open)) {
            desc = new EClassifierDescriptor(ePackage.getNsURI(), exp);
        } else if (exp.indexOf(close) != -1) {
            String ns  = exp.substring(open.length(), exp.indexOf(close));
            String name = exp.substring(exp.indexOf(close) + close.length());
            desc = new EClassifierDescriptor(ns, name);
        }
        
        if (desc == null) {
            throw new IllegalArgumentException("Illegal argument passed into function " 
                                               + Operator.INSTANCEOF.getLiteral()
                                               + "; argument is: '" + exp 
                                               + " The value has to be of the form: "
                                               + " {package_ns_uri}class_name"
                                               + " where {package_ns_uri} is optional if named class"
                                               + " resides in the same package as class of the object " 
                                               + " that owns this annotaion");
        }
        return desc;
    }
    
    @SuppressWarnings("unchecked")
    public static Object evaluate(Term term, Map context) {
        Object result = Constants.NULL;
        if (term instanceof Operand) {
            Operand op = (Operand)term;
            String value = op.getValue().toString();
            if (!Constants.NULL.getLiteral().equalsIgnoreCase(value)) {
                if (value.equalsIgnoreCase(Boolean.TRUE.toString()) 
                    || value.equalsIgnoreCase(Boolean.FALSE.toString())) {
                    if (Boolean.parseBoolean(value)) {
                        result = Boolean.TRUE;
                    } else {
                        result = Boolean.FALSE;
                    }
                } else if (value.startsWith(Constants.LITERAL_DELIM.getLiteral())) {
                    String[] tokens = value.split(Constants.LITERAL_DELIM.getLiteral());
                    result = (tokens.length > 1) ? tokens[1] : "";
                } else {
                    result = Util.getReferent(value, context);
                }
            }
        } else if (term instanceof Expression) {
            Expression e = (Expression)term;
            ExpressionRuntime runtime = DelegatingFactory.create(e.getOperator());
            runtime.run(e, context);
            result = context.get(ContextKey.RESULT);
        }
        return result;
    }
}
