/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.runtime.model.sql;

import java.util.Map;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operand;
import com.iona.soa.expression.Operator;
import com.iona.soa.expression.Term;
import com.iona.soa.expression.runtime.DelegatingFactory;
import com.iona.soa.expression.runtime.ExpressionRuntime;
import com.iona.soa.expression.runtime.model.ContextKey;
import com.iona.soa.expression.runtime.model.Util;

public class Equals implements ExpressionRuntime {

    @SuppressWarnings("unchecked")
    public void run(Expression op, Map context) {
        String lhs = evaluateLsh(op.getOperands()[0], context);
        Object rhs = Util.evaluate(op.getOperands()[1], context);
        context.put(ContextKey.RESULT, new com.iona.soa.expression.Equals(lhs, rhs));
    }
    
    protected static String evaluateLsh(Term term, Map context) {
        if (term instanceof Operand) {
            return ((Operand)term).getValue().toString();
        } else if (term instanceof Expression) {
            Expression e = (Expression)term;
            ExpressionRuntime runtime = DelegatingFactory.create(e.getOperator());
            runtime.run(e, context);
            Object result = context.get(ContextKey.RESULT);
            if (!(result instanceof String)) {
                throw new IllegalArgumentException("Illegal return type from an" 
                                                   + " argument of function " 
                                                   + Operator.EQUALS.getLiteral()
                                                   + "; return value type must be "
                                                   + " of type String but was " 
                                                   + result.toString());
            }
            return (String)result;
        }
        return null;
    }
}
