/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.runtime.model.sql;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Map;

import com.iona.soa.expression.Operator;
import com.iona.soa.expression.runtime.ExpressionRuntime;
import com.iona.soa.expression.runtime.Factory;
import com.iona.soa.repository.util.ConstructorResolver;

public class PackageFactory implements Factory {

    private static Map<Operator, Class<? extends ExpressionRuntime>> registry =  
        new HashMap<Operator, Class<? extends ExpressionRuntime>>();
    private static String namespace = "sql";
    
    static {
        registry.put(Operator.SELECT, Select.class);
        registry.put(Operator.EQUALS, Equals.class);
    }

    @SuppressWarnings("unchecked")
    public ExpressionRuntime create(Operator op) {
        Class<? extends ExpressionRuntime> c = registry.get(op);
        if (c == null) {
            throw new IllegalArgumentException("Failed to create an function" 
                                               + " implementation with name '" 
                                               + op.getLiteral()
                                               + "' and namesapce '" + getNamespace() + "'");
        }
        Constructor<? extends ExpressionRuntime> ctor = ConstructorResolver.resolve(c, new Class[]{});
        if (ctor == null) {
            throw new IllegalArgumentException("Failed to create an function" 
                                               + " implementation with name '" 
                                               + op.getLiteral()
                                               + "' and namesapce '" + getNamespace() + "'");
        }
        try {
            return (ExpressionRuntime)ctor.newInstance(new Object[]{});
        } catch (Exception e) {
            throw new IllegalArgumentException("Failed to create an function" 
                                               + " implementation with name '" 
                                               + op.getLiteral()
                                               + "' and namesapce '" + getNamespace() + "'");
        }
    }

    public String getNamespace() {
        return namespace;
    }
}
