/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.runtime.model.sql;

import java.util.Collection;
import java.util.LinkedList;
import java.util.Map;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operand;
import com.iona.soa.expression.Operator;
import com.iona.soa.expression.Term;
import com.iona.soa.expression.runtime.DelegatingFactory;
import com.iona.soa.expression.runtime.ExpressionRuntime;
import com.iona.soa.expression.runtime.model.ContextKey;
import com.iona.soa.expression.runtime.model.Util;
import com.iona.soa.resourceset.ResourceSetQueryHandler;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

public class Select implements ExpressionRuntime {
    private static String namespace = "sql";
    
    @SuppressWarnings("unchecked")
    public void run(Expression op, Map context) {
        EClass eclass = evaluateType(op.getOperands()[0], context);
        Expression where = evaluateWhere(op.getOperands()[1], context);
        EObject eobj = (EObject)context.get(ContextKey.EOBJECT);
        try {
            Collection<EObject> queryResult = 
                ResourceSetQueryHandler.executeQuery(eobj.eResource().getResourceSet(),
                                                     eclass, where, true);
            Collection<EObject> set = (Collection<EObject>)context.get(ContextKey.SET);
            if (set == null) {
                set = new LinkedList<EObject>();
                context.put(ContextKey.SET, set);
            }
            Collection<EObject> stackSet = (Collection<EObject>)context.get(ContextKey.STACKSET);
            if (stackSet == null) {
                stackSet = new LinkedList<EObject>();
                context.put(ContextKey.STACKSET, stackSet);
            }
            Collection<EObject> result = new LinkedList<EObject>();
            context.put(ContextKey.RESULT, result);
            
            for (EObject o : queryResult) {
                result.add(o);
                if (!set.contains(o)) {
                    set.add(o);
                    stackSet.add(o);
                }
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
    
    protected static EClass evaluateType(Term term, Map context) {
        if (term instanceof Expression) {
            Expression e = (Expression)term;
            ExpressionRuntime runtime = DelegatingFactory.create(e.getOperator());
            runtime.run(e, context);
            Object result = context.get(ContextKey.RESULT);
            if (result instanceof EClass) {
                return (EClass)result;
            } else {
                throw new IllegalArgumentException("Illegal return type from an" 
                                                   + " argument of function " 
                                                   + Operator.SELECT.getLiteral()
                                                   + "; return value type must be "
                                                   + " of type EClass but was " 
                                                   + result.toString());
            }

        } else if (term instanceof Operand) {
            Operand op = (Operand)term;
            return Util.getEClass(op.getValue().toString(), context);
        }
        return null;
    }
    
    protected static Expression evaluateWhere(Term term, Map context) {
        if (term instanceof Expression) {
            Expression e = (Expression)term;
            ExpressionRuntime runtime = DelegatingFactory.create(namespace, e.getOperator());
            runtime.run(e, context);
            Object result = context.get(ContextKey.RESULT);
            if (!(result instanceof Expression)) {
                throw new IllegalArgumentException("Illegal return type from an" 
                                                   + " argument of function " 
                                                   + Operator.SELECT.getLiteral()
                                                   + "; return value type must be "
                                                   + " of type Expression but was " 
                                                   + result.toString());
            }
            return (Expression)result;
        } else {
            throw new IllegalArgumentException("Illegal argument passed into function " 
                                               + Operator.SELECT.getLiteral()
                                               + "; argumument must resolve to expression"
                                               + "; argument: " + term.toString());

        }
    }
}
