/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.serialization;

import java.util.Collection;

import javax.xml.namespace.QName;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Term;
import com.iona.soa.repository.util.Out;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EPackage;

public final class ExpressionSerializer {
    private static final String SEPARATOR = "?";
    
    private ExpressionSerializer() { }
    
    public static String serialize(Term expr) {
        SerializationHandler handler = new SerializationHandler();        
        handler.handle(expr);
        return handler.getSerialization();
    }
    
    public static String serialize(Term expr, EClass cls) {
        return new QName(cls.getEPackage().getNsURI(), cls.getName()) + SEPARATOR + serialize(expr);
    }
    
    public static Term deserialize(String str) {
        return DeserializationHandler.deserialize(str);
    }
    
    public static Collection<Expression> deserializeModelQuery(String str) {
        return ModelQueryDeserializationHandler.deserialize(str);
    }

    public static Term deserialize(String str, Out<EClass> cls) {
        int idx = str.indexOf(SEPARATOR);
        if (idx < 0) {
            return deserialize(str);
        }
        String p1 = str.substring(0, idx);
        String p2 = str.substring(idx + 1);
        Term expr = DeserializationHandler.deserialize(p2);
        
        QName qn = QName.valueOf(p1);
        EPackage pkg = EPackage.Registry.INSTANCE.getEPackage(qn.getNamespaceURI());
        EClass eClass = (EClass) pkg.getEClassifier(qn.getLocalPart());        
        cls.set(eClass);
        return expr;
    }
}
