/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.serialization;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operand;
import com.iona.soa.expression.Operation;
import com.iona.soa.expression.OperationDelegate;
import com.iona.soa.expression.Term;

public class SerializationHandler implements OperationDelegate {
    private StringBuffer buffer = new StringBuffer();
    
    public boolean handle(Operation op) {
        Term[] operands = new Term[2];
        operands[0] = new Operand(op.getName());
        StringBuilder value = new StringBuilder();
        boolean first = true;
        for (Operand arg : op.getArguments()) {
            if (first) {
                first = false;
            } else {
                value.append(";");
            }
            value.append(arg.getValue());
        }
        operands[1] = new Operand(value.toString());
        performEquals(operands);
        return true;
    }
    
    public boolean handle(Expression expr) {
        switch (expr.getOperator()) {
        case AND: 
            performAnd(expr.getOperands());
            break;
        case EQUALS:
            performEquals(expr.getOperands());
            break;
        case ISNULL:
            performIsNull(expr.getOperands());
            break;
        default:
            break;
        }
        return true;
    }



    public boolean handle(Operand op) {
        try {
            buffer.append(URLEncoder.encode(op.getValue().toString(), "UTF-8"));
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            buffer.append(op.getValue());
        }
        return true;
    }

    void handle(Term term) {
        if (term instanceof Operation) {
            handle((Operation) term);
        } else if (term instanceof Expression) {
            handle((Expression) term);
        } else if (term instanceof Operand) {
            handle((Operand) term);
        }
    }

    private void performAnd(Term[] operands) {
        boolean firstTime = true;
        for (Term operand : operands) {
            if (firstTime) {
                firstTime = false;
            } else {
                buffer.append("&");
            }
            
            handle(operand);
        }
    }

    private void performEquals(Term[] operands) {
        handle(operands[0]);
        buffer.append("=");
        handle(operands[1]);
    }
   
    private void performIsNull(Term[] operands) {
        buffer.append("isnull:");
        handle(operands[0]);
    }
    
    public String getSerialization() {
        return buffer.toString();
    }
}
