/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;

public class EPackageUtil {
    
    protected EPackageUtil() {
    }        
    
    public static List<EClass> getTopLevelEClasses(EPackage pkg) {
        List<EClass> allClasses = getAllEClasses(pkg);
        Map<EClass, List<EReference>> containedByMap = getContainmentMap(pkg);
        List<EClass> topLevelClasses = new ArrayList<EClass>();
        
        for (EClass cls : allClasses) {
            if (cls.isAbstract() || cls.isInterface()) {
                continue;
            }
            // If the class does not appear in the "containments" feature of any 
            // other class in the package, consider it as "top level"
            if (isContained(containedByMap, cls) == null) {
                topLevelClasses.add(cls);
            }
        }
        
        Comparator<EClass> comparator = new Comparator<EClass>() {
            public int compare(EClass a, EClass b) {
                return a.getName().compareTo(b.getName());
            }
        };
        Collections.sort(topLevelClasses, comparator);
        
        return topLevelClasses;
    }

    @SuppressWarnings("unchecked")
    private static List<EReference> isContained(Map<EClass, List<EReference>> containedByMap, EClass cls) {
        if (containedByMap.containsKey(cls)) {
            return containedByMap.get(cls);
        }
        
        for (EClass superType : (List<EClass>) cls.getEAllSuperTypes()) {
            if (containedByMap.containsKey(superType)) {
                return containedByMap.get(superType);
            }
        }
        return null;
    }
    
    public static Map<EClass, List<EReference>> getContainmentMap(EPackage pkg) {
        Map<EClass, List<EReference>> map = new HashMap<EClass, List<EReference>>();
        List<EClass> classes = getAllEClasses(pkg);
        
        for (EClass cls : classes) {
            EList containments = cls.getEAllContainments();
            for (Iterator it = containments.iterator(); it.hasNext();) {
                EReference contained = (EReference)it.next();
                EClass containedType = contained.getEReferenceType();
                List<EReference> containedIn = isContained(map, containedType);
                if (containedIn == null) {
                    containedIn = new ArrayList<EReference>();
                    map.put(containedType, containedIn);
                }
                
                containedIn.add(contained);
            }            
        }
        
        return map;
    }
    
    public static List<EClass> getAllEClasses(EPackage pkg) {
        List<EClass> eClasses = new ArrayList<EClass>();
        for (Object obj : pkg.eContents()) {
            if (obj instanceof EClass) {
                eClasses.add((EClass)obj);
            }
        }
        return eClasses;
    }
}
