/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operand;
import com.iona.soa.expression.Operation;
import com.iona.soa.expression.OperationDelegate;
import com.iona.soa.expression.Term;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;

/** This class stems from the SQLHandler class, the sole purpose if it is to see if 
 * an expression makes sense in the context of the model.
 */
public class ExpressionValidator implements OperationDelegate {
    private final EClass resultType; 
    private final StringBuffer whereString = new StringBuffer();
    
    public ExpressionValidator(EClass type) {
        resultType = type;
    }

    public boolean handle(Operation op) {
        throw new UnsupportedOperationException();
    }
    
    public boolean handle(Expression op, Map context) {
        throw new UnsupportedOperationException();
    }    

    public boolean handle(Operand op, Map context) {
        throw new UnsupportedOperationException();
    }    
    
    public boolean handle(Expression expr) {
        switch (expr.getOperator()) {
        case AND: 
            return performAnd(expr.getOperands());
        case EQUALS:
            return performEquals(expr.getOperands());
        default:
            return false;
        }
    }

    public boolean handle(Operand op) {
        if (whereString.length() > 0 
                && whereString.charAt(whereString.length() - 1) == '=') {
            // the rhs of an equation
            whereString.append('?');
            return true;
        } else {
            return handleEqualsLHS(op.getValue());
        }
    }
    
    private boolean handleEqualsLHS(Object value) {
        String address = value.toString();
        
        List<String> indirectRefs = new LinkedList<String>();
        StringTokenizer tok = new StringTokenizer(address, ".");
        while (tok.hasMoreTokens()) {
            indirectRefs.add(tok.nextToken());
        }

        String field = indirectRefs.remove(indirectRefs.size() - 1);
        EClass lastType = handleEqualsIndirections(indirectRefs);
        if (field == null || lastType == null) {
            return false;
        }
        
        return lastType.getEStructuralFeature(field) != null;
    }

    private EClass handleEqualsIndirections(List<String> indirectRefs) {
        EClass curType = resultType;
        for (String ref : indirectRefs) {
            curType = handleEqualsIndirection(ref, curType);
            if (curType == null) {
                return null;
            }
        }
        return curType;
    }

    private EClass handleEqualsIndirection(String ref, EClass curType) {
        EStructuralFeature feature = curType.getEStructuralFeature(ref);
        if (feature instanceof EReference) {
            EReference reference = (EReference) feature;

            if (reference.getEOpposite() != null) {
                return handleToManyBidirectional(curType, reference);
            } else {
                if (reference.isMany()) {                        
                    return handleToManyUnidirectional(curType, reference);
                } else {
                    return handleToOneUnidirectional(curType, reference);
                }
            }
        } else {
            return null;
        }
    }

    private EClass handleToManyBidirectional(EClass curType, EReference reference) {
        EClass oppositeType = reference.getEReferenceType();       
        return oppositeType;
    }

    private EClass handleToManyUnidirectional(EClass curType, EReference reference) {
        EClass oppositeType = reference.getEReferenceType();        
        return oppositeType;
    }

    private EClass handleToOneUnidirectional(EClass curType, EReference reference) {
        EClass oppositeType = reference.getEReferenceType();
        return oppositeType;
    }

    private boolean handle(Term term) {
        if (term instanceof Expression) {
            return handle((Expression) term);
        } else if (term instanceof Operand) {
            return handle((Operand) term);
        }
        return false;
    }
    
    private boolean performAnd(Term[] operands) {
        boolean ret = true;
        for (Term operand : operands) {
            ret &= handle(operand);
        }
        return ret;
    }

    private boolean performEquals(Term[] operands) {
        boolean ret = handle(operands[0]);
        whereString.append("=");
        ret &= handle(operands[1]);
        return ret;
    }
}
