/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository;

import org.eclipse.emf.ecore.EObject;

/** This interface provides a generic API to persist or unPersist EMF objects.
 * It is implemented by a provider capable of doing this, e.g. the 
 * {@link com.iona.soa.repository.jdbc.resource.JDBCResourceImpl}. 
 */
public interface Persister {
    /** Persist the provided object. Any dependent or related objects will automatically also be persisted.
     * @param obj The object to persist. 
     * @throws Exception In case of IO trouble.
     */
    void persist(EObject obj) throws Exception;
    
    
    /** Delete the object from persistent storage. Note that dependent objects will not automatically be
     * deleted. 
     * @param obj The object to delete.
     * @throws Exception In case of IO trouble.
     */
    void unPersist(EObject obj) throws Exception;


    /** The persister should be capable of collecting exceptions that could happen somewhere along the way.
     * @param e The exception to add to the list.
     */
    void addException(Exception e);
}
