/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.notification;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.InternalEList;

public abstract class AbstractNotificationAdapter implements Adapter {
    private boolean resolveProxies;
    private boolean handleResourceSets;
    
    public boolean isResolveProxies() {
        return resolveProxies;
    }

    public void setResolveProxies(boolean resolveProxies) {
        this.resolveProxies = resolveProxies;
    }

    public boolean isHandleResourceSets() {
        return handleResourceSets;
    }

    public void setHandleResourceSets(boolean handleResourceSets) {
        this.handleResourceSets = handleResourceSets;
    }

    public void setTarget(Notifier newTarget) {
        Iterator<? extends Notifier> contents = newTarget instanceof EObject 
            ? resolveProxies 
                ? ((EObject)newTarget).eContents().iterator()
                : ((InternalEList<EObject>)((EObject)newTarget).eContents()).basicIterator()
            : newTarget instanceof ResourceSet 
                ? ((ResourceSet)newTarget).getResources().iterator()
                : newTarget instanceof Resource ? ((Resource)newTarget).getContents().iterator() : null;

        while (contents != null && contents.hasNext()) {
            addAdapter(contents.next());
        }
    }
    
    public final Notifier getTarget() {
        return null;
    }

    public final boolean isAdapterForType(Object type) {
        return false;
    }

    public final void notifyChanged(Notification notification) {
        Object notifier = notification.getNotifier();
        if (notifier instanceof Resource) {
            int featureId = notification.getFeatureID(Resource.class);
            if (featureId != Resource.RESOURCE__RESOURCE_SET && featureId != Resource.RESOURCE__CONTENTS
                || featureId == Resource.RESOURCE__RESOURCE_SET && !handleResourceSets) {
                return;
            }
        } else if (notifier instanceof ResourceSet && !handleResourceSets) {
            return;
        }
        
        int event = notification.getEventType();
        Object newValue = notification.getNewValue();
        Object oldValue = notification.getOldValue();
        
        switch (event) {
        case Notification.SET:
        case Notification.UNSET:
        case Notification.ADD:
        case Notification.ADD_MANY:
        case Notification.REMOVE:
        case Notification.REMOVE_MANY: {
            addAdapter(newValue);
            break;
        }
        default:
            break;
        }

        if (notifier instanceof EObject) {
            EObject eObject = (EObject)notifier;
            if (eObject.eResource() != null && ((Resource.Internal)eObject.eResource()).isLoading()) {
                return;
            }
            if (notification.getFeature() instanceof EStructuralFeature) {
                EStructuralFeature feature = (EStructuralFeature)notification.getFeature();
                switch (event) {
                case Notification.SET:
                case Notification.UNSET:
                case Notification.ADD:
                case Notification.REMOVE:
                case Notification.ADD_MANY:
                case Notification.REMOVE_MANY:
                    notifyChangedObject(event, eObject, feature, oldValue, newValue);
                    break;
                default:
                    break;
                }
            }
        } else if (notifier instanceof Resource) {
            Resource resource = (Resource)notifier;
            if (notification.getFeatureID(Resource.class) == Resource.RESOURCE__CONTENTS
                && !((Resource.Internal)resource).isLoading()) {
                switch (event) {
                case Notification.SET:
                case Notification.UNSET:
                case Notification.ADD:
                case Notification.REMOVE:
                case Notification.ADD_MANY:
                case Notification.REMOVE_MANY: 
                    notifyChangedResource(event, resource, oldValue, newValue);
                    break;
                default:
                    break;
                }
            }
        } else if (notifier instanceof ResourceSet) {
            ResourceSet resSet = (ResourceSet)notifier;
            switch (event) {
            case Notification.SET:
            case Notification.UNSET:
            case Notification.ADD:
            case Notification.REMOVE:
            case Notification.ADD_MANY:
            case Notification.REMOVE_MANY: 
                notifyChangedResourceSet(event, resSet, oldValue, newValue);
                break;
            default:
                break;
            }
        }
    }

    public abstract void notifyChangedObject(int event, EObject eObject, EStructuralFeature feature, 
                                                Object oldValue, Object newValue);
    
    public abstract void notifyChangedResource(int event, Resource resource, 
                                                  Object oldValue, Object newValue);
    
    public abstract void notifyChangedResourceSet(int event, ResourceSet resource, 
            Object oldValue, Object newValue);

    private synchronized void addAdapter(Object newValue) {
        if (newValue instanceof Notifier) {
            List<Adapter> eAdapters = ((Notifier)newValue).eAdapters();
            if (!eAdapters.contains(this)) {
                eAdapters.add(this);
            }
        } else if (newValue instanceof Collection) {
            for (Object obj : (Collection<?>)newValue) {
                addAdapter(obj);
            }
        }
    }
}
