/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.notification;

import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;

public class EObjectNotification <T extends EObject> extends NotificationImpl {
    private final T notifier;
    
    private final EStructuralFeature feature;
    
    public EObjectNotification(int eventType, 
                               T notifier, 
                               EStructuralFeature feature, 
                               Object oldValue, 
                               Object newValue) {
        super(eventType, oldValue, newValue);
        this.notifier = notifier;
        this.feature = feature;
    }

    @Override
    public EStructuralFeature getFeature() {
        return feature;
    }

    @SuppressWarnings("unchecked")
    @Override
    public int getFeatureID(Class c) {
        if (notifier != null 
                && feature != null 
                && c != null 
                && c.isAssignableFrom(notifier.getClass())) {
            return feature.getFeatureID();
        }
        return NO_FEATURE_ID;
    }

    @Override
    public T getNotifier() {
        return notifier;
    }
    
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("EObjectNotification: notifier:");
        if (notifier == null) {
            sb.append("null");
        } else {
            sb.append(EcoreUtil.getURI(notifier));
        } 
        sb.append(", feature:");
        sb.append(feature == null ? "null" : feature.getName());
        sb.append(" ");
        return sb.toString();
    }
}
