/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.notification;

import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.Resource;

public class ResourceContentsNotification<T extends Resource> extends NotificationImpl {
    public static final EReference RESOURCE_CONTENTS_FEATURE; 
    
    static {
        EcoreFactory ecoreFactory = EcoreFactory.eINSTANCE;
        
        RESOURCE_CONTENTS_FEATURE = ecoreFactory.createEReference();
        RESOURCE_CONTENTS_FEATURE.setEType(EcorePackage.eINSTANCE.getEObject());
        RESOURCE_CONTENTS_FEATURE.setContainment(true);
        RESOURCE_CONTENTS_FEATURE.setName("contents");
        RESOURCE_CONTENTS_FEATURE.setUpperBound(EStructuralFeature.UNBOUNDED_MULTIPLICITY);
    }

    private final T notifier;
    
    public ResourceContentsNotification(int eventType, 
                                        T notifier, 
                                        Object oldValue, 
                                        Object newValue) {
        super(eventType, oldValue, newValue);
        this.notifier = notifier;
    }

    @Override
    public EStructuralFeature getFeature() {
        return RESOURCE_CONTENTS_FEATURE;
    }

    @SuppressWarnings("unchecked")
    @Override
    public int getFeatureID(Class c) {
        if (notifier != null && c.isAssignableFrom(notifier.getClass())) {
            return Resource.RESOURCE__CONTENTS;
        }
        return NO_FEATURE_ID;
    }

    @Override
    public T getNotifier() {
        return notifier;
    }
    
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("EObjectNotification: notifier:");
        if (notifier == null) {
            sb.append("null");
        } else {
            sb.append(notifier.getURI());
        } 
        sb.append(", feature:");
        sb.append(getFeature().getName());
        sb.append(" ");
        return sb.toString();
    }
}
