/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.resourceset;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.ecore.resource.Resource;

public class QueryHandlerRegistry {
    private static final QueryHandlerRegistry SINGLETON = new QueryHandlerRegistry(); 
    
    private Map<Class<? extends Resource>, ResourceQueryHandler> handlers = 
        new HashMap<Class<? extends Resource>, ResourceQueryHandler>();
        
    public static QueryHandlerRegistry getInstance() {
        return SINGLETON;
    }
    public ResourceQueryHandler getQueryHandler(Resource resource) {
        Class clazz = resource.getClass();
        ResourceQueryHandler handler = null;
        do {
            handler = handlers.get(clazz);
            clazz = clazz.getSuperclass();
        } while (handler == null && clazz != null);
        return handler;
    }
    
    public void registerQueryHandler(Class<? extends Resource> resourceClass, ResourceQueryHandler handler) {
        handlers.put(resourceClass, handler);
    }
}
