/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.security;

import java.util.Collection;

import com.iona.soa.repository.SOARepository;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;

public interface SecurityService {
    String AUTHENTICATE_REQUEST_HEADER = "Authorization";
    String AUTHENTICATE_RESPONSE_HEADER = "WWW-Authenticate";
    String TOKEN_TYPE = "DEPOT_AUTH_TOKEN";
    String TRANSIENT_TOKEN_QUALIFIER = ";transient-token";
    String BASIC_AUTH_TYPE = "Basic";
    String INTERNAL_TOKEN = "internal";
    String INTERNAL_USER = "Internal";
    
    void setSOARepository(SOARepository repo);
    
    void startTransaction(String userId);
    void endTransaction();

    boolean authorizeCreate(String ssoToken, EClass eClass, EObject container);
    boolean authorizeRead(String ssoToken, EObject object);
    boolean authorizeUpdate(String ssoToken, EObject object, EStructuralFeature feature);
    boolean authorizeDelete(String ssoToken, EObject object);
    
    // Agent Authorization
    boolean authorizeRead(String ssoToken, EObject eObject, Collection<String> areas);
    
    // HTTP based Authentication
    SecurityContext authenticate(String credentials) throws AuthenticationException;
    SecurityContext authenticate(String user, String password) throws AuthenticationException;
    boolean logoutUser(String ssoToken);
    
    // Scheduler authentication
    SecurityContext authenticatePersistent(String userName, String password) throws AuthenticationException;
}
