/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.security;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Collection;
import java.util.HashSet;

import com.iona.soa.repository.EMFUtil;
import com.iona.soa.repository.util.Base64;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

public final class SecurityUtil {
    public static final String ADMIN_USER_GUID = "User-ee867ab3-2b7c-48a2-bb73-f088da067004";
    public static final String ADMIN_ROLE_GUID = "Role-40e310f2-0c71-40a1-bd5c-7b049c6a2562";
    public static final String NONE_SECAREA_ID = "None";
    public static final String AGENT_SECAREA_ID = "Agent";
    public static final String DOWNLOAD_SECAREA_ID = "Download";
    
    private static final String SECURITY_ANNOTATION = "securityHints";
    private static final String SECURITY_ANNOTATION_AREA_KEY = "area";
    private static final String DIGEST_ALGORITHM = "MD5";
    
    private SecurityUtil() {
    }


    public static byte[] calculateDigest(String message) {
        try {
            MessageDigest md = MessageDigest.getInstance(DIGEST_ALGORITHM);
            md.update(message.getBytes());
            return md.digest();
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
    }
    
    public static String calculateDigestAsHex(String message) {
        byte[] digest = calculateDigest(message);
        
        StringBuilder hexString = new StringBuilder();
        for (int i = 0; i < digest.length; i++) {
            String hex = Integer.toHexString(digest[i] & 0xff);
            if (hex.length() == 1) {
                hex = "0" + hex;
            }
            hexString.append(hex);
        }

        return hexString.toString();
    }
    
    public static String[] parseAuthHeader(String credentials) {
        String[] data = credentials == null ? null : credentials.split(" ");
        if (data == null || data.length != 2) {
            return null;
        }
        if (SecurityService.BASIC_AUTH_TYPE.equals(data[0])) {
            String creds = new String(Base64.decode(data[1]));
            int index = creds.indexOf(":");
            if (index == -1) {
                return null;
            }
            String[] ret = new String[3];
            ret[0] = data[0];
            ret[1] = creds.substring(0, index);
            ret[2] = creds.substring(index + 1);
            return ret;
        } else {
            return data;
        }
    }

    public static Collection<String> getSecurityAreas(EObject eObj, boolean incudeContainer) {
        Collection<String> ret = getSecurityAreas(eObj.eClass());
        if (incudeContainer && eObj.eContainer() != null) {
            ret.addAll(getSecurityAreas(eObj.eContainer(), incudeContainer));
        }
        return ret;
    }

    public static Collection<String> getSecurityAreas(EClass eclass) {
        Collection<String> ret = new HashSet<String>();
        ret.addAll(EMFUtil.getAnnotationValues(eclass,
                                               SECURITY_ANNOTATION,       
                                               SECURITY_ANNOTATION_AREA_KEY));
        for (EClass superClass : eclass.getESuperTypes()) {
            ret.addAll(getSecurityAreas(superClass));
        }
        return ret;
    }
}
