/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.delegate;

import java.util.Collection;
import java.util.LinkedList;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operand;
import com.iona.soa.expression.Operation;
import com.iona.soa.expression.OperationType;
import com.iona.soa.repository.EMFUtil;
import com.iona.soa.repository.EMFUtil.URIEClassEntry;
import com.iona.soa.repository.jdbc.resource.JDBCResourceImpl;
import com.iona.soa.resourceset.ResourceSetQueryHandler;

import org.eclipse.emf.ecore.EObject;

public class CrossReferencesOperationDelegate implements JDBCOperationDelegate {

    public static class Factory implements JDBCOperationDelegate.Factory {
        public JDBCOperationDelegate create(JDBCResourceImpl resource) {
            return new CrossReferencesOperationDelegate(resource);
        }
    };
    
    private JDBCResourceImpl resource;
    private Exception ex;
    private Collection<EObject> results = new LinkedList<EObject>();
    
    public CrossReferencesOperationDelegate(JDBCResourceImpl resource) {
        this.resource  = resource;
    }

    public boolean handle(Operation op) {
        try {
            // find out all dependencies
            Operation depsOp  = new Operation(OperationType._deps, (Object[])op.getArguments());
            Collection<EObject> deps =  
                ResourceSetQueryHandler.executeQuery(resource.getResourceSet(), 
                                                     depsOp, true);
            // resolve root query objects and add them into dependency list
            for (Operand o : op.getArguments()) {
                URIEClassEntry entry = EMFUtil.getObjectIdentity(o.getValue().toString());
                EObject eobj = resource.getResourceSet().getEObject(entry.getKey(), true);
                if (eobj != null) {
                    deps.add(eobj);
                }
            }
            // for each dependency get its xrefs; if an exref object is not
            // in dependencies and not already resolved then add into resultset 
            for (EObject o : deps) {
                for (EObject xref : resource.getXRefs(o, o.eClass())) {
                    if (!deps.contains(xref) && !results.contains(xref)) {
                        results.add(xref);
                    }
                }                
            }
        } catch (Exception e) {
            ex = e;
            return false;
        }
        return true;
    }

    public boolean handle(Expression e) {
        // This can be implemented so that unwanted dependencies 
        // are filtered via this expression
        throw new UnsupportedOperationException();
    }

    public boolean handle(Operand op) {
        throw new UnsupportedOperationException();
    }
    
    public Collection<EObject> getResult() {
        return results;
    }
    
    public Exception getException() {
        return ex;
    }
}
