/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.expression.delegate;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operand;
import com.iona.soa.expression.Operation;
import com.iona.soa.expression.runtime.DelegatingFactory;
import com.iona.soa.expression.runtime.ExpressionRuntime;
import com.iona.soa.expression.runtime.model.ContextKey;
import com.iona.soa.expression.serialization.ExpressionSerializer;
import com.iona.soa.repository.EMFUtil;
import com.iona.soa.repository.EMFUtil.URIEClassEntry;
import com.iona.soa.repository.jdbc.resource.JDBCResourceImpl;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;

public class DependenciesOperationDelegate implements JDBCOperationDelegate {

    public static class Factory implements JDBCOperationDelegate.Factory {
        public JDBCOperationDelegate create(JDBCResourceImpl resource) {
            return new DependenciesOperationDelegate(resource);
        }
    };

    private JDBCResourceImpl resource;
    private Exception ex;
    private Collection<EObject> results = new LinkedList<EObject>();
    
    public DependenciesOperationDelegate(JDBCResourceImpl resource) {
        this.resource  = resource;
    }

    public boolean handle(Operation op) {
        Collection<EObject> set = new LinkedList<EObject>();
        for (Operand o : op.getArguments()) {
            URIEClassEntry entry = EMFUtil.getObjectIdentity(o.getValue().toString());
            Collection<EObject> localSet = new LinkedList<EObject>();
            try {
                EObject eObject = resource.getResourceSet().getEObject(entry.getKey(), true);
                if (eObject != null) {
                    resolve(eObject, localSet, set);
                    localSet.remove(eObject);
                    results.remove(eObject);
                    for (EObject eo : localSet) {
                        if (!results.contains(eo)) {
                            results.add(eo);
                        }
                    }
                }
            } catch (Exception e) {
                ex = e;
                return false;
            }
        }
        return true;
    }
    
    @SuppressWarnings("unchecked")
    protected void resolve(EObject eobj, 
                           Collection<EObject> localSet,
                           Collection<EObject> set) throws Exception {
        if (set.contains(eobj)) {
            return;
        }
        
        set.add(eobj);
        localSet.add(eobj);

        String value = EMFUtil.getAnnotationValue(eobj.eClass(), 
                                                  EMFUtil.DELETE_ANNOTATION, 
                                                  EMFUtil.DELETE_DEPENDENCIES);
        if (value != null) {
            Collection<Expression> exps = 
                ExpressionSerializer.deserializeModelQuery(value);
            if (exps != null) {
                Map<ContextKey, Object> context = new HashMap<ContextKey, Object>();
                Collection<EObject> localset = new LinkedList<EObject>(); 
                context.put(ContextKey.EOBJECT, eobj);
                context.put(ContextKey.SET, set);
                context.put(ContextKey.STACKSET, localset);
                for (Expression exp : exps) {
                    ExpressionRuntime runtime = DelegatingFactory.create(exp.getOperator());
                    if (runtime != null) {
                        runtime.run(exp, context);
                    }
                }
                for (EObject o : localset) {
                    set.remove(o);
                    resolve(o, localSet, set);
                }
            }
        }
        for (EReference ref : (List<EReference>) eobj.eClass().getEAllReferences()) {
            if (eobj.eIsSet(ref)) {
                if (ref.isContainment()) {
                    if (ref.isMany()) {
                        for (EObject o : (List<EObject>)eobj.eGet(ref)) {
                            resolve(o, localSet, set);
                        }
                    } else {
                        EObject o = (EObject)eobj.eGet(ref);
                        resolve(o, localSet, set);
                    }
                }
            }
        } 
    }
    

    public boolean handle(Expression e) {
        // This can be implemented so that unwanted dependencies 
        // are filtered via this expression
        throw new UnsupportedOperationException();
    }

    public boolean handle(Operand op) {
        throw new UnsupportedOperationException();
    }
    
    public Collection<EObject> getResult() {
        return results;
    }
    public Exception getException() {
        return ex;
    }
}
