/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.jdbc;

import java.sql.SQLException;

import com.iona.soa.repository.jdbc.resource.JDBCResource;
import com.iona.soa.repository.notification.AbstractRepositoryAdapter;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.xmi.XMIException;

public class EMFDatabaseSerializationAdapter2 extends AbstractRepositoryAdapter {
    private JDBCResource resource;
    
    public EMFDatabaseSerializationAdapter2(JDBCResource res) {
        super(res);
        resource = res;
    }

    @SuppressWarnings("unchecked")
    @Override
    public void attachedObject(EObject eObject) {
        try {
            resource.persist(eObject);
        } catch (SQLException ex) {
            resource.getErrors().add(new XMIException(ex));
        }
    }

    @Override
    public void detachedObject(EObject eObject, EObject oldContainer, EReference containerRef) {
        detachedInternal(eObject, false);
    }

    @SuppressWarnings("unchecked")
    private void detachedInternal(EObject eObject, boolean idUnset) {
        if (resource.getContents().contains(eObject)) {
            boolean oldDeliver = resource.eDeliver();
            try {
                resource.eSetDeliver(false);
                resource.getContents().remove(eObject);
            } finally {
                resource.eSetDeliver(oldDeliver);
            }
        }
        try {
            resource.unPersist(eObject, !idUnset);
        } catch (SQLException ex) {
            resource.getErrors().add(new XMIException(ex));
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public void changedAttribute(EObject eObject, EAttribute attribute,
                                 Object oldValue, Object newValue) {
        if (attribute.isID() && newValue == null) {
            boolean prevDeliver = eObject.eDeliver();
            try {
                eObject.eSetDeliver(false);
                eObject.eSet(attribute, oldValue);
            } finally {
                eObject.eSetDeliver(prevDeliver);
            }
            detachedInternal(eObject, true);
        } else {
            PooledConnection pc = resource.getRepository().getConnectionManager().getConnection();
            try {
                EMFSQLUtil.dbUpdateSingleAttribute(eObject, attribute, pc.getConnection());
            } catch (SQLException ex) {
                resource.getErrors().add(new XMIException(ex));
            } finally {
                resource.getRepository().getConnectionManager().returnConnection(pc);
            }
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public void changedReference(EObject eObject, EReference ref,
                                 Object oldValue, Object newValue) {
        PooledConnection pc = resource.getRepository().getConnectionManager().getConnection();
        try {
            EMFSQLUtil.dbStoreReferences(eObject, ref, pc.getConnection());
        } catch (SQLException ex) {
            resource.getErrors().add(new XMIException(ex));
        } finally {
            resource.getRepository().getConnectionManager().returnConnection(pc);
        }
    }
}
