/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.jdbc;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

public class STConnectionManagerImpl implements JDBCConnectionManager {
    private PooledConnection pooledConn;
    
    public STConnectionManagerImpl(Connection conn) {
        pooledConn = new PooledConnection(conn);
    }

    public PooledConnection getConnection() {
        return pooledConn;
    }

    public void returnConnection(PooledConnection connection) {
    }

    public void startTransaction() throws SQLException {
        pooledConn.getConnection().setAutoCommit(false);
    }

    public void commitTransaction() throws SQLException {
        try {
            pooledConn.getConnection().commit();
        } finally {
            pooledConn.getConnection().setAutoCommit(true);
        }
    }

    public void rollbackTransaction() throws SQLException {
        try {
            pooledConn.getConnection().rollback();
        } finally {
            pooledConn.getConnection().setAutoCommit(true);
        }
    }

    public void shutDown() {
        Connection conn = pooledConn.getConnection();
        try {
            if (EMFSQLUtil.getDatabaseName(conn).contains("HSQL")) {
                Statement stmt = conn.createStatement();
                stmt.executeUpdate("SHUTDOWN");
                stmt.close();
            }
        } catch (SQLException e) {
            // ignore
        }
        try {
            conn.close();
        } catch (SQLException e) {
            // ignore
        }
    }
    
    public String getDatabaseURL() {
        try {
            if (null != pooledConn 
                && null != pooledConn.getConnection() 
                && null != pooledConn.getConnection().getMetaData()) {
                return pooledConn.getConnection().getMetaData().getURL();
            }
        } catch (Exception e) {
            // eat it
        }
        return "";
    }

}
