/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.jdbc.resource;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.iona.soa.repository.ModelRegistry;
import com.iona.soa.resourceset.QueryResourceSetImpl;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;

public class JDBCResourceSetImpl extends QueryResourceSetImpl {
    private boolean loaded;
    private ModelRegistry modelRegistry;
    
    @SuppressWarnings("unchecked")
    public JDBCResourceSetImpl(ModelRegistry registry) throws IOException {
        this.modelRegistry = registry;
    }
    
    @SuppressWarnings("unchecked")
    public synchronized void load() {
        if (!loaded) {
            loaded = true;
            Resource resource = createResource(URI.createURI("JDBCResource"));
            Map<URI, Resource> uriMap = new HashMap<URI, Resource>();
            for (EPackage pkg : modelRegistry.getPackages()) {
                uriMap.put(URI.createURI(pkg.getNsURI()), resource);
            }
            setURIResourceMap(uriMap);
            try {
                resource.load(Collections.EMPTY_MAP);
            } catch (IOException e) {
                // ignore
            }
            getResources().add(resource);
        }
    }
    
    public EObject findEObject(URI proxyURI) {
        Resource resource = getResource(proxyURI.trimFragment(), true);
        if (resource instanceof JDBCResource) {
            return ((JDBCResource)resource).findEObject(proxyURI);
        } else {
            return null;
        }
    }

    public void registerEObject(URI proxyURI, EObject relObj) {
        Resource resource = getResource(proxyURI.trimFragment(), true);
        if (resource instanceof JDBCResource) {
            ((JDBCResource)resource).registerEObject(proxyURI, relObj);
        }
    }  

    @SuppressWarnings("unchecked")
    public void close() {
        for (Resource res : (List<Resource>)getResources()) {
            res.unload();
        }
        getResources().clear();
        loaded = false; 
    }
}
