/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.db;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.annotation.Resource;

public class DatabaseImpl implements Database {

    protected static final Logger LOG = Logger.getLogger(DatabaseImpl.class.getName());

    protected String dbUrl; 
    protected String dbUserName; 
    protected String dbPassword; 
    protected Driver dbDriver;
    
    public DatabaseImpl(String driver) {
        try { 
            Class clazz = getClass().getClassLoader().loadClass(driver);
            Driver theDriver = (Driver)clazz.newInstance();
            init(theDriver);
        } catch (Exception ex) {
            String message = "unabled to load database driver " + driver; 
            LOG.severe(message);
            throw new RuntimeException(message);
        }
    }
    
    public DatabaseImpl(Driver theDriver) {
        try { 
            init(theDriver);
        } catch (Exception ex) {
            String message = "unabled to load database driver " 
                + theDriver.getClass().getName(); 
            LOG.severe(message);
            throw new RuntimeException(message);
        }
    }

    public DatabaseImpl(String driver, String url, String name, String passwd) {
        this(driver);
        dbUrl = url;
        dbUserName = name;
        dbPassword = passwd;
    }
    
    private void init(Driver theDriver) throws SQLException {
        DriverManager.registerDriver(theDriver);
        dbDriver = theDriver;
    }

  
    public Connection getConnection() throws SQLException {
        try {
            LOG.info("connecting to " + dbUrl);
            if (dbUserName == null) {
                return DriverManager.getConnection(dbUrl);
            } else {
                return DriverManager.getConnection(dbUrl, dbUserName, dbPassword);
            }
        } catch (SQLException ex) {
            LOG.log(Level.SEVERE, "error creating connection", ex);
            if (ex.getNextException() != null) {
                throw ex.getNextException();
            } else {
                throw ex;
            }
                
        }
    }

    public void start() {
        
    }

    public void stop() {
        if (dbDriver != null) {
            try {
                DriverManager.deregisterDriver(dbDriver);
            } catch (SQLException ex) {
                // ignore
            }
        }
    }

    @Resource
    public void setDatabaseURL(String url) {
        dbUrl = url;
    }

    public void setDatabaseLogin(String userName, String passwd) {
        dbUserName = userName;
        dbPassword = passwd;
    }

    public String getUserName() {
        return dbUserName;
    }
    
    public String getUserPassword() {
        return dbPassword;
    }

    public String getDriverName() {
        return dbDriver.getClass().getName();
    }

    public String getDatabaseURL() {
        return dbUrl;
    }
    
    public String getDatabaseType() throws SQLException {
        return DatabaseResolver.getDatabaseTypeFromURL(dbUrl);
    }
    
    public String getDatabaseHost() throws SQLException {
        return DatabaseResolver.getDatabaseHostFromURL(dbUrl);
    }
    
    public String getDatabaseName() throws SQLException {
        return DatabaseResolver.getDatabaseNameFromURL(dbUrl);
    }
    
    public void createDatabase() throws SQLException {
        // complete
    }
    
    
}
