/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.maven.resolve;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.iona.repository.maven.util.MavenDelegatingLogger;
import com.iona.repository.maven.util.MavenUtils;
import com.iona.repository.maven.util.SpiderContainerCustomizer;
import com.iona.soa.model.MavenRepository;
import com.iona.soa.repository.util.InputStreamHelper;
import com.iona.soa.repository.util.TempUtil;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.factory.ArtifactFactory;
import org.apache.maven.artifact.handler.ArtifactHandler;
import org.apache.maven.artifact.metadata.ArtifactMetadataSource;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.artifact.resolver.ArtifactCollector;
import org.apache.maven.artifact.resolver.filter.ScopeArtifactFilter;
import org.apache.maven.embedder.Configuration;
import org.apache.maven.embedder.ConfigurationValidationResult;
import org.apache.maven.embedder.ContainerCustomizer;
import org.apache.maven.embedder.DefaultConfiguration;
import org.apache.maven.embedder.MavenEmbedder;
import org.apache.maven.embedder.MavenEmbedderException;
import org.apache.maven.model.Model;
import org.apache.maven.profiles.DefaultProfileManager;
import org.apache.maven.profiles.ProfileManager;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.MavenProjectBuilder;
import org.apache.maven.project.ProjectBuildingException;
import org.apache.maven.shared.dependency.tree.DependencyNode;
import org.apache.maven.shared.dependency.tree.DependencyTreeBuilder;
import org.apache.maven.shared.dependency.tree.DependencyTreeBuilderException;
import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
import org.codehaus.plexus.util.xml.pull.XmlPullParserException;




public class Resolver {
    
    private static final Logger LOG = Logger.getLogger(Resolver.class.getName());
    private static final String SCOPE_ALL = "all";
    
    private Map<String, String> artifactDownloadUrlMap = new HashMap<String, String>();
    private Map<String, String> artifactPackagingMap = new HashMap<String, String>();

    @SuppressWarnings("unchecked")
    public DependencyNode resolve(String configPath,
                                  String groupid,
                                  String artifactId,
                                  String version,
                                  String scope,
                                  List<MavenRepository> mvnRepository) throws IOException,
                                                                              ComponentLookupException {
        ScopeArtifactFilter filter = validateScopeAndRetriveFilter(scope);

        File myFile = new File(TempUtil.createTempDir(), "pom.xml");
 
        FileOutputStream fos = new FileOutputStream(myFile);
        String repository = "";
        int count = 0;
        // TODO : The repos NEED to have meaning ful names so that the mirrors will make sense
        if (mvnRepository.size() > 0) {
            repository += "<repositories>";
            for (MavenRepository repo : (List<MavenRepository>)mvnRepository) {
                URI repoUri = repo.getUrl();
                if (repoUri != null) {
                    repository += "<repository>"
                        + "<id>repo" + count + "</id>"
                        + "<name>Maven Repository Switchboard</name>"
                        + "<layout>default</layout>"
                        + "<url>" + repoUri + "</url>"
                        + "<snapshots>"
                        + "<enabled>false</enabled>"
                        + "</snapshots>"
                        + "</repository>";
                    count++;
                }
            }
            repository += "</repositories>";
        }

        String dependency = 
            "<dependencies>" 
            + "<dependency>"
            + "<groupId>" + groupid + "</groupId>" 
            + "<artifactId>" + artifactId + "</artifactId>" 
            + "<version>" + version + "</version>" 
            + "</dependency>"
            + "</dependencies>";

        String vanillaPom = 
            "<project xmlns=\"http://maven.apache.org/POM/4.0.0\" xmlns:xsi=\"http://www.w3.org/2001/"
            + "XMLSchema-instance\" xsi:schemaLocation=\"http://maven.apache.org/POM/4.0.0 http://maven"
            + ".apache.org/maven-v4_0_0.xsd\">" 
            + "<modelVersion>4.0.0</modelVersion>"
            + "<name>DependencyResolve</name>"
            + "<artifactId>it-depo-resolve</artifactId>"
            + "<url>http://wiki.iona.com/curry-wiki/</url>"
            + "<version>1.0</version> "
            + "<groupId>com.iona</groupId>"
            + repository
            + dependency
            + "</project>";
      
        fos.write(vanillaPom.getBytes());
        fos.close();

        Configuration configuration = null;
        if (configPath == null) {
            configuration = new DefaultConfiguration().setClassLoader(MavenEmbedder.class.getClassLoader());
        } else {
            File configFile = new File(configPath);
            configuration = new DefaultConfiguration().setUserSettingsFile(configFile)
                                                      .setClassLoader(MavenEmbedder.class.getClassLoader());
        }
        configuration.setMavenEmbedderLogger(new MavenDelegatingLogger());

        MavenEmbedder mavenEmbedder = null;

        ContainerCustomizer customizer = new SpiderContainerCustomizer();
        configuration.setConfigurationCustomizer(customizer);
        ConfigurationValidationResult validationResult = MavenEmbedder.validateConfiguration(configuration);
        
        if (validationResult.isValid()) {
            
            try {
                mavenEmbedder  = new MavenEmbedder(configuration);
            } catch (MavenEmbedderException e) {
                // TODO Auto-generated catch block
                LOG.log(Level.FINE, "exception caught", e);
            }

            MavenProject mavProj = null;
            try {
                ProfileManager profManager =
                    new DefaultProfileManager(mavenEmbedder.getPlexusContainer(), null);
                MavenProjectBuilder mavenProjectBuilder =
                    (MavenProjectBuilder) mavenEmbedder.getPlexusContainer().lookup(MavenProjectBuilder.ROLE);
                mavProj = mavenProjectBuilder.buildWithDependencies(myFile, 
                                                                    mavenEmbedder.getLocalRepository(),
                                                                    profManager);
                
                Set artifacts = mavProj.getArtifacts();
                for (Iterator i = artifacts.iterator(); i.hasNext();) {
                    Artifact a = (Artifact) i.next();
                    
                    forceRetrievePackagingInfo(a, mavenEmbedder);
                    artifactDownloadUrlMap.put(buildMapId(a), getRepository(a) + getLocation(a));
                }
 
            } catch (ProjectBuildingException e) {
                LOG.log(Level.FINE, "exception caught", e);
                return null;
            }

            ArtifactRepository localRepository = mavenEmbedder.getLocalRepository(); 

            DependencyNode depNode = null;
            try {
                DependencyTreeBuilder dependencyTreeBuilder = (DependencyTreeBuilder)mavenEmbedder
                    .getPlexusContainer().lookup(DependencyTreeBuilder.ROLE);
                
                ArtifactFactory factory = 
                    (ArtifactFactory)mavenEmbedder.getPlexusContainer().lookup(ArtifactFactory.ROLE);
                
                ArtifactCollector collector = (ArtifactCollector)mavenEmbedder
                    .getPlexusContainer().lookup(ArtifactCollector.class.getName());
                
                ArtifactMetadataSource artifactMetadataSource = (ArtifactMetadataSource)mavenEmbedder
                    .getPlexusContainer().lookup(ArtifactMetadataSource.ROLE);


                depNode = dependencyTreeBuilder.buildDependencyTree(mavProj,
                                                                    localRepository,
                                                                    factory,
                                                                    artifactMetadataSource,
                                                                    filter,
                                                                    collector);
            } catch (ComponentLookupException e) {
                LOG.log(Level.FINE, "exception caught", e);
            } catch (DependencyTreeBuilderException e) {
                LOG.log(Level.FINE, "exception caught", e);
            }
            traverse(depNode);
            return depNode;
        }
        return null;
    }

    private ScopeArtifactFilter validateScopeAndRetriveFilter(String scope) {
        ScopeArtifactFilter filter = null;
        try {
            if (scope != null && scope != SCOPE_ALL) {
                assert getValidScopeValues().contains(scope);
                filter = new ScopeArtifactFilter(scope);
            }
        } catch (Exception e) {
            LOG.log(Level.SEVERE, "Scope value isn't valid", e);
        }
        return filter;
    }

    private List<String> getValidScopeValues() {
        List<String> stringRetList = new ArrayList<String>();
        stringRetList.add(Artifact.SCOPE_COMPILE);
        stringRetList.add(Artifact.SCOPE_IMPORT);
        stringRetList.add(Artifact.SCOPE_PROVIDED);
        stringRetList.add(Artifact.SCOPE_RUNTIME);
        stringRetList.add(Artifact.SCOPE_SYSTEM);
        stringRetList.add(Artifact.SCOPE_TEST);      
        stringRetList.add(SCOPE_ALL);  
        return stringRetList;
    }
    
    private String buildMapId(Artifact a) {
        return a.getGroupId() + a.getArtifactId() + a.getVersion();
    }
    
    private void forceRetrievePackagingInfo(Artifact a, MavenEmbedder mavenEmbedder) {
        try {
            String fileName = a.getFile().getName();
            String pomFileName = fileName.substring(0, fileName.lastIndexOf(".")) + ".pom";
            Model mvnModel = mavenEmbedder.readModel(new File(a.getFile().getParent(), pomFileName));
            artifactPackagingMap.put(buildMapId(a), mvnModel.getPackaging());
        } catch (XmlPullParserException e) {
            // TODO Auto-generated catch block
            LOG.log(Level.FINE, "exception caught", e);
            
        } catch (IOException e) {
            // TODO Auto-generated catch block
            LOG.log(Level.FINE, "exception caught", e);
        }
    }

    private String getLocation(Artifact a) {
        String guessedExt = 
            MavenUtils.guessFileExtensionFromMavenPackaging(artifactPackagingMap.get(buildMapId(a)));
        String extension = 
            guessedExt != null
                ? "." + guessedExt
                : a.getFile().getAbsolutePath().substring(a.getFile().getAbsolutePath().lastIndexOf("."));
        String location = a.getGroupId().replace('.', '/');
        location += "/" + a.getArtifactId() + "/" + a.getVersion() + "/";
        location += a.getArtifactId() + "-" + a.getVersion() + extension;
        return location;
    }

    public static String getRepository(Artifact a) throws IOException {
        String url = null;
        if (a.getFile() == null) {
            return null;
        }
        File newFile = null;
        FileInputStream fis = null;
        try {
            File absoluteDir = a.getFile().getParentFile();
            newFile = new File(absoluteDir.getAbsolutePath() + File.separatorChar + "mavenUrl.txt");
            fis = new FileInputStream(newFile);
        } catch (FileNotFoundException e) {
            // not in a registered maven repos...
            LOG.log(Level.FINE, "exception caught", e);
            return a.getDownloadUrl();
        }

        if (fis != null) {
            byte[] input = InputStreamHelper.drain(fis);
            fis.close();
            url = new String(input);
        }
        if (url != null && !url.endsWith("/")) {
            url += "/";
        }
        return url;
    }

    public void traverse(DependencyNode depNode) {
        if (depNode != null) {
            Artifact a = depNode.getArtifact();
            //String depNode = depNode.
            if (a != null) {
                String url = artifactDownloadUrlMap.get(buildMapId(a));
                String groupId = a.getGroupId().replace('.', '/');
                depNode.getArtifact().setResolved(url != null && url.startsWith(groupId));
                depNode.getArtifact().setDownloadUrl(url);
                
                final String packaging = artifactPackagingMap.get(buildMapId(a));
                final ArtifactHandler ah = depNode.getArtifact().getArtifactHandler();
                a.setArtifactHandler(new ArtifactHandler() {
                    public String getClassifier() { return ah.getClassifier(); }
                    public String getDirectory() { return ah.getDirectory(); }
                    public String getExtension() { return ah.getExtension(); }
                    public String getLanguage() { return ah.getLanguage(); }
                    public String getPackaging() { return packaging; }
                    public boolean isAddedToClasspath() { return ah.isAddedToClasspath(); }
                    public boolean isIncludesDependencies() { return ah.isIncludesDependencies(); }
                });

            }
        }

        for (Object obj : depNode.getChildren()) {
            DependencyNode node = (DependencyNode)obj;
            traverse(node);
        }
    }

}




