/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.maven.util;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.maven.embedder.MavenEmbedderLogger;

public class MavenDelegatingLogger implements MavenEmbedderLogger {

    private static final Logger LOG = Logger.getLogger(MavenDelegatingLogger.class.getName());

    public void close() {
        // Complete.
    }

    public void debug(String arg0) {
        LOG.log(Level.FINE, arg0);
    }

    public void debug(String arg0, Throwable arg1) {
        LOG.log(Level.FINE, arg0, arg1);
    }

    public void info(String arg0) {
        LOG.log(Level.INFO, arg0);
    }

    public void info(String arg0, Throwable arg1) {
        LOG.log(Level.INFO, arg0, arg1);
    }

    public void warn(String arg0) {
        LOG.log(Level.WARNING, arg0);
    }

    public void warn(String arg0, Throwable arg1) {
        LOG.log(Level.WARNING, arg0, arg1);
    }

    public void error(String arg0) {
        LOG.log(Level.SEVERE, arg0);
    }

    public void error(String arg0, Throwable arg1) {
        LOG.log(Level.SEVERE, arg0, arg1);
    }

    public void fatalError(String arg0) {
        LOG.log(Level.SEVERE, arg0);
    }

    public void fatalError(String arg0, Throwable arg1) {
        LOG.log(Level.SEVERE, arg0, arg1);
    }

    public int getThreshold() {
        if (LOG.isLoggable(Level.FINE)) {
            return MavenEmbedderLogger.LEVEL_DEBUG;
        } else if (LOG.isLoggable(Level.INFO)) {
            return MavenEmbedderLogger.LEVEL_INFO;
        } else if (LOG.isLoggable(Level.WARNING)) {
            return MavenEmbedderLogger.LEVEL_WARN;
        } else if (LOG.isLoggable(Level.SEVERE)) {
            return MavenEmbedderLogger.LEVEL_ERROR;
        }
        return MavenEmbedderLogger.LEVEL_DISABLED;
    }

    public boolean isDebugEnabled() {
        return LOG.isLoggable(Level.FINE);
    }

    public boolean isErrorEnabled() {
        return LOG.isLoggable(Level.SEVERE);
    }

    public boolean isFatalErrorEnabled() {
        return LOG.isLoggable(Level.SEVERE);
    }

    public boolean isInfoEnabled() {        
        return LOG.isLoggable(Level.INFO);
    }

    public boolean isWarnEnabled() {
        return LOG.isLoggable(Level.WARNING);
    }

    public void setThreshold(int arg0) {
        LOG.warning("not implemented");
    }

}
