/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.maven.util;

import java.io.File;


import org.apache.maven.embedder.Configuration;
import org.apache.maven.embedder.ConfigurationValidationResult;
import org.apache.maven.embedder.ContainerCustomizer;
import org.apache.maven.embedder.DefaultConfiguration;
import org.apache.maven.embedder.MavenEmbedder;
import org.apache.maven.embedder.MavenEmbedderException;
import org.apache.maven.embedder.MavenEmbedderLogger;


public final class MavenUtils {

    private static MavenUtils mavenUtils = new MavenUtils();
    
    private MavenUtils() {
        
    }
    
    public static MavenUtils getInstance() {
        return mavenUtils;
    }
    
    public MavenEmbedder getMavenEmbedder(String configPath) throws MavenEmbedderException {
        
        Configuration configuration = null;
        if (configPath == null) {
            configuration = new DefaultConfiguration();
            configuration.setClassLoader(MavenEmbedder.class.getClassLoader());
        } else {
            File configFile = new File(configPath);
            configuration = new DefaultConfiguration()
                .setUserSettingsFile(configFile)
                .setClassLoader(MavenEmbedder.class.getClassLoader());
        }
        MavenEmbedderLogger logger = new MavenDelegatingLogger();
        configuration.setMavenEmbedderLogger(logger);
        
        ContainerCustomizer customizer = new SpiderContainerCustomizer();
        //configuration.setConfigurationCustomizer(customizer);
        ConfigurationValidationResult validationResult = MavenEmbedder.validateConfiguration(configuration);
        
        if (validationResult.isValid()) {
            MavenEmbedder embedder = new MavenEmbedder(configuration);
            embedder.setLogger(logger);
            return embedder;
        } else {
            throw new MavenEmbedderException(validationResult.getGlobalSettingsException());
        }
    }

    public static String guessFileExtensionFromMavenPackaging(String packaging) {
        if ("jbi-component".equals(packaging)) {
            return "zip";
        } else if ("jbi-service-assembly".equals(packaging)) {
            return "zip";
        } else if ("bundle".equals(packaging)) {
            return "jar";
        } else {
            return packaging;
        }
    }
    
}
