/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.webdav.osgi;

import java.io.File;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.logging.Logger;

import com.iona.soa.repository.configuration.ConfigurationService;

import org.mortbay.jetty.servlet.DefaultServlet;
import org.osgi.service.http.HttpService;



public class RepositoryWebDavServletSupport {

    private static final Logger LOG = Logger.getLogger(RepositoryWebDavServletSupport.class.getName());
    
    private HttpService httpService;
    private ConfigurationService configurationService;
    
    public void start() {
        String storageDir = configurationService.getConfiguration().get("maven.repo.dir");
        if (!storageDir.endsWith("/")) {
            storageDir += "/";
        }
        File storageFile = new File(storageDir);
        try {
            if (!storageFile.isDirectory()) {
                LOG.info("creating directory " + storageFile);
                storageFile.mkdirs();
            }
            File depotDir = new File(storageDir + "maven-repos/depot/");
            if (!depotDir.isDirectory()) {
                LOG.info("creating directory " + depotDir);
                depotDir.mkdirs();
            }
            DefaultServlet defaultServlet = new DefaultServlet();
            Dictionary<String, String> initParams = new Hashtable<String, String>();
            initParams.put("resourceBase", storageDir);
            initParams.put("dirAllowed", "true");
            initParams.put("aliases", "true");
            httpService.registerServlet("/maven-repos/*", defaultServlet, initParams, null);
        } catch (Exception e) {
            LOG.warning("Error registering dav servlet: " + e.toString());        
        }
    }
    
    public void stop() {
        try {
            httpService.unregister("/maven-repos/*");
        } catch (Exception e) {
            LOG.warning("Error unregistering dav servlet: " + e.toString());        
        }
    }

    public HttpService getHttpService() {
        return httpService;
    }

    public void setHttpService(HttpService httpService) {
        this.httpService = httpService;
    }

    public ConfigurationService getConfigurationService() {
        return configurationService;
    }

    public void setConfigurationService(ConfigurationService configService) {
        this.configurationService = configService;
    }

}
