/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.model.util;

import com.iona.soa.model.repository.RepositoryFactory;
import com.iona.soa.model.repository.Version;
import com.iona.soa.repository.util.VersionType;

public final class VersionHelper {
    private VersionHelper() {
    }

    public static Version parseVersion(String versionString) {
        VersionType verType = VersionType.parseVersion(versionString);
        Version repoVersion = RepositoryFactory.eINSTANCE.createVersion();
        
        if (verType.getMajor() != null) {
            repoVersion.setMajor(verType.getMajor());
        }
        if (verType.getMinor() != null) {
            repoVersion.setMinor(verType.getMinor());
        }
        if (verType.getMicro() != null) {
            repoVersion.setMicro(verType.getMicro());
        }
        if (verType.getNano() != null) {
            repoVersion.setNano(verType.getNano());
        }
        if (verType.getQualifier() != null && verType.getQualifier().trim().length() > 0) {
            repoVersion.setQualifier(verType.getQualifier());
        }
        repoVersion.setOriginalString(versionString);
       
        return repoVersion;
    }

    public static VersionType versionToVersionType(Version version) {
        if (version == null) {
            return VersionType.EMPTY_VERSION;
        } else {
            return VersionType.parseVersion(versionToString(version));
        }
        
    }
    
    /** 
     * Converts a Version object to its stringified representation
     * 
     * @param newVersion
     * @return
     */
    public static String versionToString(Version version) {
        
        if (version == null) {
            return null;
        }
        
        if (version.getOriginalString() != null 
                && !new String("").equals(version.getOriginalString())) {
            return version.getOriginalString();
        }

        StringBuilder sb = new StringBuilder();
        sb.append(version.getMajor());
        if (version.getMinor() != null) {
            sb.append(VersionType.SEPARATOR);
            sb.append(version.getMinor());
            if (version.getMicro() != null) {
                sb.append(VersionType.SEPARATOR);
                sb.append(version.getMicro());
                if (version.getNano() != null) {
                    sb.append(VersionType.SEPARATOR);
                    sb.append(version.getNano());
                }
            }
        }
        
        if (version.getQualifier() != null) {
            sb.append(VersionType.SEPARATOR);
            sb.append(version.getQualifier());
        }
        return sb.toString();
    }
    
    public static int compareTo(Version lhs, Version rhs) {
        String literalLhs = versionToString(lhs);
        String literalRhs = versionToString(rhs);
        VersionType vtLhs = VersionType.parseVersion(literalLhs);
        VersionType vtRhs = VersionType.parseVersion(literalRhs);
        return vtLhs.compareTo(vtRhs);
    }
}
