/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.notification;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import com.iona.soa.model.collaboration.CollaborationPackage;
import com.iona.soa.model.events.DataChangeEvent;
import com.iona.soa.model.events.DataChangeFieldDelta;

/**
 * Holder/utility class an instance of which is supplied to the various email
 * JET emitters.
 */
public final class JetArgumentForEMail {
    
    private final DataChangeEvent event;
    
    JetArgumentForEMail(DataChangeEvent dataEvent) {
        event = dataEvent;
    }
    
    public String getTargetObjectName() {
        return event.getTargetName() != null ? event.getTargetName() : "<unknown>";
    }
    
    public String getTargetObjectType() {
        // TODO - may have to use gen.edit code to get the readble version
        // of this.
       return event.getTargetType().getName();
    }
    
    public String getUsername() {
        return event.getUserName();        
    }
    
    public String getUserEMail() {
        return event.getUserEMail();
    }
    
    public Calendar getChangeTimestamp() {
        Calendar c = Calendar.getInstance();
        c.setTimeInMillis(event.getTimestamp().getTime());
        return c;
    }
    
    public List<String> getCommentsAdded() {
        List<String> comments = new ArrayList<String>();
        for (Iterator it = event.getUpdatedFields().iterator(); it.hasNext();) {
            DataChangeFieldDelta delta = (DataChangeFieldDelta)it.next();
            if (CollaborationPackage.Literals.ICOLLABORATION__COMMENTS.equals(delta.getFieldDesc())) {
                comments.add(delta.getNewValue());
            }
        }
        return comments;
    }
    
    public List<DataChangeFieldDelta> getUpdatedFields() {
        List<DataChangeFieldDelta> updated = new ArrayList<DataChangeFieldDelta>();
        
        for (Iterator it = event.getUpdatedFields().iterator(); it.hasNext();) {
            DataChangeFieldDelta delta = (DataChangeFieldDelta)it.next();
            if (CollaborationPackage.Literals.ICOLLABORATION__COMMENTS.equals(delta.getFieldDesc())) {
                // ignore comments, they get special treatment
                continue;
            }
            updated.add(delta);
        }
        
        Collections.sort(updated, new Comparator<DataChangeFieldDelta>() {
            public int compare(DataChangeFieldDelta o1, DataChangeFieldDelta o2) {
                return o1.getFieldDesc().getName().compareTo(o2.getFieldDesc().getName());
            }
        });
        
        return updated;
    }
    
    public String stringify(List values) {
        StringBuffer sb = new StringBuffer();
        if (values != null) {
            List<String> strValues = new ArrayList<String>();
            for (Iterator it = values.iterator(); it.hasNext();) {
                strValues.add(it.next().toString());
            }
            
            Collections.sort(strValues);
            for (Iterator it = strValues.iterator(); it.hasNext();) {
                sb.append(it.next().toString());
                if (it.hasNext()) {
                    sb.append(", ");
                }
            }
        } else {
            sb.append("<no value>");
        }
        return sb.toString();
    }

    /**
     * Intended for use only if the templates are being customized in the field.
     */
    public DataChangeEvent getEvent() {
        return event;
    }
}
