/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.notification;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

import com.iona.soa.model.notification.NotificationRecipients;
import com.iona.soa.model.security.User;

/**
 * Data holder that is not meant to be used in an EMF context.  
 * Stores users and email addresses.
 * Intended for aggregation of multiple NotificationRecipients instances.
 */
final class NotificationTargets  {
    private final Set<String> emailAddresses = new HashSet<String>();
    
    private final Set<User> users = new TreeSet<User>(new Comparator<User>() {
        public int compare(User u1, User u2) {
            return u1.getGuid().compareTo(u2.getGuid());
        }
    });
    
    /**
     * Does not cater for the customRecipients, they can be merged
     * separately via the merge(Collection<User> u) method.
     */
    @SuppressWarnings("unchecked")
    void merge(NotificationRecipients recipients) {
        mergeUsers(recipients.getUsers());
        mergeEmailAddresses(recipients.getEmailAddresses());
    }

    void mergeUsers(Collection<User> us) {
        users.addAll(us);
    }
    
    void mergeEmailAddresses(Collection<String> addresses) {
        emailAddresses.addAll(addresses);
    }
    
    boolean isEmpty() {
        return users.isEmpty() && emailAddresses.isEmpty(); 
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("EmailAddresses ");
        sb.append(emailAddresses.toString());
        sb.append(", Users [");
        for (Iterator it = users.iterator(); it.hasNext();) {
            User u = (User)it.next();
            sb.append(u.getName());
            if (it.hasNext()) {
                sb.append(", ");
            }
        }
        sb.append("]");
        return sb.toString();
    }

    public Set<String> getEmailAddresses() {
        return Collections.unmodifiableSet(emailAddresses);
    }

    public Set<User> getUsers() {
        return Collections.unmodifiableSet(users);
    }
}
