/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest;

import com.iona.repository.util.i18n.Exception;
import com.iona.repository.util.i18n.Message;
import com.iona.soa.repository.status.RepositoryException;

import org.apache.abdera.protocol.server.RequestContext;

public class RESTException extends Exception {

    // HTTP Status codes.
    public static final int BAD_REQUEST = 400;
    public static final int UNAUTHORIZED = 401;
    public static final int FORBIDDEN = 403;
    public static final int NOT_FOUND = 404;
    public static final int METHOD_NOT_ALLOWED = 405;
    public static final int INTERNAL_SERVER_ERROR = 500;
    public static final int NOT_IMPLEMENTED = 501;
    public static final int SEARCH_PARSE_ERROR = 502;
    
    private static final long serialVersionUID = 4581561168911691005L;

    private final int code;
    private final RequestContext request;
    
    public RESTException(int c, Message msg) {
        super(msg);
        code = c;
        request = null;
    }
    
    public RESTException(int c, Throwable t) {
        super(t);
        code = c;
        request = null;
    }
    
    public RESTException(int c, RequestContext req, Message msg) {
        super(msg);
        code = c;
        request = req;
    }
    
    public RESTException(int c, RequestContext req, Throwable t) {
        super(t);
        code = c;
        request = req;
    }
    
    public RESTException(RepositoryException e) {
        super(e);
        code = e.getHttpReturnCode();
        request = null;
        //RepositoryStatus status = e.getStatus();
        // TODO: complete this conversion from RepositoryException
    }

    public int getCode() {
        return code;
    }

    public boolean isServerError() {
        return code >= 500;
    }
    
    public RequestContext getRequst() {
        return request;
    }
}
