/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.atom;

/**
 * Class used to control what appears in an Atom entry document.
 * 
 * @author ptaylor
 *
 */
public class AtomEntryOptions {
    
    private boolean includeResource;
    private boolean includeXhtml;
    private boolean includeSummary;
    private boolean includeRelatedLinks;
   
    /**
     * By default all options are true.
     */
    public AtomEntryOptions() {
        includeResource = true;
        includeXhtml = true;
        includeSummary = true;
        includeRelatedLinks = true;
    }

    public boolean includeResource() {
        return includeResource;
    }
    
    public AtomEntryOptions includeResource(boolean b) {
        includeResource = b;
        return this;
    }
    
    public boolean includeXhtml() {
        return includeXhtml;
    }

    public AtomEntryOptions includeXhtml(boolean b) {
        includeXhtml = b;
        return this;
    }
    
    public boolean includeSummary() {
        return includeSummary;
    }
    
    public AtomEntryOptions includeSummary(boolean b) {
        includeSummary = b;
        return this;
    }
    
    public boolean includeRelatedLinks() {
        return includeRelatedLinks;
    }
    
    public AtomEntryOptions includeRelatedLinks(boolean b) {
        includeRelatedLinks = b;
        return this;
    }
    
    public String toString() {
        return new StringBuilder()
            .append("resource: ")
            .append(includeResource() ? "yes" : "no")
            .append(", Xhtml: ")
            .append(includeXhtml() ? "yes" : "no")
            .append(", rel-links: ")
            .append(includeRelatedLinks() ? "yes" : "no")
            .append(", summary: ")
            .append(includeSummary() ? "yes" : "no")
            .toString();
            
    }
    
}
