/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.resource;

import java.util.Comparator;
import java.util.Date;

public class ResourceComparator implements Comparator<Resource> {

    private String attName;
    private boolean ascending;
    
    public ResourceComparator(String attName, boolean ascending) {
        this.attName = attName;
        this.ascending = ascending;
    }

    /**
     * Compare two resources based on a named attribute and whether the 
     * current comparator is ascending or descending. 
     * 
     * Currently only date and string attributes are supported.
     */
    public int compare(Resource r1, Resource r2) {
        try {
            Object o1 = r1.getFirstAttribute(attName).getObject();
            Object o2 = r2.getFirstAttribute(attName).getObject();
            if (o1 instanceof Date) {
                if (ascending) {
                    return ((Date) o1).compareTo((Date) o2); 
                } else {
                    return ((Date) o2).compareTo((Date) o1);
                }
            } else {
                if (ascending) {
                    return o1.toString().compareTo(o2.toString());
                } else {
                    return o2.toString().compareTo(o1.toString());
                }
            }
        } catch (Exception e) {
            return 0;
        }
    }

}
