/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.atom;

import com.iona.repository.rest.Constants;
import com.iona.repository.rest.atom.AbstractAtomEntryFactory;
import com.iona.repository.rest.atom.AtomEntryFactory;
import com.iona.repository.rest.resource.Reference;
import com.iona.repository.rest.resource.Resource;

import org.apache.abdera.model.Element;
import org.apache.abdera.model.Entry;
import org.apache.abdera.model.Link;

/**
 * Server class for creating Abdera Entry document containing a resource object.
 * The Entry documents include a Content element containing a XHTML representation
 * of the object and a Summary element.
 *
 */
public class ServerAtomEntryFactory extends AbstractAtomEntryFactory implements
        AtomEntryFactory {

    protected XhtmlWriter xhtmlWriter = new XhtmlWriter();

    public ServerAtomEntryFactory() {
    }

    protected Link getEditLink(Resource resource, String baseUri, String path) {
        return resource.isEditable() ? createLink(resource, path, "Edit") : null;
    }
    
    protected Link getDeleteLink(Resource resource, String baseUri, String path) {
        return resource.isDeletable() ? createLink(resource, path, "Delete") : null;
    }
    
    private Link createLink(Resource resource, String path, String title) {
        return abderaFactory.newLink()
            .setHref(new StringBuilder().append(path).append("/").append(getId(resource)).toString())
            .setRel(title.toLowerCase())
            .setMimeType(Constants.ATOM_CONTENT_TYPE)
            .setTitle(title + " Resource");
    }
    
    protected boolean addEditedDate() {
        return true;
    }

    protected String getXhtml(String title, Element element) {
        XhtmlWriter writer = getXhtmlWriter();
        if (writer == null) {
            return null;
        }
        return writer.toXhtml(title, element);
    }
    
    protected XhtmlWriter getXhtmlWriter() {
        return xhtmlWriter;
    }
    
    protected void addReferenceLinks(Entry entry, Resource resource, String baseUri) {
        for (Reference ref : resource.getAllReferences()) {
            Link link = getReferenceLink(ref, baseUri);
            if (link != null) {
                entry.addLink(link);
            }
        }
    }

    protected Link getReferenceLink(Reference reference, String baseUri) {
        if (reference.getHref() == null) {
            return null;
        }
        
        StringBuilder title = new StringBuilder();
        title.append(reference.getName())
             .append(": ")
             .append(reference.getText());        
        
        return abderaFactory.newLink()
               .setHref(reference.getHref())
               .setRel(Constants.REFERENCE_LINK_REL)
               .setMimeType(reference.getType())
               .setTitle(title.toString());
    }
    
    
    protected String getSummary(Resource resource) {
        return new StringBuilder()
            .append("Resource: " + resource.getName())
            .toString();
    }
}
