/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.atom;

import java.util.List;

import javax.xml.namespace.QName;


import com.iona.repository.rest.resource.Resource;

import org.apache.abdera.model.Element;

/**
 * Class for returning an XHTML rendering of an Abdera Element.  
 * 
 */
public class XhtmlWriter  {

 
    /**
     * Constructs a new XhtmlWriter object.
     * 
     */
    public XhtmlWriter() {
        // Complete.
    }


    /**
     * Returns a String representing the XHTML rendering of an Abdera Element.
     * @param title optional title for the XHTML rendering
     * @param element the Abdera Element to be rendered
     * @return a String representing the XHTML for the Element
     */
    public String toXhtml(String title, Element element) {

        StringBuilder xhtml = new StringBuilder();

        xhtml.append("<p>");
        toXhtml(element, xhtml);
        xhtml.append("</p>");

        return xhtml.toString();
    }

    private void toXhtml(Element element, StringBuilder xhtml) {

        QName qn = element.getQName();
        if (qn.equals(Resource.ATTRIBUTE_QNAME)) {
            String name = element.getAttributeValue("name");
            xhtml.append("<b>").append(XhtmlUtil.encode(name)).append(":</b> ");
            String text = element.getText();
            if (isHidden(element)) {
                text = "********";
            }
            if (text != null && !"".equals(text)) {
                String type = element.getAttributeValue("type");
                if ("xsd:anyURI".equals(type)) {
                    xhtml.append("<a type=\"")
                         .append(type)
                         .append("\" href=\"")
                         .append(text)
                         .append("\"><i>")
                         .append(XhtmlUtil.encode(text))
                         .append("</i></a>");
                } else {
                    xhtml.append("<i>").append(XhtmlUtil.encode(text)).append("</i> ");
                }
            }
        } else if (qn.equals(Resource.REFERENCE_QNAME)) {
            String name = element.getAttributeValue("name");
            xhtml.append("<b>").append(XhtmlUtil.encode(name)).append(":</b> ");
            String href = element.getAttributeValue("href");
            if (href != null) {
                String text = element.getText();
                if (text == null || "".equals(text)) {
                    text = "view";
                }
                xhtml.append("<a href=\"").append(href).append("\">")
                     .append(XhtmlUtil.encode(text)).append("</a>");
            }
            String type = element.getAttributeValue("type");
            xhtml.append(" (").append(type).append(")");
            
         
        } else if (qn.equals(Resource.RESOURCE_QNAME)) {
            String name = element.getAttributeValue("name");
            xhtml.append("<b>").append(XhtmlUtil.encode(name)).append("</b>");
            
        } else {
            xhtml.append("<b>").append(element.getQName().getLocalPart()).append(":</b> ");
            String text = element.getText();
            if (text != null && !text.equals("")) {
                xhtml.append("<i>").append(element.getText()).append("</i>");
            }
        }

        List<Element> elist = element.getElements();
        if (elist.size() == 0) {
            return;
        }

        xhtml.append("<ul>");
        for (Element e : elist) {
            xhtml.append("<li>");
            toXhtml(e, xhtml);
            xhtml.append("</li>");
        }
        xhtml.append("</ul>");
        
    }


    private boolean isHidden(Element element) {
        return "true".equals(element.getAttributeValue("hidden"));
    }

}
