/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;

import java.util.Collection;
import java.util.ResourceBundle;
import java.util.logging.Logger;

import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.repo.EClassHandler;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.rest.rt.osgi.MessageBundle;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EFactory;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;

public abstract class AbstractEClassHandler implements EClassHandler {
    
    protected static final ResourceBundle BUNDLE = MessageBundle.getInstance();
    protected static final Logger LOG = Logger.getLogger(AbstractEClassHandler.class.getName());

    @SuppressWarnings("unchecked")
    public void deleteObject(RESTContext ctx, EObject eobject) throws RESTException {
        
        if (eobject.eContainer() != null) {
            EReference ref = eobject.eContainmentFeature();
            if (ref.isMany()) {
                ((Collection<EObject>)eobject.eContainer().eGet(ref)).remove(eobject);
            } else {
                eobject.eContainer().eUnset(ref);
            }
        
        } else if (eobject.eResource() != null) {
            eobject.eResource().getContents().remove(eobject);
        
        } else {
            // TODO: throw meaningful exception
            throw new RuntimeException();
        }
    }
    
    public void addObjectToResource(RESTContext ctx,
                                    EObject eobject,
                                    Resource resource) throws RESTException {
        throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, new UnsupportedOperationException());
    }

    public EObject createObject(RESTContext ctx,
                                EFactory efactory,
                                EClass eclass,
                                String name,
                                Resource resource,
                                boolean persist) throws RESTException {
        throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, new UnsupportedOperationException());
    }
    
    public Collection<? extends EObject> createObjects(RESTContext ctx,
                                                       EFactory efactory,
                                                       EClass eclass,
                                                       Collection<Resource> resource,
                                                       boolean persist) throws RESTException {
        throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, new UnsupportedOperationException());
    }

    public EObject createObjectWithMedia(RESTContext ctx,
                                         EFactory efactory,
                                         EClass eclass) throws RESTException {
        throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, new UnsupportedOperationException());
    }
      
    public void getMedia(RESTContext ctx, EObject eobject) throws RESTException {
        throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, new UnsupportedOperationException());
    }

    public void getObjectChanges(RESTContext ctx, EObject eobject, Resource resource) throws RESTException {
        throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, new UnsupportedOperationException());
    }

    public EObject updateMedia(RESTContext ctx, EObject eobject) throws RESTException {
        throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, new UnsupportedOperationException());
    }

    public EObject createContainedObject(RESTContext ctx,
                                         EObject containingEObject,
                                         String subcollection) throws RESTException {
        throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, new UnsupportedOperationException());
    }

    public EClass getContainedClass(RESTContext ctx,
                                    EObject containingEObject,
                                    String subcollection) throws RESTException {
        throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, new UnsupportedOperationException());
    }

    @SuppressWarnings("unchecked")
    public EList getContainedObjects(RESTContext ctx, EObject eobject, String refName) throws RESTException {
        throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, new UnsupportedOperationException());
    }
}
