/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;

import com.iona.repository.rest.Constants;
import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.repo.EClassHandler;
import com.iona.repository.rest.resource.Reference;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.rest.rt.server.RESTProvider;
import com.iona.soa.model.Category;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;


/**
 * Custom handler for categories.  This only changes the type for "contained" objects
 * (ICategory.content) from ICategorisable back to ICategory to get these working as 
 * a subcollection,
 * 
 */
public class CategoryHandler extends DefaultEClassHandler implements
        EClassHandler {
    public static final String SUBCOLLECTION_CONTENT_FEATURE = "categoryContent";

    public CategoryHandler(RESTProvider a) {
        super(a);
    }

    @Override
    public void addObjectToResource(RESTContext ctx, EObject eobject, Resource resource) 
        throws RESTException {
        super.addObjectToResource(ctx, eobject, resource);
        
        Reference reference = resource.addReference(SUBCOLLECTION_CONTENT_FEATURE);
        reference.setMin(0);
        reference.setMax(-1);
        reference.setSize(((Category)eobject).getContent().size());
        reference.setType(Constants.ATOM_FEED_CONTENT_TYPE);
        String id = RestEMFUtil.getResourceId(eobject);
        reference.setHref("categories/" + id + ";" + SUBCOLLECTION_CONTENT_FEATURE);
    }
    
    
    @Override
    public EClass getContainedClass(RESTContext ctx, EObject eobject, String refName) throws RESTException {
        if (SUBCOLLECTION_CONTENT_FEATURE.equals(refName)) {
            // Just return dummy class to keep things going.
            return eobject.eClass();
        } 
        return super.getContainedClass(ctx, eobject, refName);
    }

    @SuppressWarnings("unchecked")
    @Override
    public EList getContainedObjects(RESTContext ctx, EObject eobject, String refName) throws RESTException {
        if (SUBCOLLECTION_CONTENT_FEATURE.equals(refName)) {
            Category category = (Category) eobject;
            ctx.setCollectionTitle("Category " + category.getName());
            return category.getContent();
        } 
        return super.getContainedObjects(ctx, eobject, refName);
    }
}
