/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;

import java.util.ArrayList;
import java.util.Collection;

import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.repo.EClassHandler;
import com.iona.repository.rest.resource.Attribute;
import com.iona.repository.rest.resource.Reference;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.rest.rt.server.RESTProvider;
import com.iona.soa.model.Container;
import com.iona.soa.model.repository.Property;

import org.eclipse.emf.ecore.EObject;

public class ContainerHandler extends DefaultEClassHandler implements EClassHandler {

    private static final Collection<String> READ_ONLY_FIELDS = new ArrayList<String>();
    static {
        READ_ONLY_FIELDS.add("status");
        READ_ONLY_FIELDS.add("properties");
        READ_ONLY_FIELDS.add("applications");
        READ_ONLY_FIELDS.add("artifacts");
        READ_ONLY_FIELDS.add("type");
        READ_ONLY_FIELDS.add("supportedPackageTypes");
    }
    
    public ContainerHandler(RESTProvider a) {
        super(a);
    }

    @Override
    public void addObjectToResource(RESTContext ctx,
                                    EObject eobject,
                                    Resource resource) throws RESTException {

        super.addObjectToResource(ctx, eobject, resource);

        for (Attribute attr : resource.getAllAttributes()) {
            if (READ_ONLY_FIELDS.contains(attr.getName())) {
                attr.setReadonly(true);
            }
        }

        for (Reference ref : resource.getAllReferences()) {
            if (READ_ONLY_FIELDS.contains(ref.getName())) {
                ref.setReadonly(true);
            }
        }

        if (!(eobject instanceof Container)) {
            return;
        }

        // TODO - add parameter to RESTQuery (verbose?). Only if set include
        // the properties and status.
        // Better - perhaps have a standard "summary" contained resource to put
        // stuff like this into
        Container c = (Container) eobject;

        for (Property p : c.getProperties()) {
            resource.addAttribute("property." + p.getName())
                .setReadonly(true)
                .setObject(p.getValue())
                .setType("xsd:string")
                .setMin(1)
                .setMax(1);
        }

        for (Property p : c.getStatus()) {
            resource.addAttribute("status." + p.getName())
                .setReadonly(true)
                .setObject(p.getValue())
                .setType("xsd:string")
                .setMin(1)
                .setMax(1);
        }
        
        
    }
    
    
    
}
