/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;

import java.util.Iterator;
import java.util.List;

import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.resource.Attribute;
import com.iona.repository.rest.resource.Reference;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.rest.rt.server.RESTProvider;
import com.iona.repository.util.i18n.Message;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EObjectEList;
import org.eclipse.emf.ecore.util.EcoreUtil;


public class DefaultEFeatureHandler extends AbstractEFeatureHandler {


    public DefaultEFeatureHandler(RESTProvider a) {
        super(a);        
    }

    @SuppressWarnings("unchecked")
    public void addFeatureToResource(String qualName,
                                     EStructuralFeature feature,
                                     EObject eobject,
                                     Resource resource) throws RESTException {
        
        if (feature instanceof EAttribute) {
            if (feature.isMany()) {
                EList elist = (EList) eobject.eGet(feature);
                if (elist.size() == 0) {
                    addAttributeToResource(qualName, (EAttribute) feature, null, resource);
                } else {
                    for (Object o : elist) {
                        addAttributeToResource(qualName, (EAttribute) feature, o, resource);
                    }
                }
            } else {
                Object value = eobject == null ? null : eobject.eGet(feature);
                addAttributeToResource(qualName, (EAttribute) feature, value, resource);
            }
        
        } else if (feature instanceof EReference) {
            EReference ref = (EReference) feature;
            if (ref.isContainment()) {
                if (ref.isMany()) {
                    addSubReferenceToResource(qualName, eobject, ref, resource);
                } else if (eobject != null) {
                    // EObject eo = eobject == null ? null : (EObject) eobject.eGet(ref);
                    if (eobject.eIsSet(ref)) {
                        EObject eo = (EObject) eobject.eGet(ref);
                        addContainedReferenceToResource(ref, eo, resource);
                    }
                    
                }
                
            } else {
                
                if (ref.isMany()) {

                    EList<EObject> elist = new BasicEList<EObject>((EList<EObject>) eobject.eGet(ref));
                    for (Iterator<EObject> iter = elist.iterator(); iter.hasNext();) {
                        EObject eo = iter.next();
                        if (eo.eIsProxy()) {
                            eo = EcoreUtil.resolve(eo, eobject);
                        }
                        if (RestEMFUtil.isInternal(eo)) {
                            iter.remove();
                        } else {
                            addReferenceToResource(ref, eo, ref.getLowerBound(), 
                                                   ref.getUpperBound(), resource);
                        }
                    }
                    if (elist.size() == 0) {
                        addReferenceToResource(ref, null, ref.getLowerBound(), 
                                               ref.getUpperBound(), resource);
                    }
                } else {
                    EObject eo = eobject == null ? null : (EObject) eobject.eGet(ref);
                    if (eo != null && eo.eIsProxy()) {
                        eo = EcoreUtil.resolve(eo, eobject);
                    }
                    addReferenceToResource(ref, RestEMFUtil.isInternal(eo) ? null : eo, 
                                           ref.getLowerBound(), ref.getUpperBound(), resource);
                }
            }
        } else {
            restProvider.internalError("only attributes and references are supported");
        }
    }




    private void addContainedReferenceToResource(EReference ref, EObject eobject,
                                                 Resource resource)
        throws RESTException {
        
        
        // TODO: delegate back to appropiate EClassHandler and add a prefix to the feature names.
        EClass refClass = ref.getEReferenceType();
        for (EStructuralFeature feature : refClass.getEStructuralFeatures()) {
            
            String qualName = RestEMFUtil.getQualifiedName(feature);

            if (restProvider.isFeatureExcluded(qualName)) {            
                continue;
            }

            EFeatureHandler handler = restProvider.getEFeatureHandler(qualName);
            if (handler == null) {
                handler = this;
            }

            handler.addFeatureToResource(qualName, feature, eobject, resource);
        }        
    }

    @SuppressWarnings("unchecked")
    public void getFeatureChanges(RESTContext ctx, String qualName, EStructuralFeature feature,
                               EObject eobject, Resource resource) 
        throws RESTException {

        if (feature instanceof EAttribute) {
            EAttribute eAtt = (EAttribute) feature;
            if (eAtt.isMany()) {
                List<Attribute> atts = resource.getAttributes(eAtt.getName());
                EList elist = (EList) eobject.eGet(eAtt);
                if (elist == null) {
                    restProvider.internalError("null list for attribute " + eAtt.getName());
                }
                EList newList = new BasicEList();

                // If only reference is empty remove it.
                // TODO: Once a resource is serialized as XML there is no distinction
                // between an empty string and a null string.  So may need to handle
                // that case here (&& "".equals(atts.get(0).getText())?)
                if (atts.size() == 1 && atts.get(0).getText() == null) {
                    atts.clear();
                }

                if (atts.size() == 0 && elist.size() == 0) {
                    return;
                }
                
                for (Attribute att : atts) {                    
                    newList.add(getObject(att, eAtt.getEAttributeType()));
                }
                getAttributeListChanges(ctx, eAtt, eobject, newList, elist);
            } else {             
                getAttributeChanges(ctx, (EAttribute) feature, eobject, resource);
            }

        } else if (feature instanceof EReference) {
    
            EReference eRef = (EReference) feature;
            if (eRef.isContainment()) {
                LOG.fine("containment reference; ignoring");
                return;
            }
            
            List<Reference> refs = resource.getReferences(feature.getName());
            if (eRef.isMany()) {
                EObjectEList<EObject> elist = (EObjectEList<EObject>) eobject.eGet(eRef);
                if (elist == null) {
                    restProvider.internalError("null list for reference " + eRef.getName());
                }
                EList<EObject> newList = new BasicEList<EObject>();

                // If only reference is empty remove it.
                if (refs.size() == 1 && refs.get(0).getHref() == null) {
                    refs.clear();
                }
                
                if (refs.size() == 0 && elist.size() == 0) {
                    return;
                }
                
                for (Reference ref : refs) {
                    EObject o = getReferencedObject(ref, ctx);
                    if (o != null) {
                        newList.add(o);    
                    }
                    
                }
                getReferenceListChanges(ctx, eRef, eobject, newList, elist);

            } else {
                if (refs.size() > 1) {
                    throw new RESTException(
                            RESTException.BAD_REQUEST,
                            new Message(BUNDLE, "TOO_MANY_REFS", eRef.getName()));
                }
                if (refs.size() == 0) {
                    
                    getReferenceChanges(ctx, eRef, eobject, null);
                } else {               
                    EObject oldValue = (EObject) eobject.eGet(eRef);
                    EObject newValue = getReferencedObject(refs.get(0), ctx);
                    if (oldValue != newValue) {
                        getReferenceChanges(ctx, eRef, eobject, newValue);
                    }
                }
            }
        } else {
            restProvider.internalError("only attributes and references supported");            
        }
    }
    
    

}
