/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;

import java.util.List;

import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.repo.EClassHandler;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.rest.rt.server.RESTProvider;
import com.iona.soa.model.DeploymentProfile;
import com.iona.soa.model.SOA_NetworkFactory;
import com.iona.soa.model.VersionedProfile;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EFactory;
import org.eclipse.emf.ecore.EObject;

public class DeploymentProfileHandler extends DefaultEClassHandler implements
        EClassHandler {

    public DeploymentProfileHandler(RESTProvider a) {
        super(a);
    }

    @Override
    public EObject createObject(RESTContext ctx,
                                EFactory efactory,
                                EClass eclass,
                                String name,
                                Resource resource,
                                boolean persist) throws RESTException {
        
        EObject created = super.createObject(ctx, efactory, eclass, name, resource, persist);
        
        // For now maintain a facade for the old model - one version that is active
        if (created instanceof DeploymentProfile) {
            DeploymentProfile dp = (DeploymentProfile)created;
            VersionedProfile vp = SOA_NetworkFactory.eINSTANCE.createVersionedProfile();
            vp.setName(dp.getName());
            dp.getVersions().add(vp);
            dp.setCurrent(vp);
            // Do not set active reference here.  VersionedProfile::publishNow boolean
            // is used when editing to specify this.  
        }        
        
        return created;
    }
    
    @Override
    public void getObjectChanges(RESTContext ctx,
                                 EObject eobject,
                                 Resource resource) throws RESTException {
        
        super.getObjectChanges(ctx, eobject, resource);
        
        if (eobject instanceof DeploymentProfile) {
            if (resource.getFirstReference("active") != null) {
                // A revert call from the client
                DeploymentProfile dp = (DeploymentProfile)eobject;
                dp.setCurrent(dp.getActive());
                dp.setName(dp.getCurrent().getName());
                
                // Prune anything appearing after the active profile
                int idx = dp.getVersions().indexOf(dp.getActive());
                if (idx > -1 && idx < dp.getVersions().size() - 1) {
                    List<VersionedProfile> toPrune =  
                        dp.getVersions().subList(idx + 1, dp.getVersions().size());
                    dp.getVersions().removeAll(toPrune);
                }
            }
        }
    }
}
