/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;

import com.iona.repository.rest.Constants;
import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.repo.EClassHandler;
import com.iona.repository.rest.resource.Reference;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.rest.rt.server.RESTProvider;
import com.iona.soa.model.Folder;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;


/**
 * Custom handler for folders.  This adds a contained feature called "allContent"
 * that is a merger of the "content" and "children" reference lists.
 * 
 */
public class FolderHandler extends DefaultEClassHandler implements
        EClassHandler {
    
    public static final String ALL_CONTENT_FEATURE = "allContent";

    public FolderHandler(RESTProvider a) {
        super(a);
    }

    @Override
    public void addObjectToResource(RESTContext ctx, EObject eobject, 
                                    Resource resource)
        throws RESTException {
        super.addObjectToResource(ctx, eobject, resource);
        
        Folder folder = (Folder) eobject;
        addAllContentReference(folder, resource);
    }
    
    
    @Override
    public EClass getContainedClass(RESTContext ctx, EObject eobject,
                                    String refName) 
        throws RESTException {

        if (ALL_CONTENT_FEATURE.equals(refName)) {
            // Just return dummy class to keep things going.
            return eobject.eClass();
        } 
        
        return super.getContainedClass(ctx, eobject, refName);
    }

    @SuppressWarnings("unchecked")
    @Override
    public EList getContainedObjects(RESTContext ctx, EObject eobject,
                                     String refName) 
        throws RESTException {

        if (ALL_CONTENT_FEATURE.equals(refName)) {
            Folder folder = (Folder) eobject;
            ctx.setCollectionTitle("Folder " + folder.getName());
            return getAllContents(ctx, folder);
        } 
        
        return super.getContainedObjects(ctx, eobject, refName);
    }


    @SuppressWarnings("unchecked")
    private void addAllContentReference(Folder folder, Resource resource) throws RESTException {
        
        // Merger of content and children
        
        EList children = folder.getChildren();        
        EList content = folder.getContent();
        
        Reference reference = resource.addReference(ALL_CONTENT_FEATURE);
        reference.setMin(0);
        reference.setMax(-1);
        reference.setSize(children.size() + content.size());
        reference.setType(Constants.ATOM_FEED_CONTENT_TYPE);
        
        String id = RestEMFUtil.getResourceId(folder);
        reference.setHref("folders/" + id + ";" + ALL_CONTENT_FEATURE);
    }
    
    @SuppressWarnings("unchecked")
    private EList getAllContents(RESTContext ctx, Folder folder) {
        
        EList elist = new BasicEList();
        elist.addAll(folder.getChildren());
        elist.addAll(folder.getContent());
        
        return elist;
    }


}
