/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;

import java.util.Collection;
import java.util.List;
import java.util.logging.Level;

import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.repo.EClassHandler;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.rest.rt.server.RESTProvider;
import com.iona.soa.expression.Equals;
import com.iona.soa.expression.Expression;
import com.iona.soa.model.security.ObjectPermission;
import com.iona.soa.model.security.Role;
import com.iona.soa.model.security.SecurityPackage;
import com.iona.soa.model.security.User;
import com.iona.soa.resourceset.ResourceSetQueryHandler;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EFactory;
import org.eclipse.emf.ecore.EObject;


public class ObjectPermissionHandler extends DefaultEClassHandler implements
        EClassHandler {

    public ObjectPermissionHandler(RESTProvider a) {
        super(a);
    }
    
   
    @Override
    public EObject createObject(RESTContext ctx, EFactory efactory, EClass eclass, 
                                String name, Resource resource, boolean persist)
        throws RESTException {
        
        ObjectPermission objPerm = 
            (ObjectPermission)super.createObject(ctx, efactory, eclass, name, resource, persist);
        addCurrentUser(objPerm, ctx);
        checkPermissions(objPerm);
        return objPerm;
    }

    public void getObjectChanges(RESTContext ctx,
            EObject eobject,
            Resource resource) throws RESTException {
        super.getObjectChanges(ctx, eobject, resource);
        addCurrentUser((ObjectPermission)eobject, ctx);
        checkPermissions((ObjectPermission)eobject);
    }
    
    private void checkPermissions(ObjectPermission objPerm) {
        // make sure that permission hierarchy is maintained
        Collection<User> users = objPerm.getUsersUpdate();
        Collection<Role> roles = objPerm.getRolesUpdate();
        objPerm.getUsersDelete().addAll(users);
        objPerm.getRolesDelete().addAll(roles);

        users = objPerm.getUsersDelete();
        roles = objPerm.getRolesDelete();
        objPerm.getUsersRead().addAll(users);
        objPerm.getRolesRead().addAll(roles);
    }

    private void addCurrentUser(ObjectPermission objPerm, RESTContext ctx) throws RESTException {
        try {
            String userId = RepositoryRESTAdapter.getSecurityContext(ctx).getUser();
            Expression expr = new Equals(SecurityPackage.eINSTANCE.getUser_Userid().getName(), userId);
            List<EObject> users = ResourceSetQueryHandler.executeQuery(
                    restProvider.getRepository().getResourceSet(), 
                    SecurityPackage.eINSTANCE.getUser(), 
                    expr, true);
            if (users.size() > 0) {
                User user = (User)users.get(0);
                objPerm.getUsersRead().add(user);
                objPerm.getUsersUpdate().add(user);
                objPerm.getUsersDelete().add(user);
            }
        } catch (Exception ex) {
            LOG.log(Level.WARNING, "Unable to add current user to ObjectPermission!", ex);
            throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, ex);
        }
    }
    
}
