/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;


import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.resource.Attribute;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.rest.rt.server.RESTProvider;
import com.iona.soa.security.SecurityUtil;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;

/**
 * Feature handler for passwords.  Stores the attribtue as a MD5 hash.
 *
 */
public class PasswordHandler extends DefaultEFeatureHandler implements EFeatureHandler {
    private boolean storeDigest;

    public PasswordHandler(RESTProvider a, boolean storeDigest) {
        super(a);
        this.storeDigest = storeDigest;
    }

    @Override
    public void getFeatureChanges(RESTContext ctx, String qualName, 
                           EStructuralFeature feature, EObject eobject, 
                           Resource resource)
        throws RESTException {

        EAttribute eAtt = (EAttribute) feature;
        Attribute att = resource.getFirstAttribute(eAtt.getName());
        if (att == null) {
            return;
        }
        
        String password = att.getText();
        if (password == null || "".equals(password)) {
            return;
        }
        
        LOG.info("Updating password attribute " + eAtt.getName());
        
        if (storeDigest) {
            password = SecurityUtil.calculateDigestAsHex(password);
        }
                
        eobject.eSet(eAtt, password);
        
    }
}
