/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ResourceBundle;

import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.rt.osgi.MessageBundle;
import com.iona.repository.util.i18n.Message;
import com.iona.soa.model.repository.IPersistableObject;

import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;

public final class RestEMFUtil {
    
    private static final ResourceBundle BUNDLE = MessageBundle.getInstance();

    private RestEMFUtil() {
        // Complete.
    }
    
    // TODO Merge with com.iona.soa.repository.EMFUtil
    
    
    public static void setEObjectName(EObject eobject, String name) {
    
        if (name == null) {
            return;
        }
        
        EStructuralFeature feature = eobject.eClass().getEStructuralFeature("name");
        if (feature instanceof EAttribute) {
            eobject.eSet(feature, name);
        }
    }

    public static String getResourceId(EObject eobject) throws RESTException {
               
        if (!(eobject instanceof IPersistableObject)) {
            throw new RESTException(RESTException.INTERNAL_SERVER_ERROR,
                    new Message(BUNDLE, "OBJECT_NOT_PERSIST", eobject.eClass().getName()));
        }
        IPersistableObject po = (IPersistableObject) eobject;
        
        // Assumes a GUID is always prefixed by the string "<Class>-"
        
        String id = po.getGuid();
        if (id != null) {
            int i = id.indexOf('-');
            if (i != -1) {
                id = id.substring(i + 1);            
            }
        }
        return id;
    }

    public static String getResourceName(EObject eobject) {
        
        EStructuralFeature feature = eobject.eClass().getEStructuralFeature("name");
        if (feature instanceof EAttribute && eobject.eIsSet(feature)) {
            return (String) eobject.eGet(feature).toString();        
        } else {
            return eobject.eClass().getName();
        }
    }
    

    
    public static String getQualifiedName(EStructuralFeature f) {
        return new StringBuilder()
            .append(f.getEContainingClass().getEPackage().getName())
            .append(".")
            .append(f.getEContainingClass().getName())
            .append(".")
            .append(f.getName())
            .toString();
    }


    public static String getQualifiedClassName(EClass ec) {
        return new StringBuilder()
            .append(ec.getEPackage().getName())
            .append(".")
            .append(ec.getName())
            .toString();        
    }

    public static String getObjectID(EClass ec, String resourceID) {
        return new StringBuilder()
            .append(ec.getName())
            .append("-")
            .append(resourceID)
            .toString();        
    }
    
    public static boolean isClob(EAttribute att) {
        EAnnotation annotation = att.getEAnnotation("dbhints");
        if (annotation != null) {
            Object type = annotation.getDetails().get("type");
            if (type instanceof String) {
                return "CLOB".equalsIgnoreCase((String)type);
            }
        }
        return false;
    }

    public static boolean isInternal(EObject eobj) {
        if (eobj instanceof IPersistableObject) {
            return ((IPersistableObject)eobj).isInternal();
        } else {
            return false;
        }
    }

    public static boolean isModifiable(EObject eobj) {
        if (eobj instanceof IPersistableObject) {
            return ((IPersistableObject)eobj).isModifiable();
        } else {
            return true;
        }
    }
    
    // Dump input stream from context without comsuming it.
    public void dumpInputStream(RESTContext ctx) {
        
        try {
            InputStream is = ctx.getInputStream();
            
            ByteArrayOutputStream os = new ByteArrayOutputStream(1024);
            
            byte[] buffer = new byte[1024];
            int len;

            len = is.read(buffer);
            while (len >= 0) {
                os.write(buffer, 0, len);
                len = is.read(buffer);
            }

            is.close();
            os.close();
            
            byte[] bytes = os.toByteArray();
            
            String content = new String(bytes);
            System.out.println("-------Input Stream Contents---------");
            System.out.println(content);
            System.out.println("-------------------------------------");
            
            is = new ByteArrayInputStream(bytes);
            ctx.setInputStream(is);
        } catch (IOException e) {
            e.printStackTrace();
        }
            
    }
    
    public void dump(InputStream is) {
        try {
            System.out.println("*** Content ----- start -----");
            BufferedReader br = new BufferedReader(new InputStreamReader(is));

            String line = null;
            line = br.readLine();
            while (line != null) {
                System.out.println(line);
                line = br.readLine();
            }
            br.close();
            System.out.println("*** Content ----- end -----");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
