/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;


import com.iona.repository.rest.Constants;
import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.repo.EClassHandler;
import com.iona.repository.rest.resource.Reference;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.rest.rt.server.RESTProvider;
import com.iona.soa.model.StoredQuery;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

public class StoredQueryHandler extends DefaultEClassHandler implements
        EClassHandler {
    
    public static final String QUERY_RESULT_FEATURE = "queryResult";
    
    private static final String DEFAULT_COLLECTION = "search";

    public StoredQueryHandler(RESTProvider a) {
        super(a);
    }
    
    @Override
    public void addObjectToResource(RESTContext ctx, EObject eobject, 
                                    Resource resource)
        throws RESTException {
        super.addObjectToResource(ctx, eobject, resource);
        
        StoredQuery query = (StoredQuery) eobject;
        addQueryResultFeature(ctx, query, resource);
    }
        

    @Override
    public EClass getContainedClass(RESTContext ctx, EObject eobject,
                                    String refName) 
        throws RESTException {

        if (QUERY_RESULT_FEATURE.equals(refName)) {
            // Just return dummy class to keep things going.
            return eobject.eClass();
        } 
        
        return super.getContainedClass(ctx, eobject, refName);
    }

    @SuppressWarnings("unchecked")
    @Override
    public EList getContainedObjects(RESTContext ctx, EObject eobject,
                                     String refName) 
        throws RESTException {

        if (QUERY_RESULT_FEATURE.equals(refName)) {
            StoredQuery query = (StoredQuery) eobject;
            String collection = query.getCategory();
            if (collection == null || "".equals(collection)) {
                collection = DEFAULT_COLLECTION;
            }
            ctx.setCollectionTitle("Query " 
                                   + query.getType()
                                   + "=" + query.getQuery() 
                                   + " [" + collection + "]");
            return restProvider.getQueryExecutor(collection).runStoredQuery(ctx, 
                                                                 restProvider.getEClass(collection), 
                                                                 query);
        } 
        
        return super.getContainedObjects(ctx, eobject, refName);
    }

    private void addQueryResultFeature(RESTContext ctx, StoredQuery query,
                                       Resource resource) 
        throws RESTException {
        
        Reference reference = resource.addReference(QUERY_RESULT_FEATURE);
        reference.setMin(0);
        reference.setMax(-1);
        reference.setSize(-1);
        reference.setType(Constants.ATOM_FEED_CONTENT_TYPE);
        
        String id = RestEMFUtil.getResourceId(query);
        reference.setHref("queries/" + id + ";" + QUERY_RESULT_FEATURE);

    }

}
