/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.logging.Level;


import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.RESTQuery;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.rest.rt.osgi.RestServiceBean;
import com.iona.repository.taskservice.TaskService;

import com.iona.soa.repository.SOARepository;
import com.iona.soa.repository.TransactionData;
import com.iona.soa.search.SearchService;

public class TaskServiceQueryExecutor extends RESTQueryExecutor {


    
    private RestServiceBean restServiceBean;

    public TaskServiceQueryExecutor(SOARepository soaRepo,
                                    SearchService searchService,
                                    RestServiceBean restServiceBean) {
        super(soaRepo, searchService);
        this.restServiceBean = restServiceBean;
    }

    @Override
    public Collection<Resource> queryResources(RESTContext ctx,
                                               TransactionData trnsData,
                                               RESTQuery query) throws RESTException {
        
        String queryStr = query.getDecodedParameter(RESTQuery.QUERY_SEARCH);
        return queryTaskServices(queryStr);
    }
    
    private Collection<Resource> queryTaskServices(String query) throws RESTException {

        try {
            
            List<Resource> resources = new ArrayList<Resource>();
            Collection<TaskService> taskList = restServiceBean.getTaskServices();
            
            for (TaskService task : taskList) {

                Resource resource = getResourceFactory().createResource(task.getTaskName());
                resource.setId(task.getTaskTypeID());
                addAttribute(resource, "typeid", task.getTaskTypeID(), "xsd:normalizedString");
                addAttribute(resource, "name", task.getTaskName(), "xsd:normalizedString");
                addAttribute(resource, "description", task.getTaskDescription(), "xsd:normalizedString");
                resources.add(resource);
    
            }

            return resources;
            
        } catch (Exception e) {
            getLogger().log(Level.SEVERE, "Error querying lucene index: " + e.getLocalizedMessage());
            // Just return an empty collection instead of raising an exception.
            return new ArrayList<Resource>();
        }
    }

}
