/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;

import java.sql.Timestamp;

import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.resource.Resource;
import com.iona.soa.model.repository.IPersistableObject;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;

//
// Custom handler for timestamps.  This converts the timestamp into two resource
// attributes: one being the actual Timestamp object and another the millisecond
// representation of the time.
// In the future may get rid of the Timestamp object and just leave the milisecond
// version and let the client convert the milliseconds to a displayable time.
//

public class TimestampHandler extends AbstractEFeatureHandler implements
        EFeatureHandler {

    public TimestampHandler(RepositoryRESTAdapter a) {
        super(a);
    }

    public void addFeatureToResource(String qualName, EStructuralFeature feature, EObject eobject, 
                                     Resource resource)
        throws RESTException {
        
        IPersistableObject po = (IPersistableObject) eobject;
        Timestamp ts = po.getTimestamp();
        
        resource.addAttribute("timestamp")
            .setReadonly(true)
            .setObject(ts)
            .setType("xsd:dateTime")
            .setMin(feature.getLowerBound())
            .setMax(feature.getUpperBound());
        
        resource.addAttribute("msTimestamp")
            .setReadonly(true)
            .setObject(ts == null ? null : ts.getTime())
            .setType("xsd:xint")
            .setMin(feature.getLowerBound())
            .setMax(feature.getUpperBound());
    }
    
    public void getFeatureChanges(RESTContext ctx, String qualName,
                                  EStructuralFeature feature, EObject eobject, 
                                  Resource resource)
        throws RESTException {
        // Complete
    }

}
