/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;

import java.io.IOException;

import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.repo.EClassHandler;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.rest.rt.server.RESTProvider;
import com.iona.repository.util.i18n.Message;
import com.iona.soa.expression.Equals;
import com.iona.soa.expression.Expression;
import com.iona.soa.model.security.Role;
import com.iona.soa.model.security.SecurityPackage;
import com.iona.soa.model.security.User;
import com.iona.soa.repository.EMFUtil;
import com.iona.soa.resourceset.ResourceSetQueryHandler;
import com.iona.soa.security.AuthorizationException;
import com.iona.soa.security.SecurityUtil;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EFactory;
import org.eclipse.emf.ecore.EObject;


public class UserHandler extends DefaultEClassHandler implements
        EClassHandler {

    public UserHandler(RESTProvider a) {
        super(a);
    }
    
   
    @Override
    public EObject createObject(RESTContext ctx, EFactory efactory, EClass eclass, 
                                String name, Resource resource, boolean persist)
        throws RESTException {
        
        // Validation requires that userid are unique. 
        String baseName;
        if (resource == null || resource.getAttributes("userid").size() == 0 
            || resource.getFirstAttribute("userid").getText() == null) {
            baseName = "user";
        } else {
            baseName = resource.getFirstAttribute("userid").getText();
        }
        int count = 0;
        String userId = baseName;
        int existingUsers = 0;
        do {
            Expression expr = new Equals(SecurityPackage.eINSTANCE.getUser_Userid().getName(), userId);
            try {
                existingUsers = ResourceSetQueryHandler.executeQuery(
                        restProvider.getRepository().getResourceSet(), 
                        SecurityPackage.eINSTANCE.getUser(), 
                        expr, true).size();
            } catch (Exception e) {
                // ignore
            }
            if (existingUsers == 0) {
                break;
            }
            userId =  baseName + "-" + (++count);
        } while (true);
        
        User user = (User)efactory.create(eclass);
        RestEMFUtil.setEObjectName(user, name);
        initRequiredAttributes(user);
        
        if (resource == null || resource.getAttributes("userid").size() == 0) {
            user.setUserid(userId);
        } else {
            resource.getFirstAttribute("userid").setText(userId);
            getObjectChanges(ctx, user, resource);        
        }
        
        if (persist) {
            try {
                ctx.getRepo().persist(user);
            } catch (IOException e) {
                throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, e);
            }
        }

        return user;
    }

    public void getObjectChanges(RESTContext ctx,
            EObject eobject,
            Resource resource) throws RESTException {
        super.getObjectChanges(ctx, eobject, resource);
        
        User user = (User)eobject;
        if (SecurityUtil.ADMIN_USER_GUID.equals(user.getGuid())) {
            for (Role role : user.getRoles()) {
                if (SecurityUtil.ADMIN_ROLE_GUID.equals(role.getGuid())) {
                    return;
                }
            }
            throw new AuthorizationException(
                    new Message(BUNDLE, "INTERNAL_OBJECT", "update", "User 'Admin'"));
        }
    }
    
    public void deleteObject(RESTContext ctx, EObject eobject) throws RESTException {
        if (SecurityUtil.ADMIN_USER_GUID.equals(EMFUtil.getGuid(eobject))) {
            throw new AuthorizationException(
                    new Message(BUNDLE, "INTERNAL_OBJECT", "delete", "User 'Admin'"));
        }
        super.deleteObject(ctx, eobject);
    }       
}
