/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;


import java.util.Collection;

import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.resource.Attribute;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.rest.rt.server.RESTProvider;
import com.iona.repository.util.i18n.Message;
import com.iona.soa.expression.Equals;
import com.iona.soa.expression.Expression;
import com.iona.soa.model.security.SecurityPackage;
import com.iona.soa.model.security.User;
import com.iona.soa.resourceset.ResourceSetQueryHandler;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;

/**
 * Feature handler for userIds.  Makes user that userIds are unique
 *
 */
public class UserIdHandler extends DefaultEFeatureHandler implements EFeatureHandler {


    public UserIdHandler(RESTProvider a) {
        super(a);
    }

    @Override
    public void getFeatureChanges(RESTContext ctx, String qualName, 
                           EStructuralFeature feature, EObject eobject, 
                           Resource resource)
        throws RESTException {

        EAttribute eAtt = (EAttribute) feature;
        Attribute att = resource.getFirstAttribute(eAtt.getName());
        if (att == null) {
            return;
        }
        
        String userId = att.getText();
        if (userId == null || "".equals(userId) || userId.equals(((User)eobject).getUserid())) {
            return;
        }
        
        LOG.fine("Updating userId attribute " + eAtt.getName());
                
        int existingUsers;
        Expression expr = new Equals(SecurityPackage.eINSTANCE.getUser_Userid().getName(), userId);
        try {
            Collection<EObject> eObjs = ResourceSetQueryHandler.executeQuery(
                    restProvider.getRepository().getResourceSet(), 
                    SecurityPackage.eINSTANCE.getUser(), 
                    expr, true);
            existingUsers = eObjs.contains(eobject) ? eObjs.size() - 1 : eObjs.size();
        } catch (Exception e) {
            throw new RESTException(RESTException.INTERNAL_SERVER_ERROR, e);
        }
        
        if (existingUsers != 0) {
            throw new RESTException(RESTException.BAD_REQUEST, 
                                    new Message(BUNDLE, "DUPLICATE_USERID", userId));
        }

        eobject.eSet(eAtt, userId);
        
    }
}
