/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.repo;

import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.resource.Attribute;
import com.iona.repository.rest.resource.Resource;
import com.iona.repository.util.i18n.Message;
import com.iona.soa.model.repository.RepositoryPackage;
import com.iona.soa.model.repository.Version;
import com.iona.soa.repository.model.util.VersionHelper;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;

/**
 * Handler to convert between Version objects and strings.
 * 
 * The type of the attribute is resource:version which is defined as
 * 
 * <xs:simpleType name="version">
 *   <xs:restriction base="xs:normalizedString">
 *     <xs:pattern value="[0..9][.0-9]*[.0-9a-zA-Z@]"/>    <!-- approx... -->
 *   </xs:restriction>
 * <xs:/simpleType>
 * 
 */

public class VersionHandler extends AbstractEFeatureHandler implements
        EFeatureHandler {

    public VersionHandler(RepositoryRESTAdapter a) {
        super(a);
    }

    public void addFeatureToResource(String qualName,
                                     EStructuralFeature feature, 
                                     EObject eobject,
                                     Resource resource)
        throws RESTException {
        
        Object o = eobject.eGet(feature);
        
        if (o == null) {
            resource.addAttribute("version")
                .setType("resource:version")
                .setMin(0)
                .setMax(1);
            return;
        }
        
        if (!(o instanceof Version)) {
            restProvider.internalError("object is not a version");
        }
        
        Version version = (Version) o;
        
        String versionStr = VersionHelper.versionToString(version);
        resource.addAttribute("version")
            .setType("resource:version")
            .setMin(0)
            .setMax(1)
            .setText(versionStr);
    }

    public void getFeatureChanges(RESTContext ctx,
                                  String qualName,
                                  EStructuralFeature feature,
                                  EObject eobject, 
                                  Resource resource)
        throws RESTException {
        
        Version newVersion = null;      
        Attribute resourceAtt = resource.getFirstAttribute("version");
        if (resourceAtt != null) {
        
            String versionStr = resourceAtt.getText();
            if (versionStr != null && !"".equals(versionStr)) {
                try {
                    newVersion = VersionHelper.parseVersion(versionStr);
                
                } catch (IllegalArgumentException e) {
                    throw new RESTException(RESTException.BAD_REQUEST,
                            new Message(BUNDLE, "BAD_VERSION", versionStr));            
                }
            }
        }
        
        Object o = eobject.eGet(feature);
        if (o == null && newVersion == null) {
            return;            
        }
        
        if (o == null && newVersion != null) {
            eobject.eSet(RepositoryPackage.eINSTANCE.getIVersionedObject_Version(), newVersion);
            return;
        }
        
        if (o != null && newVersion == null) {
            eobject.eSet(RepositoryPackage.eINSTANCE.getIVersionedObject_Version(), newVersion);
            return;
                       
        }


        Version oldVersion = (Version) eobject.eGet(feature);
        oldVersion.setMajor(newVersion.getMajor());
        oldVersion.setMinor(newVersion.getMinor());
        oldVersion.setMicro(newVersion.getMicro());
        oldVersion.setNano(newVersion.getNano());
        oldVersion.setQualifier(newVersion.getQualifier());
        oldVersion.setOriginalString(newVersion.getOriginalString());
    }
}
