/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.server;

import java.util.List;

import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.RESTQuery;
import com.iona.repository.rest.rt.repo.CollectionDescriptor;

public interface RESTAdapter {

    /**
     * Returns a list of all supported collections.
     * @return List of collection names; must not be null.
     */
    List<String> getCollections();
        

    /**
     * Returns the list of media types accepted by the collection.
     * @param name the collection name 
     * @return array of media types
     * @throws RESTException if any error occurs
     */
    String[] getCollectionAccepts(String name) throws RESTException;
    
    /**
     * Return a list of resources for a collection
     * @param ctx REST request context.
     * @param collection collection name
     * @param query 
     * @throws RESTException if any error occurs
     * TODO: Alterate approach if lists get too big is to pass a callback
     * that is invoked for each resource to be added to a feed.
     */
    void getCollection(RESTContext ctx, String collection, RESTQuery query) throws RESTException;
    
    /**
     * Returns a list of resources for a subcollection of a resource
     * @param ctx REST request context.
     * @param collection collection containing the resource
     * @param id resource ID
     * @param subcollection resource's sub-collection ID
     * @throws RESTException if any error occurs
     */
    void getSubCollection(RESTContext ctx,
                          String collection,
                          String id,
                          String subcollection) throws RESTException;
    
    /**
     * Returns a resource from a collection.
     * @param ctx REST request context.
     * @param collection the collection containing the resource
     * @param id the resource's identifier
     * @throws RESTException if any error occurs
     */
    void getResource(RESTContext ctx, String collection, String id) throws RESTException;

    /**
     * Updates an existing resource.
     * @param ctx REST request context.
     * @param collection the collection containing the resource
     * @param id the resource's ID 
     * @throws RESTException
     */
    void updateResource(RESTContext ctx, String collection, String id) throws RESTException;

    /**
     * Creates a new resource in a collection.
     * @param ctx REST request context.
     * @param collection the collection in which the resource is to be created.
     * @throws RESTException if any error occurs
     */
    void createResource(RESTContext ctx, String collection) throws RESTException;

    /**
     * Creates a new resources in a collection.
     * @param ctx REST request context.
     * @param collection the collection in which the resources are to be created.
     * @throws RESTException if any error occurs
     */
    void createResources(RESTContext ctx, String collection) throws RESTException;
    
    void createSubcollectionResource(RESTContext ctx,
                                     String collection,
                                     String id,
                                     String subcollection) throws RESTException;


    /**
     * Create a new resource for a new media type.
     * @param ctx REST request context.
     * @param collection the collection in which the media resource is to be created.
     * @throws RESTException if any error occurs
     */
    void createMediaResource(RESTContext ctx, String collection) throws RESTException;

    void getMediaResource(RESTContext ctx, String collection, String id) throws RESTException;

    void updateMediaResource(RESTContext ctx, String collection, String id) throws RESTException;

    /**
     * Deletes a resource from a collection
     * @param ctx REST request context.
     * @param collection the collection containing the resource
     * @param id the resource ID
     * @throws RESTException if any error occurs
     */
    void deleteResource(RESTContext ctx, String collection, String id) throws RESTException;

    CollectionDescriptor getDescriptor(String collection) throws RESTException;
}
