/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.server;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import com.iona.repository.rest.RESTContext;
import com.iona.repository.rest.RESTQuery;
import com.iona.repository.rest.resource.Resource;
import com.iona.soa.repository.SOARepository;
import com.iona.soa.security.SecurityContext;

import org.apache.abdera.protocol.server.RequestContext;
import org.apache.abdera.protocol.server.RequestContext.Scope;

public class RESTContextImpl implements RESTContext {
    
    private RequestContext request;
    private RESTQuery query;
    private Resource resource;
    private List<Resource> resourceList;
    private InputStream inputStream;
    private String contentType;
    private String collection;
    private SOARepository soaRepo;
    private String slug;
    private String collectionTitle;
    private SecurityContext securityContext;
    private boolean isEditable = true;
    
    public RESTContextImpl() {
        this.request = null;
    }
    
    public RESTContext setRequest(RequestContext r) {
        request = r;
        return this;
    }
    
    public String getSlug() {
        if (slug != null) {
            return slug;
        }
        if (request != null) {
            return request.getSlug();
        }
        return null;
    }
    
    public RESTContext setSlug(String s) {
        slug = s;
        return this;
    }
    
    public RESTContext setSecurityContext(SecurityContext s) {
        securityContext = s;
        return this;
    }
    
    public SecurityContext getSecurityContext() {
        return securityContext;
    }

    public RESTContext setSSOToken(String s) {
        //ssoToken = s;
        return this;
    }
    

    public String getContentType() {
        return contentType;
    }
    
    public RESTContext setContentType(String ct) {
        contentType = ct;
        return this;
    }
    
    public InputStream getInputStream() throws IOException {
        return inputStream;
    }

    public RESTContext setInputStream(InputStream is) {
        inputStream = is;
        return this;
    }
    
    public List<Resource> getResourceList() {
        return resourceList;
    }
    
    public RESTContext setResourceList(List<Resource> l) {
        resourceList = l;
        return this;
    }
    
    public Resource getResource() {
        return resource;
    }
    
    public RESTContext setResource(Resource r) {
        resource = r;
        return this;
    }
    
    public String getBaseUri() {
        if (request != null) {
            String basePath = request.getTargetBasePath();
            String base = request.getBaseUri().toString();
            if (basePath.startsWith("/")) {
                return base + basePath.substring(1);
            }
            return base + basePath;
        }
        return null;
    }

    public String getCollection() {
        return collection;
    }
    
    public RESTContext setCollection(String c) {
        collection = c;
        return this;
    }

    public RESTQuery getRestQuery() {
        return query;
    }
    
    public RESTContext setRestQuery(RESTQuery aQuery) {
        query = aQuery;
        return this;
    }

    public String getCollectionTitle() {
        return collectionTitle;
    }
    
    public RESTContext setCollectionTitle(String n) {
        collectionTitle = n;
        return this;
    }

    public SOARepository getRepo() {
        return soaRepo;
    }

    public RESTContext setRepo(SOARepository repo) {
        soaRepo = repo;
        return this;
    }

    public Object getRequestAttribute(String name) {
        if (request != null) {
            return request.getAttribute(Scope.REQUEST, name);
        }
        return null;
    }

    public boolean isEditable() {
        return isEditable;
    }

    public RESTContext setEditable(boolean editable) {
        isEditable = editable;
        return this;
    }
}
