/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.rest.rt.server;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.iona.repository.rest.RESTException;
import com.iona.repository.rest.RESTQuery;
import com.iona.repository.rest.rt.osgi.MessageBundle;
import com.iona.repository.util.i18n.Message;

public class RESTQueryImpl implements RESTQuery {    
    private static final ResourceBundle BUNDLE = MessageBundle.getInstance();
    private static final Logger LOG = Logger.getLogger(RESTQuery.class.getName());

    private Map<String, String> params;
    
    public RESTQueryImpl(String query) {
        
        params = new HashMap<String, String>();
     
        LOG.fine("Query string \"" + query + "\"");
        if (query != null) {
            for (String part : query.split("&")) {
                int i = part.indexOf('=');
                if (i == -1) {
                    params.put(part, "");
                } else {
                    params.put(part.substring(0, i), part.substring(i + 1));
                }
            }
        }
        
        if (LOG.isLoggable(Level.FINE) && !params.isEmpty()) {
            StringBuilder sb = new StringBuilder();
            String prefix = "";
            sb.append("Query parameters: {");
            
            for (String p : params.keySet()) {
                sb.append(prefix)
                    .append(p).append(" = \"")
                    .append(params.get(p)).append("\"");
                prefix = ", ";
            }
            sb.append("}");
            LOG.fine(sb.toString());
        }
    }
    
    public Collection<String> getParameterNames() {
        return params.keySet();
    }
    
    public String getParameter(String name) {
        return params.get(name);
    }
    
    public String getDecodedParameter(String name) {
        String p = getParameter(name);
        if (p == null) {
            return null;
        }
        
        try {
            return URLDecoder.decode(p, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            LOG.warning("UnsupportedEncodingException when decoding " + name);
            return null;
        }
    }
    
    public int getIntParameter(String name, int defaultValue) throws RESTException {
        String value = params.get(name);
        if (value == null) {
            return defaultValue;
        }
        
        try {
            return Integer.parseInt(value);
        } catch (NumberFormatException e) {
            throw new RESTException(RESTException.BAD_REQUEST,
                    new Message(BUNDLE, "BAD_INT_PARAM", name, value));
        }
    }
    
    public boolean isSet(String key) {
        return params.containsKey(key);
    }
}

