/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.search;

import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.iona.repository.event.DataChangeEventObserver;
import com.iona.soa.expression.Equals;
import com.iona.soa.expression.Expression;
import com.iona.soa.expression.Operand;
import com.iona.soa.model.events.DataChangeEvent;
import com.iona.soa.model.events.DataChangeKind;
import com.iona.soa.model.repository.RepositoryPackage;

import org.eclipse.emf.ecore.EObject;

public class IndexDataChangeEventObserver implements DataChangeEventObserver {
    
    private static final Logger LOG = Logger.getLogger(IndexDataChangeEventObserver.class.getName());
    private EObjectSearchIndex searchIndex;
    
    public IndexDataChangeEventObserver(EObjectSearchIndex index) {
        LOG.log(Level.INFO, "Registering data change event observer for LuceneSearchService");   
        searchIndex = index;
    }
    
    public void dataChanged(DataChangeEvent dataEvent) {        
        switch (dataEvent.getChangeKind().getValue()) {
        case DataChangeKind.ADD:
            searchIndex.addObject(getObjectFromRepository(dataEvent));
            break;
        case DataChangeKind.REMOVE:
            searchIndex.deleteObject(dataEvent.getTargetGuid());
            break;
        case DataChangeKind.MODIFY:
            EObject eObj = getObjectFromRepository(dataEvent);
            searchIndex.updateObject(eObj);
            break;
        default:
            break;
        }            
    }         

    private EObject getObjectFromRepository(DataChangeEvent event) {
        try {
            Operand guidOper = new Operand(RepositoryPackage.Literals.IPERSISTABLE_OBJECT__GUID.getName());
            Operand valueOper = new Operand(event.getTargetGuid());
            Expression expr = new Equals(guidOper, valueOper);
            Collection<EObject> objects = 
                searchIndex.getRepository().getResourceSet().executeQuery(event.getTargetType(), 
                                                                            expr, 
                                                                            true);
            if (!objects.isEmpty()) {
                return objects.iterator().next();
            }
        } catch (Exception ex) {
            LOG.log(Level.SEVERE, "Error occured retrieving object <" + event.getTargetGuid() + ">", ex);
        }
        return null;
    }
}
