/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.search;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import com.iona.repository.event.DataChangeAdapter;
import com.iona.soa.repository.SOARepository;
import com.iona.soa.repository.configuration.ConfigurationKeys;
import com.iona.soa.repository.configuration.ConfigurationService;
import com.iona.soa.repository.status.RepositoryException;
import com.iona.soa.repository.util.ExceptionHelper;
import com.iona.soa.search.SearchService;

import org.eclipse.emf.ecore.EObject;
import org.springframework.beans.factory.InitializingBean;

public class LuceneSearchService implements SearchService, InitializingBean {
    private static final Logger LOG = Logger.getLogger(LuceneSearchService.class.getName());

    private ConfigurationService configService;
    private EObjectSearchIndex index;

    public void setConfigurationService(ConfigurationService c) {
        LOG.fine("Configuration service available");
        configService = c;
    }

    public void setSOARepository(SOARepository repo) {
        Map<String, String> config = configService.getConfiguration();
        String indexDir = config.get(ConfigurationKeys.SEARCH_INDEX_DIR);
        if (indexDir == null) {
            indexDir = "../etc/storage/index";
        }
        try {
            index = new EObjectSearchIndex(indexDir, repo);
        } catch (IOException ioe) {
            LOG.warning("Unable to create lucene index: " + ioe);
            LOG.fine(ExceptionHelper.getStackTraceAsString(ioe));
            throw new RuntimeException(ioe);
        }
        IndexDataChangeEventObserver observer = new IndexDataChangeEventObserver(index);
        DataChangeAdapter.getInstance().registerObserver(observer);
    }

    public void afterPropertiesSet() throws Exception {
    }

    public List<EObject> search(String query) throws RepositoryException {
        return index.search(query);
    }

}
