/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.serialization;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;


import org.eclipse.emf.ecore.EAttribute;

public abstract class AbstractSerializationBase implements Configurable {
    private Map<EAttribute, AttributeHandler> handlerMap = 
        new HashMap<EAttribute, AttributeHandler>();
    private Map<Option<?>, Object> optionsMap =
        new HashMap<Option<?>, Object>();

    @SuppressWarnings("unchecked")
    public <T> T getOption(Option<T> option) {
        if (option != null) {
            if (optionsMap.containsKey(option)) {
                return (T)optionsMap.get(option);
            } else {
                return option.getDefault();
            }
        }
        return null;
    }

    public <T> boolean setOption(Option<T> option, T value) {
        if (option != null && isValidOption(option)) {
            optionsMap.put(option, value);
            return true;
        }
        return false;
    }

    public void registerAttributeHandler(AttributeHandler handler) {
        if (handler != null && handler.getAttribute() != null) {
            handlerMap.put(handler.getAttribute(), handler);
        }
    }

    protected AttributeHandler getHandler(EAttribute eAttr) {
        return handlerMap.get(eAttr);
    }
    
    private boolean isValidOption(Option option) {
        Field[] fields = getClass().getFields();
        for (Field field : fields) {
            if (Option.class.isAssignableFrom(field.getType())) {
                try {
                    if (field.get(this) == option) {
                        return true;
                    }
                } catch (Exception ex) {
                    // ignore
                }
            }
        }
        return false;
    }
}
