/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.serialization;

import java.util.Collection;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.resource.Resource;

public class ResourceEObjectMapper implements ObjectResolver, ProxyURIFactory {
    private final Resource resource;

    public ResourceEObjectMapper(Resource res) {
        resource = res;
    }

    public EObject getEObject(EClass eClass, URI uri, boolean resolve) {
        EObject ret = resource.getEObject(uri.fragment());
        if (ret == null && !resolve) {
            ret = SerializationHelper.createEProxy(eClass, uri);
        }
        return ret;
    }

    public EObject createEObject(EClass eClass, Object id) {
        URI uri = URI.createURI("#" + id);
        EObject other = getEObject(eClass, uri, true);
        return other == null ? SerializationHelper.createEObject(eClass, id) : null;
    }

    @SuppressWarnings("unchecked")
    public boolean addToResource(EObject eObj) {
        if (eObj.eResource() == null) {
            resource.getContents().add(eObj);
            return true;
        } else {
            return resource.equals(eObj.eResource());
        }
    }

    @SuppressWarnings("unchecked")
    public boolean addToContainer(EObject eObj, EObject container, EReference containerRef) {
        if (eObj.eResource() == null || resource.equals(eObj.eResource())) {
            if (containerRef.isMany()) {
                ((Collection)container.eGet(containerRef, false)).add(eObj);
            } else {
                container.eSet(containerRef, eObj);
            }
            return true;
        }
        return false;
    }

    public boolean deleteFromResource(EObject eObj) {
        if (resource.equals(eObj.eResource())) {
            if (eObj.eContainer() != null) {
                if (eObj.eContainmentFeature().isMany()) {
                    ((Collection<?>)eObj.eContainer().eGet(eObj.eContainmentFeature())).remove(eObj);
                } else {
                    eObj.eContainer().eUnset(eObj.eContainmentFeature());
                }
            } else {
                resource.getContents().remove(eObj);
            }
            return true;
        } else {
            return false;
        }
    }

    public URI getURI(EObject eObject) {
        return resource.getURI().appendFragment(resource.getURIFragment(eObject));
    }
    
    public Resource getResource() {
        return resource;
    }
}
