/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.serialization;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;

import com.iona.soa.serialization.v1.SerializationFactoryV1;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

public final class SerializationFactory {
    private SerializationFactory() {
    }
    
    public static Serializer getSerializer() {
        return getSerializer(XMLVersion.VERSION_1_0);
    }
    
    public static Serializer getSerializer(XMLVersion version) {
        switch (version) {
        case VERSION_1_0:
            return SerializationFactoryV1.createSerializer();
        default:
            return null;
        }
    }
    
    public static Deserializer getDeserializer(String xml, 
            ObjectResolver resolver) throws XMLParseException {
        try {
            return getDeserializer(new StringReader(xml), resolver);
        } catch (IOException ioe) {
            throw new XMLParseException(ioe);
        }
    }
    
    public static Deserializer getDeserializer(Reader reader, 
            ObjectResolver resolver) throws XMLParseException, IOException {
        Document doc;
        try {
            doc = new SAXBuilder().build(reader);
        } catch (JDOMException ex) {
            throw new XMLParseException(ex);
        }
        return getDeserializerInternal(doc.getRootElement(), resolver);
    }
    
    public static Deserializer getDeserializer(InputStream is, 
            ObjectResolver resolver) throws XMLParseException, IOException {
        Document doc;
        try {
            doc = new SAXBuilder().build(is);
        } catch (JDOMException ex) {
            throw new XMLParseException(ex);
        }
        return getDeserializerInternal(doc.getRootElement(), resolver);
    }

    private static Deserializer getDeserializerInternal(Element rootElement,
            ObjectResolver resolver) throws XMLParseException {
        String version = rootElement.getAttributeValue(XMLConstants.VERSION_ATTR);
        if ("1.0".equals(version)) {
            return SerializationFactoryV1.createDeserializer(rootElement, resolver);
        } else {
            throw new XMLParseException("Incompatible XML version " + version);
        }
    }
}
