/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.serialization;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.EcoreUtil;

public final class SerializationHelper {
    private SerializationHelper() {
    }

    public static Object getId(EObject eObj) {
        return eObj.eGet(eObj.eClass().getEIDAttribute(), false);
    }

    public static String getIdString(EObject eObj) {
        EAttribute id = eObj.eClass().getEIDAttribute();
        return toString(id, eObj.eGet(id, false));
    }
    
    public static EObject createEProxy(EClass eClass, URI uri) {
        EObject ret = EcoreUtil.create(eClass);
        ((InternalEObject)ret).eSetProxyURI(uri);
        String frag = uri.fragment();
        if (frag != null) {
            int idx = frag.lastIndexOf('/');
            String id;
            if (idx >= 0) {
                id = frag.substring(idx + 1);
            } else {
                id = frag;
            }
            EAttribute idAttr = eClass.getEIDAttribute();
            ret.eSet(idAttr, fromString(idAttr, id));
        }
        return ret;
    }

    public static EObject createEObject(EClass eClass, Object id) {
        EObject ret = EcoreUtil.create(eClass);
        ret.eSet(eClass.getEIDAttribute(), id);
        return ret;
    }

    public static Object fromString(EAttribute eAttr, String value) {
        return EcoreUtil.createFromString((EDataType)eAttr.getEType(), value);
    }

    public static String toString(EAttribute eAttr, Object value) {
        return EcoreUtil.convertToString((EDataType)eAttr.getEType(), value);
    }
}
