/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.serialization;

import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.jdom.output.Format;

public interface Serializer extends Configurable {
    Option<Boolean> OPTION_INCLUDE_CHILDREN = new Option<Boolean>(false);
    Option<Boolean> OPTION_INCLUDE_CONTAINER = new Option<Boolean>(false);
    Option<Boolean> OPTION_FORCE_CONTAINER_REF = new Option<Boolean>(false);
    Option<Format> OPTION_XML_FORMAT = new Option<Format>(Format.getPrettyFormat());
    Option<FeatureFilter> OPTION_FEATURE_FITLER = new Option<FeatureFilter>(null);
    
    void registerAttributeHandler(AttributeHandler handler);
    void setURIFactory(ProxyURIFactory uriFactory);
    
    void addObjects(EObject ... objects) throws XMLSerializationException;
    void deleteObjects(EObject ... objects) throws XMLSerializationException;
    void modifyObject(EObject object, 
                      EStructuralFeature feature, 
                      Object ... values) throws XMLSerializationException;
    void modifyObject(EObject object, 
                      String featureName, 
                      Object ... values) throws XMLSerializationException;
    
    String serialize()throws XMLSerializationException;
    void serialize(Writer out) throws XMLSerializationException, IOException;
    void serialize(OutputStream out) throws XMLSerializationException, IOException;

    public interface FeatureFilter {
        boolean filterAll(EStructuralFeature feature);
        boolean filter(EStructuralFeature feature, EObject eObj, Object value);
    }
}
