/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.serialization.v1;

import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.iona.soa.repository.util.CollectionsUtil;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;

public class ModificationData {
    private Map<EStructuralFeature, List<Object>> changeMap = 
        new HashMap<EStructuralFeature, List<Object>>();
    
    public void addChangeEntry(EStructuralFeature feature, Object ... values) {
        if (feature.isMany()) {
            if (!changeMap.containsKey(feature)) {
                changeMap.put(feature, new LinkedList<Object>());
            }
            if (values != null) {
                changeMap.get(feature).addAll(Arrays.asList(values));
            }
        } else {
            changeMap.put(feature, null);
        }
    }
    
    public List<EAttribute> getChangedAttributes() {
        return CollectionsUtil.filter(changeMap.keySet(), EAttribute.class);
    }
    
    public List<EReference> getChangedReferences() {
        return CollectionsUtil.filter(changeMap.keySet(), EReference.class);
    }
    
    public List<Object> getManyAttributeValues(EAttribute attribute) {
        if (!attribute.isMany() || !changeMap.containsKey(attribute)) {
            throw new IllegalArgumentException();
        }
        return changeMap.get(attribute);
    }
    
    public List<EObject> getManyReferenceValues(EReference reference) {
        if (!reference.isMany() || !changeMap.containsKey(reference)) {
            throw new IllegalArgumentException();
        }
        return CollectionsUtil.filter(changeMap.get(reference), EObject.class);
    }
}
