/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository;

import org.eclipse.emf.ecore.EClassifier;

public class EClassifierDescriptor {
    private String packageNsURI;
    private String name;
    
    public EClassifierDescriptor(EClassifier eClassifier) {
        this(eClassifier == null ? null : eClassifier.getEPackage().getNsURI(), 
             eClassifier == null ? null : eClassifier.getName());
    }
    
    public EClassifierDescriptor(String packageNsURI, String name) {
        super();
        this.packageNsURI = packageNsURI;
        this.name = name;
    }
    
    public String getName() {
        return name;
    }
    public String getPackageNsURI() {
        return packageNsURI;
    }
    
    public boolean isDescriptorFor(EClassifier eClassifier) {
        return eClassifier != null 
            && eClassifier.getEPackage().getNsURI().equals(packageNsURI) 
            && eClassifier.getName().equals(name);
    }
    
    @Override
    public int hashCode() {
        return packageNsURI == null ? 0 : packageNsURI.hashCode()
             + name == null ? 0 : name.hashCode();
    }
    
    @Override
    public boolean equals(Object other) {
        if (other instanceof EClassifierDescriptor) {
            EClassifierDescriptor otherDesc = (EClassifierDescriptor)other;
            return packageNsURI == null ? otherDesc.packageNsURI == null 
                                        : packageNsURI.equals(otherDesc.packageNsURI)
                && name == null ? otherDesc.name == null 
                                : name.equals(otherDesc.name);
        }
        return false;
    }
}
