/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import com.iona.soa.repository.util.JarIntrospector;
import com.iona.soa.repository.util.TopDownURLClassLoader;
import com.iona.soa.repository.util.VersionType;

import org.eclipse.emf.ecore.EPackage;

public final class ModelRegistryHelper {
    private static Collection<String> excludedPackages = new HashSet<String>();
    static {
        excludedPackages.add("http://com.iona.soa.registry.model");
        excludedPackages.add("http://com.iona.soa.repository.commands");
    }
    private ModelRegistryHelper() {
    }
    
    public static ModelRegistry getRegistryFromJar(File jarFile) throws Exception {
        URLClassLoader cl = new TopDownURLClassLoader(ModelRegistryHelper.class.getClassLoader(), 
                                                      jarFile.toURI().toURL());
        ModelRegistry ret = getRegistryFromJar(cl);
        return ret;
    }
    
    @SuppressWarnings("unchecked")
    public static ModelRegistry getRegistryFromJar(URLClassLoader classLoader) throws Exception {
        VersionType schemaVersion = null;
        for (URL url : classLoader.getURLs()) {
            schemaVersion = SchemaVersionChecker.getCodeSchemaVersion(url);
            if (schemaVersion != null) {
                break;
            }
        }
        String[] packageClasses = new JarIntrospector(classLoader).findClasses(EPackage.class);
        Collection<EPackage> packages = new HashSet<EPackage>();
        
        synchronized (EPackage.Registry.INSTANCE) {
            @SuppressWarnings("unused")
            Map registry = new HashMap(EPackage.Registry.INSTANCE);
            try {
                EPackage.Registry.INSTANCE.clear();
                for (String packageClass : packageClasses) {
                    Class clazz = classLoader.loadClass(packageClass);
                    if (clazz.isInterface() 
                        && clazz.getInterfaces().length == 1 
                        && clazz.getInterfaces()[0].equals(EPackage.class)) {
                        EPackage pkg = (EPackage)clazz.getDeclaredField("eINSTANCE").get(null);
                        if (excludedPackages.contains(pkg.getNsURI())) {
                            // bit of a hack... REVISIT: Maybe move out in-memory only ecores
                            // into their own jars
                            continue;
                        }
                        packages.add(pkg);
                    }
                }
                return new ModelRegistry(schemaVersion, packages);
            } finally {
                EPackage.Registry.INSTANCE.clear();
                EPackage.Registry.INSTANCE.putAll(registry);
            }
        }
    } 
}
