/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.status;

import java.io.PrintStream;
import java.io.PrintWriter;

@SuppressWarnings("serial")
public class RepositoryException extends Exception {
    protected final RepositoryStatus status;
    private final int httpReturnCode;
    
    public RepositoryException(int httpCode, RepositoryStatus status) {
        this(httpCode, status, null);
    }
    
    public RepositoryException(int httpCode, RepositoryStatus status, Throwable nested) {
        super(status == null 
              ? null 
              : status.getMessages().size() == 1 
                  ? status.getMessages().get(0).getMessage() 
                  : status.getType(),
              nested);
        httpReturnCode = httpCode;
        this.status = status;
    }

    public int getHttpReturnCode() {
        return httpReturnCode;
    }
    
    public RepositoryStatus getStatus() {
        return status;
    }

    @Override
    public void printStackTrace(PrintStream ps) {
        if (status != null && status.getMessages().size() > 0) {
            for (RepositoryStatusMessage message : status.getMessages()) {
                ps.println(message.getTrace());
            }
        } else {
            super.printStackTrace(ps);
        }
    }
    
    @Override
    public void printStackTrace(PrintWriter pw) {
        if (status != null && status.getMessages().size() > 0) {
            for (RepositoryStatusMessage message : status.getMessages()) {
                pw.println(message.getTrace());
            }
        } else {
            super.printStackTrace(pw);
        }
    }

}
