/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.status;

import java.util.Collections;
import java.util.List;

import com.iona.soa.repository.util.ExceptionHelper;

public class RepositoryStatusMessage {
    private final String message;
    private final String trace;
    private final List<String> objectHrefs;

    public RepositoryStatusMessage() {
        this((String)null, null);
    }

    public RepositoryStatusMessage(StatusMessage msg, Object ... msgArgs) {
        this((String)null, msg == null ? null : msg.getMessage(msgArgs));
    }

    public RepositoryStatusMessage(List<String> uris, StatusMessage msg, Object ... msgArgs) {
        this(getMessage(null, msg, msgArgs), null, uris);
    }

    public RepositoryStatusMessage(Throwable cause, StatusMessage msg, Object ... msgArgs) {
        this(cause == null ? null : ExceptionHelper.getStackTraceAsString(cause),
             getMessage(cause, msg, msgArgs));
    }

    public RepositoryStatusMessage(Throwable cause) {
        this(cause == null ? null : ExceptionHelper.getStackTraceAsString(cause), 
             getMessage(cause, null));
    }

    public RepositoryStatusMessage(String trace, String msg) {
        this(msg, trace, null);
    }

    @SuppressWarnings("unchecked")
    public RepositoryStatusMessage(String msg, String trace, List<String> hrefs) {
        this.message = msg;
        this.trace = trace;
        this.objectHrefs = hrefs == null ? Collections.EMPTY_LIST : hrefs;
    }

    public String getMessage() {
        return message;
    }

    public String getTrace() {
        return trace;
    }
    
    public List<String> getHrefs() {
        return objectHrefs;
    }
    
    private static String getMessage(Throwable cause, StatusMessage msg, Object ... msgArgs) {
        if (msg == null) {
            if (cause == null) {
                return "Unknown";
            } else if (cause.getMessage() == null) {
                return cause.toString();
            } else {
                return cause.getMessage();
            }
        } else {
            return msg.getMessage(msgArgs);
        }
    }
}
