/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.status;

import java.text.MessageFormat;

public enum StatusMessage {
    // Internal errors
    INTERNAL_SERVER_ERROR_MSG(0),
    ERROR_READING_RESOURCE_MSG(1),
    NO_ATTACHMENTS_MSG(0),
    WRONG_NR_ATTACHMENTS_MSG(0),
    INVALID_CHANGEDESCRIPTION_MSG(0),
    UNSUPPORTED_SOURCE_TYPE_MSG(1),
    INVALID_COMMAND_MSG(2),
    COMMAND_UNKNOWN_MSG(3),
    COMMAND_HANDLE_ERROR_MSG(3),
    NO_COMMANDS_MSG(0),
    UNEXPECTED_COMMAND_MSG(1),
    BAD_CONTENT_ID_MSG(1),
    ERROR_SAVING_ATTACHMENT_MSG(1),
    ERROR_CREATING_COPY_MSG(1),
    NO_GUID_IN_ATTACHMENT_MSG(0),
    NO_CONTENT_ID_IN_ATTACHMENT_MSG(0),
    MESSAGE_PARSE_ERROR_MSG(0),
    ATTACHMENT_PARSE_ERROR_MSG(0),
    ERROR_UPDATING_DOCUMENT_URL(1),
    NO_DOCUMENT_FOUND_MSG(1),
    NO_HEADER_AVAILABLE_MSG(1),
    INVALID_MIME_TYPE_MSG(0),
    DOCUMENT_UNINITIALIZED_MSG(0),
    MISSING_ATTACHMENT_INFO(1),
    UNABLE_CONNECT_TO_DATABASE(1),
    
    // HTTP errors
    RESOURCE_NOT_FOUND_MSG(1),
    BAD_REQUEST_METHOD_MSG(2),
    BAD_REQUEST_URL_MSG(1),
    MISSING_REQUEST_BODY_MSG(0),
    CONTEXT_NOT_SUPPORTED_MSG(1),

    // Security errors
    NOT_AUTHENTICATED_MSG(0),
    CHANGE_NOT_AUTHORIZED_MSG(0),
    UPDATE_NOT_AUTHORIZED_MSG(1),
    READ_NOT_AUTHORIZED_MSG(1),
    
    // Query errors
    INVALID_QUERY(1),
    NULL_QUERY(0),
    UNSUPPORTED_QUERY_LANG(1),
    
    // Operation errors
    UNSUPPORTED_OPERATION(1),
    
    // Validation errors
    INVALID_ATTRIBUTE_CHANGE_MSG(1),
    INVALID_FEATURE_CHANGE_MSG(1),
    VERSION_COUNT_MISMATCH_MSG(2),
    UNRESOLVABLE_OBJECT_MSG(1),
    MISSING_VERSION_ATTR_MSG(0),
    AMBIGUOUS_VERSION_NUMBERS_MSG(2),
    NON_ZERO_NEW_OBEJCT_VERSION_MSG(2),
    VERSION_NUMBER_MISMATCH_MSG(3),
    CANT_MODIFY_UNMODIFIABLE_MSG(1),
    CANT_DELETE_USR_MSG(1),
    CANT_DELETE_PARTICIPANT_WITH_DEPLOYED_PIS_MSG(1),
    ALL_SERVICES_MUST_BELONG_IN_ASSEMBLY(1),
    DUPLICATE_NAMED_OBJECT_FOUND(1),
    DUPLICATE_NAMED_OBJECT_IN_FOLDER(2);
    
    
    final int numArgs;
    private final String message;
    
    StatusMessage(int numArgs) {
        this.numArgs = numArgs;
        message = MessageHelper.getMessage(this);
    }
    
    public String getMessage(Object ... args) {
        assert numArgs == args.length;
        if (numArgs != args.length) {
            throw new RuntimeException("ExceptionMessage " + name() + " expects " 
                    + numArgs + " arguments but was given " + args.length + ".");
        }
        if (numArgs > 0) {
            return MessageFormat.format(message, args);
        } else {
            return message;
        }
    }
}
