/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.util;

import java.io.FilterReader;
import java.io.IOException;
import java.io.Reader;

public class CRLFFilterReader extends FilterReader {
    int nextVal;

    public CRLFFilterReader(Reader in) {
        super(in);
        try {
            nextVal = super.read();
        } catch (IOException ex) {
            nextVal = -1;
        }
    }

    public int read() throws IOException {
        int thisVal = nextVal;
        nextVal = super.read();
        
        if (thisVal == 13) {
            if (nextVal == 10) {
                nextVal = super.read();
            }
            thisVal = 10;
        } else if (thisVal == 10) {
            if (nextVal == 13) {
                nextVal = super.read();
            }
        }
        return thisVal;
    }

    public int read(char cbuf[], int off, int len) throws IOException {
        if (len == 0) {
            return 0;
        } else if (nextVal == -1) {
            return -1;
        }
        
        char[] temp = new char[len];
        int chars = super.read(temp, 0, len);
        if (chars == -1) {
            cbuf[off] = nextVal == 13 ? (char)10 : (char)nextVal;
            nextVal = -1;
            return 1;
        }
        
        int ret = 0;
        
        // checkstype don't like for(...) when the control var is modified... 
        // Hence the while(...) loop. Doh!
        int i = 0;
        while (i < chars) {
            int thisVal = nextVal;
            nextVal = temp[i];
            
            if (thisVal == 13) {
                if (nextVal == 10) {
                    nextVal = ++i < chars ? temp[i] : super.read();
                }
                thisVal = 10;
            } else if (thisVal == 10) {
                if (nextVal == 13) {
                    nextVal = ++i < chars ? temp[i] : super.read();
                }
            }
            cbuf[off + ret++] = (char)thisVal;
            ++i;
        }
        if (chars < len) {
            if (nextVal != -1) {
                cbuf[off + ret++] = nextVal == 13 ? (char)10 : (char)nextVal;
                nextVal = -1;
            }
        } else if (ret < len) {
            int more = read(cbuf, off + ret, len - ret);
            if (more > 0) {
                ret += more;
            }
        }
        return ret;
    }
}
