/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.util;

import java.io.CharArrayWriter;
import java.io.PrintWriter;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.iona.soa.repository.status.RepositoryException;
import com.iona.soa.repository.status.RepositoryStatusMessage;

public final class ExceptionHelper {
    
    private ExceptionHelper() {
    }
    
    public static String getStackTraceAsString(Throwable t) {
        CharArrayWriter cw = new CharArrayWriter();
        PrintWriter p = new PrintWriter(cw);
        t.printStackTrace(p);
        
        char[] chars = cw.toCharArray();
        StringBuffer sb = new StringBuffer();
        
        for (int i = 0; i < chars.length; i++) {
            if ((int)chars[i] == 10 || (int)chars[i] == 13) {
                if (i + 1 < chars.length) {
                    if ((int)chars[i + 1] == 10 || (int)chars[i + 1] == 13) {
                        continue;
                    }
                }
                sb.append((char)10);
            } else {
                sb.append(chars[i]);
            }
        }
        if (t.getCause() != null) {
            sb.append("\nNested Exception:\n");
            sb.append(getStackTraceAsString(t.getCause()));
        }
        return sb.toString();
    }

    public static String getStackTraceAsString(Thread t) {
        StringBuffer sb = new StringBuffer();
        sb.append(t).append("\n");
        StackTraceElement[] trace = t.getStackTrace();
        for (StackTraceElement ste : trace) {
            sb.append("\tat ").append(ste).append("\n");
        }
        return sb.toString();
    }

    public static void logRepositoryException(RepositoryException re, Logger logger, Level logLevel) {
        for (RepositoryStatusMessage message : re.getStatus().getMessages()) {
            logger.log(logLevel, "{0}: {1}", new Object[] {re.getStatus().getType(), message.getMessage()});
            logger.log(Level.FINE, "{0}: {1}", new Object[] {re.getStatus().getType(), message.getTrace()});
        }
    }

    public static RepositoryException extractRepositoryException(Exception ex) {
        RepositoryException re; 
        
        if (ex instanceof RepositoryException) {
            re = (RepositoryException)ex;
        } else if (ex instanceof RuntimeException && ex.getCause() instanceof RepositoryException) {
            re = (RepositoryException)ex.getCause();
        } else {
            re = null;
        }
        return re;
    }
}
