/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.util;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * This is a utility class that can be used to get a list of all the File
 * entries in directory tree under a certain root File.
 */
public class FileTreeList {
    List<File> fileList = new ArrayList<File>();

    /**
     * Constructor. Constructs the list of files.
     * @param f
     *            The root to use.
     */
    @SuppressWarnings("unchecked")
    public FileTreeList(File f) {
        String[] names = f.list();

        if (names == null) {
            fileList.add(f);
            return;
        }

        for (String name : names) {
            File curFile = new File(f, name);

            if (curFile.isDirectory()) {
                fileList.addAll(new FileTreeList(curFile).getFiles());
            } else {
                fileList.add(curFile);
            }
        }
        fileList.add(f);
    }

    /**
     * Get the list of File objects. This list is ordered such that it can be
     * used to call delete on every file object to remove the whole tree. This
     * means that for every directory the contents are returned first, and the
     * actual directory itself is returned last.
     * 
     * @return The list of files.
     */
    public List<File> getFiles() {
        return fileList;
    }
}
