/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.repository.util;

import java.net.MalformedURLException;
import java.net.URL;

public final class URLUtil {
    private static final String JAR_URL_PREFIX = "jar:";
    private URLUtil() {
    }
    
    public static boolean isJarURL(URL url) {
        return url.toString().startsWith(JAR_URL_PREFIX);
    }
    
    public static String classURLToJarURL(String clURL) {
        int i = clURL.lastIndexOf("!/");
        if (i >= 0) {
            clURL = clURL.substring(0, i);
        }
        
        if (clURL.indexOf("!/") == -1) {
            if (clURL.startsWith(JAR_URL_PREFIX)) {
                clURL = clURL.substring(4);
            } 
        }
        return clURL;

    }
    
    public static URL classURLToJarURL(URL clURL) throws MalformedURLException {
        String jarUrl = classURLToJarURL(clURL.toString());
        return new URL(jarUrl);
    }
    
    public static URL getBaseURLForClass(Class clazz) {
        String resourceName = "/" + clazz.getName().replace(".", "/") + ".class";
        URL url = clazz.getResource(resourceName);
        if (url != null) {
            if (isJarURL(url)) {
                try {
                    return classURLToJarURL(url);
                } catch (MalformedURLException ex) {
                    // ignore
                }
            } else {
                String urlStr = url.toString();
                try {
                    return new URL(urlStr.substring(0, urlStr.length() - resourceName.length()));
                } catch (MalformedURLException ex) {
                    // ignore
                }
            }
        }
        return null;
    }
}