/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.base.client.atom;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.xml.client.Document;
import com.google.gwt.xml.client.Element;
import com.google.gwt.xml.client.Node;
import com.google.gwt.xml.client.NodeList;
import com.google.gwt.xml.client.XMLParser;
import com.google.gwt.xml.client.impl.DOMParseException;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.base.client.remote.RemoteError;

public class AtomParser {

    private Document document;
    
    public AtomParser(String msgText) {
        try {
            document = XMLParser.parse(msgText);
        } catch (DOMParseException e) {
            document = null;
        }
    }

    public boolean isError() {
        return document != null && document.getElementsByTagName("error").getLength() == 1;
    }

    public boolean isFeed() {
        return document != null && document.getElementsByTagName("feed").getLength() == 1;
    }
    
    public boolean isEntry() {
        return document != null && !isFeed() && document.getElementsByTagName("entry").getLength() == 1;
    }
    
    public RemoteError parseError() {
        
        if (!isError()) {
            return null;
        }

        NodeList items = document.getElementsByTagName("error");
        Element errElt = (Element)items.item(0);
        
        String errCode = getElementTextContent(errElt.getElementsByTagName("code").item(0));
        String errMsg = getElementTextContent(errElt.getElementsByTagName("message").item(0));

        return new RemoteError(errCode, errMsg);
    }

    public ReposObj parseEntry() {
        
        return isEntry() ? parseEntry((Element)document.getElementsByTagName("entry").item(0)) : null;
    }
    
    public ReposObjCollection parseFeed() {
        
        if (!isFeed()) {
            return null;
        }

        NodeList items = document.getElementsByTagName("feed");
        Element feedElt = (Element)items.item(0);
        
        ReposObjCollection objCollection = new ReposObjCollection(feedElt);
        objCollection.setBaseLink(feedElt.getAttribute("xml:base"));
        if (!objCollection.getBaseLink().endsWith("/")) {
            objCollection.setBaseLink(objCollection.getBaseLink() + "/");
        }
        
        NodeList childNodes = feedElt.getChildNodes();
        for (int i = 0; i < childNodes.getLength(); i++) {
            Node childNode = childNodes.item(i);
            if ("link".equals(childNode.getNodeName())) {
                Element linkElt = (Element)childNode;
                if ("self".equals(linkElt.getAttribute("rel"))) {
                    String href = getHref(childNode);                    
                    objCollection.setId(href);
                    
                    String selfLink = objCollection.getBaseLink() + href;
                    int index = selfLink.indexOf(";");
                    objCollection.setContainerLink(index > 0 ? selfLink.substring(0, index) : null);
                    objCollection.setContainedList(index > 0);
                    objCollection.setSelfLink(selfLink);
                } else if ("create".equals(linkElt.getAttribute("rel"))) {
                    objCollection.setCreateAllowed(true);
                
                }
            }
        }

        objCollection.setCaption(getElementTextContent(feedElt.getElementsByTagName("title").item(0)));
        objCollection.setTimeStamp(getElementTextContent(feedElt.getElementsByTagName("updated").item(0)));
        
        NodeList paginationNodes = feedElt.getElementsByTagName("totalResults");
        if (paginationNodes.getLength() > 0) {
            objCollection.setTotalSize(getElementIntContent(paginationNodes.item(0)));
        }
        paginationNodes = feedElt.getElementsByTagName("itemsPerPage");
        if (paginationNodes.getLength() > 0) {
            objCollection.setPaginationPageSize(getElementIntContent(paginationNodes.item(0)));
        }
        paginationNodes = feedElt.getElementsByTagName("startIndex");
        if (paginationNodes.getLength() > 0) {
            objCollection.setPaginationStartIndex(getElementIntContent(paginationNodes.item(0)));
        }

        
        NodeList entryNodes = document.getElementsByTagName("entry");
        List<ReposObj> parsedObjs = new ArrayList<ReposObj>(entryNodes.getLength());
        for (int i = 0; i < entryNodes.getLength(); i++) {
            parsedObjs.add(parseEntry((Element)entryNodes.item(i)));
        }
        objCollection.setTheReposObjs(parsedObjs);
        
        return objCollection;        
    }

    private ReposObj parseEntry(Element entryNode) {
        
        ReposObj parsedObj = new ReposObj(entryNode);
        
        String entryBaseLink = entryNode.getAttribute("xml:base");
        if (!entryBaseLink.endsWith("/")) {
            entryBaseLink = entryBaseLink + "/";
        }
        parsedObj.setServerBaseLink(entryBaseLink);
        
        NodeList childNodes = entryNode.getChildNodes();
        for (int i = 0; i < childNodes.getLength(); i++) {
            Node childNode = childNodes.item(i);
            
            String nodeName = childNode.getNodeName();
            if ("title".equals(nodeName)) {
                parsedObj.setCaption(getElementTextContent(childNode));             
            
            } else if ("updated".equals(nodeName)) {
                parsedObj.setLastUpdatedTimeStamp(getElementTextContent(childNode));              
            
            } else  if ("id".equals(nodeName)) {
                parsedObj.setId(getElementTextContent(childNode));             
            
            } else if ("link".equals(nodeName)) {
                
                Element linkElt = (Element)childNode;
                
                if ("edit".equals(linkElt.getAttribute("rel"))) {
                    parsedObj.setEditLink(parsedObj.getServerBaseLink() + getHref(childNode));
                
                } else if ("delete".equals(linkElt.getAttribute("rel"))) {
                    parsedObj.setDeleteLink(parsedObj.getServerBaseLink() + getHref(childNode));
                
                } else if ("it_feed".equals(linkElt.getAttribute("rel"))) {
                    String href = getHref(childNode);
                    parsedObj.setFeedSubscriptionLink(parsedObj.getServerBaseLink() + href);
                
                } else if ("alternate".equals(linkElt.getAttribute("rel"))
                            && linkElt.getChildNodes().getLength() > 0) {
                    
                    if (childNode.getAttributes().getNamedItem("href") != null) {
                        parsedObj.setSelfLink(parsedObj.getServerBaseLink() 
                            + getHref(childNode));
                    }
                   
                    Element resourceElt = (Element)linkElt.getElementsByTagName("resource").item(0);
                    
                    if (resourceElt != null) {

                        parsedObj.setCaption(resourceElt.getAttribute("name"));
                        parsedObj.setEClass(resourceElt.getAttribute("eclass"));
                        parsedObj.setId(resourceElt.getAttribute("id"));
                        
                        for (int j = 0; j < resourceElt.getChildNodes().getLength(); j++) {
                            Element attributeElt = (Element)resourceElt.getChildNodes().item(j);
                            
                            String attrName = attributeElt.getNodeName();
                            if ("attribute".equals(attrName) || "reference".equals(attrName)) {
                                parseAttribute(parsedObj, attributeElt, parsedObj.getServerBaseLink());
                            }
                        }
                    }
                }
            }
        }
        
        return parsedObj;
    }

    private void parseAttribute(ReposObj reposObj, Element attributeElt, String baseHref) {
        String attrName = attributeElt.getAttribute("name");
        
        if (reposObj.getAttribute(attrName) != null && attributeElt.getFirstChild() != null) {
            ReposObjAttribute existingObjAttr = reposObj.getAttribute(attrName);
            
            if (existingObjAttr.isReference()) {
                existingObjAttr.addRefValue(attributeElt.getAttribute("href"),
                                            getElementTextContent(attributeElt));
            } else {
                existingObjAttr.addSimpleValue(getElementTextContent(attributeElt));
            }
             
        } else {
            ReposObjAttribute objAttr = new ReposObjAttribute(attributeElt);

            objAttr.setName(attrName);
            objAttr.setMulti("-1".equals(attributeElt.getAttribute("max")));
            objAttr.setOptional("0".equals(attributeElt.getAttribute("min")));
            objAttr.setHidden("true".equals(attributeElt.getAttribute("hidden")));
            objAttr.setReadOnly("true".equals(attributeElt.getAttribute("readonly")));

            objAttr.setPrimitiveType(attributeElt.getAttribute("type"));
            objAttr.setRefEClass(attributeElt.getAttribute("eclass"));
            
            boolean isReference = "reference".equals(attributeElt.getNodeName());
            objAttr.setReference(isReference);
            
            if (isReference) {
                String collectionHref = attributeElt.getAttribute("collectionHref");
                String href = attributeElt.getAttribute("href");
                
                if (collectionHref != null) {
                    objAttr.setCollectionLink(baseHref + collectionHref);
                    objAttr.setCollection(false);
                } else if (href != null) {
                    objAttr.setCollectionLink(baseHref + href);
                    objAttr.setCollection(true);
                    String size = attributeElt.getAttribute("size");
                    if (size != null) {
                        objAttr.setCollectionSize(Integer.parseInt(size));
                    }
                }
            }            
            
            if (attributeElt.getFirstChild() != null) {
                if (objAttr.isReference()) {
                    objAttr.addRefValue(attributeElt.getAttribute("href"),
                                        getElementTextContent(attributeElt));
                } else {
                    objAttr.addSimpleValue(getElementTextContent(attributeElt));
                }
            }
            
            reposObj.addAttribute(objAttr);
        }
    }
    
    private String getHref(Node node) {
        return node.getAttributes().getNamedItem("href").getNodeValue();
    }

    private String getElementTextContent(Node elt) {
        return elt != null && elt.getFirstChild() != null ? elt.getFirstChild().getNodeValue() : "";
    }
    
    private int getElementIntContent(Node elt) {
        return Integer.parseInt(getElementTextContent(elt));
    }

}
