/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.base.client.model;

public final class DeploymentProfileHelper {
    private DeploymentProfileHelper() {        
    }
    
    public static boolean isDeployed(ReposObj deploymentProfile) {
        String activeHref = deploymentProfile.getAttribute("active").getValue();
        return activeHref != null;
    }
    
    public static boolean hasChangesForDeployment(ReposObj deploymentProfile) {
        if (isDeployed(deploymentProfile)) {
            String activeHref = deploymentProfile.getAttribute("active").getValue();
            String currentHref = deploymentProfile.getAttribute("current").getValue();
            if (activeHref.equals(currentHref)) {
                // the latest one is active, so no changes
                return false;
            }
        }
        return true;
    }
    
    /**
     * @param deploymentProfile
     * @return the full href of the VersionedProfile that will be returned
     * for editing etc..
     */
    public static String getCurrentVersionRef(ReposObj deploymentProfile) {
        String currentProfileHRef = deploymentProfile.getAttribute("current").getValue();
        return currentProfileHRef;
    }

    /**
     * Returns the guid of the current VersionedProfile.  Needed for
     * ModelModificationTaskService that works with guid, not hrefs.
     *  
     * @param deploymentProfile
     * @return the guid of the VersionedProfile that will be returned
     * for editing etc..
     */
    public static String getCurrentVersionGuid(ReposObj deploymentProfile) {
        String currentProfileHRef = getCurrentVersionRef(deploymentProfile);
        return currentProfileHRef == null 
            ? null : "VersionedProfile-" + currentProfileHRef.substring(currentProfileHRef.lastIndexOf('/') + 1);
    }
    
    public static void markForPublish(ReposObj deploymentProfile) {
        deploymentProfile.getAttribute("publishNow").setSimpleValue("true");
    }
}
