/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.base.client.model;

import java.util.ArrayList;
import java.util.List;

/**
 * notifies interested parties of changes to specific repository objects
 * @see IReposObjChangeListener
 */
public final class ReposObjChangeNotifier implements IReposObjChangeListener {
    
    public static final ReposObjChangeNotifier INSTANCE = new ReposObjChangeNotifier();
    
    private List<IReposObjChangeListener> listeners;
    
    private ReposObjChangeNotifier() {
        listeners = new ArrayList<IReposObjChangeListener>();
    }

    public void addListener(IReposObjChangeListener aListener) {
        if (aListener != null) {
            listeners.add(aListener);
        }
    }
    
    public void removeListener(IReposObjChangeListener aListener) {
        if (aListener != null) {
            listeners.remove(aListener);
        }
    }
    
    public void reposObjCreated(ReposObj createdObj, String containingCollectionUrl, int index) {
        for (IReposObjChangeListener listener : listeners) {
            listener.reposObjCreated(createdObj, containingCollectionUrl, index);
        }
    }

    public void reposObjDeleted(ReposObj deletedObj) {
        for (IReposObjChangeListener listener : listeners) {
            listener.reposObjDeleted(deletedObj);
        }
    }

    public void reposObjUpdated(ReposObj updatedObj) {
        for (IReposObjChangeListener listener : listeners) {
            listener.reposObjUpdated(updatedObj);
        }
    }

}
