/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.base.client.util;

import java.util.HashMap;
import java.util.Map;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.RequestException;
import com.google.gwt.http.client.Response;

public final class PropertyHelper {
    private PropertyHelper() {
    }
    
    public static void loadProperties(final String propertyFileUrl, final PropertyCallBack callBack) {
        Log.debug("\n\nGET property       ==> " + propertyFileUrl);
        
        try {
            RequestBuilder rb = new RequestBuilder(RequestBuilder.GET, propertyFileUrl);
            rb.setHeader("Cache-Control", "max-age=0, no-store");
            rb.setTimeoutMillis(0);

            rb.sendRequest(null, new RequestCallback() {

                public void onError(Request request, Throwable exception) {
                    Log.debug("GET (ERROR) ==> " + exception.getMessage(), exception);
                    callBack.propertiesLoaded(null);
                }

                public void onResponseReceived(Request request, Response response) {
                    Log.debug("GET (STATUS CODE) ==>" + response.getStatusCode());
                    Log.debug("GET (STATUS TEXT) ==>" + response.getStatusText());
                    Log.debug("GET (RESPONSE) ==>" + response.getText());
                    
                    Map<String, String> properties = parsePropertiesFile(response.getText());
                    callBack.propertiesLoaded(properties);
                }
            });
        } catch (RequestException e) {
            Log.debug("GET (ERROR) ==> " + e.getMessage(), e);
            callBack.propertiesLoaded(null);
        }
    }

    private static Map<String, String> parsePropertiesFile(String text) {
        Map<String, String> ret = new HashMap<String, String>();
        text = text.replaceAll("\r\n", "\n")
                   .replaceAll("\r", "\n")
                   .replaceAll("\\\\:", ":");
        String[] properties = text.split("\n");
        
        for (int i = 0; i < properties.length; i++) {
            int index = properties[i].indexOf("=");
            if (index >= 0) {
                String key = properties[i].substring(0, index);
                String value = properties[i].substring(index + 1);
                ret.put(key, value);
            } else {
                ret.put(properties[i], null);
            }
        }
        return ret;
    }
}
