/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client;

import com.allen_sauer.gwt.log.client.Log;
import com.extjs.gxt.ui.client.XDOM;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.Html;
import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.Cookies;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.RootPanel;
import com.iona.soa.web.repository.base.client.atom.AtomClient;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.base.client.remote.ICollectionCallBack;
import com.iona.soa.web.repository.base.client.remote.RemoteClient;
import com.iona.soa.web.repository.main.client.admin.AdminSubModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiCollectionCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ISubModule;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.provisioning.ProvisioningSubModule;
import com.iona.soa.web.repository.main.client.root.FlowNavBar;
import com.iona.soa.web.repository.main.client.root.TopToolBar;
import com.iona.soa.web.repository.main.client.root.WorkspacePanel;

/**
 * the unique entry point of the ARR web tooling
 */
public class MainModule implements EntryPoint {

    //public static final String DOCS_SERVER_URL = "contextSample/";
    public static final String DOCS_SERVER_URL = "/org.eclipse.help.webapp/";
    
    public static final String ATOM_SERVER_URL = "/repository/atom";
    // public static final String ATOM_SERVER_URL = "http://localhost:9091/repository/atom";
    //public static final String ATOM_SERVER_URL = "https://localhost:8443/repository/atom";

    private static MainModule singleton;

    private static RemoteClient remoteClient;
    
    private static ReposObj currentUser;
    
    private WorkspacePanel wsPanel;


    /**
     * Gets the singleton app instance.
     */
    public static MainModule get() {
        return singleton;
    }
    
    public static RemoteClient getRemoteClient() {
        return remoteClient;
    }
    
    public static void setRemoteClient(RemoteClient newRemoteClient) {
        remoteClient = newRemoteClient;
    }
    
    public static ReposObj getCurrentUser() {
        return currentUser;
    }
    
    public void onModuleLoad() {
        Log.setUncaughtExceptionHandler();

        DeferredCommand.addCommand(new Command() {
            public void execute() {
                myModuleLoad();
            }
        });
    }

    /**
     * This method constructs the application user interface by instantiating controls and hooking up event
     * listeners.
     */
    public void myModuleLoad() {
        singleton = this;
        
        setRemoteClient(new AtomClient());
        
        String serverURL = ATOM_SERVER_URL;
        if (serverURL.startsWith("/")) {
            String baseUrl = GWT.getHostPageBaseURL();
            if (baseUrl.indexOf("/", 8) > 0) {
                baseUrl = baseUrl.substring(0, baseUrl.indexOf("/", 8));
                serverURL = baseUrl + serverURL;
            }
        }
        ReposHandle.INSTANCE.connect(serverURL);

        Html logoPanel = new Html("<img src=\"images/arr_logo.gif\"></img>"
                                  + "<div id=\"arr-status-bar\" class=\"x-hidden\">"
                                  + "<span id=\"arr-status-text\"></span></div>");
        logoPanel.addStyleName("arr-logo-panel");

        final ISubModule[] modules = {
            new ProvisioningSubModule(),
            new AdminSubModule()
        };
        ContentPanel toolbarPanel = new ContentPanel();
        toolbarPanel.setId("arr-toolbar-panel");
        toolbarPanel.setHeaderVisible(false);
        toolbarPanel.setBorders(false);
        toolbarPanel.setBodyBorder(false);
        toolbarPanel.add(new TopToolBar(modules));
        
        ContentPanel logoToolbarHolder = new ContentPanel();
        logoToolbarHolder.setId("arr-logo-toolbar-holder");
        logoToolbarHolder.setBorders(false);
        logoToolbarHolder.setBodyBorder(false);
        logoToolbarHolder.setHeaderVisible(false);
        logoToolbarHolder.add(toolbarPanel);
        logoToolbarHolder.add(logoPanel);

        FlowNavBar navBar = new FlowNavBar();
        wsPanel = new WorkspacePanel(modules, navBar);

        HtmlSnippets snippets = GWT.create(HtmlSnippets.class);
        Html footer = new Html(snippets.footerCopyrightNotice());
        footer.setId("arr-footer");
        
        // WorkspacePanel is the controller, processing all
        // requests to visit links and knows both what the active context is.
        // It will detect if there are any unsaved changes.
        Window.addWindowCloseListener(wsPanel);
        
        ContentPanel all = new ContentPanel();
        all.setBorders(false);
        all.setBodyBorder(false);        
        all.setHeaderVisible(false);
        all.setId("arr-main");
        all.setBodyStyleName("arr-main-body");
        all.add(logoToolbarHolder);
        all.add(navBar);
        all.add(wsPanel);
        all.add(footer);
        RootPanel.get("arr-main-placeholder").add(all);
        
        DeferredCommand.addCommand(new Command() {
            public void execute() {
                getWorkspaceControl().activateSubModule(modules[0]);
                hideLoadingMessage();
            }
        });

        DeferredCommand.addCommand(new Command() {
            public void execute() {
                loadCurrentUserObj();
            }
        });
    }

    public WorkspacePanel getWorkspaceControl() {
        return wsPanel;
    }
    
    private void hideLoadingMessage() {
        Element loadingElem = XDOM.getElementById("arr-loading");
        loadingElem.setAttribute("visible", "false");
        loadingElem.getParentElement().removeChild(loadingElem);
    }

    private void loadCurrentUserObj() {
        String userId = Cookies.getCookie("userId");
        if (userId != null) {
            String queryURL = EClassHelper.getCollectionLinkForClass(EClassHelper.USER_ECLASS)
                + "?emfql=userid=" + userId;
            
            ICollectionCallBack callback = new AbstractUiCollectionCallBack() {
                @Override
                public void processReposObjCollection(ReposObjCollection list) {
                    if (list.getTheReposObjs().size() == 1) {
                        currentUser = list.getTheReposObjs().get(0);
                    }
                    Utils.closeWaitDialog();
                }
            };
            Utils.showWaitDialog();
            getRemoteClient().getReposObjList(queryURL, 
                                              null, 
                                              null,
                                              callback);
        }
    }

}
