/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.admin.actions;

import java.util.List;

import com.extjs.gxt.ui.client.widget.Component;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.ISelectionSensitiveContextAction;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.ISelectionProvider.ISelectionListener;

public class RunTaskNowAction implements ISelectionSensitiveContextAction, ISelectionListener {

    Component renderedActionWidget;
    ReposObj currentObj;
    
    public RunTaskNowAction() {
        // do nothing
    }    

    public String getCaption() { return "Run now"; }
    public String getToolTip() { return "Run the selected task now"; }
    public String getIconCs() { return "run-task-now-icon"; }

    public void setRenderedWidget(Component widget) {
        renderedActionWidget = widget;
        if ((currentObj != null) && (!"RUNNING".equals(currentObj.getAttributeValue("state")))) {
            enableAction(currentObj);
        } else {
            disableAction();
        }
    }

    public void onSelectionChange(List<ReposObj> selection) {
        if (selection.size() == 1 
            && selection.get(0).isEditAllowed()
            && !"RUNNING".equals(selection.get(0).getAttributeValue("state"))) { 
            enableAction(selection.get(0));            
        } else {
            disableAction();
        }
    }    

    public void doIt() {
        ReposObjAttribute stateAttr = currentObj.getAttribute("runNow");
        if (stateAttr == null) {
            stateAttr = new ReposObjAttribute();
            /*stateAttr.setMulti(false);
            stateAttr.setPrimitiveType(ReposObjAttribute.SIMPLE_TYPE_SHORT_STRING);*/
            stateAttr.setName("runNow");
            currentObj.addAttribute(stateAttr);
        }
        stateAttr.setSimpleValue("true");
        
        Utils.showWaitDialog();
        MainModule.getRemoteClient().updateReposObj(currentObj, new AbstractUiObjCallBack() {
            public void processReposObj(ReposObj anObj) {
                Utils.closeWaitDialog();
            }            
        });
    }

    private void disableAction() {
        if (renderedActionWidget != null) {
            renderedActionWidget.disable();
        }
    }

    private void enableAction(ReposObj obj) {
        currentObj = obj;
        if (renderedActionWidget != null) {
            renderedActionWidget.enable();
        }
    }
}
