/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.admin.panels;

import java.util.HashMap;
import java.util.Map;

import com.extjs.gxt.ui.client.store.Store;
import com.google.gwt.dom.client.Element;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.ActionColumnConfig;
import com.iona.soa.web.repository.main.client.base.ui.GridConfig;
import com.iona.soa.web.repository.main.client.base.ui.ScheduledTaskPanelHelper;
import com.iona.soa.web.repository.main.client.base.ui.BaseObjectDialogWindow.AfterCreationCallBack;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator.DefaultColumnIds;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.ObjectCreationDialog;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.panels.ScheduledTaskPanel;

public final class ScheduledTaskGridConfig extends GridConfig {
    //static Store taskIdMappingStore;
    static final Map<String, String> TASK_ID_DISPLAY_NAME_MAP = new HashMap<String, String>();
    static final Map<String, String[]> TASK_ID_PARAMS_MAP = new HashMap<String, String[]>();

    @Override
    protected ActionColumnConfig<ReposObjDataModel> addEditColumn() {
        ActionColumnConfig.ActionColumnCallback<ReposObjDataModel> callback =
            new ActionColumnConfig.ActionColumnCallback<ReposObjDataModel>() {
                public void actionClicked(ReposObjDataModel record, Element anchorElement) {
                    ScheduledTaskPanelHelper.openEditScheduledTaskWindow(record.getPOJO(), new AfterCreationCallBack(){

                        public void onDialogClosure(ReposObj theObj) {
                            gridPanel.getStore().getLoader().load();
                        }});
                }
            };
        
        ActionColumnConfig<ReposObjDataModel> column =
            new ActionColumnConfig<ReposObjDataModel>(getGrid(), DefaultColumnIds.EDIT) {
            protected String getActionLabel(ReposObjDataModel record) {
                ReposObj obj = record.getPOJO();
                return obj.isEditAllowed() ? "Edit" : "View";
            }
        };
        column.setActionCallback(callback);
        
        columns.add(column);
        return column;
    }
         

    @Override
    protected void addCustomColumns() {
        SimpleColumnConfig typeColumnConfig =
            new SimpleColumnConfig("typeID", "Type") {
                public Object getCellData(ReposObj obj) {
                    String val = obj.getAttributeValue("typeID");
                    String dn = TASK_ID_DISPLAY_NAME_MAP.get(val);
                    if (dn == null) {
                        return val;
                    } else {
                        return dn;
                    }
                }                
            };
        typeColumnConfig.setWidth(200);
        columns.add(typeColumnConfig);
        
        columns.add(new SimpleColumnConfig("lastRun", "Last Run") {
            @Override
            public Object getCellData(ReposObj obj) {
                return formatDate(obj.getAttributeValue("lastRun"));
            }                
        });
        
        columns.add(new SimpleColumnConfig("duration", "Duration (mins)") {
            @Override
            public Object getCellData(ReposObj obj) {
                return formatDuration(obj);
            }            
        });

        columns.add(new SimpleColumnConfig("nextRun", "Next Run") {
            @Override
            public Object getCellData(ReposObj obj) {
                return formatDate(obj.getAttributeValue("nextRun"));
            }                
        });
        
        columns.add(new SimpleColumnConfig("state", "State"));
    }

    String formatDate(String date) {
        if (date == null) {
            return "";
        }
        
        int idx = date.indexOf('.');
        if (idx == -1) {
            return date;
        } else {
            return date.substring(0, idx);
        }
    }

    String formatDuration(ReposObj obj) {
        try {
            int duration = Integer.parseInt(obj.getAttributeValue("duration"));
            duration /= 60000;
            return "" + duration;
        } catch (Throwable th) {
            return "";
        }
    }
}
