/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public final class EClassHelper {
    public static final String FOLDER_ECLASS = "model.Folder";
    public static final String IFOLDER_ECLASS = "folder.IFolder";
    public static final String CATEGORY_ECLASS = "model.Category";
    public static final String ICATEGORY_ECLASS = "folder.ICategory";
    public static final String QUERY_ECLASS = "model.StoredQuery";
    public static final String ARTIFACT_ECLASS = "model.Artifact";
    public static final String ARTIFACT_TYPE_ECLASS = "model.ArtifactType";
    public static final String APPLICATION_ECLASS = "model.Application";
    public static final String DEPLOYMENT_PROFILE_ECLASS = "model.DeploymentProfile";
    public static final String VERSIONED_PROFILE_ECLASS = "model.VersionedProfile";
    public static final String CONTAINER_ECLASS = "model.Container";
    public static final String SERVICE_ECLASS = "model.Service";
    public static final String POLICY_ECLASS = "model.PolicyStatement";
    public static final String CONTRACT_ECLASS = "model.InterfaceDescriptionDocument";
    public static final String USER_ECLASS = "security.User";
    public static final String ROLE_ECLASS = "security.Role";
    public static final String DOCUMENT_ECLASS = "document.Document";
    public static final String MAVENREPO_ECLASS = "model.MavenRepository";
    public static final String PROPERTY_ECLASS = "repository.Property";
    public static final String SCHEDULED_TASK_ECLASS = "scheduling.ScheduledTask";    
    public static final String SCHEDULED_ONCEOFF_TASK_ECLASS = "scheduling.OnceOffTask";
    public static final String SCHEDULED_INTERVAL_REPEATED_TASK_ECLASS = "scheduling.IntervalRepeatedTask";
    public static final String SCHEDULED_DAILY_TASK_ECLASS = "scheduling.DailyRepeatedTask";
    public static final String SCHEDULED_WEEKLY_TASK_ECLASS = "scheduling.WeeklyRepeatedTask";
    public static final String SCHEDULED_MONTHLY_TASK_ECLASS = "scheduling.MonthlyRepeatedTask";
    public static final String AUDIT_EVENT_ECLASS = "repository.AuditEvent";
    
    // Non-EClass collections
    public static final String OSGI_TASK_SERVICES = "osgi.taskservices";

    private static final Map<String, ClassInfo> ECLASS_TO_INFO_MAP = new HashMap<String, ClassInfo>();
    private static final String DEFAULT_ECLASS = "default";
    
    static {
        ECLASS_TO_INFO_MAP.put(FOLDER_ECLASS,                 new ClassInfo("folders", "Folder", "images/folder_obj_small.gif", "folder-icon", "create-folder-icon", null, false));
        ECLASS_TO_INFO_MAP.put(CATEGORY_ECLASS,               new ClassInfo("categories", "Category", "folder_obj_small.gif", "folder-icon", "create-folder-icon", null, false));
        ECLASS_TO_INFO_MAP.put(QUERY_ECLASS,                  new ClassInfo("queries", "Query", "images/query.gif", "query-icon", "create-query-icon", null, false));
        ECLASS_TO_INFO_MAP.put(ARTIFACT_ECLASS,               new ClassInfo("artifacts", "Artifact", "images/artifact.gif", "artifact-icon", "create-artifact-icon", "artifacts_main", false));
        ECLASS_TO_INFO_MAP.put(ARTIFACT_TYPE_ECLASS,          new ClassInfo("artifacttypes", "Artifact Type", "images/artifact.gif", "artifact-icon", "create-artifact-icon", null, false));
        ECLASS_TO_INFO_MAP.put(APPLICATION_ECLASS,            new ClassInfo("applications", "Application", "images/application.gif", "application-icon", "create-application-icon", "apps_main", true));
        ECLASS_TO_INFO_MAP.put(DEPLOYMENT_PROFILE_ECLASS,     new ClassInfo("profiles", "Deployment Profile", "images/profile.gif", "profile-icon", "create-profile-icon", "deploy_main", true));
        ECLASS_TO_INFO_MAP.put(VERSIONED_PROFILE_ECLASS,      new ClassInfo("versionedprofiles", "Deployment Profile", "images/profile.gif", "profile-icon", "create-profile-icon", "deploy_main", false));
        ECLASS_TO_INFO_MAP.put(CONTAINER_ECLASS,              new ClassInfo("containers", "Container", "images/agent.gif", "agent-icon", "create-icon", "containers_main", true));
        ECLASS_TO_INFO_MAP.put(USER_ECLASS,                   new ClassInfo("users", "User", "images/user.gif", "user-icon", "create-user-icon", "users_main", true));
        ECLASS_TO_INFO_MAP.put(ROLE_ECLASS,                   new ClassInfo("roles", "Role", "images/role.gif", "role-icon", "create-role-icon", "roles_main", true));
        ECLASS_TO_INFO_MAP.put(DOCUMENT_ECLASS,               new ClassInfo("documents", "Document", "images/file_obj_small.gif", "file-icon", "create-file-icon", null, false));
        ECLASS_TO_INFO_MAP.put(MAVENREPO_ECLASS,              new ClassInfo("mavenrepositories", "Maven Repository", "images/maven_repository.gif", "maven-repository-icon", "create-icon", "maven_main", true));
        ECLASS_TO_INFO_MAP.put(PROPERTY_ECLASS,               new ClassInfo("properties", "Property", "images/artifact.gif", "artifact-icon", "create-icon", "property_main", false));
        ECLASS_TO_INFO_MAP.put(SCHEDULED_TASK_ECLASS,         new ClassInfo("scheduledtasks", "Scheduled Task", "images/scheduled_task.gif", "scheduled-task-icon", "create-icon", "sched_main", false));
        ECLASS_TO_INFO_MAP.put(SCHEDULED_ONCEOFF_TASK_ECLASS, new ClassInfo("onceofftasks", "Once-off Task", "images/scheduled_task.gif", "scheduled-task-icon", "create-icon", "sched_main", false));
        ECLASS_TO_INFO_MAP.put(SCHEDULED_INTERVAL_REPEATED_TASK_ECLASS, new ClassInfo("intervalrepeatedtasks", "Interval Repeated Task", "images/scheduled_task.gif", "scheduled-task-icon", "create-icon", "sched_main", false));
        ECLASS_TO_INFO_MAP.put(SCHEDULED_DAILY_TASK_ECLASS,   new ClassInfo("dailyrepeatedtasks", "Daily Repeated Task", "images/scheduled_task.gif", "scheduled-task-icon", "create-icon", "sched_main", false));
        ECLASS_TO_INFO_MAP.put(SCHEDULED_WEEKLY_TASK_ECLASS,  new ClassInfo("weeklyrepeatedtasks", "Weekly Repeated Task", "images/scheduled_task.gif", "scheduled-task-icon", "create-icon", "sched_main", false));
        ECLASS_TO_INFO_MAP.put(SCHEDULED_MONTHLY_TASK_ECLASS, new ClassInfo("monthlyrepeatedtasks", "Monthly Repeated Task", "images/scheduled_task.gif", "scheduled-task-icon", "create-icon", "sched_main", false));
        ECLASS_TO_INFO_MAP.put(OSGI_TASK_SERVICES,            new ClassInfo("taskservices", "Task Service", "images/file_obj_small.gif", "file-icon", "create-icon", null, false));
        ECLASS_TO_INFO_MAP.put(AUDIT_EVENT_ECLASS,            new ClassInfo("auditevents", "Audit Events", "images/file_obj_small.gif", "file-icon", "create-icon", null, false));
        ECLASS_TO_INFO_MAP.put(DEFAULT_ECLASS,                new ClassInfo(null, "Unknown", "images/file_obj_small.gif", "file-icon", "create-icon", null, false));
    }
    
    private EClassHelper() {
    }
    
    public static String getCollectionLinkForClass(String eClass) {
        return ReposHandle.INSTANCE.getServerBaseUrl() + getClassInfo(eClass).context;
    }
    
    public static String getClassForCollectionLink(String link) {
        if (!link.startsWith(ReposHandle.INSTANCE.getServerBaseUrl())) {
            return null;
        }
        
        int queryIdx = link.indexOf('?');
        if (queryIdx > 0) {
            link = link.substring(0, queryIdx);
        }
        
        String bareName = link.substring(ReposHandle.INSTANCE.getServerBaseUrl().length());
        int slashIdx = bareName.indexOf('/');
        if (slashIdx > 0) {
            bareName = bareName.substring(0, slashIdx);
        }
        
        for (Iterator<Map.Entry<String, ClassInfo>> it = ECLASS_TO_INFO_MAP.entrySet().iterator();
                it.hasNext();) {
            Map.Entry<String, ClassInfo> entry = it.next();
            if (entry.getValue() instanceof ClassInfo) {
                if (bareName.equals(((ClassInfo) entry.getValue()).context)) {
                    return entry.getKey().toString();
                }
            }
        }
        return null;
    }

    public static String getContextForClass(String eClass) {
        return getClassInfo(eClass).context;
    }
    
    public static String getNameForClass(String eClass) {
        return getClassInfo(eClass).singleName;
    }
    
    public static String getPluralNameForClass(String eClass) {
        return getClassInfo(eClass).pluralName;
    }
    
    public static String getIconFileForClass(String eClass) {
        return getClassInfo(eClass).iconFile;
    }
    
    public static String getIconClsForClass(String eClass) {
        return getClassInfo(eClass).iconCls;
    }
    
    public static String getAddIconClsForClass(String eClass) {
        return getClassInfo(eClass).addIconCls;
    }
    
    public static boolean isCategorizable(String eClass) {
        return getClassInfo(eClass).categorizable;
    }
    
    public static boolean isCollectionURL(String url) {
        // TODO REVISIT ...
        // Could iterate through known classes
        return url.endsWith("s") || url.endsWith("s/") || url.indexOf("?") > 0 || url.indexOf(";") > 0;
    }
    
    public static String getHelpBaseContextId(String eClass) {
        return getClassInfo(eClass).helpBaseContextId;
    }

    public static String getEClassForContext(String context) {
        for (Iterator<Map.Entry<String, ClassInfo>> it = ECLASS_TO_INFO_MAP.entrySet().iterator();
            it.hasNext();) {
            Map.Entry<String, ClassInfo> entry = it.next();
            if (entry.getValue() instanceof ClassInfo) {
                if (context.equals(((ClassInfo) entry.getValue()).context)) {
                    return entry.getKey().toString();
                }
            }
        }
        return null;
    }
    
    private static ClassInfo getClassInfo(String eClass) {
        ClassInfo info = (ClassInfo)ECLASS_TO_INFO_MAP.get(eClass);
        if (info == null) {
            info = (ClassInfo)ECLASS_TO_INFO_MAP.get(DEFAULT_ECLASS);
        }
        return info;
    }
    
    private static class ClassInfo {
        public final String context;
        public final String singleName;
        public final String pluralName;
        public final String iconFile;
        public final String iconCls;
        public final String addIconCls;
        public final String helpBaseContextId;
        public final boolean categorizable;
        
        public ClassInfo(String context,
                         String name,
                         String iconFile,
                         String iconCls,
                         String addIconCls,
                         String helpBaseContextId,
                         boolean categorizable) {
            
            this.context = context;
            this.singleName = name;
            if (name.endsWith("y")) {
                // Query -> Queries, Policy -> Policies, etc.
                pluralName = name.substring(0, name.length() - 1) + "ies";
            } else {
                // Folder -> Folders, etc.
                pluralName = name + "s";
            }
            this.iconFile = iconFile;
            this.iconCls = iconCls;
            this.addIconCls = addIconCls;
            this.helpBaseContextId = helpBaseContextId;
            this.categorizable = categorizable;
        }
    }
}
