/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.MenuEvent;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.extjs.gxt.ui.client.widget.menu.Item;
import com.extjs.gxt.ui.client.widget.menu.Menu;
import com.extjs.gxt.ui.client.widget.menu.MenuItem;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;

public class GridContextMenuAdapter implements Listener<MenuEvent> {
    

    private Menu rootMenu = new Menu();
    private Grid<ReposObjDataModel> anchor;
    private List<ReposObjDataModel> records;
    private Map<Menu, Collection<CallBack>> menuItems = new HashMap<Menu, Collection<CallBack>>();
    
    public GridContextMenuAdapter(Grid<ReposObjDataModel> anAnchor) {
        rootMenu.addListener(Events.BeforeShow, this);
        anchor = anAnchor;
    }
    
    public void registerContextMenuItem(Menu menu, final CallBack callBack) {
        if (menu == null) {
            menu = rootMenu;
        }
        if (!menuItems.containsKey(menu)) {
            menuItems.put(menu, new LinkedList<CallBack>());
        }
        menuItems.get(menu).add(callBack);
    }
    
    public Menu getRootMenu() {
        return rootMenu;
    }

    public void resetContextMenu(Menu menu) {
        if (menuItems.containsKey(menu)) {
            menuItems.get(menu).clear();
        }
    }

    @SuppressWarnings("unchecked")
    public void handleEvent(MenuEvent ge) {
        onRowContextMenu(ge);
    }
    
    public void onRowContextMenu(MenuEvent ge) {
        ge.stopEvent();
        
        List<ReposObjDataModel> selectedRecords = anchor.getSelectionModel().getSelectedItems();
        ReposObjDataModel contextRecord = anchor.getSelectionModel().getSelectedItem();
        boolean found = false;
        for (ReposObjDataModel selectedRecord : selectedRecords) {
           
            if (contextRecord.getPOJO().getSelfLink().equals(selectedRecord.getPOJO().getSelfLink())) {
                found = true;
                break;
            }
        }
        
        if (!found) {
            anchor.getSelectionModel().select(contextRecord);
        }
        
        records = anchor.getSelectionModel().getSelectedItems();
        
        tweakMenu(rootMenu);
    }
    
    private boolean tweakMenu(Menu menu) {
        if (menu == null) {
            return false;
        }
        
        menu.removeAll();
        Collection<CallBack> items = menuItems.get(menu);
        if (items == null) {
            return false;
        }
        
        boolean ret = false;
        for (Iterator<CallBack> iter = items.iterator(); iter.hasNext();) {
            final CallBack callBack = iter.next();
            Item item = callBack.createMenuItem(records);
            if (item == null) {
                continue;
            }
            
            item.addSelectionListener(new SelectionListener<ComponentEvent>() {
                @Override
                public void componentSelected(ComponentEvent ce) {
                    callBack.menuItemSelected(records);
                }
            });
            
            if (item.isEnabled()) {
                ret = true;
                if (item instanceof MenuItem
                        && ((MenuItem)item).getSubMenu() != null) {
                    tweakMenu(((MenuItem)item).getSubMenu());
                }
            }
            menu.add(item);
        }
        return ret;
    }
    
    public interface CallBack {
        Item createMenuItem(List<ReposObjDataModel> selectedRecords);
        void menuItemSelected(List<ReposObjDataModel> records);
    }

    
}
