/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base;

/**
 * Abstraction of the process of editing an "input".  Subset of an Eclipse 
 * interface with the same name.
 * <p>
 * External entities to an editor typically need to know if the editor is "dirty", i.e.
 * has unsaved changes.  Gwt-Ext does provide a Form metaphor that will provide this; however
 * the number of "Field" implementations is limited and it is difficult to adapt non "Field" 
 * classes to behave like Fields.  For example, GridPanel objects backed by stores
 * are typically found in an editor, yet are difficult to make work with the Form metaphor. 
 * </p>
 * This interface and the associated IFormPart interface aim to provide a more complete
 * form framework (for our needs).  
 * 
 * @see com.iona.soa.web.repository.main.client.base.ui.editor.widgets.FieldWidgetHelper
 *  
 * for the <code>adapt</code> method that adapts standard Field widgets to IFormPart instances.
 */
public interface IManagedForm {

    /**
     * @return true if any form part has a change that needs to be written
     * to the model.
     */
    boolean isDirty();
    
    /**
     * Add a part that will be thereafter "managed" by this form.  For example, a 
     * call to commit on the form will result in a call to commit on the part.
     * 
     * @param formPart
     * @return the part just added.
     */
    IFormPart addPart(IFormPart formPart);
    
    IFormPart removePart(IFormPart formPart);
    
    /**
     * All pending changes are pushed into the model.
     */
    void commit(boolean refresh);
    
    /**
     * Update all form parts with the data from the model.
     */
    void refresh();
    
    void close();
    
    /**
     * Set the input object that the form is editing.
     * 
     * @param input
     */
    void setInput(Object input);
}
