/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base;

import java.util.ArrayList;
import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.History;
import com.google.gwt.user.client.HistoryListener;
import com.iona.soa.web.repository.base.client.model.IReposObjChangeListener;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjChangeNotifier;
import com.iona.soa.web.repository.main.client.MainModule;

/*
 * track a fixed number of links i in the chronological order in which they gets reported
 */
public final class LinkHistoryTracker implements HistoryListener {

    public static final LinkHistoryTracker INSTANCE = new LinkHistoryTracker();

    private static final String HISTORY_TOKEN_SEPARATOR = "-__-";
    
    private static List<ISubModule> knownContexts;
    
    private List<String> orphanLinks;

    private String newlyAddedToken;
    
    private IReposObjChangeListener orphanTrackingListener = new IReposObjChangeListener() {
        public void reposObjCreated(ReposObj createdObj, String containingCollectionUrl, int index) {
            // nothing required
        }

        public void reposObjDeleted(ReposObj deletedObj) {
            orphanLinks.add(deletedObj.getSelfLink());
        }
        
        public void reposObjUpdated(ReposObj updatedObj) {
            // nothing required
        }
    };
    
    private LinkHistoryTracker() { 
        knownContexts = new ArrayList<ISubModule>();
        
        orphanLinks = new ArrayList<String>();
        
        ReposObjChangeNotifier.INSTANCE.addListener(orphanTrackingListener);
        History.addHistoryListener(this);
    }

    public void reportLink(String link) {
        reportLink(link, null);
    }
    
    public void reportLink(String link, String filterQuery) {
        ISubModule context = MainModule.get() != null 
            ? MainModule.get().getWorkspaceControl().getActiveSubModule() : null;
        String token = makeHistoryToken(link, filterQuery, context);

        if (!token.equals(History.getToken())) {
            Log.debug("\nhistory token recorded      ==> " + token);
            newlyAddedToken = token;
            History.newItem(token, false);
        }
    }

    public void walkToOlderLink() {
        Log.debug("\nhistory ==> walking back");
        History.back();
    }

    /**
     * HistoryListener implementation
     */
    public void onHistoryChanged(String historyToken) {

        if (isValidToken(historyToken) && !historyToken.equals(newlyAddedToken)) {
            
            if (orphanLinks.contains(getLinkFromHistoryToken(historyToken))) {
                History.back();
                return;
            }
            Log.debug("\nhistory token loaded      ==> " + historyToken);
            
            String filerQuery = getFilterQueryFromHistoryToken(historyToken);
            ISubModule context = getContextFromHistoryToken(historyToken);
            String link = getLinkFromHistoryToken(historyToken);
            if (filerQuery != null && filerQuery.length() > 0) {
                MainModule.get().getWorkspaceControl().openCollection(link, filerQuery, context, true);
            } else {
                MainModule.get().getWorkspaceControl().openLink(link, context, true);
            }
        }
    }
    
    private String makeHistoryToken(String link, String filterQuery, ISubModule context) {
        
        int contextIndex = knownContexts.indexOf(context);
        if (contextIndex == -1) {
            knownContexts.add(context);
            contextIndex = knownContexts.indexOf(context);
        }
        return link
               + HISTORY_TOKEN_SEPARATOR
               + (filterQuery == null ? "" : filterQuery)
               + HISTORY_TOKEN_SEPARATOR
               + contextIndex;
    }
    
    private String getLinkFromHistoryToken(String token) {
        return token.substring(0, token.indexOf(HISTORY_TOKEN_SEPARATOR));
    }

    private String getFilterQueryFromHistoryToken(String token) {
        return token.substring(token.indexOf(HISTORY_TOKEN_SEPARATOR) + HISTORY_TOKEN_SEPARATOR.length(),
                               token.lastIndexOf(HISTORY_TOKEN_SEPARATOR));
    }

    private ISubModule getContextFromHistoryToken(String token) {
        int contextIndex = Integer.parseInt(token.substring(token.lastIndexOf(HISTORY_TOKEN_SEPARATOR)
                                                            + HISTORY_TOKEN_SEPARATOR.length()));
        return knownContexts.get(contextIndex);
    }

    private boolean isValidToken(String token) {
        int index = token.lastIndexOf(HISTORY_TOKEN_SEPARATOR);
        return (index == token.length() - (HISTORY_TOKEN_SEPARATOR.length() + 2))
                || (index == token.length() - (HISTORY_TOKEN_SEPARATOR.length() + 1));
    }
}
