/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base;

import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.base.client.remote.IObjCallBack;
import com.iona.soa.web.repository.main.client.MainModule;

public final class ReposObjHelper {
    public static final String SUB_COLLECTION_SEPARATOR = ";";
    public static final String FOLDER_ALLCONTENT_URL_SUFFIX = SUB_COLLECTION_SEPARATOR + "allContent";
    public static final String FOLDER_CHILDREN_URL_SUFFIX = SUB_COLLECTION_SEPARATOR + "children";
    public static final String QUERY_RESULT_URL_SUFFIX = SUB_COLLECTION_SEPARATOR + "queryResult";
    public static final String CATEGORY_CONTENT_URL_SUFFIX = SUB_COLLECTION_SEPARATOR + "categoryContent";
    private static final String FOLDER_CONTEXT = EClassHelper.getContextForClass(EClassHelper.FOLDER_ECLASS);
    
    private ReposObjHelper() {
    }
    
    public static boolean isFolderObj(ReposObj obj) {
        return obj == null ? false : isFolderType(obj.getEClass());
    }
    
    public static void renameObject(String link,
                                    final String oldName,
                                    final String newName,
                                    final IObjCallBack callBack) {
        
        link = ReposObjHelper.getSelfLink(link);
        MainModule.getRemoteClient().getReposObj(link, new IObjCallBack() {
            public void processReposObj(ReposObj anObj) {
                ReposObjAttribute attr = anObj.getAttribute("name");
                if (attr == null) {
                    callBack.processConnectionError(new RuntimeException("Object has no name"));
                    return;
                }
                String currentName = attr.getValue();
                if (oldName == null && currentName != null
                    || oldName != null && !oldName.equals(currentName)) {
                    callBack.processConnectionError(new RuntimeException("Object out of date"));
                    return;
                }
                
                attr.setSimpleValue(newName);
                MainModule.getRemoteClient().updateReposObj(anObj, callBack);
            }
            public void processConnectionError(Throwable e) {
                callBack.processConnectionError(e);
            }
            public void processResponseError(String anErrorCode, String anErrorMsg) {
                callBack.processResponseError(anErrorCode, anErrorMsg);
            }
        });
    }

    public static void createObject(String collectionName, 
                                    String objectName,
                                    final IObjCallBack callBack) {
        createObjectInFolder(collectionName, objectName, null, callBack);
    }
    
    public static void createObjectInFolder(String collectionName, 
                                            String objectName,
                                            String parentFolderLink, 
                                            final IObjCallBack callBack) {
        ReposObj templateObj = null;
        if (parentFolderLink != null || objectName != null) {
            templateObj = new ReposObj();
            if (objectName != null) {
                ReposObjAttribute nameAttr = new ReposObjAttribute();
                nameAttr.setName("name");
                nameAttr.setSimpleValue(objectName);
                templateObj.addAttribute(nameAttr);
            }
            if (parentFolderLink != null) {
                ReposObjAttribute parentAttr = new ReposObjAttribute();
                parentAttr.setReference(true);
                if (FOLDER_CONTEXT.equals(collectionName)) {
                    parentAttr.setName("parent");
                } else {
                    parentAttr.setName("belongsTo");
                }
                parentAttr.setRefValue(ReposHandle.getRelativeLink(parentFolderLink), "");
                templateObj.addAttribute(parentAttr);
            }
        }

        MainModule.getRemoteClient().createReposObj(ReposHandle.INSTANCE.getServerBaseUrl(),
                                             collectionName,
                                             templateObj,
                                             callBack);
    }
    
    public static String getSelfLink(String subCollectionLink) {
        if (subCollectionLink != null) {
            int index = subCollectionLink.indexOf(SUB_COLLECTION_SEPARATOR);
            if (index > 0) {
                subCollectionLink = subCollectionLink.substring(0, index);
            }
        }
        return subCollectionLink;
    }
    
    public static boolean isEClassLink(String aLink, String eClass) {
        String contextForClass = EClassHelper.getContextForClass(eClass);
        if (ReposHandle.isAbsoluteLink(aLink)) {
            return aLink.startsWith(ReposHandle.INSTANCE.getServerBaseUrl() + contextForClass);
        } else {
            return aLink.startsWith(contextForClass);
        }
    }
    
    public static boolean isFolderType(String eClass) {
        if (eClass != null 
            && (EClassHelper.FOLDER_ECLASS.equals(eClass) || EClassHelper.IFOLDER_ECLASS.equals(eClass))) {
            return true;
        }
        return false;
    }
    
    public static boolean isCategoryType(String eClass) {
        if (eClass != null 
            && (EClassHelper.CATEGORY_ECLASS.equals(eClass)
                    || EClassHelper.ICATEGORY_ECLASS.equals(eClass))) {
            return true;
        }
        return false;
    }
}
