/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base;

import java.util.Map;

public final class ReposURL {
    private final String url;
    private final String eClass;
    private final Map<String, String> parametersMap;
    
    public ReposURL(String reposURL) {
        url = reposURL;
        eClass = EClassHelper.getClassForCollectionLink(url);
        parametersMap = null;
    }

    /**
     * @param parameters in URL form e.g. "?section=applications&action=create"
     */
    public ReposURL(String reposURL, Map<String, String> parameters) {
        url = reposURL;
        eClass = EClassHelper.getClassForCollectionLink(url);
        parametersMap = parameters;
    }
    
    public String getURL() {
        return url;
    }
    
    public String getEClass() {
        return eClass;
    }
    
    public boolean represents(String anEClass) {
        // TODO - could this be implemented as eClass == this.eClass?
        return ReposObjHelper.isEClassLink(url, anEClass);        
    }

    public boolean hasParameter(String key) {
        return parametersMap == null ? false : parametersMap.containsKey(key);
    }
    
    public String getParameterValue(String key) {
        return parametersMap == null ? null : parametersMap.get(key);        
    }
    
    @Override
    public boolean equals(Object other) {
        if (other == this) {
            return true;
        } else if (other instanceof ReposURL) {
            return url.equals(((ReposURL)other).url);
        }
        
        return false;
    }

    @Override
    public String toString() {
        return super.toString();
    }
}
