/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.XDOM;
import com.extjs.gxt.ui.client.core.El;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.WindowEvent;
import com.extjs.gxt.ui.client.fx.Fx;
import com.extjs.gxt.ui.client.fx.FxConfig;
import com.extjs.gxt.ui.client.fx.SingleStyleEffect;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.MessageBox.MessageBoxType;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.Window;

public final class Utils {
    private static MessageBox openErrorMessage;
    private Utils() { }
    
    public static synchronized void handleErrorMsg(String msg, final boolean logout) {
        if (openErrorMessage == null) {
            openErrorMessage = new MessageBox();
            openErrorMessage.setType(MessageBoxType.ALERT);
            openErrorMessage.setIcon(MessageBox.ERROR);
            openErrorMessage.addListener(Events.Close, new Listener<WindowEvent>() {
                public void handleEvent(WindowEvent be) {
                    openErrorMessage = null;
                    if (logout) {
                        Window.open("\\ui\\Session?action=logout", "_self", ""); 
                    }
                }
            });
            openErrorMessage.setButtons(MessageBox.OK);
            openErrorMessage.setTitle("Error");
            openErrorMessage.setMessage(msg);
            openErrorMessage.show();
        }
    }
    
    public static void showWaitDialog() {
        // TODO - perhaps should make a proper panel in MainModule?
        Element statusHolderElem = XDOM.getElementById("arr-status-bar");
        if (statusHolderElem != null) {
            statusHolderElem.setClassName("");
            updateWaitDialogMsg("Contacting server...");
        }
    }
    
    public static void updateWaitDialogMsg(String newMsg) {
        Element statusTextElem = XDOM.getElementById("arr-status-text");
        if (statusTextElem != null) {
            statusTextElem.setInnerText(newMsg);
        }
    }
    
    public static void closeWaitDialog() {
        Timer t = new Timer() {
            @Override
            public void run() {
                Element statusHolderElem = XDOM.getElementById("arr-status-bar");
                if (statusHolderElem != null) {
                    updateWaitDialogMsg("");
                    statusHolderElem.setClassName("x-hidden");
                }
            }
        };
        t.schedule(100);
    }

    public static El highlight(El theElt) {
        FxConfig cfg = new FxConfig(10000);
        Fx fx = new Fx(cfg);
        fx.run(new HighLightEffect(theElt));
        
        return theElt;
    }
    
    private static class HighLightEffect extends SingleStyleEffect {

        private String initialBgColor;
        
        public HighLightEffect(El el) {
            super(el, "background-color", 0, 255);
        }

        @Override
        public void increase(double value) {
            int intValue = 0;
            while (value >= 0 && intValue < value) {
                intValue++;
            }
            String hex = Integer.toHexString(255 - intValue + 1).toUpperCase();
            el.dom.getStyle().setProperty("background-color", "#00" + hex + hex);
        }

        public void onComplete() {
            el.dom.getStyle().setProperty("filter", "");
            el.dom.getStyle().setProperty("background-color", initialBgColor);
        }

        public void onStart() {
            initialBgColor = el.getStyleAttribute("background-color");
            el.dom.getStyle().setProperty("background-color", "#FF0000");
            el.setVisible(true);
        }
    }
}
