/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.gxtdatamodels;

import com.extjs.gxt.ui.client.data.BaseModelData;
import com.extjs.gxt.ui.client.data.ModelData;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.ReposObjHelper;

@SuppressWarnings("serial")
public class ReposObjDataModel extends BaseModelData {

    public static final String FIELD_POJO = "POJO";
    public static final String FIELD_ECLASS = "eclass";
    public static final String FIELD_CAPTION = "caption";
    public static final String FIELD_NAME = "name";
    public static final String FIELD_VALUE = "value";
    public static final String FIELD_TIMESTAMP = "timestamp";
    public static final String FIELD_TIMESTAMP_IN_MS = "msTimestamp";
    public static final String FIELD_LAST_MODIFIED_BY = "lastModifiedBy";
    public static final String FIELD_RSS_FEED = "rss";
    public static final String FIELD_SELF_LINK = "selfLink";
    public static final String FIELD_EDIT_LINK = "editLink";
    // Allow models to be searched by proper
    public static final String FIELD_ID = "id";
    public static final String FIELD_HREF = "href";

    public ReposObjDataModel(ReposObj aReposObj) {
        super();
        setPOJO(aReposObj);
        
        initModelData(aReposObj, this);
    }

    protected static void initModelData(ReposObj aReposObj, ModelData md) {
        if (aReposObj != null) {
            md.set(FIELD_CAPTION, aReposObj.getCaption());
            md.set(FIELD_ECLASS, aReposObj.getEClass());
            md.set(FIELD_TIMESTAMP, aReposObj.getLastUpdatedTimeStamp());
            md.set(FIELD_RSS_FEED, aReposObj.getFeedSubscriptionLink());
            md.set(FIELD_EDIT_LINK, aReposObj.getEditLink());
            md.set(FIELD_ID, aReposObj.getId());
            
            String relevantSelfLink;
            if (ReposObjHelper.isFolderObj(aReposObj)) {
                relevantSelfLink = aReposObj.getSelfLink() + ReposObjHelper.FOLDER_ALLCONTENT_URL_SUFFIX;
            } else if (EClassHelper.QUERY_ECLASS.equals(aReposObj.getEClass())) {
                relevantSelfLink = aReposObj.getSelfLink() + ReposObjHelper.QUERY_RESULT_URL_SUFFIX;
            } else {
                relevantSelfLink = aReposObj.getSelfLink();
            }
            md.set(FIELD_SELF_LINK, relevantSelfLink);
            md.set(FIELD_HREF, ReposHandle.getRelativeLink(relevantSelfLink));
            
            for (String attrName : aReposObj.getAttributes().keySet()) {
                ReposObjAttribute anAttr = aReposObj.getAttribute(attrName);
                if (anAttr != null && !anAttr.isReference() && !anAttr.isMulti()) {
                    md.set(attrName, anAttr.getValue());
                }
                if (anAttr != null && anAttr.isCollection()) {
                    md.set(attrName, anAttr.getCollectionLink());
                }
            }
        }
    }

    public ReposObj getPOJO() {
        return (ReposObj) get(FIELD_POJO);
    }

    public void setPOJO(ReposObj value) {
        set(FIELD_POJO, value);
    }
    
    protected String getPojoAttrValue(String anAtrrName) {
        return getPOJO() == null ? null : getPOJO().getAttributeValue(anAtrrName);
    }
    
    protected void setPojoAttrValue(String anAtrrName, String aValue) {
        if (getPOJO() != null) {
            getPOJO().getAttribute(anAtrrName).setSimpleValue(aValue);
        }
    }

}
