/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui;

import java.util.ArrayList;
import java.util.List;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.event.GridEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.extjs.gxt.ui.client.widget.layout.FitLayout;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Element;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.main.client.UIMessages;
import com.iona.soa.web.repository.main.client.base.ICollectionPanel;
import com.iona.soa.web.repository.main.client.base.ISelectionProvider;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;

public abstract class BaseCollectionPanel extends ContentPanel
                                          implements ICollectionPanel, ISelectionProvider {

    protected static final UIMessages MESSAGES = GWT.create(UIMessages.class);
    
    protected final EditorGrid<ReposObjDataModel> gridPanel;
    protected final SimpleGridConfigurator gridConfig;
    private final List<ISelectionListener> selectionListeners = new ArrayList<ISelectionListener>();

    public BaseCollectionPanel(SimpleGridConfigurator aGridConfig) {
        
        addStyleName("arr-collection-panel");
        //setHeader(false);
        //setClosable(false);
        //setTitleCollapse(false);
        setCollapsible(false);
        setBodyBorder(false);
        setBorders(false);
        setHeaderVisible(false);
        
        gridConfig  = aGridConfig;
        gridPanel = gridConfig.getGrid();
        gridPanel.addStyleName("arr-collection-panel-grid");
        
        addSelectionSupport(gridPanel);
     
        setLayout(new FitLayout());
        add(gridPanel);
    }
    
    @Override
    protected void onRender(Element parent, int pos) {
        super.onRender(parent, pos);
        doFirstLoad();
        attachStoreAndGridSynchronizer();
    }


    protected abstract void attachStoreAndGridSynchronizer();

//    protected abstract String getContentLabel();

    /**
     * @see com.iona.soa.web.repository.main.client.base.IEditorPanel
     */
    public final boolean isDirty() {
        // Collection panels by default don't allow editing.
        return false;
    }
    
    /**
     * ICollectionPanel impl 
     */
    public final ListStore<ReposObjDataModel> getStore() {
        return gridPanel.getStore();
    }

    public final EditorGrid<ReposObjDataModel> getGrid() {
        return gridPanel;
    }

    /**
     * ISelectionProvider implementation
     */
    public final void addSelectionListener(ISelectionListener listener) {
        selectionListeners.add(listener);
    }

    /**
     * ISelectionProvider implementation
     */
    public final void removeSelectionListener(ISelectionListener listener) {
        selectionListeners.remove(listener);
    }

    private void addSelectionSupport(Grid<ReposObjDataModel> gp) {
        gp.addListener(Events.RowClick, new Listener<GridEvent>() {
            public void handleEvent(GridEvent ge) {
                List<ReposObjDataModel> selectedItems = ge.grid.getSelectionModel().getSelectedItems();
                List<ReposObj> selectedObjs = new ArrayList<ReposObj>(selectedItems.size());
                for (ReposObjDataModel item : selectedItems) {
                    selectedObjs.add(item.getPOJO());
                }
                
                for (ISelectionListener l : selectionListeners) {
                    l.onSelectionChange(selectedObjs);
                }
            }
        });
    }
    
    /**
     * ISelectionProvider implementation
     */
    public final List<ReposObj> getSelection() {
        
        List<ReposObj> selection = new ArrayList<ReposObj>();
        for (ReposObjDataModel record : gridPanel.getSelectionModel().getSelection()) {
            selection.add(record.getPOJO());
        }
        return selection;
    }

    protected void doFirstLoad() {
        reloadData();
    }
    
    public final void reloadData() {
        getStore().getLoader().load();
    }
    
    /* May reuse in SimpleGridConfig::configure - allow specific grids to mask themselves
     * for example, nested grids in an editor
    protected void attachStoreLoadingDecorator(Store store) {
        
        final String itemsStr = getContentLabel();
        store.addStoreListener(new StoreListenerAdapter() {
            private boolean loading;
            @Override
            public boolean doBeforeLoad(Store store) {
                Timer timer = new Timer() {
                    public void run() {
                        if (loading) {                            
                            getEl().mask("Loading " + itemsStr + "...", 
                                         "arr-collection-panel-loading-mask");
                        }
                    }
                };
                loading = true;
                timer.schedule(200);
                return true;
            }
    
            @Override
            public void onLoad(Store store, Record[] records) {
                loading = false;
                unmask();
            }
    
            @Override
            public void onLoadException(Throwable error) {
                loading = false;
                unmask();
            }
            
            private void unmask() {
                DeferredCommand.addCommand(new Command() {
                    public void execute() {
                        getEl().unmask();
                    }
                });
            }
        });
    }
    */   
}