/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui;

import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.Window;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.AbstractReposObjCreationPanel;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.CreationPanelRegistry;

public abstract class BaseObjectDialogWindow extends Window {

    private static final int WINDOW_HEIGHT = 300;

    private static final int WINDOW_WIDTH = 500;
    
    public BaseObjectDialogWindow(String anEClass, 
                                  AfterCreationCallBack aCallBack, Object panelDefaults) {
        initialise(anEClass, aCallBack, panelDefaults);
    }

    public BaseObjectDialogWindow(String anEClass, 
                                  AfterCreationCallBack aCallBack) {
        initialise(anEClass, aCallBack, null);
    }

    private void initialise(String eClass, 
                            final AfterCreationCallBack callback, 
                            Object panelDefaults) {
        final AbstractReposObjCreationPanel cp 
            = CreationPanelRegistry.getPanel(eClass, panelDefaults);

        setWidth(cp == null ? WINDOW_WIDTH : cp.getRequiredWidth());
        setHeight(cp == null ? WINDOW_HEIGHT : cp.getRequiredHeight());
        setTitle(getTitleMessage(EClassHelper.getNameForClass(eClass)));
        setHeading(getTitleMessage(EClassHelper.getNameForClass(eClass)));
        setBorders(true);
        setBodyBorder(true);
        setFrame(true);

        setModal(true);
        setMaximizable(false);
        setCloseAction(CloseAction.HIDE); // TODO
        setPlain(true);

        add(cp);

        // setIconCls(EClassHelper.getAddIconClsForClass(eClass));
        Button ok = new Button("OK");
        ok.addSelectionListener(new SelectionListener<ButtonEvent>() {
            @Override
            public void componentSelected(ButtonEvent ce) {
                onOkSelected(cp, callback);
            }
        });
        addButton(ok);

        Button cancel = new Button("Cancel");
        cancel.addSelectionListener(new SelectionListener<ButtonEvent>() {
            @Override
            public void componentSelected(ButtonEvent ce) {
                close();
            }
        });
        addButton(cancel);
    }

    protected abstract String getTitleMessage(String objectName);

    protected abstract void onOkSelected(AbstractReposObjCreationPanel panel,
                                         AfterCreationCallBack callback);

    public interface AfterCreationCallBack {
        void onDialogClosure(ReposObj theObj);
    }
}
