/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui;

import java.util.List;

import com.extjs.gxt.ui.client.store.Store;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.extjs.gxt.ui.client.widget.menu.CheckMenuItem;
import com.extjs.gxt.ui.client.widget.menu.Item;
import com.extjs.gxt.ui.client.widget.menu.Menu;
import com.extjs.gxt.ui.client.widget.menu.MenuItem;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.base.client.model.ReposObjChangeNotifier;
import com.iona.soa.web.repository.base.client.remote.IDeletionCallBack;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.GridContextMenuAdapter;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.ReposObjHelper;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;

public final class CollectionContextMenuHelper {
    private CollectionContextMenuHelper() {
    }
    
    public static void addRenameContextMenu(final EditorGrid<ReposObjDataModel> gridPanel, 
                                            GridContextMenuAdapter menuAdapter) {
        
        GridContextMenuAdapter.CallBack callBack = new GridContextMenuAdapter.CallBack() {
            
            public Item createMenuItem(List<ReposObjDataModel> selectedRecords) {
                MenuItem ret = new MenuItem("Rename");
                if (selectedRecords.size() == 1 && selectedRecords.get(0).getPOJO().isEditAllowed()) {
                    ret.setIconStyle("rename-obj-icon");
                } else {
                    ret.setIconStyle("rename-obj-off-icon");
                    ret.disable();
                }
                return  ret;
            }
            
            public void menuItemSelected(List<ReposObjDataModel> records) {
                if (records.size() == 1) {
                    int rowIdx = gridPanel.getStore().indexOf(records.get(0));
                    int colIdx = 0; //collectionPanel.getGrid().getColumnModel().getIndexById("NAME_INDEX");
                    gridPanel.startEditing(rowIdx, colIdx);
                }
            }
        };

        menuAdapter.registerContextMenuItem(null, callBack);
        
    }
    
    public static void addEditContextMenu(final EditorGrid<ReposObjDataModel> gridPanel, 
                                          GridContextMenuAdapter menuAdapter) {
        
        GridContextMenuAdapter.CallBack callBack = new GridContextMenuAdapter.CallBack() {
        
            public Item createMenuItem(List<ReposObjDataModel> selectedRecords) {
                MenuItem ret = new MenuItem("Edit");
                if (selectedRecords.size() == 1) {
                    if (!selectedRecords.get(0).getPOJO().isEditAllowed()) {
                        ret = new MenuItem("View");
                    }
                    ret.setIconStyle("edit-obj-icon");
                } else {
                    ret.setIconStyle("edit-obj-off-icon");
                    ret.disable();
                }
                return  ret;
            }
            
            public void menuItemSelected(List<ReposObjDataModel> records) {
                if (records.size() == 1) {
                    String link = ReposObjHelper.getSelfLink(records.get(0).getPOJO().getSelfLink());
                    MainModule.get().getWorkspaceControl().openLink(link, true);
                }
            }
        };
        
        menuAdapter.registerContextMenuItem(null, callBack);
    }
    
    public static void addDeleteContextMenu(final EditorGrid<ReposObjDataModel> gridPanel, 
                                            GridContextMenuAdapter menuAdapter) {
        
        GridContextMenuAdapter.CallBack callBack = new GridContextMenuAdapter.CallBack() {
        
            public Item createMenuItem(List<ReposObjDataModel> selectedRecords) {
                MenuItem ret;
                if (selectedRecords.size() < 2) {
                    ret = new MenuItem("Delete");
                } else {
                    ret = new MenuItem("Delete All");
                }
                for (ReposObjDataModel record : selectedRecords) {
                    if (!record.getPOJO().isDeleteAllowed()) {
                        ret.setIconStyle("delete-off-icon");
                        ret.disable();
                        return ret;
                    }
                }
                ret.setIconStyle("delete-icon");
                return  ret;
            }
           
            public void menuItemSelected(List<ReposObjDataModel> records) {
                if (records.size() == 0) {
                    return;
                }
                
                final ReposObj[] objs = new ReposObj[records.size()];
                int i = 0;
                for (ReposObjDataModel rcd : records) {
                    objs[i++] = rcd.getPOJO();
                }

                MainModule.get().getWorkspaceControl().deleteObjects(objs, new IDeletionCallBack() {
                    public void processSuccess() { }
                    public void processConnectionError(Throwable e) { }
                    public void processResponseError(String anErrorCode, String anErrorMsg) { }
                }, true);
            }
        };
        menuAdapter.registerContextMenuItem(null, callBack);
    }

    public static void addCategoriesContextMenu(WorkspaceCollectionPanel panel,
                                                final Store<ReposObjDataModel> categoriesStore) {
        
        final EditorGrid<ReposObjDataModel> gridPanel = panel.getGrid(); 
        final GridContextMenuAdapter menuAdapter = panel.getContextMenuAdapter();
        final Menu menu = new Menu();
        
        menuAdapter.registerContextMenuItem(null, new GridContextMenuAdapter.CallBack() {
            
            public Item createMenuItem(List<ReposObjDataModel> selectedRecords) {
                menuAdapter.resetContextMenu(menu);
                MenuItem ret = new MenuItem("Categories");
                ret.setIconStyle("folder-icon");
                ret.setSubMenu(new Menu());
                if (categoriesStore.getModels().size() > 0) {
                    addCategoriesSubMenu(gridPanel, 
                                         menuAdapter, 
                                         ret.getSubMenu(), 
                                         categoriesStore, 
                                         selectedRecords);
                } else {
                    ret.disable();
                }
                return ret;
            }
            
            public void menuItemSelected(List<ReposObjDataModel> records) {
                // do nothing
            }
        });
    }    

    private static void addCategoriesSubMenu(final EditorGrid<ReposObjDataModel> gridPanel,
                                             final GridContextMenuAdapter menuAdapter, 
                                             final Menu anchor,
                                             final Store<ReposObjDataModel> categoriesStore, 
                                             List<ReposObjDataModel> selectedRecords) {
        
        List<ReposObjDataModel> categories = categoriesStore.getModels();
        for (ReposObjDataModel categoryRcd : categories) {
            final ReposObj category = categoryRcd.getPOJO();
            
            boolean itemChecked = false;
            boolean itemEnabled = true;
            for (int j = 0; j < selectedRecords.size(); j++) {
                ReposObj obj = selectedRecords.get(j).getPOJO();
                if (j == 0) {
                    itemChecked = isInCategory(obj, category.getSelfLink());
                } else if (itemEnabled && isInCategory(obj, category.getSelfLink()) != itemChecked) {
                    itemEnabled = false;
                }
            }
            
            final boolean itemCheckedFin = itemChecked;
            final boolean itemEnabledFin = itemEnabled;
            
            menuAdapter.registerContextMenuItem(anchor, new GridContextMenuAdapter.CallBack() {
                public Item createMenuItem(List<ReposObjDataModel> selectedRecords) {
                    CheckMenuItem ret = new CheckMenuItem(category.getCaption());
                    if (itemEnabledFin) {
                        ret.setChecked(itemCheckedFin);
                    } else {
                        ret.disable();
                    }
                    return  ret;
                }
                public void menuItemSelected(List<ReposObjDataModel> records) {
                    updateCategory(gridPanel, category, records, !itemCheckedFin, categoriesStore);
                }
            });
        }
    }

    private static void updateCategory(final EditorGrid<ReposObjDataModel> gridPanel,
                                       final ReposObj category, 
                                       final List<ReposObjDataModel> records,
                                       final boolean addCategory,
                                       final Store<ReposObjDataModel> categoriesStore) {
        
        ReposObjAttribute contentAttr = category.getAttribute("content");
        for (ReposObjDataModel record : records) {
            if (addCategory) {
                contentAttr.addRefValue(record.getPOJO());
            } else {
                contentAttr.removeRefValue(ReposHandle.getRelativeLink(record.getPOJO().getSelfLink()));
            }
        }

        // Update the category and reload the store containing the records added
        // to the category.  Fixes http://jira.boston.amer.iona.com:8080/browse/REPO-306.
        // In addition, DeploymentProfiles were proving troublesome - the POST from
        // the client was sending back an update to the "active" reference - why..
        MainModule.getRemoteClient().updateReposObj(category, new AbstractUiObjCallBack() {
            @Override
            public void processReposObj(ReposObj updatedCategory) {
                // Allow listeners to realize that the category object was updated
                ReposObjChangeNotifier.INSTANCE.reposObjUpdated(updatedCategory);
                gridPanel.getStore().getLoader().load();
            }
        });
    }
    
    private static boolean isInCategory(ReposObj obj, String catLink) {
        ReposObjAttribute attr = obj.getAttribute("categories");
        if (attr != null) {
            for (String val : attr.getValues()) {
                if (catLink.contains(val)) {
                    return true;
                }
            }
        }
        return false;
    }
}
