/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.WindowEvent;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.Component;
import com.extjs.gxt.ui.client.widget.Dialog;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.iona.soa.web.repository.base.client.model.DeploymentProfileHelper;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.remote.RemoteClient;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.admin.actions.RunTaskNowAction;
import com.iona.soa.web.repository.main.client.base.AbstractUiDeletionCallBack;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.ContextActionSeparator;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ICollectionPanel;
import com.iona.soa.web.repository.main.client.base.IContextAction;
import com.iona.soa.web.repository.main.client.base.IContextActionWithId;
import com.iona.soa.web.repository.main.client.base.IContextAreaContributor;
import com.iona.soa.web.repository.main.client.base.IEditorPanel;
import com.iona.soa.web.repository.main.client.base.IManagedForm;
import com.iona.soa.web.repository.main.client.base.ISelectionProvider;
import com.iona.soa.web.repository.main.client.base.ISelectionProvider.ISelectionListener;
import com.iona.soa.web.repository.main.client.base.ISelectionSensitiveContextAction;
import com.iona.soa.web.repository.main.client.base.LinkHistoryTracker;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.ReposURL;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.BaseObjectDialogWindow.AfterCreationCallBack;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.ObjectCreationDialog;
import com.iona.soa.web.repository.main.client.base.ui.editor.custom.DeploymentProfileEditor;
import com.iona.soa.web.repository.main.client.provisioning.actions.AddMavenArtifactContextAction;
import com.iona.soa.web.repository.main.client.provisioning.actions.IndexLaterAction;
import com.iona.soa.web.repository.main.client.provisioning.actions.IndexNowAction;
import com.iona.soa.web.repository.main.client.provisioning.actions.UploadArtifactContextAction;

public class DefaultContextAreaContributor implements IContextAreaContributor {
    /*
     * Important - keep IContextAreaContributor implementations state-less.
     */
    
    public IContextAction getCreateAction(ReposURL contextURL, ICollectionPanel contextPanel) {
        if (!EClassHelper.QUERY_ECLASS.equals(contextURL.getEClass())
                && !EClassHelper.CONTAINER_ECLASS.equals(contextURL.getEClass())) {
            return new DefaultContextAreaContributor.CreateContextAction(contextURL, contextPanel);
        }
        return null;
    }
    
    public List<IContextAction> getCollectionActions(ReposURL contextURL, 
                                                     ICollectionPanel contextPanel) {
        List<IContextAction> actions = new ArrayList<IContextAction>();
        
        actions.add(new DefaultContextAreaContributor.ReloadContextAction(contextPanel));
        actions.add(new DefaultContextAreaContributor.BackContextAction());
        return actions;
    }

    public List<IContextAction> getEditorActions(ReposURL contextURL, IEditorPanel editorPanel) {
        List<IContextAction> actions = new ArrayList<IContextAction>();
        actions.add(new UndoContextAction(contextURL, editorPanel));
        DeleteContextAction deleteAction = new DeleteContextAction(contextURL, editorPanel);
        
        if (!(editorPanel instanceof DeploymentProfileEditor)) {
            if (editorPanel instanceof ISelectionProvider) {
                ((ISelectionProvider)editorPanel).addSelectionListener(deleteAction);
            }
            actions.add(deleteAction);
        }
        return actions;
    }

    public String getHelpId(ReposURL contextURL) {
        return EClassHelper.getHelpBaseContextId(contextURL.getEClass());
    }
    
    
    // TODO - redo this
    public static ListStore<ReposObjDataModel> getStoreFromContentPanel(IEditorPanel contextPanel) {
        if (contextPanel instanceof WorkspaceCollectionPanel) {
            return ((WorkspaceCollectionPanel)contextPanel).getGrid().getStore();
        }
        return null;
    }

    public static final class NullContextAreaContributor implements IContextAreaContributor {

        public IContextAction getCreateAction(ReposURL contextURL, ICollectionPanel contextPanel) {
            return null;
        }

        public List<IContextAction> getCollectionActions(ReposURL contextURL,
                                                                  ICollectionPanel contextPanel) {
            return Collections.emptyList();
        }

        public List<IContextAction> getEditorActions(ReposURL contextURL, IEditorPanel editorPanel) {
            return Collections.emptyList();
        }

        public String getHelpId(ReposURL contextURL) {
            return null;
        }
    }
    
    public static final class ArtifactContextAreaContributor extends DefaultContextAreaContributor {
        
        @Override
        public IContextAction getCreateAction(ReposURL arg0, ICollectionPanel arg1) {
            return new AddMavenArtifactContextAction(arg1);
        }
        
        @Override
        public List<IContextAction> getCollectionActions(ReposURL contextURL, 
                                                                  ICollectionPanel contextPanel) {
            List<IContextAction> actions = new ArrayList<IContextAction>();
            
            actions.add(new UploadArtifactContextAction(contextURL, contextPanel));
            actions.add(new DefaultContextAreaContributor.CreateContextAction(contextURL, contextPanel));
            
            actions.add(new DefaultContextAreaContributor.ReloadContextAction(contextPanel));
            actions.add(new DefaultContextAreaContributor.BackContextAction());

            return actions;
        }
    }
    
    public static final class DeploymentProfileContextAreaContributor extends DefaultContextAreaContributor {
        @Override
        public List<IContextAction> getCollectionActions(ReposURL contextURL, 
                                                         ICollectionPanel contextPanel) {
            List<IContextAction> actions = super.getCollectionActions(contextURL, contextPanel);
            
            actions.add(new ContextActionSeparator());
            
            PublishNowContextAction publishAction = new PublishNowContextAction(contextPanel);
            contextPanel.addSelectionListener(publishAction);
            actions.add(publishAction);

            SchedulePublishContextAction schedulePublishAction 
                = new SchedulePublishContextAction(contextPanel);
            contextPanel.addSelectionListener(schedulePublishAction);
            actions.add(schedulePublishAction);
            
            return actions;
        }
    }
    
    public static final class MavenRepoContextAreaContributor extends DefaultContextAreaContributor {
        @Override
        public List<IContextAction> getCollectionActions(ReposURL contextURL, 
                                                         ICollectionPanel contextPanel) {
            List<IContextAction> actions = super.getCollectionActions(contextURL, contextPanel);
            
            actions.add(new ContextActionSeparator());
            IndexNowAction indexAction =
                new IndexNowAction(contextPanel);
            contextPanel.addSelectionListener(indexAction);
            actions.add(indexAction);
            
            IndexLaterAction scheduleTask = new IndexLaterAction(contextPanel);
            contextPanel.addSelectionListener(scheduleTask);
            actions.add(scheduleTask);
            
            return actions;
        }
    }

    public static final class ScheduledTaskContextAreaContributor extends DefaultContextAreaContributor {
        
        @Override
        public IContextAction getCreateAction(ReposURL contextURL, ICollectionPanel contextPanel) {
            return null;
        }        
        
        @Override
        public List<IContextAction> getCollectionActions(ReposURL contextURL, 
                                                                  ICollectionPanel contextPanel) {
            List<IContextAction> actions = super.getCollectionActions(contextURL, contextPanel);
            
            actions.add(new ContextActionSeparator());
            RunTaskNowAction runNowAction = new RunTaskNowAction();
            contextPanel.addSelectionListener(runNowAction);
            actions.add(runNowAction);
            
            return actions;
        }
        
        @Override
        public List<IContextAction> getEditorActions(ReposURL contextURL, IEditorPanel editorPanel) {
            List<IContextAction> actions = super.getEditorActions(contextURL, editorPanel);
            if (editorPanel instanceof ISelectionProvider) {
                actions.add(new ContextActionSeparator());
                RunTaskNowAction runNowAction = new RunTaskNowAction();
                ((ISelectionProvider)editorPanel).addSelectionListener(runNowAction);
                actions.add(runNowAction);
            }
            return actions;
        }
    }

    // Collection Actions
    public static final class CreateContextAction extends AbstractCreateObjectAction implements IContextActionWithId {
        private ReposURL contextURL;
        private String eClass;
        private String name;
        private String icon;
        private ListStore<ReposObjDataModel> store;
        public CreateContextAction(ReposURL ctxURL, ICollectionPanel contextPanel) {
            super(contextPanel);
            contextURL = ctxURL;
            eClass = contextURL.getEClass();
            name = EClassHelper.getNameForClass(eClass);
            icon = EClassHelper.getAddIconClsForClass(eClass);
            store = DefaultContextAreaContributor.getStoreFromContentPanel(contextPanel);
        }
        public String getCaption() { return "Add " + name; }
        public String getToolTip() { return "Create a new entry for the '" + name + "' collection"; }
        public String getIconCs() { return icon; }
        public void doIt() { 
            MainModule.get().getWorkspaceControl().createReposObj(contextURL, store); 
        }
        public String getId() {
            return "create";
        }
    }

    private static final class ReloadContextAction implements IContextAction {
        private ListStore<ReposObjDataModel> store;
        public ReloadContextAction(IEditorPanel contextPanel) {
            store = DefaultContextAreaContributor.getStoreFromContentPanel(contextPanel);
        }
        public String getCaption() { return "Refresh / Reload"; }
        public String getToolTip() { return "Reload the collection"; }
        public String getIconCs() { return "refresh-icon"; }
        public void doIt() { store.getLoader().load(); }
    }

    private static final class BackContextAction implements IContextAction {
        public String getCaption() { return "Back"; }
        public String getToolTip() { return "Back in history"; }
        public String getIconCs() { return "back-icon"; }
        public void doIt() { LinkHistoryTracker.INSTANCE.walkToOlderLink(); }
    }

    // Editor Actions
    private static final class UndoContextAction implements IContextAction {
        private ReposURL objURL;
        private IManagedForm managedForm;
        UndoContextAction(ReposURL url, IEditorPanel panel) {
            objURL = url;
            managedForm = (IManagedForm)panel;
        }
        public String getCaption() { return "Undo / Reload"; }
        public String getToolTip() { return "Discard any Changes and reload the data from the editor"; }
        public String getIconCs() { return "refresh-icon"; }
        
        public void doIt() {
            Utils.showWaitDialog();
            MainModule.getRemoteClient().getReposObj(objURL.getURL(), new AbstractUiObjCallBack() {
                public void processReposObj(final ReposObj loadedObj) {
                    Utils.closeWaitDialog();
                    managedForm.setInput(loadedObj);
                }
            });
        }
    }
        
    private static final class DeleteContextAction implements ISelectionSensitiveContextAction, 
                                                              ISelectionListener {
        private Component renderedActionWidget;
        private ReposObj selected;
        
        DeleteContextAction(ReposURL url, IEditorPanel panel) {
        }
        
        public String getCaption() { return "Delete"; }
        public String getToolTip() { return "Delete the current resources from the server"; }
        public String getIconCs() { return "delete-icon"; }
        public void doIt() {
            if (selected != null) {
                delete(selected);
            }
        }
        public void setRenderedWidget(Component widget) {
            renderedActionWidget = widget;
            renderedActionWidget.disable();
        }
        
        public void onSelectionChange(List<ReposObj> selection) {
            selected = null;
            if (selection.size() == 1 && selection.get(0).isDeleteAllowed()) {
                selected = selection.get(0);
                renderedActionWidget.enable();
            } else {
                renderedActionWidget.disable();
            }
        }

        private void delete(final ReposObj toDelete) {
            Listener<WindowEvent> callback = new Listener<WindowEvent>() {
                public void handleEvent(WindowEvent we) {
                    if (Dialog.YES.equals(we.buttonClicked.getItemId())) {
                        Utils.showWaitDialog();
                        MainModule.getRemoteClient().deleteReposObj(toDelete,
                                                                    new AbstractUiDeletionCallBack() {
                                public void processSuccess() {
                                    Utils.closeWaitDialog();
                                    // Close the editor
                                    LinkHistoryTracker.INSTANCE.walkToOlderLink();
                                }
                            });
                    }
                }
            };

            String title = "Deleting \"" + toDelete.getCaption() + "\"?";
            String msg = "Are you sure you want to delete \"" + toDelete.getCaption() + "\"";
            MessageBox.confirm(title, msg, callback);
        }        
    }
    
    // TODO - all ISelectionSensitiveContextAction should extend an Abstract implementation
    //
    private static class PublishNowContextAction implements 
        ISelectionSensitiveContextAction, ISelectionListener {
        
        protected ReposObj selected;
        protected ICollectionPanel panel;
        
        private Component renderedActionWidget;
        
        PublishNowContextAction(ICollectionPanel contextPanel) {
            panel = contextPanel;
        }

        public String getCaption() {
            return "Publish now";
        }

        public String getToolTip() {
            return "Makes changes available for deployment";
        }

        public String getIconCs() {
            return "publish-now-icon";
        }

        public void doIt() {
            if (selected != null) {
                publish(selected);
            }
        }

        public void setRenderedWidget(Component widget) {
            renderedActionWidget = widget;
            renderedActionWidget.disable();
        }

        public void onSelectionChange(List<ReposObj> selection) {
            selected = null;
            if (selection.size() == 1 
                    && DeploymentProfileHelper.hasChangesForDeployment(selection.get(0))
                    && selection.get(0).isEditAllowed()) {
                selected = selection.get(0);
                renderedActionWidget.enable();
            } else {
                renderedActionWidget.disable();
            }
        }

        protected void publish(ReposObj deploymentProfile) {
            final RemoteClient client = MainModule.getRemoteClient();
            
            Utils.showWaitDialog();
            String currentRef = DeploymentProfileHelper.getCurrentVersionRef(deploymentProfile);

            client.getReposObj(ReposHandle.getAbsoluteLink(currentRef), 
                               new AbstractUiObjCallBack() {
                    @Override
                    public void processReposObj(ReposObj currentProfile) {
                        DeploymentProfileHelper.markForPublish(currentProfile);
                        
                        client.updateReposObj(currentProfile, new AbstractUiObjCallBack() {
                            @Override
                            public void processReposObj(ReposObj updatedProfile) {
                                // TODO - Only really need to reload one row - the selected row
                                getStoreFromContentPanel(panel).getLoader().load();
                                Utils.closeWaitDialog();
                            }
                        });
                    }
                });
        }
    }

    private static class SchedulePublishContextAction extends PublishNowContextAction {
        
        SchedulePublishContextAction(ICollectionPanel contextPanel) {
            super(contextPanel);
        }

        @Override
        public String getCaption() {
            return "Publish later";
        }

        @Override
        public String getToolTip() {
            return "Schedule a publish";
        }

        @Override
        public String getIconCs() {
            return "schedule-publish-icon";
        }
        
        @Override
        protected void publish(ReposObj deploymentProfile) {
            ScheduledTaskPanelHelper schedulerHelper = new ScheduledTaskPanelHelper();
            AfterCreationCallBack onExitCallBack =  new ObjectCreationDialog.AfterCreationCallBack() {
                public void onDialogClosure(ReposObj anObj) {
                    getStoreFromContentPanel(panel).getLoader().load();
                    Utils.closeWaitDialog();
                }                
            };
            schedulerHelper.fillModelModificationTaskPanel(deploymentProfile, onExitCallBack);
        }

    }
        

    
}
