/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui;

import java.util.Date;

import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.ColumnData;
import com.extjs.gxt.ui.client.widget.grid.GridCellRenderer;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;

public class GridConfig extends SimpleGridConfigurator {
    
    public static String getSelfLink(ReposObjDataModel record) {
        return record.getPOJO().getSelfLink();
    }

    public static String getCaption(ReposObjDataModel record) {
        return record.getPOJO().getCaption();
    }

    @Override
    protected void addVisibleColumns() {
        // The original behaviour of GridConfig - non "compact" mode, as in all columns are shown
        addNameColumn();
        addCustomColumns();
        addEditColumn();
        addDeleteColumn(true);
        addRssColumn();
    }
    
    protected void addCustomColumns() {
        addTimestampColumn(false);
        addLastModifiedByColumn();
    }

    protected final ColumnConfig addTimestampColumn(boolean hidden) {
        SimpleColumnConfig column = new SimpleColumnConfig(DefaultColumnIds.TIMESTAMP, 
                                                           hidden ? null : "Last edited");
        columns.add(column);
        return column;
    }

    protected final ColumnConfig addLastModifiedColumn() {
        SimpleColumnConfig column = new SimpleColumnConfig(DefaultColumnIds.LAST_MODIFIED, "Last modified") {
            @Override
            public Object getCellData(ReposObj obj) {
                String millis = obj.getAttributeValue("msTimestamp");
                return DateTimeFormat.getMediumDateTimeFormat().format(new Date(Long.parseLong(millis)));
            }
        };
        column.setWidth(150);
        columns.add(column);
        return column;
    }
    
    protected final ColumnConfig addLastModifiedByColumn() {
        SimpleColumnConfig column = new SimpleColumnConfig(DefaultColumnIds.LAST_MODIFIED_BY,
                                                           "Last modified by") {
            @Override
            public Object getCellData(ReposObj obj) {
                return obj.getAttributeValue("lastEditedBy");
            }
        };
        columns.add(column);
        return column;
    }
    
    protected final void addRssColumn() {
        RssColumnConfig column = new RssColumnConfig(DefaultColumnIds.RSS_FEED, "");
        columns.add(column);
    }
    
    private static final class RssColumnConfig extends SimpleColumnConfig {        
        RssColumnConfig(String id, String caption) {
            super(id, caption);
            setRenderer(new GridCellRenderer<ReposObjDataModel>() {
                
                public String render(ReposObjDataModel record,
                                     String property,
                                     ColumnData config,
                                     int rowIndex,
                                     int colIndex,
                                     ListStore<ReposObjDataModel> store) {
                    String feedSubscriptionUrl = record.get(property);
                    return HTML_SNIPPETS.rssColumn(feedSubscriptionUrl);
                }
            });
            setSortable(false);
            setWidth(25);
        }
        
        public Object getCellData(ReposObj obj) {
            return obj.getFeedSubscriptionLink();
        }
    }   
}
