/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * 
 */
package com.iona.soa.web.repository.main.client.base.ui;

import java.util.Date;

import com.extjs.gxt.ui.client.Style.SortDir;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.ColumnData;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.extjs.gxt.ui.client.widget.grid.GridCellRenderer;
import com.extjs.gxt.ui.client.widget.grid.GridGroupRenderer;
import com.extjs.gxt.ui.client.widget.grid.GroupColumnData;
import com.extjs.gxt.ui.client.widget.grid.GroupingView;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;

/**
 * GridConfig for a grid that wishes to group entries by date and time.
 * Sub-classes should override <code>addVisibleColumns</code> and include calls
 * to <code>addGroupByDateColumn</code> and <code>addTimeColumn</code> with its
 * own columns.
 */
public class GroupByDateTimeGridConfig extends SimpleGridConfigurator {
    
    protected static final class GroupByColumnIds {
        static final String TIME = "time";
        private GroupByColumnIds() { }
    }
    
    private final DateTimeFormat dateValueFormat = DateTimeFormat.getFormat("yyMMdd");
    
    private final DateTimeFormat dateDisplayFormat = DateTimeFormat.getFormat("E, MMM dd");
    
    private final DateTimeFormat prevYearDateDisplayFormat = DateTimeFormat.getFormat("E, MMM dd yyyy");
    
    private final DateTimeFormat timeDisplayFormat = DateTimeFormat.getFormat("HH:mm:ss");
    
    @Override
    protected void addVisibleColumns() {
        addGroupByDateColumn();
    }

    @Override
    public void configure(EditorGrid<ReposObjDataModel> grid, RemoteRestDataProxy dataProxy) {
        
        setSortInfo(DefaultColumnIds.TIMESTAMP, SortDir.DESC);
        setGroupingField(DefaultColumnIds.TIMESTAMP);
        super.configure(grid, dataProxy);
        
        grid.setHideHeaders(true);
        grid.setAutoHeight(false); // we want to limit it
        grid.setHeight(300);
        
        GroupingView view = new GroupingView();
        configure(view, grid);
        grid.setView(view);
    }

    protected void configure(GroupingView view, final Grid<ReposObjDataModel> grid) {
        view.setForceFit(true);
        view.setGroupRenderer(new GridGroupRenderer() {
            public String render(GroupColumnData data) {
                String f = grid.getColumnModel().getColumnById(data.field).getHeader();  
                String l = data.models.size() == 1 ? "Item" : "Items";  
                return f + ": " + data.group + " (" + data.models.size() + " " + l + ")"; 
            }
        });
        view.setShowGroupedColumn(false);
        view.setEnableGroupingMenu(false);
        //view.setShowGroupName(false);
        

    }
    
    protected ColumnConfig addGroupByDateColumn() {
        // We have to use the same column id as is being set as the sort column. The sort column
        // is also sent to the server in the query.
        // A kludge to get around fact that when grouping is set, sort order appears ignored.
        SimpleColumnConfig dateColumn = new SimpleColumnConfig(DefaultColumnIds.TIMESTAMP, "Date") {
            public Object getCellData(ReposObj obj) {
                String millis = obj.getAttributeValue("msTimestamp");
                return dateValueFormat.format(new Date(Long.parseLong(millis)));
            }
        };
        
        dateColumn.setRenderer(new GridCellRenderer<ReposObjDataModel>() {
            @SuppressWarnings("deprecation")
            public String render(ReposObjDataModel record,
                                 String property,
                                 ColumnData config,
                                 int rowIndex,
                                 int colIndex,
                                 ListStore<ReposObjDataModel> store) {
                
                ReposObj obj = record.getPOJO();
                String millis = obj.getAttributeValue("msTimestamp");
                Date d = new Date(Long.parseLong(millis));
                Date today = new Date(System.currentTimeMillis());
                today.setHours(0);
                today.setMinutes(0);
                today.setSeconds(0);
                if (d.before(today)) {
                    if (d.getYear() < today.getYear()) {
                        return prevYearDateDisplayFormat.format(d);                            
                    }
                    return dateDisplayFormat.format(d);
                }
                return "Today";
            }
            
        });
        columns.add(dateColumn);
        return dateColumn;
    }
    
    protected ColumnConfig addTimeColumn() {
        SimpleColumnConfig timeColumn = new SimpleColumnConfig(GroupByColumnIds.TIME, "Time") {
            public Object getCellData(ReposObj obj) {
                String millis = obj.getAttributeValue("msTimestamp");
                return timeDisplayFormat.format(new Date(Long.parseLong(millis)));
            }
        };
        timeColumn.setWidth(70);
        columns.add(timeColumn);
        return timeColumn;
    }
}