/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui;

import java.util.ArrayList;
import java.util.List;

import com.extjs.gxt.ui.client.core.El;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.FxEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.WidgetListener;
import com.extjs.gxt.ui.client.fx.FxConfig;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.BoxComponent;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.ColumnModel;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.google.gwt.dom.client.Element;
import com.iona.soa.web.repository.base.client.model.IReposObjChangeListener;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjChangeNotifier;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.main.client.base.IFilterable;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.ReposObjHelper;
import com.iona.soa.web.repository.main.client.base.ReposURL;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.RemoteRestDataProxy.RemoteRestDataReader;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator.SimpleColumnConfig;

public class RemoteCollectionPanel extends BaseCollectionPanel implements IFilterable  {
    
    protected final RemoteRestDataProxy dataProxy;
    
    private final ReposURL originalURL;
    private final List<ICollectionListener> collectionListeners = new ArrayList<ICollectionListener>();
    
    public RemoteCollectionPanel(SimpleGridConfigurator gridConfig, RemoteRestDataProxy proxy) {
        super(gridConfig);
        
        dataProxy = proxy;
        
        dataProxy.setLoadListener(new ICollectionListener() {
            public void onCollectionLoaded(ReposObjCollection collection) {
                for (ICollectionListener listener : collectionListeners) {
                    listener.onCollectionLoaded(collection);
                }
            }
        });
        
        originalURL = dataProxy.getResourceURL(); 
        if (originalURL == null) {
            throw new IllegalStateException("A URL must be set in the proxy before creating the panel");
        }
    }

    public void addCollectionListener(ICollectionListener listener) {
        collectionListeners.add(listener);
    }

    public void removeCollectionListener(ICollectionListener listener) {
        collectionListeners.remove(listener);
    }

    /**
     * IFilterable implementation
     */
    public void setFilterByQuery(String filterQuery) {
        dataProxy.setFilterQuery(filterQuery);
        gridPanel.getStore().getLoader().load();
    }
    
    /**
     * IFilterable implementation
     */
    public void setFilterByCategory(String filterCategoryURL, String categoryName) {
        if (filterCategoryURL == null) {
            // Revert to original collection url
            if (!dataProxy.getResourceURL().equals(originalURL)) {
                dataProxy.setResourceURL(originalURL);
                gridPanel.getStore().getLoader().load();
            }
        } else {
            // Alter the url used by the proxy and reload
            ReposURL filterURL = new ReposURL(filterCategoryURL 
                                              + ReposObjHelper.CATEGORY_CONTENT_URL_SUFFIX);
            if (!dataProxy.getResourceURL().equals(filterURL)) {
                dataProxy.setResourceURL(filterURL);
                dataProxy.setFilterQuery(null);
                gridPanel.getStore().getLoader().load();
            }
        }
    }
    
    
    @Override
    protected void attachStoreAndGridSynchronizer() {
        attachReposChangeListener(gridPanel, createSynchronizer());
    }

    protected StoreAndGridSynchronizer createSynchronizer() {
        return new StoreAndGridSynchronizer(gridPanel, dataProxy);
    }
    
    public static void attachReposChangeListener(BoxComponent container, 
                                                 final IReposObjChangeListener listener) {
        // Bind the remote client and the listener together
        ReposObjChangeNotifier.INSTANCE.addListener(listener);

        // Remove the binding when the container is being destroyed
        container.addWidgetListener(new WidgetListener() {
            @Override
            public void widgetDetached(ComponentEvent ce) {
                ReposObjChangeNotifier.INSTANCE.removeListener(listener);
            }
        });
    }
    
    public static class StoreSynchronizer implements IReposObjChangeListener {

        protected ListStore<ReposObjDataModel> store;

        private RemoteRestDataProxy dataProxy;
        
        private ColumnModel columnModel;
        
        public StoreSynchronizer(ListStore<ReposObjDataModel> aStore,
                                 RemoteRestDataProxy aDataProxy,
                                 ColumnModel aColumnModel) {
            store = aStore;
            dataProxy = aDataProxy;
            columnModel = aColumnModel;
        }
        
        public void reposObjCreated(ReposObj createdObj,
                                    String containingCollectionUrl,
                                    int locationIndex) {
            // Add if on page 1 and num items < page size and object class is acceptable
            // otherwise reload
            String storeUrl = ReposHandle.getRelativeLink(dataProxy.getResourceURL().getURL());
            String eventUrl = ReposHandle.getRelativeLink(containingCollectionUrl);
            
            if (storeUrl.equals(eventUrl)) {
                store.insert(RemoteRestDataReader.createModel(createdObj, columnModel),
                             locationIndex < 0 ? store.getCount() : locationIndex);
            }
            
        }

        public void reposObjDeleted(final ReposObj deletedObj) {
            ReposObjDataModel r = findRecord(deletedObj);
            if (r != null) {
                store.remove(r);
            }
        }

        public void reposObjUpdated(ReposObj updatedObj) {
            ReposObjDataModel r = findRecord(updatedObj);
            if (r != null) {
                int recordIndex = store.indexOf(r);
                store.remove(r);
                ReposObjDataModel model = new ReposObjDataModel(updatedObj);
                for (int i = 0; i < columnModel.getColumnCount(); i++) {
                    ColumnConfig cfg = columnModel.getColumn(i);
                    if (cfg instanceof SimpleColumnConfig) {
                        model.set(cfg.getId(), ((SimpleColumnConfig)cfg).getCellData(updatedObj));
                    }
                }
                store.insert(model, recordIndex);
            }
        }
        
        protected final ReposObjDataModel findRecord(ReposObj deletedObj) {
            List<ReposObjDataModel> records = store.getModels();                
            for (ReposObjDataModel record : records) {
                ReposObj obj = record.getPOJO();
                if (deletedObj.getSelfLink().equals(obj.getSelfLink())) {
                    return record;
                }
            }
            
            return null;
        }
    }
    
    public static class StoreAndGridSynchronizer extends StoreSynchronizer {
        protected Grid<ReposObjDataModel> grid;
        
        public StoreAndGridSynchronizer(Grid<ReposObjDataModel> gridPanel,
                                        RemoteRestDataProxy dataProxy) {
            super(gridPanel.getStore(), dataProxy, gridPanel.getColumnModel());
            grid = gridPanel;
        }

        @Override
        public void reposObjCreated(ReposObj createdObj, String parentCollection, int locationIndex) {
            super.reposObjCreated(createdObj, parentCollection, locationIndex);
            renderCoolHighlightingEffect(createdObj);
        }
        
        @Override
        public void reposObjDeleted(final ReposObj deletedObj) {
            renderCoolFadingEffect(deletedObj,
                                   new Listener<FxEvent>() {
                    public void handleEvent(FxEvent be) {}
                });
        }

        @Override
        public void reposObjUpdated(ReposObj updatedObj) {
            super.reposObjUpdated(updatedObj);
            grid.getView().refresh(false);
            renderCoolHighlightingEffect(updatedObj);
        }

        private void renderCoolFadingEffect(ReposObj targetedObj, Listener<FxEvent> fxCompleteListener) {
            ReposObjDataModel r = findRecord(targetedObj);
            if (r != null) {
                int index = grid.getStore().indexOf(r);
                Element rowElement = grid.getView().getRow(index);
                if (rowElement != null) {
                    El el = new El((com.google.gwt.user.client.Element)rowElement);
                    el.fadeOut(new FxConfig(fxCompleteListener));
                }
                // Now get it removed from the store 
                // needs to be done synchronously instead of in the fxCompleteListener
                // as the Listener isn't always called (Ext bug?) See REPO-812
                store.remove(r);
            }
        }

        protected void renderCoolHighlightingEffect(ReposObj targetedObj) {
            ReposObjDataModel r = findRecord(targetedObj);
            if (r != null) {
                int index = grid.getStore().indexOf(r);
                Element rowElement = grid.getView().getRow(index);
                if (rowElement != null) {
                    El el = new El((com.google.gwt.user.client.Element)rowElement);
                    Utils.highlight(el);
                }
            }
        }
    }
}
