/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui;

import java.util.ArrayList;
import java.util.List;

import com.extjs.gxt.ui.client.Style.SortDir;
import com.extjs.gxt.ui.client.data.BasePagingLoadResult;
import com.extjs.gxt.ui.client.data.DataReader;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.ColumnModel;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.google.gwt.user.client.rpc.AsyncCallback;

import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.base.client.remote.ICollectionCallBack;
import com.iona.soa.web.repository.base.client.remote.PagingInfo;
import com.iona.soa.web.repository.base.client.remote.RemoteClient;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiCollectionCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ReposURL;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.ICollectionPanel.ICollectionListener;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator.DefaultColumnIds;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator.SimpleColumnConfig;

// TODO : this class should be made to extend RemoteRestListDataProxy
public class RemoteRestDataProxy extends AbstractRemoteDataProxy<PagingLoadConfig, PagingLoadResult<ReposObjDataModel>> {
    protected ReposObjToRecordMapper reposObjMapper;   
    private ICollectionListener loadListener;
    
    public interface ReposObjToRecordMapper {
        //Record getRecordDef();
        
        Object[] createRecordData(ReposObj obj);
    }
    
    /**
     * Clients must call setResourceURL and setRecordMapper
     */
    public RemoteRestDataProxy(RemoteClient client) {
        super(client);
    }

    public static RemoteRestDataProxy createProxyForEClass(String eClass) {
        RemoteRestDataProxy dataProxy = new RemoteRestDataProxy(MainModule.getRemoteClient());
        dataProxy.setResourceURL(new ReposURL(EClassHelper.getCollectionLinkForClass(eClass)));
        return dataProxy;
    }

    public static RemoteRestDataProxy createProxyForEClass(String eClass, String emfQuery) {
        RemoteRestDataProxy dataProxy = new RemoteRestDataProxy(MainModule.getRemoteClient());
        String url = EClassHelper.getCollectionLinkForClass(eClass) + "?emfql=" + emfQuery;
        dataProxy.setResourceURL(new ReposURL(url));
        return dataProxy;
    }

    public static RemoteRestDataProxy createProxyForCollection(String collectionURL) {
        RemoteRestDataProxy dataProxy = new RemoteRestDataProxy(MainModule.getRemoteClient());
        dataProxy.setResourceURL(new ReposURL(collectionURL));
        return dataProxy;
    }

    public final RemoteRestDataProxy setRecordMapper(ReposObjToRecordMapper mapper) {
        reposObjMapper = mapper;
        return this;
    }

    public void setLoadListener(ICollectionListener listener) {
        loadListener = listener;
    }

    public void load(DataReader<PagingLoadConfig, PagingLoadResult<ReposObjDataModel>> reader,
                     PagingLoadConfig loadConfig,
                     AsyncCallback<PagingLoadResult<ReposObjDataModel>> callback) {

        Utils.showWaitDialog();
        
        if (getSortInfo() != null) {
            loadConfig.setSortInfo(getSortInfo());
            setSortInfo(null);
        }
        
        String sort = loadConfig.getSortInfo().getSortField();
        sort = sort == null ? PagingInfo.SORT_BY_TITLE : sort;
        sort = DefaultColumnIds.NAME.equals(sort) ? PagingInfo.SORT_BY_TITLE : sort;
        
        SortDir sortDir = loadConfig.getSortInfo().getSortDir();
        boolean isAscending = sortDir == null ? true : (SortDir.DESC == sortDir ? false : true);            
        
        boolean actuallyPaged = loadConfig.getLimit() == 0;
        PagingInfo pagingData = new PagingInfo(actuallyPaged
                                                   ? 0
                                                   : loadConfig.getOffset() / loadConfig.getLimit(),
                                               actuallyPaged ? -1 : loadConfig.getLimit(),
                                               sort,
                                               isAscending);
        
        getRemoteClient().getReposObjList(getResourceURL().getURL(), 
                                          getFilterQuery(), 
                                          pagingData, 
                                          getLoadingCallBack(reader, callback));
    }

    protected ICollectionCallBack getLoadingCallBack(
            final DataReader<PagingLoadConfig, PagingLoadResult<ReposObjDataModel>> reader,
            final AsyncCallback<PagingLoadResult<ReposObjDataModel>> callback) {
        
        return new AbstractUiCollectionCallBack() {
            public void processReposObjCollection(ReposObjCollection queriedList) {
                try {
                    PagingLoadResult<ReposObjDataModel> loadResult;
                    
                    if (reader != null) {
                        loadResult = reader.read(null, queriedList);
                        
                    } else {
                        
                        List<ReposObj> allReposObjs = queriedList.getTheReposObjs();
                        List<ReposObjDataModel> gridData =
                            new ArrayList<ReposObjDataModel>(allReposObjs.size());
                        for (ReposObj aReposObj : allReposObjs) {
                            gridData.add(new ReposObjDataModel(aReposObj));
                        }
                        
                        loadResult = new BasePagingLoadResult<ReposObjDataModel>(gridData);
                    }
                    
                    loadResult.setOffset(queriedList.getPaginationStartIndex());
                    loadResult.setTotalLength(queriedList.getTotalSize());
                    callback.onSuccess(loadResult);
                
                    if (loadListener != null) {
                        loadListener.onCollectionLoaded(queriedList);
                    }
                } finally {
                    Utils.closeWaitDialog();
                }
            }
        };
    }
    
    public static class RemoteRestDataReader implements DataReader<PagingLoadConfig,
                                                        PagingLoadResult<ReposObjDataModel>> {
        
        Grid<ReposObjDataModel> gridPanel;
        
        public RemoteRestDataReader(Grid<ReposObjDataModel> aGrid) {
            gridPanel = aGrid;
        }
        
        public PagingLoadResult<ReposObjDataModel> read(PagingLoadConfig loadConfig, Object data) {
            
            List<ReposObj> allReposObjs = ((ReposObjCollection)data).getTheReposObjs();
            List<ReposObjDataModel> gridData = new ArrayList<ReposObjDataModel>(allReposObjs.size());
            for (ReposObj aReposObj : allReposObjs) {
                gridData.add(createModel(aReposObj, gridPanel.getColumnModel()));
            }

            return new BasePagingLoadResult<ReposObjDataModel>(gridData);
        }

        public static ReposObjDataModel createModel(ReposObj aReposObj, ColumnModel columnModel) {
            ReposObjDataModel model = new ReposObjDataModel(aReposObj);
            
            for (int i = 0; columnModel != null && i < columnModel.getColumnCount(); i++) {
                ColumnConfig cfg = columnModel.getColumn(i);
                if (cfg instanceof SimpleColumnConfig) {
                    Object cellData = ((SimpleColumnConfig)cfg).getCellData(aReposObj);
                    if (cellData != null) {
                        model.set(cfg.getId(), cellData);
                    }
                }
            }
            return model;
        }
    }
}
