/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui;

import java.util.ArrayList;
import java.util.List;

import com.extjs.gxt.ui.client.Style.SortDir;
import com.extjs.gxt.ui.client.data.BaseListLoadResult;
import com.extjs.gxt.ui.client.data.BasePagingLoadResult;
import com.extjs.gxt.ui.client.data.DataReader;
import com.extjs.gxt.ui.client.data.ListLoadConfig;
import com.extjs.gxt.ui.client.data.ListLoadResult;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.base.client.remote.ICollectionCallBack;
import com.iona.soa.web.repository.base.client.remote.PagingInfo;
import com.iona.soa.web.repository.base.client.remote.RemoteClient;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiCollectionCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ReposURL;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator.DefaultColumnIds;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator.SimpleColumnConfig;

public class RemoteRestListDataProxy extends AbstractRemoteDataProxy<ListLoadConfig, ListLoadResult<ReposObjDataModel>> {
    /**
     * Clients must call setResourceURL and setRecordMapper
     */
    public RemoteRestListDataProxy(RemoteClient client) {
        super(client);
    }
    
    public static RemoteRestListDataProxy createProxyForEClass(String eClass) {
        RemoteRestListDataProxy dataProxy = new RemoteRestListDataProxy(MainModule.getRemoteClient());
        dataProxy.setResourceURL(new ReposURL(EClassHelper.getCollectionLinkForClass(eClass)));
        return dataProxy;
    }

    public static RemoteRestListDataProxy createProxyForCollection(String collectionURL) {
        RemoteRestListDataProxy dataProxy = new RemoteRestListDataProxy(MainModule.getRemoteClient());
        dataProxy.setResourceURL(new ReposURL(collectionURL));
        return dataProxy;
    }

    public void load(DataReader<ListLoadConfig, ListLoadResult<ReposObjDataModel>> reader,
                     ListLoadConfig loadConfig,
                     AsyncCallback<ListLoadResult<ReposObjDataModel>> callback) {

        Utils.showWaitDialog();
        
        if (getSortInfo() != null) {
            loadConfig.setSortInfo(getSortInfo());
            setSortInfo(null);
        }
        
        String sort = loadConfig.getSortInfo().getSortField();
        sort = sort == null ? PagingInfo.SORT_BY_TITLE : sort;
        sort = DefaultColumnIds.NAME.equals(sort) ? PagingInfo.SORT_BY_TITLE : sort;
        
        SortDir sortDir = loadConfig.getSortInfo().getSortDir();
        boolean isAscending = sortDir == null ? true : (SortDir.DESC == sortDir ? false : true);            
        
        PagingInfo pagingData = new PagingInfo(0, -1, sort, isAscending);
        
        getRemoteClient().getReposObjList(getResourceURL().getURL(), 
                                          getFilterQuery(), 
                                          pagingData, 
                                          getLoadingCallBack(reader, callback));
    }

    protected ICollectionCallBack 
    getLoadingCallBack(final DataReader<ListLoadConfig, 
                       ListLoadResult<ReposObjDataModel>> reader,
                       final AsyncCallback<ListLoadResult<ReposObjDataModel>> callback) {
        
        return new AbstractUiCollectionCallBack() {
            public void processReposObjCollection(ReposObjCollection queriedList) {
                try {
                    ListLoadResult<ReposObjDataModel> loadResult;
                    
                    if (reader != null) {
                        loadResult = reader.read(null, queriedList);
                        
                    } else {
                        
                        List<ReposObj> allReposObjs = queriedList.getTheReposObjs();
                        List<ReposObjDataModel> gridData =
                            new ArrayList<ReposObjDataModel>(allReposObjs.size());
                        for (ReposObj aReposObj : allReposObjs) {
                            gridData.add(new ReposObjDataModel(aReposObj));
                        }
                        
                        loadResult = new BaseListLoadResult<ReposObjDataModel>(gridData);
                    }
                    
                    callback.onSuccess(loadResult);
                } finally {
                    Utils.closeWaitDialog();
                }
            }
        };
    }

    public static class RemoteRestDataReader implements DataReader<ListLoadConfig,
                                                            ListLoadResult<ReposObjDataModel>> {
        
        Grid<ReposObjDataModel> gridPanel;
        
        public RemoteRestDataReader(Grid<ReposObjDataModel> aGrid) {
            gridPanel = aGrid;
        }
        
        public ListLoadResult<ReposObjDataModel> read(ListLoadConfig loadConfig, Object data) {
            
            List<ReposObj> allReposObjs = ((ReposObjCollection)data).getTheReposObjs();
            List<ReposObjDataModel> gridData = new ArrayList<ReposObjDataModel>(allReposObjs.size());
            for (ReposObj aReposObj : allReposObjs) {
                ReposObjDataModel model = new ReposObjDataModel(aReposObj);
                
                for (int i = 0; i < gridPanel.getColumnModel().getColumnCount(); i++) {
                    ColumnConfig cfg = gridPanel.getColumnModel().getColumn(i);
                    if (cfg instanceof SimpleColumnConfig) {
                        model.set(cfg.getId(), ((SimpleColumnConfig)cfg).getCellData(aReposObj));
                    }
                }
                
                gridData.add(model);
            }

            return new BasePagingLoadResult<ReposObjDataModel>(gridData);
        }
    }
}
