/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui;

import com.google.gwt.core.client.GWT;
import com.iona.soa.web.repository.base.client.model.DeploymentProfileHelper;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.UIMessages;
import com.iona.soa.web.repository.main.client.base.AbstractUiCollectionCallBack;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.ui.BaseObjectDialogWindow.AfterCreationCallBack;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.ObjectCreationDialog;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.panels.ScheduledTaskPanel.ScheduledTaskDefaults;
import com.iona.soa.web.repository.main.client.base.ui.editor.ObjectEditDialog;

public final class ScheduledTaskPanelHelper {

    public ScheduledTaskPanelHelper() {
    }
    
    static public void openEditScheduledTaskWindow(
            final ReposObj prototypeObj,
            final AfterCreationCallBack onExitCallBack) {
        
        ScheduledTaskDefaults defaults = new ScheduledTaskDefaults();
        defaults.setTaskPrototypeObj(prototypeObj);
        
        BaseObjectDialogWindow dialog = new ObjectEditDialog(EClassHelper.SCHEDULED_TASK_ECLASS,
                onExitCallBack, defaults);

        dialog.setTitle(defaults.getTaskName());
        dialog.show();
    }
    
    static public void openEditModelModificationTaskWindow(
            final ReposObj deploymentProfile,
            final AfterCreationCallBack onExitCallBack,
            final ReposObj scheduleObject) {
        if (scheduleObject == null) {
            openPrototypeModelModTaskWindow(deploymentProfile, onExitCallBack);
            return;
        } else {
            openEditScheduledTaskWindow(scheduleObject, onExitCallBack);
        }
    }
    

    public void fillModelModificationTaskPanel(final ReposObj deploymentProfile, final AfterCreationCallBack onExitCallBack) {

        MainModule.getRemoteClient().getReposObjList(ReposHandle.getAbsoluteLink("onceofftasks"),
            null,
            null,
            new AbstractUiCollectionCallBack() {
                @Override
                public void processReposObjCollection(ReposObjCollection list) {
                    boolean foundRelatedObject = false;
                    ReposObj returnedObj = null;
                    for (ReposObj overrideObj : list.getTheReposObjs()) {
                        
                        ReposObjAttribute paramAttr = overrideObj.getAttribute("parameters");
                        String deploymentProfileGuid = deploymentProfile.getAttributeValue("guid");
                        String values[] = paramAttr.getValues();
                        for (String guid : values){
                            if (guid.contains(deploymentProfileGuid)) {
                                returnedObj = overrideObj;
                                foundRelatedObject = true;
                                openEditModelModificationTaskWindow(deploymentProfile,
                                        onExitCallBack, returnedObj);
                                break;
                            }
                        }
                    }
                    if (!foundRelatedObject){
                        openPrototypeModelModTaskWindow(deploymentProfile,
                                    onExitCallBack);
                        }
                }
    
    
            }
        );
    }
    
    static private ScheduledTaskDefaults getScheduledTaskDefaults(ReposObj currentObj, ReposObj taskPrototypeObj,
            AfterCreationCallBack onExitCallBack) {


        String[] parameters = new String[7];
        ReposObjAttribute parameterListAttr = taskPrototypeObj.getAttribute("parameters");
        if (parameterListAttr != null && parameterListAttr.getValues().length  == parameters.length) {
            String[] paramList = parameterListAttr.getValues();
            int i = 0;
            for (String param : paramList) {
                parameters[i] = param;
                i++;
            }
        } else {
            if (currentObj.getEClass().equals(EClassHelper.DEPLOYMENT_PROFILE_ECLASS)) {
                // probably not going to go in here 
                parameters[3] = currentObj.getAttributeValue("guid"); 
                parameters[6] = DeploymentProfileHelper.getCurrentVersionGuid(currentObj);        
            }
            
            if (currentObj.getEClass().equals(EClassHelper.VERSIONED_PROFILE_ECLASS)) {
                // work out how to get the guid of the parent deployment profile
                parameters[3] = currentObj.getAttributeValue("parentGuid");
                parameters[6] = currentObj.getAttributeValue("guid");        
            }
    
            parameters[0] = "SET";
            parameters[1] = "http://com.iona.soa.model";
            parameters[2] = "DeploymentProfile";
            parameters[4] = "REFERENCE";
            parameters[5] = "active";
        }
        UIMessages messages = GWT.create(UIMessages.class);
        String taskName = messages.defaultSchedulePublishTaskName(currentObj.getAttributeValue("name"));
        ScheduledTaskDefaults defaults = new ScheduledTaskDefaults()
            .setTaskType("com.iona.repository.scheduler.taskservice.ModelModificationTaskService")
            .setTaskParameters(parameters)
            .setTaskName(taskName)
            .setTaskPrototypeObj(taskPrototypeObj);
        return defaults;

    }
    
    static private void openNewSchedulerWindow(
            final ReposObj deploymentProfile,
            final AfterCreationCallBack onExitCallBack,
            final ReposObj prototypeObj) {
        ScheduledTaskDefaults defaults = getScheduledTaskDefaults(deploymentProfile, prototypeObj, onExitCallBack);
        BaseObjectDialogWindow dialog = new ObjectCreationDialog(EClassHelper.SCHEDULED_TASK_ECLASS,
                onExitCallBack, defaults);
        dialog.setTitle(defaults.getTaskName());
        dialog.show();
    }
    
    static private void openPrototypeModelModTaskWindow(
            final ReposObj deploymentProfile,
            final AfterCreationCallBack onExitCallBack) {
        String url = EClassHelper.getCollectionLinkForClass(EClassHelper.SCHEDULED_ONCEOFF_TASK_ECLASS)
            + "/prototype"; // prototype id
        MainModule.getRemoteClient().getReposObj(url, new AbstractUiObjCallBack() {
            public void processReposObj(final ReposObj prototypeObj) {
                UIMessages messages = GWT.create(UIMessages.class);
                String taskName = messages.defaultSchedulePublishTaskName(deploymentProfile.getAttributeValue("name"));
                prototypeObj.getAttribute("name").setSimpleValue(taskName);
                openNewSchedulerWindow(deploymentProfile,
                    onExitCallBack, prototypeObj);
            }
        });
    }
    



}