/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui;

import java.util.ArrayList;
import java.util.List;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.Style.HorizontalAlignment;
import com.extjs.gxt.ui.client.Style.SortDir;
import com.extjs.gxt.ui.client.data.BasePagingLoadConfig;
import com.extjs.gxt.ui.client.data.BasePagingLoader;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.extjs.gxt.ui.client.data.PagingLoader;
import com.extjs.gxt.ui.client.data.SortInfo;
import com.extjs.gxt.ui.client.event.BaseEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.store.GroupingStore;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.PagingToolBar;
import com.extjs.gxt.ui.client.widget.form.TextField;
import com.extjs.gxt.ui.client.widget.grid.CellEditor;
import com.extjs.gxt.ui.client.widget.grid.CheckBoxSelectionModel;
import com.extjs.gxt.ui.client.widget.grid.CheckColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.ColumnData;
import com.extjs.gxt.ui.client.widget.grid.ColumnModel;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.extjs.gxt.ui.client.widget.grid.GridCellRenderer;
import com.extjs.gxt.ui.client.widget.grid.GridSelectionModel;
import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.main.client.HtmlSnippets;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.RemoteRestDataProxy.RemoteRestDataReader;

public class SimpleGridConfigurator {

    public final class DefaultColumnIds {
        
        public static final String NAME = "caption";
        public static final String TIMESTAMP = "timestamp";
        public static final String LAST_MODIFIED = "lastModified";
        public static final String LAST_MODIFIED_BY = "lastModifiedBy";
        public static final String RSS_FEED = "rss";
        public static final String SELF_LINK = "selfLink";
        public static final String ECLASS = "eclass";
        public static final String POJO = "pojo";
        public static final String EDIT = "edit";
        public static final String DELETE = "delete";
        
        private DefaultColumnIds() { }
    }

    protected static final HtmlSnippets HTML_SNIPPETS = GWT.create(HtmlSnippets.class);

    static final int PAGINATION_PAGE_SIZE = 15;

    protected List<SimpleColumnConfig> columns = new ArrayList<SimpleColumnConfig>();
    protected EditorGrid<ReposObjDataModel> gridPanel;
    private String groupColumnId;
    private SortInfo defaultSortInfo = new SortInfo(DefaultColumnIds.NAME, SortDir.ASC);


    
    /**
     * One of the configure methods must be called after using this constructor
     */
    public SimpleGridConfigurator() {
    }

    /**
     * If the grid is to use grouping, set the column id before calling configure.
     * 
     * @use GroupingView::setHideGroupedColumn to hide the data in the grouping column
     * if required.
     * 
     * @param columnId that the data will be grouped using.
     */
    public final void setGroupingField(String columnId) {
        groupColumnId = columnId;
    }
    
    public final void setSortInfo(String columnId, SortDir dir) {
        defaultSortInfo = new SortInfo(columnId, dir);
    }
    
    public void configure(EditorGrid<ReposObjDataModel> grid,
                          RemoteRestDataProxy dataProxy) {
        configure(grid, dataProxy, false);
    }

    public void configure(EditorGrid<ReposObjDataModel> grid,
                          RemoteRestDataProxy dataProxy,
                          boolean includeCheckboxColumn) {
        if (gridPanel != null) {
            return;
        }
        gridPanel = grid;
        
        gridPanel.setAutoWidth(true);
        gridPanel.setAutoHeight(true);
        
        gridPanel.setBorders(false);  
        gridPanel.setHideHeaders(false);
        gridPanel.setIntStyleAttribute("margins", 0);
        
        // Some columns rely on the gridPanel being set so they can attach grid cell
        // listeners
        addVisibleColumns();
        addHiddenColumns();

        boolean expandColSet = false;
        for (SimpleColumnConfig column : columns) {
            if (column.getId().equals(getExpandableColumnId())) {
                expandColSet = true;
                gridPanel.setAutoExpandColumn(getExpandableColumnId());
                break;
            }
        }
        if (!expandColSet && !columns.isEmpty()) {
            gridPanel.setAutoExpandColumn(columns.get(0).getId());
        }
        
        gridPanel.reconfigure(prepareStore(dataProxy),
                              prepareColumnModel(includeCheckboxColumn));
    }

    public String getExpandableColumnId() {
        return GridConfig.DefaultColumnIds.NAME;
    }

    /**
     * Note that the store to be set on the gridPanel first before calling this method.
     * 
     * @param gp
     * @param entriesEClass
     * @return
     */
    public final PagingToolBar attachPagingToolbar(Grid<ReposObjDataModel> gp,
                                                   ContentPanel gridContainer,
                                                   String entriesEClass) {
        return attachPagingToolbar(gp,
                                   gridContainer,
                                   entriesEClass,
                                   SimpleGridConfigurator.PAGINATION_PAGE_SIZE);
    }
    
    /**
     * Note that the store to be set on the gridPanel first before calling this method.
     * 
     * @param gp
     * @param entriesEClass
     * @return
     */
    @SuppressWarnings("unchecked")
    public final PagingToolBar attachPagingToolbar(Grid<ReposObjDataModel> gp, 
                                                   ContentPanel gridContainer,
                                                   String entriesEClass,
                                                   int pagingSize) {
        final PagingToolBar pagingToolbar = new PagingToolBar(pagingSize);
        pagingToolbar.bind((PagingLoader<BasePagingLoadConfig>)gp.getStore().getLoader());
        
        
        // TODO - move to snippets
        final String entriesLabel = EClassHelper.getPluralNameForClass(entriesEClass);
        pagingToolbar.getMessages().setEmptyMsg("No " + entriesLabel + " found");        
        pagingToolbar.getMessages().setDisplayMsg(entriesLabel + " {0} to {1} out of {2}");
        gridContainer.setTopComponent(pagingToolbar);
        
        return pagingToolbar;
    }
    
    public final EditorGrid<ReposObjDataModel> getGrid() {
        return gridPanel;
    }

    protected void addVisibleColumns() {
        addNameColumn();
    }
    
    protected ColumnConfig addNameColumn() {
        return addNameColumn(true);
    }

    protected final CaptionColumnConfig addNameColumn(boolean displayImage) {
        CaptionColumnConfig column = new CaptionColumnConfig(getGrid(),
                                                             DefaultColumnIds.NAME,
                                                             "Name",
                                                             displayImage);
        
        columns.add(column);
        return column;
    }

    protected ActionColumnConfig<ReposObjDataModel> addDeleteColumn(final boolean askConfirmation) {
        ActionColumnConfig.ActionColumnCallback<ReposObjDataModel> callback =
            new ActionColumnConfig.ActionColumnCallback<ReposObjDataModel>() {
                public void actionClicked(ReposObjDataModel record, Element anchorElement) {
                    MainModule.get().getWorkspaceControl().deleteObjects(new ReposObj[] {record.getPOJO()},
                                                                         null,
                                                                         askConfirmation);
                }
            };  
        
        ActionColumnConfig<ReposObjDataModel> column =
            new ActionColumnConfig<ReposObjDataModel>(getGrid(), DefaultColumnIds.DELETE) {
                @Override
                protected String getActionLabel(ReposObjDataModel record) {
                    ReposObj obj = record.getPOJO();
                    return obj.getSelfLink() == null || !obj.isDeleteAllowed() 
                           ? null : super.getActionLabel(record);
                }
            };
        
        column.setActionLabel("Delete").setActionCallback(callback);
        column.setResizable(true);
        column.setWidth(50);
        
        columns.add(column);
        return column;
    }
    
    protected ActionColumnConfig<ReposObjDataModel> addEditColumn() {
        ActionColumnConfig.ActionColumnCallback<ReposObjDataModel> callback =
            new ActionColumnConfig.ActionColumnCallback<ReposObjDataModel>() {
                public void actionClicked(ReposObjDataModel record, Element anchorElement) {
                    MainModule.get().getWorkspaceControl().openLink(record.getPOJO().getSelfLink(), true);
                }
            };
        
        ActionColumnConfig<ReposObjDataModel> column =
            new ActionColumnConfig<ReposObjDataModel>(getGrid(), DefaultColumnIds.EDIT) {
            protected String getActionLabel(ReposObjDataModel record) {
                ReposObj obj = record.getPOJO();
                return obj.isEditAllowed() ? "Edit" : "View";
            }
        };
        column.setActionCallback(callback);
        
        columns.add(column);
        return column;
    }
    
    protected void addHiddenColumns() {
        addSelfLinkColumn();
        addObjectTypeColumn();
        addPojoColumn();        
    }

    protected final void addPojoColumn() {
        columns.add(new SimpleColumnConfig(DefaultColumnIds.POJO, null));
    }

    protected final void addObjectTypeColumn() {
        columns.add(new SimpleColumnConfig(DefaultColumnIds.ECLASS, null));
    }

    protected final void addSelfLinkColumn() {
        columns.add(new SimpleColumnConfig(DefaultColumnIds.SELF_LINK, null));
    }

    private ColumnModel prepareColumnModel(boolean usingCheckboxes) {
        List<ColumnConfig> visibleColumns = new ArrayList<ColumnConfig>();
    
        if (usingCheckboxes) {
            visibleColumns.add(new CheckColumnConfig());
            gridPanel.setSelectionModel(new CheckBoxSelectionModel<ReposObjDataModel>());  
        } else {
            gridPanel.setSelectionModel(new GridSelectionModel<ReposObjDataModel>());
        }
        
        for (SimpleColumnConfig column : columns) {
            if (!column.isHidden()) {
                visibleColumns.add(column);
            }
        }
        
        return new ColumnModel(visibleColumns);
    }

    private ListStore<ReposObjDataModel> prepareStore(RemoteRestDataProxy dataProxy) {
        
        if (dataProxy == null) {
            return new ListStore<ReposObjDataModel>();
        }
        
        if (dataProxy instanceof RemoteRestDataProxy) {
            // setting on store doesn't seem to work
            ((RemoteRestDataProxy)dataProxy).setSortInfo(defaultSortInfo);
        }
        
        RemoteRestDataReader reader = new RemoteRestDataReader(gridPanel);
        
        PagingLoader<PagingLoadConfig> loader =
            new BasePagingLoader<PagingLoadConfig, PagingLoadResult<ReposObjDataModel>>(dataProxy, reader);  
        loader.setRemoteSort(true);  
        loader.setOffset(PAGINATION_PAGE_SIZE);
        loader.setSortDir(defaultSortInfo.getSortDir());
        loader.setSortField(defaultSortInfo.getSortField());
        //loader.setLimit(0);
        
        ListStore<ReposObjDataModel> store = null;
        if (groupColumnId != null) {
            store = new GroupingStore<ReposObjDataModel>(loader);
            ((GroupingStore<ReposObjDataModel>)store).groupBy(groupColumnId);
        } else {
            store = new ListStore<ReposObjDataModel>(loader);
        }
        
        return store;
    }
    
    public static class SimpleColumnConfig extends ColumnConfig {
        static final int DEFAULT_COLUMN_WIDTH = 120;
        
        public SimpleColumnConfig(String id, String caption) {
            this(id, caption, DEFAULT_COLUMN_WIDTH);
            setDataIndex(id);
            setHidden(caption == null);
        }
        
        public SimpleColumnConfig(String id, String caption, int width) {
            super(id, caption, width);
            setSortable(true);
            setMenuDisabled(true);
            setDataIndex(id);

            setHidden(caption == null);
            
            //setCss("vertical-align:middle;");
        }
        
        public Object getCellData(ReposObj obj) {
            // by default expect the models to be populated already...
            return null;
        }

    }

    /**
     * Assumes that a ReposObj has an attribute with the supplied column id.
     * The value of that attribute is displayed.  
     */
    public static class AttributeColumnConfig extends SimpleColumnConfig {
        public AttributeColumnConfig(String id, String caption) {
            super(id, caption, DEFAULT_COLUMN_WIDTH);
        }
        
        public AttributeColumnConfig(String id, String caption, int width) {
            super(id, caption, width);
        }
        
        public final Object getCellData(ReposObj obj) {
            return obj.getAttribute(getId()).getValue();
        }

    }
    
    protected static final class CaptionColumnConfig extends ActionColumnConfig<ReposObjDataModel> {
        
        private boolean showImage; 
        
        CaptionColumnConfig(Grid<ReposObjDataModel> grid, String id, String caption, boolean displayImage) {
            super(grid, id);
            setHeader(caption);
            showImage = displayImage;
            
            setAlignment(HorizontalAlignment.LEFT);

            ActionColumnConfig.ActionColumnCallback<ReposObjDataModel> callback =
                new ActionColumnConfig.ActionColumnCallback<ReposObjDataModel>() {
                    public void actionClicked(ReposObjDataModel record, Element anchorElement) {
                        String absoluteLink = ReposHandle.getAbsoluteLink(record.getPOJO().getSelfLink());
                        MainModule.get().getWorkspaceControl().openLink(absoluteLink, true);
                    }
                };
            setActionCallback(callback);
            setSortable(true);
            
            final TextField<String> tf = new TextField<String>();
            tf.addListener(Events.Focus, new Listener<BaseEvent>() {
                public void handleEvent(BaseEvent be) {
                    tf.selectAll();
                }
            });
            setEditor(new CellEditor(tf));  
        }
        
        @Override
        protected GridCellRenderer<ReposObjDataModel> createRenderer() {
            return new GridCellRenderer<ReposObjDataModel>() {
                
                public String render(ReposObjDataModel record,
                                     String property,
                                     ColumnData config,
                                     int rowIndex,
                                     int colIndex,
                                     ListStore<ReposObjDataModel> store) {
                        
                    String caption = record.get(ReposObjDataModel.FIELD_CAPTION);
                    if (showImage) {
                        String eClass = record.get(ReposObjDataModel.FIELD_ECLASS);
                        String imageUrl = EClassHelper.getIconFileForClass(eClass);
                        return HTML_SNIPPETS.captionColumnWithImage(imageUrl,
                                                                    getAnchorTagCssClass(),
                                                                    caption);
                    }
                    
                    return HTML_SNIPPETS.captionColumnNoImage(getAnchorTagCssClass(), 
                                                              caption);
                }

                
            };
        }
            
        public Object getCellData(ReposObj obj) {
            return obj.getCaption() == null ? "<unspecified>" : obj.getCaption();
        }
    }
}