/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.XDOM;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.BoxComponent;
import com.extjs.gxt.ui.client.widget.Component;
import com.extjs.gxt.ui.client.widget.Window;
import com.extjs.gxt.ui.client.widget.Window.CloseAction;
import com.extjs.gxt.ui.client.widget.grid.ColumnData;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.extjs.gxt.ui.client.widget.grid.GridCellRenderer;
import com.extjs.gxt.ui.client.widget.layout.FitLayout;
import com.extjs.gxt.ui.client.widget.tips.ToolTip;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;

public final class ToolTipActionColumnConfig extends ActionColumnConfig<ReposObjDataModel> {
    
    public interface ToolTipActionColumnProvider {
        boolean actionRequired(ReposObjDataModel record);
        
        String getActionLabel();
        
        ActionColumnCallback<ReposObjDataModel> getActionCallback();
        
        int getToolTipContentsWidthHint();
        
        BoxComponent createToolTipContents(ReposObjDataModel record);
    }
    
    public abstract static class AbstractToolTipActionColumnProviderAdapter
            implements ToolTipActionColumnProvider {
        
        public ActionColumnCallback<ReposObjDataModel> getActionCallback() {
            return new ActionColumnCallback<ReposObjDataModel>() {
                public void actionClicked(ReposObjDataModel record,
                                          com.google.gwt.dom.client.Element anchorElement) {
                    BoxComponent contents = createToolTipContents(record);
                    contents.setAutoHeight(true);
                    contents.setAutoWidth(true);
                    
                    openWindow(anchorElement, 
                               record.getPOJO().getCaption() + ": " + getActionLabel(), 
                               contents);
                }
            };
        }

        private void openWindow(com.google.gwt.dom.client.Element anchorElement,
                                String title,
                                BoxComponent panel) {
            final Window window = new Window();
            window.setAutoHide(true);
            window.setClosable(false);
            window.setFrame(true);
            window.setModal(false);
            window.setCloseAction(CloseAction.CLOSE);
            window.setHeading(title);
            window.setHeaderVisible(true);
            window.setResizable(true);
            
            window.setLayout(new FitLayout());

            
            window.add(panel);
            window.setWidth(getToolTipContentsWidthHint());
            
            // TODO support for the anchor element?
            window.show();
            window.alignTo((com.google.gwt.user.client.Element)anchorElement,
                           "br",
                           new int[] {0 - getToolTipContentsWidthHint(), 0});
        }
    }
    
    private ToolTipActionColumnProvider provider;
    
    public ToolTipActionColumnConfig(Grid<ReposObjDataModel> grid,
                                     String columnId) {
        super(grid, columnId);
    }
    
    /**
     * Calling this method (required) negates the need to call setActionLabel and setActionCallback
     * in the base class.
     *  
     * @param actionProvider
     */
    public void setActionProvider(ToolTipActionColumnProvider actionProvider) {
        provider = actionProvider;
        
        setActionLabel(provider.getActionLabel());
        setActionCallback(provider.getActionCallback());
        setWidth((provider.getActionLabel().length() * 8) + 16); // allow for toolTip
    }
    
    @Override
    protected GridCellRenderer<ReposObjDataModel> createRenderer() {
        return new ToolTipActionColumnRenderer();
    }
    
    private final class ToolTipActionColumnRenderer implements GridCellRenderer<ReposObjDataModel> {
        
        public String render(final ReposObjDataModel record,
                             String property,
                             ColumnData config,
                             int rowIndex,
                             int colIndex,
                             ListStore<ReposObjDataModel> store) {
            if (record == null || !provider.actionRequired(record)) {
                return "";
            }
            
            final String imgCssId = getTooltipImgCssId(record);
            
            DeferredCommand.addCommand(new Command() {
                public void execute() {
                    createTooltip(imgCssId, record);
                }
            });
            
            StringBuffer sb = new StringBuffer();
            sb.append(snippets.anchorTagWithCssCls(provider.getActionLabel(),
                                                   getAnchorTagCssClass()));
            sb.append(snippets.tooltipImage(imgCssId));
            return sb.toString();
        }
        
        private ToolTip createTooltip(String relativeCssId, final ReposObjDataModel record) {
           
            Component anchorCpt = new Component() { };
            anchorCpt.setElement(XDOM.getElementById(relativeCssId));
            
            ToolTip t = new ToolTip(anchorCpt) {
                @Override
                public void handleEvent(ComponentEvent ce) {
                    int eventId = ce.type;
                    
                    /*if (Events.Render == eventId) {
                        onRender(ce.component);
                    } else if (Events.Render == eventId) {
                        onRender(ce.component);
                    } else */
                    if (Events.Show == eventId) {
                        onShow(ce.component);
                    } else if (Events.Hide == eventId) {
                        onHide(ce.component);
                    } else {
                        super.handleEvent(ce);
                    }
                }
                
                
                /*public void onRender(final Component component) {
                    // Seems to be only done once
                    component.addClassOnOver("hasMouseFocus", true);
                    component.el().addListener("mouseout", new EventCallback() {
                        public void execute(EventObject e) {
                            component.hide();
                        }
                    });
                }

                public boolean doBeforeHide(final Component component) {
                    if (component.el().hasStyleName("hasMouseFocus")) {
                        return false;
                    }
                    return true;
                }*/
                
                public void onShow(Component component) {
                    ToolTip tooltip = (ToolTip)component;
                    BoxComponent contents = provider.createToolTipContents(record);
                    contents.setAutoHeight(true);
                    contents.setAutoWidth(true);
                    tooltip.add(contents);
                    tooltip.layout();
                }
                
                public void onHide(Component component) {
                    ToolTip t = (ToolTip)component;
                    t.removeAll(); // destroys contents, re-created in onShow
                }
            };
            
            //t.setDefaultAlign("t-b?");
            t.setLayout(new FitLayout());
            //t.setDraggable(true);
            t.setHideDelay(300);
            t.setAutoHide(true);
            t.setWidth(provider.getToolTipContentsWidthHint()); 
            t.setTitle(record.getPOJO().getCaption() + ": " + provider.getActionLabel()); // TODO
            
            return t;
        }
        
        private String getTooltipImgCssId(ReposObjDataModel record) {
            // column id plus object id
            return getId() + "-" + record.getPOJO().getId();
        }
    }
}
