/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.data.BasePagingLoadConfig;
import com.extjs.gxt.ui.client.event.BaseEvent;
import com.extjs.gxt.ui.client.event.GridEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.widget.PagingToolBar;
import com.extjs.gxt.ui.client.widget.grid.CellEditor;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid.ClicksToEdit;
import com.extjs.gxt.ui.client.widget.toolbar.AdapterToolItem;
import com.extjs.gxt.ui.client.widget.toolbar.SeparatorToolItem;
import com.google.gwt.user.client.ui.HTML;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.GridContextMenuAdapter;
import com.iona.soa.web.repository.main.client.base.ReposObjHelper;
import com.iona.soa.web.repository.main.client.base.ReposURL;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator.DefaultColumnIds;

/**
 * Base Panel to display collections of repository object. it can handle any collection by default but can be
 * extended to provide specific behaviour when the default one isn't appropriate. In such a case overloading
 * the following methods is expected:
 *  - setListFields()
 *  - createContextActions()
 *  
 * the ordering of the list is based on the text in the 1st column
 */
public class WorkspaceCollectionPanel extends RemoteCollectionPanel {

    private final GridContextMenuAdapter menuAdapter;
    
    private final String collectionName;
    
    public WorkspaceCollectionPanel(SimpleGridConfigurator gridConfig, RemoteRestDataProxy dataProxy) {
        
        super(gridConfig, dataProxy);
        
        setHeaderVisible(false);
        
        collectionName = EClassHelper.getPluralNameForClass(dataProxy.getResourceURL().getEClass());
        setTitle(collectionName);
        
        if (gridPanel instanceof EditorGrid) {
            addEditSupport(gridPanel);
            addRenamingSupport(gridPanel);
        }
        
        menuAdapter = addContextMenu(gridPanel, dataProxy.getResourceURL());
        
        
        PagingToolBar pagingToolbar = gridConfig.attachPagingToolbar(gridPanel,
                                                                     this,
                                                                     dataProxy.getResourceURL().getEClass());
        
        HTML rssLink = new HTML("<a style=\"text-decoration: none;\" href=\""
                                + dataProxy.getResourceURL().getURL()
                                + "\" target=\"_blank\"><img src=\"images/rss.gif\"</a>");
        pagingToolbar.add(new SeparatorToolItem());
        pagingToolbar.add(new AdapterToolItem(rssLink));
    }

    @Override
    protected void doFirstLoad() {
        BasePagingLoadConfig loadConfig =
            new BasePagingLoadConfig(0, SimpleGridConfigurator.PAGINATION_PAGE_SIZE);
        gridPanel.getStore().getLoader().load(loadConfig);
    }
    
    @Override
    public void setFilterByCategory(String filterCategoryURL, String categoryName) {
        super.setFilterByCategory(filterCategoryURL, categoryName);
        setTitle(MESSAGES.collectionTitleByCategory(collectionName, categoryName));
    }

    @Override
    public void setFilterByQuery(String filterQuery) {
        super.setFilterByQuery(filterQuery);
        if ("".equals(filterQuery) || filterQuery == null) {
            setTitle(collectionName);
        } else {
            setTitle(MESSAGES.collectionTitleByQuery(collectionName, filterQuery));
        }
    }

    final GridContextMenuAdapter getContextMenuAdapter() {
        return menuAdapter;
    } 
    
    /**
     * Decorates a GridPanel
     * 
     * @param gp
     * @param eClass
     */
    private static void addEditSupport(final EditorGrid<ReposObjDataModel> gp) {
        gp.setClicksToEdit(ClicksToEdit.TWO);
        gp.getView().addListener(Events.RowDoubleClick, new Listener<BaseEvent>() {  
            public void handleEvent(BaseEvent be) {
                ReposObjDataModel record = gp.getSelectionModel().getSelectedItem();
                gp.getView().getFiresEvents();
                String selfLink = (String)record.get(ReposObjDataModel.FIELD_SELF_LINK);
                MainModule.get().getWorkspaceControl().openLink(selfLink, true);
            }  
        }); 
    }

    /**
     * Decorates a GridPanel
     * 
     * @param gp
     * @param eClass
     */
    private static GridContextMenuAdapter addContextMenu(EditorGrid<ReposObjDataModel> gp,
                                                         ReposURL collectionURL) {
        
        GridContextMenuAdapter cma = new GridContextMenuAdapter(gp);
        if (gp instanceof EditorGrid) {
            CollectionContextMenuHelper.addRenameContextMenu((EditorGrid<ReposObjDataModel>)gp, cma);
        }
        CollectionContextMenuHelper.addEditContextMenu(gp, cma);
        CollectionContextMenuHelper.addDeleteContextMenu(gp, cma);
        
        gp.setContextMenu(cma.getRootMenu());
        return cma;
    }
    
    /**
     * Decorates a GridPanel
     * 
     * @param gp
     * @param eClass
     */
    private static void addRenamingSupport(EditorGrid<ReposObjDataModel> agp) {
        
        final EditorGrid<ReposObjDataModel> gp = agp;
        
        CellEditor renamingEditor = gp.getColumnModel().getColumnById(DefaultColumnIds.NAME).getEditor();
        
        if (renamingEditor == null) {
            return;
        }
        
        // TODO : is that still required?
        gp.addListener(Events.BeforeEdit, new Listener<GridEvent>() {
            public void handleEvent(GridEvent ge) {
                ge.doit = ge.grid.getSelectionModel().getSelectedItems().size() == 1;
            }
        });
 
        gp.addListener(Events.AfterEdit,
                       new Listener<GridEvent>() {
                
                public void handleEvent(GridEvent ge) {
                    Utils.showWaitDialog();
                    gp.getStore().commitChanges();
                    String selfLink = (String)ge.record.getModel().get(ReposObjDataModel.FIELD_SELF_LINK);
                    ReposObjHelper.renameObject(selfLink, 
                                                (String)ge.startValue,
                                                (String)ge.value, 
                                                new AbstractUiObjCallBack() {
                            public void processReposObj(ReposObj anObj) {
                                Utils.closeWaitDialog();
    
                            }
                        });
                }
            });
        
        
          /*  public void onAfterEdit(GridPanel grid,
                                    Record record,
                                    String field,
                                    Object newValue,
                                    Object oldValue,
                                    int rowIndex,
                                    int colIndex) {
                Utils.showWaitDialog();
                grid.getStore().commitChanges();
                ReposObjHelper.renameObject(GridConfig.getSelfLink(record), 
                                            (String)oldValue, (String)newValue, 
                                            new AbstractUiObjCallBack() {
                        public void processReposObj(ReposObj anObj) {
                            Utils.closeWaitDialog();
                            // reload();  TODO - update listener may now handle this
                        }
                    });
            }
            
        });*/
    }
}
