/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.creationpanels;

import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.Window;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.layout.FitLayout;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.ui.editor.AfterClosureCallBack;

/**
 * A simple example of an 'about' dialog box.
 */
public class AttributeCreationDialog extends Window {

    private static final int WINDOW_HEIGHT = 300;
    private static final int WINDOW_WIDTH = 500;

    private AbstractReposObjCreationPanel cp;
    
    private ReposObjAttribute updatedAttr;
    private AfterClosureCallBack callBack;
    
    private Button btnOk;
    
    public AttributeCreationDialog(ReposObjAttribute anUpdatedAttr, AfterClosureCallBack aCallBack) {
        new AttributeCreationDialog(anUpdatedAttr, null, aCallBack);
    }
    
    public AttributeCreationDialog(ReposObjAttribute anUpdatedAttr,
                                   AbstractReposObjCreationPanel aCreationPanel,
                                   AfterClosureCallBack aCallBack) {

        updatedAttr = anUpdatedAttr;
        callBack = aCallBack;
        
        if (aCreationPanel == null) {
            cp = CreationPanelRegistry.getPanel(updatedAttr.getRefEClass());
        } else {
            cp = aCreationPanel;
        }
        
        setWidth(cp == null ? WINDOW_WIDTH : cp.getRequiredWidth());
        setHeight(cp == null ? WINDOW_HEIGHT : cp.getRequiredHeight());
        setLayout(new FitLayout());
        
        
        setStyleAttribute("padding", "0");
        setBorders(true);
        setBodyBorder(true);
        setFrame(true);
        setModal(true);
        setMaximizable(false);
        setHeaderVisible(true);
        setClosable(false);

        setCloseAction(CloseAction.HIDE);
        setPlain(true);
        
        // Set the dialog's caption.
        String attrName = updatedAttr.getName();
        setHeading(updatedAttr.isMulti()
                    ? "Add a new value to the " + attrName + " field"
                    : "Set the value for the " + attrName + " field");

        /*LayoutContainer panel = new LayoutContainer();
        panel.setLayout(new FitLayout());
        panel.setBorders(false);
        panel.setStyleAttribute("padding", "0");
        panel.add(cp);
        */add(cp);
        
        //getButtonBar().setMinButtonWidth(20);
        
        
        
        btnOk = new Button("Ok", new SelectionListener<ButtonEvent>() {
            
            @Override
            public void componentSelected(ButtonEvent ce) {
                btnOk.disable();
                commit();
            }
        });
        btnOk.setIconStyle("ok-icon");
        btnOk.setId("ok-icon");
        addButton(btnOk);
        //btnOk.disable();
        
        Button btnCancel = new Button("Cancel", new SelectionListener<ButtonEvent>() {
            @Override
            public void componentSelected(ButtonEvent ce) {
                close();
            }
        });
        btnCancel.setIconStyle("cancel-icon");
        btnCancel.setId("cancel-icon");
        addButton(btnCancel);
    }

    private void commit() {
        
        ReposObj value = cp.getData();
        
        if (value.getSelfLink() != null)  {
            MainModule.getRemoteClient().updateReposObj(value,
                                                        new AbstractUiObjCallBack() {
                    public void processReposObj(ReposObj anObj) {
                        callBack.onDialogClosure();
                        close();
                    }
                });
        } else {
            MainModule.getRemoteClient().createReposObj(updatedAttr.getCollectionLink(),
                                                        value,
                                                        new AbstractUiObjCallBack() {
                    public void processReposObj(ReposObj anObj) {
                        callBack.onDialogClosure();
                        close();
                    }
                });
        }
    }
}
