/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.creationpanels.panels;

import java.util.ArrayList;
import java.util.List;

import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.store.Store;
import com.extjs.gxt.ui.client.widget.form.ComboBox;
import com.extjs.gxt.ui.client.widget.form.FormPanel;
import com.extjs.gxt.ui.client.widget.form.TextField;
import com.extjs.gxt.ui.client.widget.form.Validator;
import com.extjs.gxt.ui.client.widget.form.ComboBox.TriggerAction;
import com.extjs.gxt.ui.client.widget.layout.FitLayout;
import com.google.gwt.user.client.ui.Widget;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.AbstractReposObjCreationPanel;

public class PropertyPanel extends AbstractReposObjCreationPanel {

    
    private ComboBox<ReposObjDataModel> fieldName;
    private TextField<String> fieldValue;

    public PropertyPanel(Store<ReposObjDataModel> aStore, String[] propertyKeySuggestions) {
        setLayout(new FitLayout());
        setMonitorResize(false);
        setHeaderVisible(false);
        setFrame(true);
        
        setStyleAttribute("padding", "0px");

        FormPanel fp = new FormPanel();
        
        fp.setHeaderVisible(false);
        fp.setFrame(true);
        fp.setBodyBorder(false);
        fp.setStyleAttribute("padding", "0px");
        
        fieldName = new ComboBox<ReposObjDataModel>();  
        fieldName.setForceSelection(false);  
        fieldName.setMinChars(1);  
        fieldName.setFieldLabel("Name");  
        fieldName.setName("name");  
        
        fieldName.setDisplayField(ReposObjDataModel.FIELD_NAME);  
        fieldName.setTriggerAction(TriggerAction.ALL);  
        fieldName.setEmptyText("Enter Property Name");  
        fieldName.setLoadingText("Searching...");  
        fieldName.setTypeAhead(true);  
        fieldName.setSelectOnFocus(true);  
        
        fieldName.setValidator(new Validator<String, TextField<String>>() {
            public String validate(TextField<String> field, String value) {
                return value.matches("[a-zA-Z0-9_\\-\\.]+")
                    ? null
                    : "The property name contains invalid characters";
            }
        });
        
        populateCombo(aStore, propertyKeySuggestions);
        
        fp.add(fieldName);
        
        fieldValue = new TextField<String>();
        fieldValue.setFieldLabel("Value");
        fieldValue.setName("value");
        fieldValue.setAllowBlank(false);
        fp.add(fieldValue);
        
        add(fp);

    }

    private void populateCombo(Store<ReposObjDataModel> aStore, String[] propertyKeySuggestions) {
        
        // load existing props details
        List<String> existingProps = new ArrayList<String>();
        ListStore<ReposObjDataModel> comboStore = new ListStore<ReposObjDataModel>();
        if (aStore != null) {
            List<ReposObjDataModel> rcds = aStore.getModels();
            for (ReposObjDataModel rcd : rcds) {
                String propName = rcd.get(ReposObjDataModel.FIELD_NAME);
                if (propName != null && propName.indexOf("<b>") != -1) {
                    propName = propName.substring(3, propName.length() - 4);
                }
                existingProps.add(propName);
                comboStore.add(new ReposObjDataModel(rcd.getPOJO()));
            }
        }
        
        // load property key suggestions
        if (propertyKeySuggestions != null && propertyKeySuggestions.length > 0) {
            for (String suggestion : propertyKeySuggestions) {
                if (!existingProps.contains(suggestion)) {
                    existingProps.add(suggestion);
                    ReposObjDataModel suggestionDataModel = new ReposObjDataModel(null);
                    suggestionDataModel.set(ReposObjDataModel.FIELD_NAME, suggestion);
                    comboStore.add(suggestionDataModel);
                }
            }
        }
        

        fieldName.setStore(comboStore); 
    }

    public ReposObj getData() {
        
        String propName = fieldName.getRawValue();
        String propValue = fieldValue.getValue();
        
        ReposObj data = fieldName.getValue() == null ? null : fieldName.getValue().getPOJO();
        
        if (data != null) {
            data.getAttribute("value").setSimpleValue(propValue);
            
        } else {
            data = new ReposObj();
    
            ReposObjAttribute propNameAttribute = new ReposObjAttribute();
            propNameAttribute.setName(ReposObjDataModel.FIELD_NAME);
            propNameAttribute.setSimpleValue(propName);
            data.addAttribute(propNameAttribute);
    
            ReposObjAttribute propValAttribute = new ReposObjAttribute();
            propValAttribute.setName(ReposObjDataModel.FIELD_VALUE);
            propValAttribute.setSimpleValue(propValue);
            data.addAttribute(propValAttribute);
        }
        
        return data;
    }

    public boolean validate() {
        // TODO Auto-generated method stub
        return false;
    }

    public boolean remove(Widget child) {
        // TODO Auto-generated method stub
        return false;
    }


    public int getRequiredHeight() {
        return 160;
    }

    public int getRequiredWidth() {
        return 380;
    }

}
