/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.creationpanels.panels;

import java.util.Arrays;
import java.util.Date;



import com.extjs.gxt.ui.client.data.BaseModelData;
import com.extjs.gxt.ui.client.data.ModelData;

import com.extjs.gxt.ui.client.event.SelectionChangedEvent;
import com.extjs.gxt.ui.client.event.SelectionChangedListener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.form.CheckBox;
import com.extjs.gxt.ui.client.widget.form.ComboBox;
import com.extjs.gxt.ui.client.widget.form.DateField;
import com.extjs.gxt.ui.client.widget.form.Field;
import com.extjs.gxt.ui.client.widget.form.FieldSet;
import com.extjs.gxt.ui.client.widget.form.FormPanel;
import com.extjs.gxt.ui.client.widget.form.NumberField;
import com.extjs.gxt.ui.client.widget.form.SimpleComboBox;
import com.extjs.gxt.ui.client.widget.form.SimpleComboValue;
import com.extjs.gxt.ui.client.widget.form.TextArea;
import com.extjs.gxt.ui.client.widget.form.TextField;
import com.extjs.gxt.ui.client.widget.form.TimeField;
import com.extjs.gxt.ui.client.widget.layout.CardLayout;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.ui.Label;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ScheduledTaskType;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.AbstractReposObjCreationPanel;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.FieldWidgetHelper;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.RefValAttrField;

public class ScheduledTaskPanel extends AbstractReposObjCreationPanel {
    
    public static final String OPTION_TASK_TYPE = "typeID";
    public static final String OPTION_TASK_PARAMETERS = "parameters";

    
    FieldSet scheduleAtFixedInt;
    FieldSet scheduleOn;
    FieldSet addEveryDayDetailsFieldSet;
    FieldSet addDayOfWeekDetailsFieldSet;
    FieldSet addDayOfMonthDetailsFieldSet;
    FieldSet activeScheduleFieldSet;
    TextField<String> nameTF;
    SimpleComboBox<String> taskTypeCombo;  
    TextArea taskParametersTxt; 
    SimpleComboBox<ScheduledTaskType>  scheduleTypeCombo;

    CardLayout detailsCardLayout;
    NumberField intervalNF;
    ComboBox<ModelData> intervalUnitsCombo;
    TextField<String> passwordTF;
    TextField<String> daysTF;
    TimeField onceOffTimeTF;
    DateField onceOffDateDF;
    TimeField dailyTimeTF;    
    TimeField dowTimeTF;    
    TimeField domTimeTF;    
    CheckBox mondayCB;
    CheckBox tuesdayCB;
    CheckBox wednesdayCB;
    CheckBox thursdayCB;
    CheckBox fridayCB;
    CheckBox saturdayCB;
    CheckBox sundayCB;
    RefValAttrField userField;
    
    private final ScheduledTaskDefaults defaults;
    private final ReposObj taskObj;
    

    /**
     * Constructor for generic task, i.e. type of task isn't known.
     */
    public ScheduledTaskPanel() {
        this(new ReposObj(), new ScheduledTaskDefaults());
    }
    
    /**
     * Constructor for task, task type is set via arguments.
     * @param args
     */
    public ScheduledTaskPanel(ScheduledTaskDefaults defaultValues) {
        defaults = defaultValues;
        taskObj = defaultValues.getTaskPrototype() == null 
            ? new ReposObj() : defaultValues.getTaskPrototype();
        initialize();
    }
    
    /**
     * Constructor for editing an existing task object.
     * @param input
     */
    public ScheduledTaskPanel(ReposObj input, ScheduledTaskDefaults defaultValues) {
        defaults = defaultValues; 
        taskObj = input;
        initialize();
    }
    
    private void initialize() {
        this.setHeaderVisible(false);
        detailsCardLayout = new CardLayout();
        defaults.applyDefaultValues(taskObj); // TOSO - change
        FormPanel form = new FormPanel();
        form.setHeaderVisible(false);
        form.setLabelWidth(35);
        form.setLabelWidth(100);
        form.setBorders(false);

        form.setHeight(height);
        
        add(form);

        addNameField(form);
        addScheduleTypeCombo(form);
        addRunWhenSection(form);
        addRunAsSection(form);
    }


    private void addNameField(FormPanel parent) {
        nameTF = new TextField<String>();
        nameTF.setFieldLabel("Name");
        nameTF.setName("name");
        
        nameTF.setWidth(250);
        nameTF.setValue(taskObj.getAttributeValue("name"));
        parent.add(nameTF);
    }
 
    private void addScheduleTypeCombo(FormPanel form) {
        
        scheduleTypeCombo = new SimpleComboBox<ScheduledTaskType>();
        if (defaults.getTaskPrototype() != null) {
            String eclass = defaults.getTaskPrototype().getEClass();
            ScheduledTaskType[] list = ScheduledTaskType.values();
            for (ScheduledTaskType st : list) {
                if (st.getClassName().equals(eclass)){
                    scheduleTypeCombo.add(st);
                    scheduleTypeCombo.setSimpleValue(st);
                }
            }

        } else {
            scheduleTypeCombo.add(Arrays.asList(ScheduledTaskType.values()));
        }
        scheduleTypeCombo.setFieldLabel("Run task");
        scheduleTypeCombo.setName("runtask");
        scheduleTypeCombo.setForceSelection(true);
        scheduleTypeCombo.setEmptyText("select when to run task");
        scheduleTypeCombo.setTypeAhead(true);
        scheduleTypeCombo.setSelectOnFocus(true);
        scheduleTypeCombo.setWidth(250);


        SelectionChangedListener<SimpleComboValue<ScheduledTaskType>> listener = 
            new SelectionChangedListener<SimpleComboValue<ScheduledTaskType>>(){


            public void selectionChanged(
                    SelectionChangedEvent<SimpleComboValue<ScheduledTaskType>> se) {
                if (activeScheduleFieldSet != null) {
                    activeScheduleFieldSet.setVisible(false);
                }

                switch(se.getSelectedItem().getValue()){
                case SPECIFIC_DATE_SELECTION:
                    activeScheduleFieldSet = scheduleOn;
                    parseSpecificDateSelection();
                    break;
                case FIXED_INTERVAL_SELECTION:
                    activeScheduleFieldSet = scheduleAtFixedInt;
                    parseIntervalRepeatedTask();
                    break;
                case EVERY_DAY_SELECTION:
                    activeScheduleFieldSet = addEveryDayDetailsFieldSet;
                    parseDailyTask();
                    break;
                case WEEKDAY_SELECTION:
                    activeScheduleFieldSet = addDayOfWeekDetailsFieldSet;     
                    parseWeeklyTask();
                    break;
                case MONTHDAY_SELECTION:
                    activeScheduleFieldSet = addDayOfMonthDetailsFieldSet;
                    parseMonthlyTask();
                    break;
                default:

                    break;
                }
                activeScheduleFieldSet.setVisible(true);
            }

            
        };

        scheduleTypeCombo.addSelectionChangedListener(listener);
       

        
        if (defaults.scheduleTypeDefaulted()) {
            scheduleTypeCombo.setRawValue(defaults.getScheduleType());
        }
        
        /*
        if (defaults.getTaskPrototype() == null) {
            form.add(scheduleTypeCombo);
        }*/
        form.add(scheduleTypeCombo);

        if (defaults.scheduleTypeDefaulted()) {
            DeferredCommand.addCommand(new Command() {
                public void execute() {
                    // TODO:
                    //    scheduleTypeSelected(defaults.getScheduleType());
                }
            });
        }
    }
   

    private void addRunAsSection(FormPanel form) {
        if (taskObj.getEditLink() == null) {
            taskObj.setEditLink(""); // RefValAttrField won't work otherwise 
        }
        
        FieldSet fs = FieldWidgetHelper.createFieldSet("Run as the following user", 90, 175);
        ReposObjAttribute userAttribute = taskObj.getAttribute("user");
        passwordTF = new TextField<String>();
        userField = new RefValAttrField(userAttribute, "User");
        
        userField.setWidth(300);
        fs.add(userField);

        passwordTF.setFieldLabel("Password"); // TODO bind id on form to "password" ?
        passwordTF.setPassword(true);
        passwordTF.setWidth(250);
        fs.add(passwordTF);
        form.add(fs);
    }
    
    private void addRunWhenSection(FormPanel form) {
        // which panel is displayed is driven by scheduleTypeSelected
        LayoutContainer detailPanel = new LayoutContainer();

        //detailPanel.setLayout(detailsCardLayout);
        detailPanel.setBorders(false);
        detailPanel.setEnabled(true);
        //detailsCardLayout.
        form.add(detailPanel);
        
        addSpecificDateDetails(detailPanel);        
        addFixedIntervalDetails(detailPanel);
        
        addEveryDayDetails(detailPanel);
        addDayOfWeekDetails(detailPanel);
        addDayOfMonthDetails(detailPanel);
        form.add(new Label());
        //detailsCardLayout.setActiveItem(scheduleAtFixedInt);
        
    }
    
    private void addSpecificDateDetails(LayoutContainer panel) {

        scheduleOn = FieldWidgetHelper.createFieldSet("Schedule on", 90, 175);
        scheduleOn.setVisible(false);
        onceOffDateDF = new DateField(); // "Date", "sddate", 150
        onceOffDateDF.setFieldLabel("Date");
        onceOffDateDF.setName("date");

        onceOffDateDF.setEmptyText("Select a date");
        scheduleOn.add(onceOffDateDF);


        onceOffTimeTF = new TimeField(); // "Time", "sdtime", 150
        onceOffTimeTF.setFieldLabel("Time");
        onceOffTimeTF.setFormat(DateTimeFormat.getFormat("HH:mm"));
        scheduleOn.add(onceOffTimeTF);
        
        if (EClassHelper.SCHEDULED_ONCEOFF_TASK_ECLASS.equals(taskObj.getEClass())) {
            String scheduleDate = taskObj.getAttributeValue("scheduleDate");
            String[] parts = scheduleDate.split(" ");
            final String date = parts[0];
            String[] timeParts = parts[1].split(":");
            final String time = timeParts[0] + ":" + timeParts[1];
            DeferredCommand.addCommand(new Command() {
                public void execute() {
                    Date dateVal = new Date(date);
                    onceOffDateDF.setValue(dateVal);
                    //onceOffTimeTF
                    onceOffTimeTF.setRawValue(time);
                }
            });
        }
        
        panel.add(scheduleOn);
    }

    private void addFixedIntervalDetails(LayoutContainer  panel) {     
        scheduleAtFixedInt = FieldWidgetHelper.createFieldSet("Schedule at Fixed Intervals", 90, 175);
        scheduleAtFixedInt.setVisible(false);
        panel.add(scheduleAtFixedInt);
        
        intervalNF = new NumberField();
        intervalNF.setFieldLabel("Every");
        // "Every", "every" // TODO - field binding to form using id "every"        
        intervalNF.setWidth(50);
        scheduleAtFixedInt.add(intervalNF);


        BaseModelData minutes = new BaseModelData();
        minutes.set("id", "MINUTES");
        minutes.set("text", "minutes");
        BaseModelData hours = new BaseModelData();
        hours.set("id", "HOURS");
        hours.set("text", "hours");
        ListStore<ModelData> store = new ListStore<ModelData>();
        store.add(minutes);
        store.add(hours);
        

        intervalUnitsCombo = new ComboBox<ModelData>();
        intervalUnitsCombo.setFieldLabel("Units");
        intervalUnitsCombo.setName("units");
        
        intervalUnitsCombo.setDisplayField("text");
        intervalUnitsCombo.setValueField("id");
        intervalUnitsCombo.setFieldLabel("Units");
        intervalUnitsCombo.setForceSelection(true);
        intervalUnitsCombo.setStore(store);
        intervalUnitsCombo.setEmptyText("select units");
        intervalUnitsCombo.setTypeAhead(true);
        intervalUnitsCombo.setSelectOnFocus(true);
        intervalUnitsCombo.setWidth(100);
        
        scheduleAtFixedInt.add(intervalUnitsCombo);        
    }

    private FieldSet addEveryDayDetails(LayoutContainer  panel) {

        addEveryDayDetailsFieldSet = FieldWidgetHelper.createFieldSet("Schedule Every Day", 90, 175);        
        addEveryDayDetailsFieldSet.setVisible(false);
        panel.add(addEveryDayDetailsFieldSet);
        
        dailyTimeTF = new TimeField(); // "Time", "dailytime", 100
        dailyTimeTF.setFieldLabel("Time");
        dailyTimeTF.setFormat(DateTimeFormat.getFormat("HH:mm"));
        addEveryDayDetailsFieldSet.add(dailyTimeTF);
        return addEveryDayDetailsFieldSet;
    }
    
    private void addDayOfWeekDetails(LayoutContainer  panel) {

        addDayOfWeekDetailsFieldSet = FieldWidgetHelper.createFieldSet("Schedule at fixed Days of the Week", 90, 175);
        addDayOfWeekDetailsFieldSet.setVisible(false);
        panel.add(addDayOfWeekDetailsFieldSet);

        dowTimeTF = new TimeField();  // "Time", "weektime", 100
        dowTimeTF.setFieldLabel("Time");
        dowTimeTF.setFormat(DateTimeFormat.getFormat("HH:mm"));
        addDayOfWeekDetailsFieldSet.add(dowTimeTF);
         
        mondayCB = new CheckBox(); // "Monday", "monday"
        mondayCB.setFieldLabel("Monday");
        addDayOfWeekDetailsFieldSet.add(mondayCB);        
        tuesdayCB = new CheckBox(); // "Tuesday", "tuesday"
        addDayOfWeekDetailsFieldSet.add(tuesdayCB);
        tuesdayCB.setFieldLabel("Tuesday");
        wednesdayCB = new CheckBox(); // "Wednesday", "wednesday"
        wednesdayCB.setFieldLabel("Wednesday");
        addDayOfWeekDetailsFieldSet.add(wednesdayCB);
        thursdayCB = new CheckBox(); // "Thursday", "thursday"
        thursdayCB.setFieldLabel("Thursday");
        addDayOfWeekDetailsFieldSet.add(thursdayCB);
        fridayCB = new CheckBox(); // "Friday", "friday"
        fridayCB.setFieldLabel("Friday");
        addDayOfWeekDetailsFieldSet.add(fridayCB);
        saturdayCB = new CheckBox(); // "Saturday", "saturday"
        saturdayCB.setFieldLabel("Saturday");
        addDayOfWeekDetailsFieldSet.add(saturdayCB);
        sundayCB = new CheckBox(); // "Sunday", "sunday"
        sundayCB.setFieldLabel("Sunday");
        addDayOfWeekDetailsFieldSet.add(sundayCB);        
    }

    private void addDayOfMonthDetails(LayoutContainer panel) {
        addDayOfMonthDetailsFieldSet = FieldWidgetHelper.createFieldSet("Schedule at fixed Days of the Month", 90, 175);
        addDayOfMonthDetailsFieldSet.setVisible(false);
        panel.add(addDayOfMonthDetailsFieldSet);

        domTimeTF = new TimeField();  // "Time", "monthtime", 100
        domTimeTF.setFieldLabel("Time");
        domTimeTF.setFormat(DateTimeFormat.getFormat("HH:mm"));
        addDayOfMonthDetailsFieldSet.add(domTimeTF);
        
        daysTF = new TextField<String>();
        daysTF.setFieldLabel("Days");   // form id = "days" ?
        daysTF.setWidth(190);
        daysTF.setToolTip("Separate days with comma's as in 1, 15");
        addDayOfMonthDetailsFieldSet.add(daysTF);
    }

    public ReposObj getData() {
        return taskObj;
    }

    public int getRequiredHeight() {
        return 400;
    }

    public int getRequiredWidth() {
        return 450;
    }

    public boolean validate() {
        
        try {
            if (scheduleTypeCombo != null) {
                switch (scheduleTypeCombo.getSelection().get(0).getValue()) {
                    case SPECIFIC_DATE_SELECTION:
                        validateOnceOffTask();
                        break;
                    case FIXED_INTERVAL_SELECTION:
                        validateIntervalRepeatedTask();
                        break;
                    case EVERY_DAY_SELECTION:
                        validateDailyTask();
                        break;
                    case WEEKDAY_SELECTION:
                        validateWeeklyTask();
                        break;
                    case MONTHDAY_SELECTION:
                        validateMonthlyTask();
                        break;
                    default:
                        throw new ValidationException("Please specify when to run this task.");
                }
            }
            requiredAttr(nameTF);
            setAttr("name", nameTF);
            
            validateTaskTypeAndParameterAttributes();
            
            requiredAttr(userField);
            setAttr("user", userField);
            
            requiredAttr(passwordTF);
            setAttr("password", passwordTF);
            
        } catch (ValidationException e) {
            MessageBox.alert("Validation error", e.getMessage(), null);
            return false;
        }
                
        return true;
    }

    private void setAttr(String attrName, Field<?> field) {
        setAttr(attrName, field.getRawValue());
    }
    
    private void setAttr(String attrName, RefValAttrField value) {
        // What?
        value.getAttribute().setName(attrName);
        taskObj.addAttribute(value.getAttribute());
    }
    
    private void setAttr(String attrName, String value) {
        if (taskObj.getAttribute(attrName) == null) {
            ReposObjAttribute attr = new ReposObjAttribute();
            attr.setName(attrName);
            taskObj.addAttribute(attr);            
        }
        taskObj.getAttribute(attrName).setSimpleValue(value);
    }

    private ReposObjAttribute setMultiValueAttr(String attrName, Field<?> field, String regExp) {
        ReposObjAttribute attr = new ReposObjAttribute();
        attr.setName(attrName);
        attr.setMulti(true);
        
        String [] values = field.getRawValue().split(regExp);
        for (int i = 0; i < values.length; i++) {            
            if (values[i] == null) {
                attr.addSimpleValue("");
            } else {
                String v = values[i].trim();
                if (v.length() > 0) {
                    attr.addSimpleValue(v);
                }
            }
        }
        taskObj.addAttribute(attr);
        return attr;
    }
    
    private ReposObjAttribute setMultiValueAttr(String attrName, String[] values) {
        ReposObjAttribute attr = new ReposObjAttribute();
        attr.setName(attrName);
        attr.setMulti(true);
        for (int i = 0; i < values.length; i++) {
            if (values[i] == null) {
                attr.addSimpleValue("");
            } else {
                String v = values[i].trim();
                if (v.length() > 0) {
                    attr.addSimpleValue(v);
                }
            }
        }
        taskObj.addAttribute(attr);
        return attr;
    }

    @SuppressWarnings("deprecation")
    private void validateOnceOffTask() throws ValidationException {        
        taskObj.setEClass(EClassHelper.SCHEDULED_ONCEOFF_TASK_ECLASS);
        StringBuffer sb = new StringBuffer();
        
        requiredAttr(onceOffDateDF);
        Date date = onceOffDateDF.getValue();
        sb.append(date.getYear() + 1900);
        sb.append('-');
        if (date.getMonth() < 10) {
            sb.append('0');
        }
        sb.append(date.getMonth() + 1);
        sb.append('-');
        if (date.getDate() < 10) {
            sb.append('0');
        }
        sb.append(date.getDate());
        sb.append(' ');
        
        requiredAttr(onceOffTimeTF);
        sb.append(onceOffTimeTF.getRawValue());
        sb.append(":00");
        
        setAttr("scheduleDate", sb.toString());
    }
    
    private void parseSpecificDateSelection() {
        ReposObjAttribute scheduleString = taskObj.getAttribute("scheduleDate");
        if (scheduleString != null) {
            String scheduleDate = scheduleString.getValue();
            String[] dateArray = scheduleDate.split("-");
            String[] dateTime = dateArray[2].split(" ");
            String[] time = dateTime[1].split(":");
            
            //scheduleDate
            String year = scheduleDate.substring(0, scheduleDate.lastIndexOf("-"));
            int yearVal = Integer.parseInt(dateArray[0]) - 1900;
            int monthVal = Integer.parseInt(dateArray[1]) - 1;
            int dateVal = Integer.parseInt(dateTime[0]);
    
            Date date = new Date();
            
            date.setYear(yearVal);
            date.setMonth(monthVal);
            date.setDate(dateVal);
            
            BaseModelData timeAsModelData = new BaseModelData();
            timeAsModelData.set("text", time[0] + ":" + time[1]);
            onceOffTimeTF.setValue(timeAsModelData);
            onceOffDateDF.setValue(date);
        }
    }

    private void validateIntervalRepeatedTask() throws ValidationException {
        taskObj.setEClass(EClassHelper.SCHEDULED_INTERVAL_REPEATED_TASK_ECLASS);
        
        requiredAttr(intervalNF);
        setAttr("interval", intervalNF);
        String val = intervalUnitsCombo.getValue().get("id");
        setAttr("timeUnit", val);
    }

    private void parseIntervalRepeatedTask() {
        ReposObjAttribute interval = taskObj.getAttribute("interval");
        if (interval != null) {
            String intervalString = interval.getValue();
            intervalNF.setValue(Integer.parseInt(intervalString));
        }
        
        ReposObjAttribute timeUnit = taskObj.getAttribute("timeUnit");
        if (timeUnit != null) {
            
            ListStore<ModelData> list = intervalUnitsCombo.getStore();
            for (int count = 0 ; count < list.getCount() ; count++) {
                ModelData modelData = list.getAt(count);
                String id = modelData.get("id");
                String val = timeUnit.getValue();
                if (id.equals(val)){
                    intervalUnitsCombo.setValue(modelData);
                }
            }
        }
        
    }
    
    private void requiredAttr(Field<?> field) throws ValidationException {
        if ("".equals(field.getRawValue().trim())) {
            throw new ValidationException("Please specify a value for '" + field.getFieldLabel() + "'");
        }
    }
    
    private void requiredAttr(RefValAttrField field) throws ValidationException {
        if (field.getAttribute().getValues().length != 1) {
            throw new ValidationException("Please specify a value for '" + field.getFieldLabel() + "'");
        }
    }


    private void validateDailyTask() throws ValidationException {
        taskObj.setEClass(EClassHelper.SCHEDULED_DAILY_TASK_ECLASS);
        
        validateTimeField(dailyTimeTF);
    }

    private void validateTimeField(TimeField tf) throws ValidationException {
        requiredAttr(tf);
        String [] td = tf.getRawValue().split(":");
        if (td.length != 2) {
            throw new ValidationException("Invalid time format: " + tf.getRawValue());
        }
        
        setAttr("hour", td[0]);
        setAttr("minute", td[1]);
    }

    private void parseDailyTask() {
        parseTimeField(dailyTimeTF);
    }
    
    private void parseTimeField(TimeField timeField){
        ReposObjAttribute hourAttr = taskObj.getAttribute("hour");
        ReposObjAttribute minuteAttr = taskObj.getAttribute("minute");
        if (hourAttr != null && minuteAttr != null) {
            String hour = hourAttr.getValue();
            while (hour.length() < 2) {
                hour = "0" + hour;
            }
            String minute = minuteAttr.getValue();
            while (minute.length() < 2) {
                minute =  minute + "0";
            }
            String time = hour + ":" + minute;
            BaseModelData timeAsModelData = new BaseModelData();
            timeAsModelData.set("text", time);
            timeField.setValue(timeAsModelData);
        }
    }
    
    private void validateWeeklyTask() throws ValidationException {
        validateTimeField(dowTimeTF);
        
        taskObj.setEClass(EClassHelper.SCHEDULED_WEEKLY_TASK_ECLASS);
        taskObj.removeAttribute("weekdays");
        ReposObjAttribute attr = new ReposObjAttribute();
        attr.setName("weekdays");
        attr.setMulti(true);
        
        readCheckBox(attr, mondayCB, "MONDAY");
        readCheckBox(attr, tuesdayCB, "TUESDAY");
        readCheckBox(attr, wednesdayCB, "WEDNESDAY");
        readCheckBox(attr, thursdayCB, "THURSDAY");
        readCheckBox(attr, fridayCB, "FRIDAY");
        readCheckBox(attr, saturdayCB, "SATURDAY");
        readCheckBox(attr, sundayCB, "SUNDAY");
        
        if (attr.getValues().length < 1) {
            throw new ValidationException("Please specify at least one day.");
        }

        taskObj.addAttribute(attr);

    }
    
    private void parseWeeklyTask() {
        parseTimeField(dowTimeTF);
        ReposObjAttribute attr = taskObj.getAttribute("weekdays");
        if (attr != null) {
            String daysOfTheWeek[] = attr.getValues();
            
            for (String dayOfTheWeek : daysOfTheWeek) {
                if(dayOfTheWeek.equals("MONDAY")) {
                    mondayCB.setValue(true);   
                    continue;
                }
                if(dayOfTheWeek.equals("TUESDAY")) {
                    tuesdayCB.setValue(true);
                    continue;
                    
                }
                if(dayOfTheWeek.equals("WEDNESDAY")) {
                    wednesdayCB.setValue(true);
                    continue;
                }
                if(dayOfTheWeek.equals("THURSDAY")) {
                    thursdayCB.setValue(true);
                    continue;
                }
                if(dayOfTheWeek.equals("FRIDAY")) {
                    fridayCB.setValue(true);
                    continue;
                }
                if(dayOfTheWeek.equals("SATURDAY")) {
                    saturdayCB.setValue(true);
                    continue;
                }
                if(dayOfTheWeek.equals("SUNDAY")) {
                    sundayCB.setValue(true);
                    continue;
                }
            }
        }

    }

    
    private void readCheckBox(ReposObjAttribute attr, CheckBox cb, String value) {
        if (cb.getValue()) {
            attr.addSimpleValue(value);
        } 
    }

    private void validateMonthlyTask() throws ValidationException {
        validateTimeField(domTimeTF);
        
        taskObj.setEClass(EClassHelper.SCHEDULED_MONTHLY_TASK_ECLASS);
        
        requiredAttr(daysTF);        
        ReposObjAttribute attr = setMultiValueAttr("days", daysTF, "\\D");   
        String[] values = attr.getValues();
        for (int i = 0; i < values.length; i++) {
            int v = 0;            
            try {
                v = Integer.parseInt(values[i]);
            } catch (NumberFormatException nfe) {
                // skip
            }
            if (v < 1 || v > 31) {
                throw new ValidationException("Days not correctly formatted,"
                                              + "please use numbers between 1 and 31 separated by comma's.");
            }
        }        
    }
    
    private void parseMonthlyTask() {
        parseTimeField(domTimeTF);
        ReposObjAttribute days = taskObj.getAttribute("days");
        String daysVals = "";
        if (days != null) {
            String [] values = days.getValues();
            if (values.length > 0 ) {
                daysVals = values[0];
            }
            for (int i = 1; i < values.length; i++) {            
                daysVals += "," + values[i];
            }
        }
        
        daysTF.setValue(daysVals);
    }

    private void validateTaskTypeAndParameterAttributes() throws ValidationException {
        if (defaults.taskTypeDefaulted()) {
            setAttr(OPTION_TASK_TYPE, defaults.getTaskType());
            setMultiValueAttr(OPTION_TASK_PARAMETERS, defaults.getTaskParameters());
        } else {
//            requiredAttr(taskTypeCombo);
//            setAttr(OPTION_TASK_TYPE, taskTypeCombo);
//            setMultiValueAttr(OPTION_TASK_PARAMETERS, taskParametersTxt, "[\\r\\n]");
//            requiredAttr(scheduleTypeCombo);
        }
    }

    public static final class ScheduledTaskDefaults {
        private String taskType;
        private String taskName;
        private String[] taskParameters;
        private String scheduleType;
        private ReposObj taskPrototype;
        
        
        public ScheduledTaskDefaults setTaskType(String taskType) {
            this.taskType = taskType;
            return this;
        }
        
        public ScheduledTaskDefaults setTaskParameters(String[] taskParameters) {
            this.taskParameters = taskParameters;
            return this;
        }
        
        public ScheduledTaskDefaults setTaskName(String name) {
            this.taskName = name;
            return this;
        }
        
        public ScheduledTaskDefaults setTaskPrototypeObj(ReposObj taskPrototypeObj) {
            this.taskPrototype = taskPrototypeObj;
            return this;
        }
        
        
        public String getScheduleType() {
            return null;
        }

        void applyDefaultValues(ReposObj task) {
            setDefaultTaskName(task);
            setDefaultUser(task);
        }
        
        private void setDefaultTaskName(ReposObj task) {
            if (task.getAttribute("name") == null) {
                ReposObjAttribute nameAttr = new ReposObjAttribute();
                nameAttr.setName("name");
                task.addAttribute(nameAttr);
                task.getAttribute("name").setSimpleValue(getTaskName());
            } 
            //task.getAttribute("name").setSimpleValue(getTaskName());
        }

        private void setDefaultUser(ReposObj task) {
            ReposObjAttribute userAttribute = task.getAttribute("user");
            if (userAttribute == null) {
                userAttribute = new ReposObjAttribute();
                userAttribute.setName("user")
                    .setReference(true)
                    .setMulti(false)
                    .setCollectionLink(EClassHelper.getCollectionLinkForClass(EClassHelper.USER_ECLASS));
                
                // ReposObjAttribute needs its parent
                task.addAttribute(userAttribute); 
            }
            
            if (MainModule.getCurrentUser() != null) {
                task.getAttribute("user").setRefValue(MainModule.getCurrentUser());
            }
        }
        
        public String getTaskType() {
            return taskType;
        }
        
        public String[] getTaskParameters() {
            return taskParameters == null ? new String[0] : taskParameters;
        }

        boolean scheduleTypeDefaulted() {
            return getScheduleType() != null;
        }
        
        boolean taskTypeDefaulted() {
            return taskType != null;
        }
        
        ReposObj getTaskPrototype() {
            return taskPrototype;
        }
        

        
        public String getTaskName() {
            return taskName == null ? "" : taskName;
        }        
    }
    
    @SuppressWarnings("serial")
    private static final class ValidationException extends Exception {
        private ValidationException(String reason) {
            super(reason);
        }
    }
}
