/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.widget.Component;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.form.Field;
import com.extjs.gxt.ui.client.widget.form.FieldSet;
import com.extjs.gxt.ui.client.widget.layout.ColumnData;
import com.extjs.gxt.ui.client.widget.layout.ColumnLayout;
import com.extjs.gxt.ui.client.widget.layout.FormData;
import com.extjs.gxt.ui.client.widget.layout.FormLayout;
import com.google.gwt.user.client.ui.Widget;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.IFormPart;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.ui.editor.EditorLayoutRegistry.EditorColumn;
import com.iona.soa.web.repository.main.client.base.ui.editor.EditorLayoutRegistry.EditorLayoutDescriptor;
import com.iona.soa.web.repository.main.client.base.ui.editor.EditorLayoutRegistry.EditorLayoutRegistryCallBack;
import com.iona.soa.web.repository.main.client.base.ui.editor.EditorLayoutRegistry.EditorSection;
import com.iona.soa.web.repository.main.client.base.ui.editor.EditorLayoutRegistry.FieldGroup;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.AbstractReposObjFormEditor;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.CollectionValAttrField;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.ConstraintAttrField;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.FieldWidgetHelper;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.LinkField;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.MultiValueAttributeField;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.PropertyAttrField;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.RefValAttrField;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.SingleRefValAttrField;

/**
 * Base editor for repository object. 
 * 
 * The layout of the editor is defined by the eClass attribute of the object being loaded, provided 
 * that a layout has been defined for that particular eClass.  Otherwise it will use a default layout 
 * (1 section, 1 column, ordered alphabetically). To define specific layouts for specific objects see 
 * <code>EditorLayoutRegistry</code>
 */
public class BaseReposObjEditor extends AbstractReposObjFormEditor {

    private EditorLayoutDescriptor layoutDescriptor;

    protected FieldSet sectionControlCurrentlyUnderConstruction;

    public BaseReposObjEditor(final String objURL) {

        super();
        
        addListener(Events.Render, new Listener<ComponentEvent>() {
            public void handleEvent(ComponentEvent be) {
                Utils.showWaitDialog();
                
                MainModule.getRemoteClient().getReposObj(objURL, new AbstractUiObjCallBack() {
                    public void processReposObj(final ReposObj loadedObj) {
                        Utils.closeWaitDialog();
                        setInput(loadedObj);
                    }
                });
            }
        });
    }

    protected void doFillForm() {
        if (getInput() != null) {
            
            EditorLayoutRegistry.INSTANCE.getLayout(getInput(), 
                                                    new EditorLayoutRegistryCallBack() {
                    public void layoutLoaded(EditorLayoutDescriptor desc) {
                        layoutDescriptor = desc;
                        doFillForm2();                                    
                    }
                });
        }
    }
    
    private void doFillForm2() {
        //editableFields = new HashMap<BoxComponent, ReposObjAttribute>();
            
        setTitle("");
        if (getInput() == null) {
            return;
        }

//        String titleAsLink = "<a style=\"text-decoration: none;\" href=\""
//                             + getInput().getSelfLink()
//                             + "\" target=\"_blank\">&nbsp;</a>"
//                             + getInput().getCaption();
//        setTitle(titleAsLink);
        
        if (layoutDescriptor == null) {
            applyDefaultLayout();
        } else {
            applyCustomLayout(layoutDescriptor);
        }
        
        addObjectPermissionsPanel();
        
        doLayout();
    }

    private void applyDefaultLayout() {
        LayoutContainer panel = new LayoutContainer();
        panel.setLayout(new FormLayout(LabelAlign.RIGHT));
        
        panel.setAutoWidth(true);
        panel.setAutoHeight(true);
        panel.setMonitorWindowResize(true);
        
        panel.setStyleAttribute("margin", "0px");
        panel.addStyleName("arr-editor-panel");
        
        add(panel, new FormData("100% -10"));
        String[] attrNames = new String[getInput().getAttributes().size()];
        getInput().getAttributes().keySet().toArray(attrNames);
        Arrays.sort(attrNames, new Comparator<String>() {
            public int compare(String s1, String s2) {
                if (s1 == null && s2 != null) {
                    return -1;
                }
                
                if (s1 != null && s2 == null) {
                    return 1;
                }
                
                return s1.toLowerCase().compareTo(s2.toLowerCase());
            }
        });
        
        for (int i = 0; i < attrNames.length; i++) {
            addControlForAttribute(panel, attrNames[i], null);
        }
    }

    private void applyCustomLayout(EditorLayoutDescriptor desc) {
        List<String> unusedAttrNames = new ArrayList<String>(getInput().getAttributes().size());
        unusedAttrNames.addAll(getInput().getAttributes().keySet());
        
        final Widget[] sections = new Widget[desc.getSectionCount()];
        int sectionIndex = 0;
        desc.resetIndex();
        sections[sectionIndex++] = createSectionControl(desc.nextSection(), true, unusedAttrNames);
        while (desc.hasNext()) {
            sections[sectionIndex++] = createSectionControl(desc.nextSection(), false, unusedAttrNames);
        }

        for (Iterator<String> iter = desc.getIgnoredFields().iterator(); iter.hasNext();) {
            unusedAttrNames.remove(iter.next());
        }
        
        for (int i = 0; i < sections.length; i++) {
            //sections[i].setVisible(false);
            add(sections[i]);
        }
        
        if (unusedAttrNames.size() > 0) {
            EditorSection othersSection = new EditorSection();
            othersSection.setCaption("Other");
            EditorColumn column = new EditorColumn();

            for (String attrName : unusedAttrNames) {
                column.addField(attrName);
            }

            othersSection.addColumn(column);

            FieldSet ctrl = createSectionControl(othersSection, false, null);
            //ctrl.setCollapsed(true);
            add(ctrl);
        }
    }

    private FieldSet createSectionControl(EditorSection aSection,
                                          boolean ignoreTitle,
                                          List<String> unusedAttrNames) {
        
        FieldSet sectionControl = FieldWidgetHelper.createFieldSet();
        sectionControlCurrentlyUnderConstruction = sectionControl;
        sectionControl.setHeading(ignoreTitle ? "" : aSection.getCaption());
        sectionControl.setCollapsible(!ignoreTitle);
        
        sectionControl.addStyleName("arr-section-control");
        
        LayoutContainer topPanel = new LayoutContainer();
        topPanel.setBorders(false);
        
        topPanel.setLayout(new ColumnLayout()); 
        sectionControl.add(topPanel);
        
        aSection.resetIndex();
        int columnIndex = 0;
        while (aSection.hasNext()) {
            Component column = createColumnControl(aSection.nextColumn(), unusedAttrNames);
            column.addStyleName(columnIndex++ == 0 ? "arr-first-column" 
                    : (columnIndex == aSection.getColumnCount() - 1 ? "arr-last-column" : "arr-column"));
            
            column.setStyleAttribute("padding", aSection.hasNext() ? "10px 10px 0px 0px" : "10px 0px 0px 0px");
            column.setStyleAttribute("margin", "0px");
            topPanel.add(column, new ColumnData(.98d / aSection.getColumnCount()));
        }

        return sectionControl;
    }
    
    private Component createColumnControl(EditorColumn aCol, List<String> unusedAttrNames) {
        LayoutContainer columnControl = new LayoutContainer();
        columnControl.setBorders(false);
        columnControl.setAutoHeight(true);
        columnControl.addStyleName("arr-column-control");
        
        FormLayout formLayout = new FormLayout();
        columnControl.setLayout(formLayout);
        
        aCol.resetIndex();
        while (aCol.hasNext()) {
            Object aColItem = aCol.nextItem();
            
            if (aColItem instanceof String) {
                addControlForAttribute(columnControl, (String)aColItem, unusedAttrNames);
            } else {
                columnControl.add(createFieldGroupControl((FieldGroup)aColItem, unusedAttrNames),
                                  new FormData("-5"));
            }
        }
        return columnControl;
    }

    private FieldSet createFieldGroupControl(FieldGroup aGroup, List<String> unusedAttrNames) {
        FieldSet groupControl = new FieldSet();
        
        groupControl.setHeading(aGroup.getCaption() == null ? "" : aGroup.getCaption());
        groupControl.setCollapsible(false);
        groupControl.setExpanded(true);
        groupControl.setBorders(false);
        groupControl.setAutoHeight(true);
        
        groupControl.setStyleAttribute("margin", "0px");
        groupControl.setStyleAttribute("padding", "0px");
        groupControl.addStyleName("arr-group-control");
        
        groupControl.setLayout(new FormLayout());
        
        aGroup.resetIndex();
        while (aGroup.hasNext()) {
            addControlForAttribute(groupControl, aGroup.nextFieldId(), unusedAttrNames);
        }
        
        return groupControl;
    }

    private void addControlForAttribute(LayoutContainer container, 
                                        String attrName, 
                                        List<String> unusedAttrNames) {
        Widget fieldControl = createAttrField(attrName);
        
        if (fieldControl != null) {
            
            if (fieldControl instanceof Field) {
                // to allow space for the "error" icon to render during validation failures
                container.add(fieldControl, new FormData("-20"));
            } else {
                container.add(fieldControl, new FormData("100%"));
            }
            
            if (unusedAttrNames != null) {
                unusedAttrNames.remove(attrName);
            }
        }
    }

    private Widget createAttrField(String attrName) {
        ReposObjAttribute attr = getInput().getAttribute(attrName);
        if (attr == null) {
            return null;
        }

        IFormPart formPart = handleNonFieldAttribute(attr);
        if (formPart != null) {
            // Assumes that formPart impl is silently handling the read-only case.
            if (formPart instanceof Component) {
                ((Component)formPart).setStyleAttribute("marginBottom", "4px");
            }
            addPart(formPart);  
            return (Widget)formPart;
        }
        
        Field<?> field = FieldWidgetHelper.createField(attr);
        if (field != null) {
            if (!field.isReadOnly()) {
                addPart(FieldWidgetHelper.adapt(field));
            }
            return field;
        }

        return null;
    }
    
    // Non field as in the return type is not a UI Field widget
    protected IFormPart handleNonFieldAttribute(final ReposObjAttribute reposObjAttribute) {
        IFormPart formPart = null;
        
        if (EClassHelper.PROPERTY_ECLASS.equals(reposObjAttribute.getRefEClass())
                    && "constraints".equals(reposObjAttribute.getName())) {
            formPart = new ConstraintAttrField(reposObjAttribute, 
                                               FieldWidgetHelper.getLabel(reposObjAttribute));
        } else if (EClassHelper.PROPERTY_ECLASS.equals(reposObjAttribute.getRefEClass())) {
            formPart = new PropertyAttrField(reposObjAttribute, 
                                             FieldWidgetHelper.getLabel(reposObjAttribute));
        } else if (reposObjAttribute.isCollection()) {            
            // nested collection (a.k.a: contained references)
            formPart = new CollectionValAttrField(reposObjAttribute, 
                                                  FieldWidgetHelper.getLabel(reposObjAttribute));
//        } else if (ReposObjHelper.isFolderType(reposObjAttribute.getRefEClass())
//                   && !reposObjAttribute.isMulti()) {        
//            formPart = new FolderAttrField(reposObjAttribute, 
//                                           FieldWidgetHelper.getLabel(reposObjAttribute));        
        } else if (reposObjAttribute.isReference()) {            
            // non-contained references
            if (reposObjAttribute.isMulti()) {
                formPart = new RefValAttrField(reposObjAttribute, 
                                               FieldWidgetHelper.getLabel(reposObjAttribute));
            } else {
                formPart = new SingleRefValAttrField(reposObjAttribute, 
                                                     FieldWidgetHelper.getLabel(reposObjAttribute));
            }

//        } else if (ReposObjAttribute.SIMPLE_TYPE_SHORT_STRING.equals(reposObjAttribute.getPrimitiveType())
//                    && reposObjAttribute.isMulti()
//                    && "tags".equals(reposObjAttribute.getName())) {            
//            formPart = new TagsField(reposObjAttribute, 
//                                     FieldWidgetHelper.getLabel(reposObjAttribute));
//            
        } else if (ReposObjAttribute.SIMPLE_TYPE_URI.equals(reposObjAttribute.getPrimitiveType())) {
            formPart = new LinkField(reposObjAttribute);            
        } else if (reposObjAttribute.isMulti()) {
            formPart = new MultiValueAttributeField(reposObjAttribute, 
                                                    FieldWidgetHelper.getLabel(reposObjAttribute));
        }
        
        return formPart;
    }
}
