/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor;

import java.util.Arrays;
import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.extjs.gxt.ui.client.Style.HorizontalAlignment;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.store.Store;
import com.extjs.gxt.ui.client.store.StoreEvent;
import com.extjs.gxt.ui.client.store.StoreSorter;
import com.extjs.gxt.ui.client.widget.Dialog;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.form.AdapterField;
import com.extjs.gxt.ui.client.widget.form.ListField;
import com.extjs.gxt.ui.client.widget.layout.AnchorData;
import com.extjs.gxt.ui.client.widget.layout.FitLayout;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.Widget;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.DualList;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.DualList.Direction;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.DualList.Mode;

/**
 * A simple example of an 'about' dialog box.
 */
public class BaseSelectRefDialog extends Dialog {

    private static final int WINDOW_HEIGHT = 300;
    private static final int WINDOW_WIDTH = 500;

    private ReposObjAttribute updatedAttr;
    private ReposObjCollection candidates;
    private AfterClosureCallBack callBack; // this is for when ok is selected
    
    private DualList<ReposObjDataModel> itemSelector;
    private AdapterField selectorField;
    
    public BaseSelectRefDialog(ReposObjAttribute anUpdatedAttr,
                               ReposObjCollection possibleHrefs,
                               AfterClosureCallBack aCallBack) {

        candidates = possibleHrefs;
        updatedAttr = anUpdatedAttr;
        callBack = aCallBack;
        
        setWidth(WINDOW_WIDTH);
        setHeight(WINDOW_HEIGHT);
        //setAutoWidth(true);
        //setAutoHeight(true);
        
        //setLayout(new AnchorLayout());
        setLayout(new FitLayout());
          
        
        setStyleAttribute("paddings", "2");
        String eClass = EClassHelper.getClassForCollectionLink(possibleHrefs.getSelfLink());
        setIconStyle(EClassHelper.getIconClsForClass(eClass));
        //setBorders(true);
        setBodyBorder(false);
        setFrame(true);
        setModal(true);
        setMaximizable(false);
        setCollapsible(false);

        setCloseAction(CloseAction.HIDE);
        
        //setPlain(false);
        
        if (anUpdatedAttr == null || !anUpdatedAttr.isReference()) {
            setTitle("ERROR!!!!!!!!!!!");
            HTML text = new HTML("this dialog is incompatible with the attribute selected: " + anUpdatedAttr);
            add(text);

        } else {
            Log.debug("\n\n BaseSelectRefDialog creating UI");
            
            // Set the dialog's caption.
            String attrName = updatedAttr.getName();
            setHeading(updatedAttr.isMulti()
                        ? "Modify the " + attrName + (attrName.endsWith("s") ? " selection" : "s selection")
                        : "Select a new " + attrName);

            //add(createForm(anUpdatedAttr), new AnchorData("100% -100"));
            add(createForm(anUpdatedAttr));
            
            setButtons(OKCANCEL);
            setButtonAlign(HorizontalAlignment.RIGHT);
            
            Button btnOk = getButtonBar().getButtonById(Dialog.OK);
            btnOk.setIconStyle("ok-icon");
            btnOk.addSelectionListener(new SelectionListener<ComponentEvent>() {
                @Override
                public void componentSelected(ComponentEvent ce) {
                    updateAttrSelection();
                    callBack.onDialogClosure();
                    close();
                }
            });
            
            Button btnCancel = getButtonBar().getButtonById(Dialog.CANCEL);
            btnCancel.setIconStyle("cancel-icon");
            btnCancel.addSelectionListener(new SelectionListener<ComponentEvent>() {
                @Override
                public void componentSelected(ComponentEvent ce) {
                    close();
                }
            });
            Log.debug("\n\n BaseSelectRefDialog done creating UI");
            
            populate();
        }
    }

    private Widget createForm(ReposObjAttribute anUpdatedAttr) {
        Log.debug("\n\n BaseSelectRefDialog creating form");
        itemSelector = new DualList<ReposObjDataModel>(Mode.APPEND, Direction.LEFT, true);    
        selectorField = new AdapterField(itemSelector);  
        
        //itemSelector.setMode(Mode.INSERT);
                        
        selectorField.setFieldLabel("ext references");  
        selectorField.setName("extref");
        
        selectorField.setHideLabel(false);
        selectorField.setMessageTarget("side");  

        ListField<ReposObjDataModel> fromList = itemSelector.getFromList();
        fromList.setFieldLabel("available " + anUpdatedAttr.getName() + "(s)");
        fromList.setToolTip("available " + anUpdatedAttr.getName() + "(s)");
        fromList.setHideLabel(false);
        fromList.setDisplayField(ReposObjDataModel.FIELD_CAPTION);  
        ListStore<ReposObjDataModel> store = new ListStore<ReposObjDataModel>();  
        store.setStoreSorter(new StoreSorter<ReposObjDataModel>());  
        //store.add(TestData.getShortStocks());  
        fromList.setStore(store); 
        
        
        ListField<ReposObjDataModel> toList = itemSelector.getToList();
        toList.setFieldLabel("assigned " + anUpdatedAttr.getName() + "(s)");
        toList.setToolTip("assigned " + anUpdatedAttr.getName() + "(s)");
        toList.setHideLabel(false);
        toList.setDisplayField(ReposObjDataModel.FIELD_CAPTION);  
        store = new ListStore<ReposObjDataModel>();  
        store.setStoreSorter(new StoreSorter<ReposObjDataModel>());  
        //store.add(TestData.getShortStocks());  
        toList.setStore(store); 
        
        toList.getStore().addListener(Store.DataChanged, new Listener<StoreEvent<ReposObjDataModel>>() {
            public void handleEvent(StoreEvent<ReposObjDataModel> be) {
                validate();
            }
        });

        Log.debug("\n\n BaseSelectRefDialog finished creating form");
        return itemSelector;
    }

    protected void validate() {
        
        List<ReposObjDataModel> values = getSelectedReferences();
        
        if (values.size() == 0 && !updatedAttr.isOptional()) {
            selectorField.forceInvalid(updatedAttr.getName()
                                      + " is a mandatory piece of information. Choose at least one value.");
            //btnOk.disable();
        
        } else if (values.size() > 1 && !updatedAttr.isMulti()) {
            selectorField.forceInvalid(updatedAttr.getName() + " does not accept multiple values.");
            //btnOk.disable();
        
        } else {
            selectorField.clearInvalid();
            //btnOk.enable();
        }

    }

    private void populate() {
        
        Log.debug("\n\n BaseSelectRefDialog populating lists");
        final DualList<ReposObjDataModel> selector = itemSelector;
        
        List<String> preSelectedValues = Arrays.asList(updatedAttr.getValues());
        for (ReposObj entry : candidates.getTheReposObjs()) {
            
            // so that an obj cannot reference itself
            String selfLink = entry.getSelfLink();
            if (updatedAttr.getParent() == null
                    || !updatedAttr.getParent().getSelfLink().equals(selfLink)) {
            
                String href = makeCandidateHrefRelative(selfLink);
    
                if (preSelectedValues.contains(href)) {
                    selector.getToList().getStore().add(new ReposObjDataModel(entry));
                } else {
                    selector.getFromList().getStore().add(new ReposObjDataModel(entry));
                }
            }
        }

        Log.debug("\n\n BaseSelectRefDialog done populating lists");
    }

    private String makeCandidateHrefRelative(String selfLink) {
        return (selfLink != null && selfLink.startsWith(candidates.getBaseLink()))
                ? selfLink.substring(candidates.getBaseLink().length())
                : selfLink;
    }

    private void updateAttrSelection() {
        updatedAttr.clearValues();
        
        List<ReposObjDataModel> values = getSelectedReferences();
        
        for (ReposObjDataModel selection : values) {
            updatedAttr.addRefValue(makeCandidateHrefRelative(selection.getPOJO().getSelfLink()),
                                    selection.getPOJO().getCaption());
        }
    }

    private List<ReposObjDataModel> getSelectedReferences() {
        return itemSelector.getToList().getStore().getModels();
    }
    
    public boolean onKeyDownPreview(char key, int modifiers) {
        switch (key) {
        case KeyboardListener.KEY_ENTER:
            updateAttrSelection();
            callBack.onDialogClosure();
            close();
            break;
        case KeyboardListener.KEY_ESCAPE:
            close();
            break;
        default:
            break;

        }

        return true;
    }

/*    private class MyDualListField extends DualListField<ReposObjDataModel> {
        
        @Override
        protected void initLists() {
            fromField.setAutoHeight(false);
            fromField.setAutoWidth(false);
            toField.setAutoHeight(false);
            toField.setAutoWidth(false);
            
            toField.setHideLabel(false);
            toField.setFieldLabel("target");
            
            fromField.setHeight(WINDOW_HEIGHT - 75);
            fromField.setWidth((WINDOW_WIDTH / 2) - 45);  
            
            toField.setHeight(WINDOW_HEIGHT - 75);
            toField.setWidth((WINDOW_WIDTH / 2) - 45);  

        }
    }
*/    

}
