/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.GridEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.grid.ColumnData;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.extjs.gxt.ui.client.widget.grid.GridCellRenderer;
import com.extjs.gxt.ui.client.widget.layout.ColumnLayout;
import com.extjs.gxt.ui.client.widget.layout.TableLayout;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Widget;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.base.client.remote.ICollectionCallBack;
import com.iona.soa.web.repository.base.client.remote.IDeletionCallBack;
import com.iona.soa.web.repository.base.client.remote.IObjCallBack;
import com.iona.soa.web.repository.main.client.HtmlSnippets;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.UIMessages;
import com.iona.soa.web.repository.main.client.base.AbstractUiCollectionCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.IFormPart;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.RemoteCollectionPanel;
import com.iona.soa.web.repository.main.client.base.ui.RemoteRestDataProxy;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.FieldWidgetHelper;

public class ObjectPermissionsSummaryPanel extends LayoutContainer implements IFormPart {
    private static final HtmlSnippets SNIPPETS = GWT.create(HtmlSnippets.class);

    private static final UIMessages MESSAGES = GWT.create(UIMessages.class);
    
    private static final String[] ATTRIBUTE_NAMES  = {
            "rolesRead", "rolesUpdate", "rolesDelete",
            "usersRead", "usersUpdate", "usersDelete"
    };

    
    private HTML editAccessLink;
    
    private Button restoreDefaultsButton;
    
    private HTML accessSummaryLabel;
    
    private LayoutContainer editPanel;

    private EditAccessPanel editPanelContents;
    
    private String objectTypeLabel;

    private ReposObj editing;
    
    private ReposObj objectPermissions;
    private ReposObj originalObjectPermissions;

    private boolean customSecurityApplied;
    
    public ObjectPermissionsSummaryPanel(ReposObj editedObj) {
        this.editing = editedObj;
        objectTypeLabel = EClassHelper.getNameForClass(editing.getEClass()).toLowerCase();

        // Has the objectControl reference been set?
        customSecurityApplied = getObjectControlAttribute().getCollectionSize() > 0;

        editAccessLink = new HTML();
        editAccessLink.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                if (editPanel == null) {
                    fetchPermissions();
                } else {
                    showHideEditPanel();
                }                
            }
        });
        restoreDefaultsButton = new Button(MESSAGES.restoreDefaults());                
        restoreDefaultsButton.addListener(Events.Select, new Listener<ButtonEvent>() {
            public void handleEvent(ButtonEvent be) {
                restoreDefaults();
            }
        });
        restoreDefaultsButton.addStyleName("arr-object-permissions-defaults-btn");
        restoreDefaultsButton.setVisible(false);
        
        accessSummaryLabel = new HTML();
        //accessSummaryLabel.addClass("arr-object-permissions-defaulted");        
        updateConfigureAccessLink(false);
        updateAccessInfoLabel(false);
        
        LayoutContainer holder = new LayoutContainer();
        holder.setBorders(false);
        holder.setLayout(new TableLayout(2));
        
        holder.add(restoreDefaultsButton);
        holder.add(editAccessLink);
        add(holder);
        add(accessSummaryLabel);
    }

    // IFormPart implementation
    public void commit() {
        if (editPanelContents != null) {
            editPanelContents.commit();
        }
    }

    // IFormPart implementation
    public boolean isDirty() {
        if (editPanelContents == null || originalObjectPermissions == null) {
            return false;
        }
        for (String attr : ATTRIBUTE_NAMES) {
            String[] origVals = originalObjectPermissions.getAttribute(attr).getValues();
            String[] currentVals = objectPermissions.getAttribute(attr).getValues();
            if (origVals.length != currentVals.length) {
                return true;
            }
            Collection<String> temp = Arrays.asList(currentVals);
            for (String s : origVals) {
                if (!temp.contains(s)) {
                    return true;
                }
            }
        }
        return false;
    }

    // IFormPart implementation
    public void refresh() {
        // do nothing
    }

    // IFormPart implementation
    public void setFormInput(Object input) {
        // do nothing
    }
    
    /**
     * Call from editor it's nested in, supplying the containing object 
     */
    public void saveChanges(final ReposObj beingSaved, final IObjCallBack callback) {
        if (!isDirty()) {
            // The "Configure" link was never clicked or
            // no actual changes made, just continue saving
            callback.processReposObj(beingSaved);
        } else if (objectPermissions.getSelfLink() != null)  {
            if (customSecurityApplied) {
                // pre-existent object was changed
                MainModule.getRemoteClient().updateReposObj(objectPermissions, 
                                                            callback);
            } else {
                // pre-existent object was reset to default
                MainModule.getRemoteClient().deleteReposObj(objectPermissions, new IDeletionCallBack() {
                    public void processSuccess() {
                        callback.processReposObj(objectPermissions);
                    }
                    public void processConnectionError(Throwable e) {
                        callback.processConnectionError(e);
                    }
                    public void processResponseError(String anErrorCode, String anErrorMsg) {
                        callback.processResponseError(anErrorCode, anErrorMsg);
                    }
                });                           
            }
        } else {
            // Need to create new ObjectPermission object
            MainModule.getRemoteClient().createReposObj(getObjectControlAttribute().getCollectionLink(),
                                                        objectPermissions,
                                                        callback);
        }
    }

    private void fetchPermissions() {
        if (customSecurityApplied) {
            String objectControlURL = getObjectControlAttribute().getCollectionLink();
            objectControlURL = ReposHandle.getAbsoluteLink(objectControlURL);
            MainModule.getRemoteClient().getReposObjList(objectControlURL,
                                                         null,
                                                         null,
                                                         new AbstractUiCollectionCallBack() {
                    @Override
                    public void processReposObjCollection(ReposObjCollection list) {
                        objectPermissions = list.getTheReposObjs().get(0);
                        originalObjectPermissions = objectPermissions.deepCopy(); 
                        showHideEditPanel();
                    }
                });
        } else {
            // Get the default permission object and use as initial contents for 
            // a new objectControl.
            fetchDefaultPermissions(new AbstractUiCollectionCallBack() {
                @Override
                public void processReposObjCollection(ReposObjCollection list) {
                    List<ReposObj> response = list.getTheReposObjs();
                    if (response.size() == 1) {
                        objectPermissions = response.get(0);
                        objectPermissions.setSelfLink(null);
                        originalObjectPermissions = objectPermissions.deepCopy(); 
                        showHideEditPanel();
                    }
                }
            });
        }
    }

    private void restoreDefaults() {
        fetchDefaultPermissions(new AbstractUiCollectionCallBack() {
            @Override
            public void processReposObjCollection(ReposObjCollection list) {
                List<ReposObj> response = list.getTheReposObjs();
                if (response.size() == 1) {
                    customSecurityApplied = false;
                    restoreDefaultsButton.setEnabled(customSecurityApplied);
                    ReposObj defaultPermissions  = response.get(0);
                    for (String attrName : ATTRIBUTE_NAMES) {
                        ReposObjAttribute attrib = objectPermissions.getAttribute(attrName);
                        attrib.clearValues();
                        for (String href : defaultPermissions.getAttribute(attrName).getValues()) {
                            attrib.addRefValue(href, "");
                        }
                    }
                    editPanelContents.reload();
                }
            }
        });
    }
    
    private void fetchDefaultPermissions(ICollectionCallBack callback) {
        String objectControlURL = getObjectControlAttribute().getCollectionLink();
        objectControlURL = ReposHandle.getAbsoluteLink(objectControlURL);
        objectControlURL = objectControlURL.replaceFirst(";objectControl", 
            ";defaultPermission");
        MainModule.getRemoteClient().getReposObjList(objectControlURL,
                                                     null,
                                                     null,
                                                     callback);
    }
    
    private void addEditPanel() {
        editPanelContents = new EditAccessPanel();
        
        editPanel = new LayoutContainer();
        editPanel.setBorders(false);
        editPanel.setStyleAttribute("margins", "12px 0px 0px 0px");
        editPanel.add(editPanelContents);
        editPanel.setVisible(false);
        editPanel.addStyleName("arr-object-permissions-edit");

        add(editPanel);
        layout();
    }
    
    private void showHideEditPanel() {
        
        boolean configuring = editPanel == null ? false : editPanel.isVisible();
        if (editPanel == null) {
            addEditPanel();
        }
        configuring = !configuring; // change current editing state
        
        updateConfigureAccessLink(configuring);
        updateAccessInfoLabel(configuring);
        
        restoreDefaultsButton.setVisible(configuring);
        restoreDefaultsButton.setEnabled(customSecurityApplied);
        editPanel.setVisible(configuring);  
        editPanel.layout();
        
    }
    
    private void updateConfigureAccessLink(boolean configuring) {
        String cssCls = "arr-action-link arr-specify-access-permissions";
        if (configuring) {
            editAccessLink.setHTML(SNIPPETS.anchorTagWithCssCls(MESSAGES.iAmDoneConfiguring(),
                                                                cssCls));                
        } else {
            String configureAccessSecurity = MESSAGES.configureAccessSecurity(objectTypeLabel);
            editAccessLink.setHTML(SNIPPETS.anchorTagWithCssCls(configureAccessSecurity, cssCls));
        }
    }
    
    private void updateAccessInfoLabel(boolean configuring) {
        // If objectControl ref set, then definitely custom access defined.
        //
        // Otherwise:
        // We need to consider the state of the objectPermisions object.  It 
        // will be non-null if the user selects the "configure" link.
        // Then we decide if the objectPermisions object actually contains any
        // values, i.e. the user has selected roles/users.  Cheat by using a boolean
        // flag that is set to true whenever a checkbox change is made.
        
        if (customSecurityApplied) {
            accessSummaryLabel.setHTML(MESSAGES.customAccessSpecified(objectTypeLabel));            
        } else {
            accessSummaryLabel.setHTML(MESSAGES.defaultAccessInEffect());
        }
        
        accessSummaryLabel.setVisible(!configuring);
    }
    
    private ReposObjAttribute getObjectControlAttribute() {
        return editing.getAttribute("objectControl");
    }
    
    private final class EditAccessPanel extends LayoutContainer {
        private EditorGrid<ReposObjDataModel> rolesGrid;
        private EditorGrid<ReposObjDataModel> usersGrid;
        
        // Ideally rolesGrid would be the actual part.  But we'd need to get
        // BaseCollectionPanel behaving very well in terms of implementing IFormPart
        private IFormPart rolesFormPart;
        private IFormPart usersFormPart;
        
        EditAccessPanel() {
            
            setLayout(new ColumnLayout());
            setStyleAttribute("margin", "10px");
            setBorders(false);
            
            RemoteCollectionPanel rolesPanel = cratePanel(EClassHelper.ROLE_ECLASS);
            rolesGrid = rolesPanel.getGrid();
            LayoutContainer left = createContainer(MESSAGES.specifyRolesWithAccess());
            left.add(rolesPanel);
            left.addStyleName("arr-object-permissions-roles");

            RemoteCollectionPanel usersPanel = cratePanel(EClassHelper.USER_ECLASS);
            usersGrid = usersPanel.getGrid();
            LayoutContainer right = createContainer(MESSAGES.specifyUsersWithAccess());
            right.add(usersPanel);
            right.addStyleName("arr-object-permissions-users");
            
            add(left, new com.extjs.gxt.ui.client.widget.layout.ColumnData(.5));
            add(right, new com.extjs.gxt.ui.client.widget.layout.ColumnData(.5));
            
            rolesFormPart = FieldWidgetHelper.adapt(rolesGrid);
            usersFormPart = FieldWidgetHelper.adapt(usersGrid);
        }
        
        private LayoutContainer createContainer(String title) {
            LayoutContainer p = new LayoutContainer();
            p.setBorders(false);
            HTML lbl = new HTML();
            lbl.setHTML(title);
            p.add(lbl);
            return p;
        }
        
        private RemoteCollectionPanel cratePanel(String eClass) {
            RemoteRestDataProxy dataProxy = RemoteRestDataProxy.createProxyForEClass(eClass);
            
            SimpleGridConfigurator gridConfig = new AccessGridConfig(EClassHelper.getContextForClass(eClass));
            gridConfig.configure(new EditorGrid<ReposObjDataModel>(null, null), dataProxy);
            
            RemoteCollectionPanel panel = new RemoteCollectionPanel(gridConfig, dataProxy);
            panel.getGrid().setHideHeaders(false);
            
            panel.setHeaderVisible(false);
            return panel;
        }
        
        void reload() {
            rolesGrid.getStore().getLoader().load();
            usersGrid.getStore().getLoader().load();
        }
        
        void commit() {
            rolesFormPart.commit();
            usersFormPart.commit();
        }
        
        boolean isDirty() {
            return false;
        }
    }
    
    private final class AccessGridConfig extends SimpleGridConfigurator {
        private String eClassPlural; // Roles or Users
        
        AccessGridConfig(String targetClassPlural) {
            eClassPlural = targetClassPlural;
        }
        
        @Override
        protected void addVisibleColumns() {
            addNameColumn(false);
            
            columns.add(new CRUDOperationColumnConfig(CRUDOperation.READ,
                                                      eClassPlural + CRUDOperation.READ.getLabel()));
            columns.add(new CRUDOperationColumnConfig(CRUDOperation.UPDATE,
                                                      eClassPlural + CRUDOperation.UPDATE.getLabel()));
            columns.add(new CRUDOperationColumnConfig(CRUDOperation.DELETE,
                                                      eClassPlural + CRUDOperation.DELETE.getLabel()));
        }

        @Override
        public void configure(final EditorGrid<ReposObjDataModel> grid,
                              RemoteRestDataProxy dataProxy,
                              boolean includeCheckboxColumn) {
            super.configure(grid, dataProxy, includeCheckboxColumn);
            
            grid.addStyleName(eClassPlural); // makes grid identifable in selenium
            grid.setAutoHeight(true);
            
            Listener<GridEvent> listener =
                new Listener<GridEvent>() {
                
                    public void handleEvent(GridEvent ge) {
                        if (ge.colIndex > 0) { // any column other than name

                            ReposObjDataModel selectedItem = 
                                (ReposObjDataModel)ge.grid.getStore().getModels().get(ge.rowIndex);
                            ReposObj userOrRole = selectedItem.getPOJO();
                            String attributeId = grid.getColumnModel().getColumnId(ge.colIndex);
                            //@SuppressWarnings("unused")
                            boolean isChecked = changeCheckedState(attributeId, userOrRole);
                            selectedItem.set(grid.getColumnModel().getColumnId(ge.colIndex), 
                                                    Boolean.valueOf(isChecked).toString());
                            grid.getView().refresh(false);
                            customSecurityApplied = true;
                            restoreDefaultsButton.setEnabled(customSecurityApplied);
                        }
                        
                        ge.doit = false;
                    }
                };
            
            grid.addListener(Events.RowClick, listener);
        }
        
        boolean changeCheckedState(String attributeId, ReposObj userOrRole) {
            ReposObjAttribute attribute = objectPermissions.getAttribute(attributeId);
            if (attributeContainsRef(attributeId, userOrRole)) {
                String userOrRoleRef = ReposHandle.getRelativeLink(userOrRole.getSelfLink());
                attribute.removeRefValue(userOrRoleRef);
            } else {
                attribute.addRefValue(userOrRole);
            }
            return attributeContainsRef(attributeId, userOrRole);
        }
        
        private boolean attributeContainsRef(String attributeId,
                                             ReposObj ref) {
            
            ReposObjAttribute attribute = objectPermissions.getAttribute(attributeId);
            String userOrRoleRef = ReposHandle.getRelativeLink(ref.getSelfLink());
            String[] hrefs = attribute.getValues();
            for (int i = 0; i < hrefs.length; i++) {
                if (userOrRoleRef.equals(hrefs[i])) {
                    return true;
                }
            }
            
            return false;
        }
        
        private class CRUDOperationColumnConfig extends SimpleColumnConfig {
            public CRUDOperationColumnConfig(CRUDOperation type,
                                             String attributeId) {
                super(attributeId, type.getLabel());
                setSortable(false);
                setWidth(50);
                setRenderer(new GridCellRenderer<ReposObjDataModel>() {
                    public String render(ReposObjDataModel record,
                                         String property,
                                         ColumnData config,
                                         int rowIndex,
                                         int colIndex,
                                         ListStore<ReposObjDataModel> store) {
                        String prop = record.get(property);
                        boolean checked = Boolean.parseBoolean(prop);
                        String tooltip = "";
                        // Include the header (Read/Update/Delete) in the css class for
                        // easy identification in Selenium tests. 
                        String cssCls = "checkbox " + getHeader();
                        
                        return SNIPPETS.imageTagWithCssCls("images/default/menu/"
                                                    + (checked ? "checked.gif" : "unchecked.gif"), 
                                                    tooltip, 
                                                    cssCls);
                    }
                });
            }

            @Override
            public Object getCellData(ReposObj userOrRole) {
                String attributeId = getId();
                // Our RecordDef uses String types for all fields
                return Boolean.valueOf(attributeContainsRef(attributeId, userOrRole)).toString();
            }            
        }
    }
    
    private static final class CRUDOperation {
        public static final CRUDOperation CREATE = new CRUDOperation("Create");
        public static final CRUDOperation READ = new CRUDOperation("Read");
        public static final CRUDOperation UPDATE = new CRUDOperation("Update");
        public static final CRUDOperation DELETE = new CRUDOperation("Delete");
        
        private final String type;
        
        private CRUDOperation(String type) {
            this.type = type;
        }
        
        public String getLabel() {
            return type;
        }
        
        public String toString() {
            return type;
        }
    }
}
