/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor.custom;

import java.util.ArrayList;
import java.util.List;

import com.extjs.gxt.ui.client.core.El;
import com.extjs.gxt.ui.client.widget.Component;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.form.FieldSet;
import com.extjs.gxt.ui.client.widget.layout.ColumnData;
import com.extjs.gxt.ui.client.widget.layout.TableLayout;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.base.client.remote.ICollectionCallBack;
import com.iona.soa.web.repository.base.client.remote.RemoteClient;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiCollectionCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.IFormPart;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.ui.editor.BaseReposObjEditor;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.FieldWidgetHelper;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.RefValAttrField;
import com.iona.soa.web.repository.main.client.provisioning.panels.MavenRepoSearchAndResultsPanel;
import com.iona.soa.web.repository.main.client.provisioning.panels.MavenRepoSearchAndResultsPanel.AddArtifactsCallback;

public class ApplicationEditor extends BaseReposObjEditor {

    protected LayoutContainer nestedFormPartHolder;
    
    private IFormPart nestedFormPart;
    
    public ApplicationEditor(String objURL) {
        super(objURL);
    }

    // Non field as in the return type is not a UI Field widget
    @Override
    protected IFormPart handleNonFieldAttribute(final ReposObjAttribute reposObjAttribute) {
        if (EClassHelper.ARTIFACT_ECLASS.equals(reposObjAttribute.getRefEClass())
                && "artifacts".equals(reposObjAttribute.getName())) {
            
            final FieldSet relevantSection = sectionControlCurrentlyUnderConstruction;
            // We're in the application editor
            return new RefValAttrField(reposObjAttribute, 
                                       FieldWidgetHelper.getLabel(reposObjAttribute)) {
                @Override
                protected void runAddCommand() {
                    openArtifactSearchWindow(this, relevantSection, reposObjAttribute);
                }
            };
        }        
        return super.handleNonFieldAttribute(reposObjAttribute);
    }
            
    private void openArtifactSearchWindow(final RefValAttrField field,
                                          final FieldSet anchor,
                                          final ReposObjAttribute owningRefList) {        
        AddArtifactsCallback addCallback = new AddArtifactsToAppCallbackImpl(owningRefList, field);
        
        insertFormPart(field,
                       anchor, 
                       new MavenRepoSearchAndResultsPanel(addCallback), 
                       "Select artifacts to add to this application");
    }
    
    // TODO - make it a IFormPart that implements isDirty
    private void insertFormPart(Component forField,
                                final FieldSet anchor,
                                final IFormPart formPart, 
                                String editorTitle) {
        if (nestedFormPart != null) {
            return;
        }
        
        El parent = forField.el().findParent("fieldset", 10);

        if (parent == null) {
            return;
        }
        
        
        
        nestedFormPartHolder = getFormPartHolder(editorTitle, formPart);
        
        //nestedFormPartHolder.setHeight(1000);
        
        /*nestedFormPartHolder.render(parent.dom);
        parent.appendChild(nestedFormPartHolder.getElement());
        ComponentHelper.doAttach(nestedFormPartHolder);*/
        //nestedFormPartHolder.layout();
        
        anchor.add(nestedFormPartHolder, new ColumnData(1));
        anchor.layout();
        DeferredCommand.addCommand(new Command() {
            public void execute() {
                nestedFormPartHolder.getElement().scrollIntoView();
            }
        });
        
        // Get it managed in terms of isDirty
        addPart(nestedFormPart);
    }

    private LayoutContainer getFormPartHolder(String editorTitle, final IFormPart formPart) {
        
        nestedFormPart = formPart;
        
        LayoutContainer holder = new LayoutContainer();
        holder.addStyleName("arr-inner-editor");
        holder.setBorders(true);
        holder.setAutoHeight(true);
        holder.setAutoWidth(true);        
        
        LayoutContainer titleHolder = new LayoutContainer();
        titleHolder.setLayout(new TableLayout(2));
        titleHolder.addStyleName("arr-inner-editor-title");
        titleHolder.setBorders(false);
        HTML cancelLink = new HTML(SNIPPETS.anchorTagWithCssCls("Cancel",
                                                                "arr-action-link arr-cancel-action-link"));
        cancelLink.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                removeNestedFormPart();
            }
        });
        titleHolder.add(new Label(editorTitle));
        titleHolder.add(cancelLink);
        
        holder.add(titleHolder);
        holder.add((Widget)nestedFormPart);
        
        return holder;
    }

    private void removeNestedFormPart() {
        if (nestedFormPart != null) {
            removePart(nestedFormPart);
            nestedFormPart = null;
            
            nestedFormPartHolder.setVisible(false);
            nestedFormPartHolder.removeFromParent();
            //nestedFormPartHolder.destroy();
            nestedFormPartHolder = null;
        }
    }
    
    private static boolean isPersistedArtifact(ReposObj artifact) {
        String guid = artifact.getAttributeValue("guid"); 
        return guid != null && !"".equals(guid); 
    }
    
    private static void importArtifacts(final List<ReposObj> artifacts, 
                                        final ICollectionCallBack createdCallback) {
        final RemoteClient client = MainModule.getRemoteClient();
        final String artifactCollectionURL =
            EClassHelper.getCollectionLinkForClass(EClassHelper.ARTIFACT_ECLASS);
         
        client.createReposObjs(artifactCollectionURL, 
                               artifacts,
                               createdCallback);
    }    
    
    private final class AddArtifactsToAppCallbackImpl implements AddArtifactsCallback {
        private final ReposObjAttribute artifactsRefList;
        private final RefValAttrField uiField;
        
        AddArtifactsToAppCallbackImpl(ReposObjAttribute owningRefList,
                                      RefValAttrField appsField) {
            artifactsRefList = owningRefList;
            uiField = appsField;
        }
        
        public void add(List<ReposObj> artifacts) {
            if (artifacts.isEmpty()) {
                removeNestedFormPart();
                return;
            }
            
            // We hijacked the editing part of the default field, so need
            // to let it know what happened - it represents us in the editor.
            uiField.markAsDirty();
            
            boolean needToImport = false;
            for (ReposObj artifact : artifacts) {
                if (!isPersistedArtifact(artifact)) {
                    needToImport = true;
                    break;
                }
            }
            
            if (needToImport) {
                final List<ReposObj> toImport = new ArrayList<ReposObj>();
                final List<ReposObj> toAdd = new ArrayList<ReposObj>();
                for (ReposObj artifact : artifacts) {
                    if (!isPersistedArtifact(artifact)) {
                        toImport.add(artifact); // import first, then add to "toAdd" list later
                    } else {
                        toAdd.add(artifact); // add directly
                    }
                }
                DeferredCommand.addCommand(new Command() {
                    public void execute() {
                        nestedFormPartHolder.el().mask("Importing artifacts...");
                        importArtifacts(toImport, new AbstractUiCollectionCallBack(nestedFormPartHolder) {
                            @Override
                            public void processReposObjCollection(ReposObjCollection list) {
                                nestedFormPartHolder.el().unmask();
                                removeNestedFormPart();
                                toAdd.addAll(list.getTheReposObjs()); // add the imported ones
                                new AddArtifactsToAppCommand(toAdd,
                                                             artifactsRefList,
                                                             uiField).execute();
                            }
                        });                                
                    }                        
                });
            } else {
                removeNestedFormPart();
                // Give more time to allow visuals to refresh
                DeferredCommand.addCommand(new AddArtifactsToAppCommand(artifacts, 
                                                                        artifactsRefList, 
                                                                        uiField));
            }
        }
        
        public void cancel() {
            removeNestedFormPart();
        }
        
        public boolean alreadyInUse(ReposObj artifact) {
            String[] hrefs = artifactsRefList.getValues();
            String absoluteRef = ReposHandle.getAbsoluteLink(artifact.getSelfLink());
            for (int i = 0; i < hrefs.length; i++) {
                String absoluteExistingRef = ReposHandle.getAbsoluteLink(hrefs[i]);
                if (absoluteRef.equals(absoluteExistingRef)) {
                    // would be nice if it displayed "Already exists" - would
                    // have to remove the anchor also.
                    // Alternatively filter the record out.
                    return true;
                }
            }
            return false;
        }
        
    }
    
    private static final class AddArtifactsToAppCommand implements Command {
        private final List<ReposObj> artifacts;
        private final ReposObjAttribute artifactsRefList;
        private final RefValAttrField uiField;
        
        AddArtifactsToAppCommand(List<ReposObj> artifactsToAdd,
                                 ReposObjAttribute owningRefList,
                                 RefValAttrField appsField) {
            artifacts = artifactsToAdd;
            artifactsRefList = owningRefList;
            uiField = appsField;
        }
        
        public void execute() {
            for (ReposObj artifact : artifacts) {
                artifactsRefList.addRefValue(artifact);
            }
            uiField.refresh();
        }
    }
}
