/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor.custom;

import java.util.HashMap;
import java.util.Map;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.data.BasePagingLoader;
import com.extjs.gxt.ui.client.data.LoadEvent;
import com.extjs.gxt.ui.client.data.ModelData;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.LoadListener;
import com.extjs.gxt.ui.client.event.WindowEvent;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.store.StoreEvent;
import com.extjs.gxt.ui.client.store.StoreListener;
import com.extjs.gxt.ui.client.widget.Dialog;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.form.Field;
import com.extjs.gxt.ui.client.widget.form.FieldSet;
import com.extjs.gxt.ui.client.widget.form.ListField;
import com.extjs.gxt.ui.client.widget.form.Radio;
import com.extjs.gxt.ui.client.widget.form.RadioGroup;
import com.extjs.gxt.ui.client.widget.layout.ColumnData;
import com.extjs.gxt.ui.client.widget.layout.FormData;
import com.google.gwt.user.client.Element;

import com.google.gwt.user.client.ui.Label;

import com.iona.soa.web.repository.base.client.model.DeploymentProfileHelper;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiCollectionCallBack;
import com.iona.soa.web.repository.main.client.base.AbstractUiDeletionCallBack;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.IManagedForm;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.ReposURL;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.MultiFieldPanel;
import com.iona.soa.web.repository.main.client.base.ui.RemoteRestDataProxy;
import com.iona.soa.web.repository.main.client.base.ui.ScheduledTaskPanelHelper;
import com.iona.soa.web.repository.main.client.base.ui.BaseObjectDialogWindow.AfterCreationCallBack;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.ObjectCreationDialog;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.AbstractReposObjFormEditor;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.ConstraintOverrideAttrField;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.DualList;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.FieldWidgetHelper;
import com.iona.soa.web.repository.main.client.base.ui.editor.widgets.PropertyOverrideAttrField;

public class DeploymentProfileEditor extends AbstractReposObjFormEditor {

    private Field<String> nameField;
    private Radio scheduled;
    private Radio scheduleOnSave;
    private Radio deployNow;
    private Radio deployLater;

    private RadioGroup radioGroup;
    private boolean performedFindRelatedObject;

    private ReposObj relatedObj;
    private String parentGuid;
    
    public DeploymentProfileEditor(ReposURL profileURL) {
        super();
        
        addListener(Events.Render, new LoadHelper(profileURL));
    }

    protected void doFillForm() {


        if (findRelatedObjectPerformed() == false) {
            prepareDeploymentPanel();
            return;
        }
        ReposObjAttribute nameAttr = getInput().getAttribute("name");
        nameField = FieldWidgetHelper.createField(nameAttr);
        nameField.setFieldLabel("Profile name");
        nameField.setLabelSeparator("");

        add(nameField, new FormData("50%"));

        ConfigurationsContainer configurationOverrides = new ConfigurationsContainer();
        ApplicationsContainer appsContainer = new ApplicationsContainer(configurationOverrides);
        RadioGroup group = createDeployRadioGroup();
        
        add(appsContainer);
        add(group, new FormData("100%"));
        add(configurationOverrides);
        addObjectPermissionsPanel();
    }

    private RadioGroup createDeployRadioGroup() {
        class ColorRadio extends Radio{
            @Override
            public void setValue(Boolean value) {
                super.setValue(value);
                if (value) {
                    setStyleAttribute("color", "red");
                } else {
                    setStyleAttribute("color", "black");
                }
            }
        };
        
        RadioGroup group = new RadioGroup();
        deployNow = new ColorRadio();
        deployLater = new ColorRadio();
        scheduled = new ColorRadio();
        scheduleOnSave = new ColorRadio();
        deployNow.setBoxLabel("Deploy On Save");
        deployLater.setBoxLabel("Don't Schedule Deployment");
        // check to see if there is a schedule in place already
        ReposObj overrideObj = (ReposObj)getRelatedObject();
        if (overrideObj != null) {
            String time = overrideObj.getAttributeValue("nextRun");
            scheduled.setBoxLabel("Scheduled for " + time);
            scheduled.setValue(true);
            scheduled.setStyleAttribute("color", "red");
            scheduleOnSave.setBoxLabel("Reschedule Deploy on Save");
            group.add(scheduled);
            group.add(scheduleOnSave);
            group.add(deployNow);
            group.add(deployLater);
        
        } else {
            scheduleOnSave.setBoxLabel("Schedule Deploy on Save");
            group.add(scheduleOnSave);
            deployLater.setValue(true);
            group.add(deployNow);
            group.add(deployLater);
        }
        group.setLabelSeparator("");
        return group;
    }

    // Lst method called in construction, after all loading etc.. has been done
    private void applicationsPanelReady(DualList<ModelData> applicationsList) {
        // Now make base IManagedForm aware of the parts.
        addPart(FieldWidgetHelper.adapt(nameField));
        addPart(FieldWidgetHelper.adapt(applicationsList.getToList()));
        // createConfigPanels adds its own panels as parts.
    }

    

    
    private class ApplicationsContainer extends LayoutContainer {
        private DualList<ModelData> applicationsList;

        private final ConfigurationsContainer overridesContainer;
        
        public ApplicationsContainer(ConfigurationsContainer overrides) {
            overridesContainer = overrides;
        }
        
        @Override
        protected void onRender(Element parent, int index) {
            super.onRender(parent, index);
            add(createApplicationsPanel());
        }

        private DualList<ModelData> createApplicationsPanel() {
            applicationsList = new DualList<ModelData>();
            applicationsList.addStyleName("arr-profile-applications");
            applicationsList.getFromList().setFieldLabel("All applications");
            applicationsList.getFromList().setEmptyText("No applications available");
            applicationsList.getToList().setFieldLabel("Applications in this profile");
            applicationsList.getToList().setEmptyText("None");
            // LHS = chosen applications, i.e. in this profile
            // RHS = available applications
            // applicationsPanel.getFromList().setEmptyText("None"); // chosen
            //applicationsPanel.getToList().setEmptyText("No applications available"
            // );

            ListField<ModelData> chosenApps = applicationsList.getToList();
            chosenApps.setDisplayField("name");
            ListStore<ModelData> store = new ListStore<ModelData>();
            chosenApps.setStore(store);

            ListField<ModelData> availableApps = applicationsList.getFromList();
            availableApps.setDisplayField("name");
            RemoteRestDataProxy proxy = RemoteRestDataProxy.createProxyForEClass(EClassHelper.APPLICATION_ECLASS);

            BasePagingLoader<PagingLoadConfig, PagingLoadResult<ReposObjDataModel>> loader =
                new BasePagingLoader<PagingLoadConfig, PagingLoadResult<ReposObjDataModel>>(proxy);
            loader.setRemoteSort(true);

            store = new ListStore<ModelData>(loader);
            availableApps.setStore(store);

            availableApps.getStore().getLoader().addLoadListener(new LoadListener() {
                @Override
                public void loaderLoad(LoadEvent le) {
                    applicationsLoaded();
                }
            });

            loader.load();
            return applicationsList;
        }

        private void applicationsLoaded() {
            // Prune applications available list, moving items to the chosen
            // list if they are in the applications reference list in the profile.
            ListStore<ModelData> availableApps = applicationsList.getFromList().getStore();
            ListStore<ModelData> chosenApps = applicationsList.getToList().getStore();
            ReposObjAttribute attr = getInput().getAttribute("applications");
            for (String href : attr.getValues()) {
                ModelData app = availableApps.findModel(ReposObjDataModel.FIELD_HREF, href);
                if (app != null) {
                    availableApps.remove(app);
                    chosenApps.add(app);
                }
            }

            // Now that the apps are loaded and in the correct list, add a 
            // listener to react to the add/remove from/to buttons
            chosenApps.addStoreListener(new StoreListener<ModelData>() {
                @Override
                public void storeAdd(StoreEvent<ModelData> se) {
                    ModelData model = (ModelData) se.store.getModels().get(se.index);
                    ReposObj app = model.get(ReposObjDataModel.FIELD_POJO);
                    applicationAddedCallback(app);
                }

                @Override
                public void storeRemove(StoreEvent<ModelData> se) {
                    ModelData model = se.model;
                    ReposObj app = model.get(ReposObjDataModel.FIELD_POJO);
                    applicationRemovedCallback(app);
                }
            });
            
            loadConfigOverrides();
        }

        private void loadConfigOverrides() {
            AbstractUiCollectionCallBack overridesLoadedCallback = new AbstractUiCollectionCallBack() {
                @Override
                public void processReposObjCollection(ReposObjCollection list) {
                    // The chosen apps have been moved to this the LHS list after load.
                    ListStore<ModelData> configuredApps = applicationsList.getToList().getStore();
                    
                    // Match each override obj to its application
                    for (ReposObj overrideObj : list.getTheReposObjs()) {
                        String linkedAppHref = overrideObj.getAttributeValue("application");
                        linkedAppHref = ReposHandle.getRelativeLink(linkedAppHref);
                        ModelData app = configuredApps.findModel("href", linkedAppHref);
                        if (app != null) {
                            ReposObj application = app.get(ReposObjDataModel.FIELD_POJO);
                            overridesContainer.addConfiguration(application, overrideObj);
                        }
                    }
                    applicationsPanelReady(applicationsList);
                }
            };            
            
            // load the app cfg panels
            String overridesHref =
                getInput().getAttribute("configurationOverride").getCollectionLink();
            
            MainModule.getRemoteClient().getReposObjList(ReposHandle.getAbsoluteLink(overridesHref), null, null,
                                                         overridesLoadedCallback);
        }
        
        private void applicationAddedCallback(final ReposObj application) {
            getInput().getAttribute("applications").addRefValue(application);

            String overridesSubColHref = getInput().getAttribute(
                    "configurationOverride").getCollectionLink();
            ReposObj newOverride = new ReposObj();
            ReposObjAttribute appRefAttr = new ReposObjAttribute();
            appRefAttr.setReference(true);
            appRefAttr.setMulti(false);
            appRefAttr.setCollection(false);
            appRefAttr.setName("application");
            appRefAttr.setRefValue(ReposHandle.getRelativeLink(application.getSelfLink()),
                    application.getCaption());
            newOverride.addAttribute(appRefAttr);

            AbstractUiObjCallBack cb = new AbstractUiObjCallBack() {
                @Override
                public void processReposObj(ReposObj configOverride) {
                    overridesContainer.addConfiguration(application, configOverride);
                }
            };
            MainModule.getRemoteClient().createReposObj(
                    ReposHandle.getAbsoluteLink(overridesSubColHref), newOverride,
                    cb);
        }

        private void applicationRemovedCallback(final ReposObj application) {
            AbstractUiDeletionCallBack cb = new AbstractUiDeletionCallBack() {
                @Override
                public void processSuccess() {
                    final String relativeLink = ReposHandle.getRelativeLink(application.getSelfLink());
                    getInput().getAttribute("applications").removeRefValue(relativeLink);

                    overridesContainer.removeConfiguration(application);
                }
            };
            MainModule.getRemoteClient().deleteReposObj(overridesContainer.getOverride(application), cb);
        }
    }
    
    /**
     * Container of configuration overrides UI for all applications in the profile.
     */
    private static class ConfigurationsContainer extends LayoutContainer {
        private final Map<ReposObj, ApplicationConfigFieldSet> configOverridePanels;

        public ConfigurationsContainer() {
            configOverridePanels = new HashMap<ReposObj, ApplicationConfigFieldSet>();
        }
        
        @Override
        protected void onRender(Element parent, int index) {
            super.onRender(parent, index);
            add(new Label("Configuration Details"));
        }
    

        void addConfiguration(ReposObj application, ReposObj configOverride) {
            final ApplicationConfigFieldSet configPanel 
                = new ApplicationConfigFieldSet(application, configOverride);
//            configPanel.addPartsTo(this);
            
            configOverridePanels.put(application, configPanel);
            
            add(configPanel);
            layout();
        }
        
        void removeConfiguration(ReposObj application) {
            ApplicationConfigFieldSet configPanel = configOverridePanels.remove(application);
            remove(configPanel);
            layout();
//          configPanel.removePartsFrom(this);
        }
        
        ReposObj getOverride(ReposObj application) {
            return configOverridePanels.get(application).getOverride();            
        }
    }
    

    @Override
    public void onSaveButtonClick(final boolean closeAfterCommit) {
        // check to see if the radio button is selected if so 
        //scheduled;
        if (scheduled != null) {
            if (scheduled.getValue()) {
                commit(closeAfterCommit);
            }
        }
        if (scheduleOnSave.getValue()) {
            ReposObj versionedProfile = getInput();
            //commit(closeAfterCommit);
            AfterCreationCallBack onExitCallBack;
            if (closeAfterCommit) {
                onExitCallBack = new ObjectCreationDialog.AfterCreationCallBack() {
                    public void onDialogClosure(ReposObj anObj) {
                        Utils.closeWaitDialog();
                        commit(true);
                    }                
                };
            } else {
                onExitCallBack = new ObjectCreationDialog.AfterCreationCallBack() {
                    public void onDialogClosure(ReposObj anObj) {
                        Utils.closeWaitDialog();
                        resetFindRelatedObjectPerformed();
                        refreshForm();
                        commit(false);
                    }                
                };
            }


            ScheduledTaskPanelHelper.openEditModelModificationTaskWindow(versionedProfile,
                                                              onExitCallBack,
                                                              relatedObj);
        }
        ReposObj scheduledObj = getRelatedObject();
        
        if (deployNow.getValue()) {
            if (scheduledObj != null) {
                removeScheduledObject(scheduledObj,
                    "By deploying the profile now the deployment scheduled for " 
                    + scheduledObj.getAttributeValue("nextRun")
                    + " will be cancelled, "
                    + "do you want to proceed with deploy now?",
                    closeAfterCommit);
            } else {
                publishAndSaveChanges();
                commit(closeAfterCommit);
            }
        }
        
        if (deployLater.getValue()) {
            if (scheduledObj != null) {
                removeScheduledObject(scheduledObj,
                                      "The scheduled deployment is being cancelled ,"
                                          + " are you sure you want to proceed?",
                                      closeAfterCommit);
            } else {
                commit(closeAfterCommit);
            }
        }

    }
    
    /**
     * Override UI for a single application
     */
    private static class ApplicationConfigFieldSet extends FieldSet {
        private final ReposObj configOverride;
        
        private final PropertyOverrideAttrField overrideField;

        private final ConstraintOverrideAttrField constraintOverrideField;
        
        ApplicationConfigFieldSet(ReposObj application, ReposObj override) {
            configOverride = override;
            
            setTitle(application.getCaption());
            addStyleName("arr-application-config");
            setBorders(false);
            //setPaddings(0, 12, 0, 0);

            ReposObjAttribute configDefaultsAttr = application.getAttribute("configuration");
            ReposObjAttribute configOverridesAttr = configOverride.getAttribute("configuration");
            overrideField = new PropertyOverrideAttrField(
                    configOverridesAttr, "Application Config",
                    configDefaultsAttr.getCollectionLink());

            /*namePanel.addToRow(new Label(" "), 15); */

            ReposObjAttribute constraintDefaultsAttr = application.getAttribute("constraints");
            ReposObjAttribute constraintOverridesAttr = configOverride.getAttribute("constraints");
            constraintOverrideField = new ConstraintOverrideAttrField(
                    constraintOverridesAttr, "Deployment constraints",
                    constraintDefaultsAttr.getCollectionLink());
            
            MultiFieldPanel namePanel = new MultiFieldPanel();
            namePanel.addToRow(overrideField, new ColumnData(0.5));
            namePanel.addToRow(constraintOverrideField, new ColumnData(0.5));
            add(namePanel);
        }
        
        ReposObj getOverride() {
            return configOverride;
        }

        void addPartsTo(IManagedForm form) {
            form.addPart(overrideField);
            form.addPart(constraintOverrideField);
        }
        
        void removePartsFrom(IManagedForm form) {
            form.removePart(overrideField);
            form.removePart(constraintOverrideField);
        }
    }


        

        
        private boolean findRelatedObjectPerformed() {
            return performedFindRelatedObject;
        }
        
        private boolean resetFindRelatedObjectPerformed() {
            return performedFindRelatedObject = false;
        }
        
        private void setRelatedObject(ReposObj obj) {
            relatedObj = obj;
            setInput(getInput());
        }
        
        private ReposObj getRelatedObject() {
            return relatedObj;
        }

        private void prepareDeploymentPanel() {
            MainModule.getRemoteClient().getReposObjList(ReposHandle.getAbsoluteLink("onceofftasks"),
                null,
                null,
                new AbstractUiCollectionCallBack() {
                    @Override
                    public void processReposObjCollection(ReposObjCollection list) {
                        performedFindRelatedObject = true;
                        ReposObj returnedObj = null;
                        for (ReposObj overrideObj : list.getTheReposObjs()) {
                            ReposObjAttribute paramAttr = overrideObj.getAttribute("parameters");
                            String versionedProfileGuid = getInput().getAttributeValue("guid");
                            String values[] = paramAttr.getValues();
                            for (String guid : values) {
                                if (guid.contains(versionedProfileGuid)) {
                                    returnedObj = overrideObj;
                                    break;
                                }
                            }
                        }
                        setRelatedObject(returnedObj);
                    }
                }
            );
        }

        private RadioGroup createDeploymentPanel() {
            //deploymentfieldSet = new FieldSet();
            radioGroup = new RadioGroup();
            deployNow = new Radio();
            deployNow.setBoxLabel("Deploy On Save");
            
            deployLater = new Radio();
            deployLater.setBoxLabel("Don't Schedule Deployment");
            
            radioGroup.add(deployNow);
            radioGroup.add(deployLater);
            
            // check to see if there is a schedule in place already
            ReposObj overrideObj = (ReposObj)getRelatedObject();
            if (overrideObj != null) {
                String time = overrideObj.getAttributeValue("nextRun");
                scheduled = new Radio();
                scheduled.setBoxLabel("Scheduled for " + time);

                scheduleOnSave = new Radio();
                scheduleOnSave.setBoxLabel("Edit Deploy Schedule on Save");

                radioGroup.add(scheduleOnSave);
            
            } else {
                scheduleOnSave = new Radio();
                scheduleOnSave.setBoxLabel("Schedule on Save");
                
                scheduleOnSave.setName("deployGroup");
                //deploymentfieldSet.add(scheduleOnSave);
                deployLater.setValue(true);
            }
            radioGroup.add(scheduled);

            return radioGroup;
        }
        
        private void removeScheduledObject(final ReposObj toDelete, String message, Boolean closeAfterCommit) {
            if (closeAfterCommit) {
                MessageBox.confirm("Deleting existing schedule", 
                                   message ,
                                   new Listener<WindowEvent>() {
                    
                        public void handleEvent(WindowEvent we) {
                            if (Dialog.NO.equals(we.buttonClicked.getItemId())) {
                                return;
                            }
                            Utils.showWaitDialog();
                            MainModule.getRemoteClient().deleteReposObj(toDelete, 
                                new AbstractUiDeletionCallBack() {
                                    public void processSuccess() {
                                        Utils.closeWaitDialog();                                        
                                        commit(true);
                                    }
                                }
                            );
                        }
                    }
                );
            } else {
                MessageBox.confirm("Deleting existing schedule", 
                        message ,
                        new Listener<WindowEvent>() {
         
             public void handleEvent(WindowEvent we) {
                 if (Dialog.NO.equals(we.buttonClicked.getItemId())) {
                     return;
                 }
                 Utils.showWaitDialog();
                 MainModule.getRemoteClient().deleteReposObj(toDelete, 
                     new AbstractUiDeletionCallBack() {
                         public void processSuccess() {
                             Utils.closeWaitDialog();
                             resetFindRelatedObjectPerformed();
                             refreshForm();
                             commit(false);
                             
                         }
                     }
                 );
             }
         }
     );

                
            }
        }
        
        private void publishAndSaveChanges() {
            if (getInput() == null) {
                return;
            }

            getInput().getAttribute("publishNow").setSimpleValue("true");
        }

    
    /**
     * The URL supplied to this panel can be either for a DeploymentProfile or a
     * VersionedProfile.
     * <p>
     * The panel is intended for editing a VersionedProfile, so if a
     * DeploymentProfile URL is supplied, a 2 phase load happens whereby the
     * deployment profile is loaded first in order to get the URL of the version
     * profile to load and edit.
     */
    private class LoadHelper implements Listener<ComponentEvent> {
        private final ReposURL profileURL;

        LoadHelper(ReposURL profileURL) {
            this.profileURL = profileURL;
        }

        public void handleEvent(ComponentEvent be) {
            if (EClassHelper.DEPLOYMENT_PROFILE_ECLASS.equals(profileURL
                    .getEClass())) {
                loadDeploymentProfile(profileURL);
            } else {
                loadVersionedProfile(profileURL);
            }
        }

        private void loadVersionedProfile(ReposURL aProfileURL) {
            loadVersionedProfile(aProfileURL, null);
        }

        private void loadDeploymentProfile(ReposURL deploymentProfileURL) {
            MainModule.getRemoteClient().getReposObj(
                    deploymentProfileURL.getURL(), new AbstractUiObjCallBack() {
                        public void processReposObj(final ReposObj loaded) {
                            deploymentProfileLoaded(loaded);
                        }
                    });
        }

        private void deploymentProfileLoaded(ReposObj deploymentProfile) {
            String currentProfileHRef = DeploymentProfileHelper
                    .getCurrentVersionRef(deploymentProfile);
            String guidOfParentDeploymentProfile = deploymentProfile
                    .getAttributeValue("guid");
            if (guidOfParentDeploymentProfile != null) {
                parentGuid = guidOfParentDeploymentProfile;
            } else {
                guidOfParentDeploymentProfile = parentGuid;
            }
            if (currentProfileHRef != null && !"".equals(currentProfileHRef)) {
                loadVersionedProfile(new ReposURL(ReposHandle
                        .getAbsoluteLink(currentProfileHRef)),
                        guidOfParentDeploymentProfile);
            }
        }

        private void loadVersionedProfile(ReposURL versionedProfileURL,
                final String guidOfParentDeploymentProfile) {

            AbstractUiObjCallBack loadedCb = new AbstractUiObjCallBack() {
                @Override
                public void processReposObj(final ReposObj loaded) {
                    if (guidOfParentDeploymentProfile != null) {
                        ReposObjAttribute anAttr = new ReposObjAttribute();
                        anAttr.setName("parentGuid");
                        anAttr.addSimpleValue(guidOfParentDeploymentProfile);
                        loaded.addAttribute(anAttr);
                    }

                    // This call triggers creation of the UI.
                    setInput(loaded);
                }
            };
            MainModule.getRemoteClient().getReposObj(
                    versionedProfileURL.getURL(), loadedCb);
        }
    }
    
    @Override
    public void setInput(Object formInput) {
        ReposObj loaded = (ReposObj)formInput;
        ReposObjAttribute attr = loaded.getAttribute("parentGuid");
        if (attr == null || attr.getValue() == null || attr.getValue().equals("")) {
            ReposObjAttribute anAttr = new ReposObjAttribute();
            anAttr.setName("parentGuid");
            anAttr.addSimpleValue(parentGuid);
            loaded.addAttribute(anAttr);
        }
        input = loaded;
        refreshForm();
        
    }
    


    
    }
