/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor.widgets;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.WindowEvent;
import com.extjs.gxt.ui.client.widget.Dialog;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.form.FieldSet;
import com.extjs.gxt.ui.client.widget.form.FormPanel;
import com.google.gwt.core.client.GWT;
import com.iona.soa.web.repository.base.client.model.IReposObjChangeListener;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjChangeNotifier;
import com.iona.soa.web.repository.main.client.HtmlSnippets;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.UIMessages;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.IEditorPanel;
import com.iona.soa.web.repository.main.client.base.IFormPart;
import com.iona.soa.web.repository.main.client.base.IManagedForm;
import com.iona.soa.web.repository.main.client.base.ISelectionProvider;
import com.iona.soa.web.repository.main.client.base.LinkHistoryTracker;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.ui.editor.ObjectPermissionsSummaryPanel;

public abstract class AbstractReposObjFormEditor extends FormPanel 
                                                 implements IEditorPanel, 
                                                            ISelectionProvider, 
                                                            IManagedForm, 
                                                            IReposObjChangeListener {

    protected static final UIMessages MESSAGES = GWT.create(UIMessages.class);
    
    protected static final HtmlSnippets SNIPPETS = GWT.create(HtmlSnippets.class);
    
    protected ReposObj input;
    
    private final List<ISelectionListener> selectionListeners = new ArrayList<ISelectionListener>();
    
    private final List<IFormPart> formParts = new ArrayList<IFormPart>();

    private ObjectPermissionsSummaryPanel objectPermissionsPanel;
    private Button saveButton;
    private Button saveAndCloseButton;
    private boolean obsoleted;

    protected AbstractReposObjFormEditor() {
        addStyleName("arr-editor");
        setBorders(false);
        setBodyBorder(false);
        //setHeaderVisible(false);
        setLabelAlign(LabelAlign.RIGHT);
        
        setStyleAttribute("padding", "0");
        setStyleAttribute("margin", "0");
        
        // Needed by FormPanel - adding widgets post render drives it 
        // nuts unless at least one was there already
        /*Label lbl = new Label(" ");
        lbl.getElement().setId("wibble");
        add(lbl); */
        //setWaitMsgTarget("wibble");
        
        // Note: we don't fill the form until the input is loaded.
        
        ReposObjChangeNotifier.INSTANCE.addListener(this);
        Listener<ComponentEvent> listener = new Listener<ComponentEvent>() {
            public void handleEvent(ComponentEvent be) {
                ReposObjChangeNotifier.INSTANCE.removeListener(AbstractReposObjFormEditor.this);
            }
        };
        addListener(Events.Detach, listener);
        addListener(Events.Hide, listener);
        addListener(Events.BeforeRemove, listener);
        addButtons();
    }

    /**
     * ISelectionProvider implementation
     */
    public final void addSelectionListener(ISelectionListener listener) {
        selectionListeners.add(listener);
    }
    
    /**
     * ISelectionProvider implementation
     */
    public final void removeSelectionListener(ISelectionListener listener) {
        selectionListeners.remove(listener);
    }

    /**
     * ISelectionProvider implementation
     */
    public final List<ReposObj> getSelection() {
        if (input != null) {
            return Collections.singletonList(input);
        }
        return Collections.emptyList();
    }

    /**
     * @see IReposObjChangeListener
     */
    public void reposObjCreated(ReposObj createdObj, String containingCollectionUrl, int index) {
        // nothing required
    }

    /**
     * @see IReposObjChangeListener
     */
    public void reposObjDeleted(ReposObj deletedObj) {
        if (getInput() != null && getInput().getId().equals(deletedObj.getId())) {
            formParts.clear();  // Any changes made will be lost
            // the editor is closed externally, don't do it here
        }
    }

    /**
     * @see IReposObjChangeListener
     */
    public void reposObjUpdated(ReposObj updatedObj) {
        // nothing required as the editor should only reacts to updates it initiates itself (which it does
        // already)
    }
    
    protected abstract void doFillForm();
    
    public final void close() {
        if (isDirty()) {
            MainModule.get().getWorkspaceControl().displayUnsavedChangesDialog(new Listener<WindowEvent>() {
                public void handleEvent(WindowEvent we) {
                    if (Dialog.YES.equals(we.buttonClicked.getItemId())) {
                        obsoleted = true;
                        LinkHistoryTracker.INSTANCE.walkToOlderLink();
                    }
                }
            });
        } else {
            LinkHistoryTracker.INSTANCE.walkToOlderLink();
        }
    }
    
    protected final ReposObj getInput() {
        return input;
    }
    
    /**
     * @see com.iona.soa.web.repository.main.client.base.IEditorPanel
     * @see com.iona.soa.web.repository.main.client.base.IManagedForm
     */
    public final boolean isDirty() {
        if (obsoleted) {
            return false;
        }
        for (IFormPart formPart : formParts) {
            if (formPart.isDirty()) {
                return true;
            }
        }
        return false;
    }

    public final IFormPart addPart(IFormPart formPart) {
        formParts.add(formPart);
        formPart.setFormInput(getInput());
        return formPart;
    }

    public final IFormPart removePart(IFormPart formPart) {
        formParts.remove(formPart);
        return formPart;
    }
    
    public void onSaveButtonClick(final boolean closeAfterCommit) {
        commit(closeAfterCommit);
    }

    public void commit(final boolean closeAfterCommit) {
        for (IFormPart formPart : formParts) {
            if (formPart.isDirty()) {
                formPart.commit();
            }
        }
        
        if (objectPermissionsPanel == null) {
            doCommit(closeAfterCommit);  
        } else {
            objectPermissionsPanel.saveChanges(getInput(), new AbstractUiObjCallBack() {
                public void processReposObj(ReposObj queriedObj) {
                    doCommit(closeAfterCommit);
                }
            });            
        }
    }

    protected void doCommit(final boolean closeAfterCommit) {
        if (getInput() == null) {
            return;
        }

        Utils.showWaitDialog();
        MainModule.getRemoteClient().updateReposObj(getInput(), new AbstractUiObjCallBack() {
            public void processReposObj(ReposObj savedObj) {
                try {
                    setInput(savedObj);
                    if (closeAfterCommit) {
                        close();
                    } else {
                        refresh();
                    }
                } finally {
                    Utils.closeWaitDialog();
                }
            }
        });
    }

    public void refresh() {
        for (IFormPart formPart : formParts) {
            formPart.refresh();
        }
    }

    public void setInput(Object formInput) {
        input = (ReposObj)formInput;
        
        refreshForm();
    }

    protected void refreshForm() {
        removeAll(true);
        formParts.clear();
        
        fillForm();
        
        // After setting the "editing" object there is now a 
        // selection to notify listeners about.
        notifySelectionListeners();
    }

    protected void addObjectPermissionsPanel() {
        if (getInput().getAttribute("objectControl") != null) {
            FieldSet section = FieldWidgetHelper.createFieldSet();
            section.setBorders(true);
            section.setTitle(MESSAGES.objectSecurityTitle());
            section.setCollapsible(false);
            section.setStyleName("arr-object-permissions-panel");
            
            objectPermissionsPanel = new ObjectPermissionsSummaryPanel(getInput());
            section.add(objectPermissionsPanel);
            add(section);
            addPart(objectPermissionsPanel);
        }
    }    
    
    private void notifySelectionListeners() {
        List<ReposObj> selection = getSelection();
        for (ISelectionListener l : selectionListeners) {
            l.onSelectionChange(selection);
        }
    }
    
    private void fillForm() {
        if (getInput() != null && getInput().isEditAllowed()) {
            saveButton.enable();
            saveAndCloseButton.enable();
        } else {
            saveButton.disable();
            saveAndCloseButton.disable();
        }
        try {
            setHeading(getInput().getCaption());
            doFillForm();
        } finally {
            doLayout();            
        }
    }    

    private void addButtons() {
        saveButton = new Button("Save");
        saveButton.addListener(Events.Select, new Listener<ButtonEvent>() {
            public void handleEvent(ButtonEvent be) {
                onSaveButtonClick(false);
            }
        });
        saveButton.disable();
        addButton(saveButton);
        
        saveAndCloseButton = new Button("Save & Close");
        saveAndCloseButton.addListener(Events.Select, new Listener<ButtonEvent>() {
            public void handleEvent(ButtonEvent be) {
                onSaveButtonClick(true);
            }
        });
        saveAndCloseButton.disable();
        addButton(saveAndCloseButton);
        
        Button closeButton = new Button("Close");
        closeButton.addListener(Events.Select, new Listener<ButtonEvent>() {
            public void handleEvent(ButtonEvent be) {
                close();
            }
        });
        addButton(closeButton);
    }        
}
