/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor.widgets;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.Style.Scroll;
import com.extjs.gxt.ui.client.event.GridEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.widget.BoxComponent;
import com.extjs.gxt.ui.client.widget.ScrollContainer;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid.ClicksToEdit;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.IFormPart;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.MultiFieldPanel;
import com.iona.soa.web.repository.main.client.base.ui.RemoteCollectionPanel;
import com.iona.soa.web.repository.main.client.base.ui.RemoteRestDataProxy;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator.DefaultColumnIds;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.AbstractReposObjCreationPanel;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.AttributeCreationDialog;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.CreationPanelRegistry;
import com.iona.soa.web.repository.main.client.base.ui.editor.AfterClosureCallBack;

public class CollectionValAttrField extends MultiFieldPanel implements IFormPart {
    
    protected BoxComponent dataPanel;
    
    protected Label lblAttr;

    private ReposObj input;
    
    private final String attributeName;
    
    public CollectionValAttrField(ReposObjAttribute reposObjAttribute, String label) {
        attributeName = reposObjAttribute.getName();
        setFormInput(reposObjAttribute.getParent()); // allow subclasses access asap
        
        setBorders(false);
        setAutoHeight(false);
        setAutoWidth(false);
        setStyleName("x-form-item");
        
        
        if (label == null) {
            lblAttr = new Label(reposObjAttribute.getName() + ":");
        } else {
            lblAttr = new Label(label + ":");
        }
        lblAttr.setStyleName("x-form-item-label");
        addToRow(lblAttr, FieldWidgetHelper.FORM_LABEL_WIDTH);

        /*Panel elementPanel = new Panel();
        elementPanel.setBaseCls("x-plain");
        elementPanel.addClass("arr-form-element");
        elementPanel.setHeader(false);*/
        
        dataPanel = createDataPanel();
        //elementPanel.add(dataPanel);
        
        //addToRow(anchoringPanel, new ColumnLayoutData(1));
        addToRow(dataPanel, null);
        
        Widget buttonWidget = createButtonWidget();
        if (buttonWidget != null) {
            addToRow(buttonWidget, FieldWidgetHelper.EDITOR_BTNS_DEFAULT_WIDTH);
        }
    }

    protected String getAddIcon() {
        return "images/create.gif";
    }
    
    public final ReposObjAttribute getAttribute() {
        return input.getAttribute(attributeName);
    }

    protected final ReposObj getInput() {
        return input;
    }
    
    public String getFieldLabel() {
        return lblAttr.getText();
    }
    
    
    protected BoxComponent createDataPanel() {
        
        RemoteRestDataProxy dataProxy = createDataProxy();
        final EditorGrid<ReposObjDataModel> gridPanel = new EditorGrid<ReposObjDataModel>(null, null);
        
        SimpleGridConfigurator gridConfig = getGridConfig();
        gridConfig.configure(gridPanel, dataProxy);
        
        // Overrides for us
        gridPanel.setAutoHeight(true);
        gridPanel.setHideHeaders(!showGridHeaders());
        gridPanel.setStripeRows(true);
        gridPanel.setAutoExpandColumn(DefaultColumnIds.NAME); 
        gridPanel.addStyleName("force-hand-pointer");
        gridPanel.disableTextSelection(true);
        
        // add in line editing support...
        Listener<GridEvent> editorGridListener = getEditorGridListener();
        if (editorGridListener != null && !getAttribute().isReadOnly()) {
            gridPanel.setClicksToEdit(ClicksToEdit.ONE);
            gridPanel.addListener(Events.AfterEdit, editorGridListener);
        }
        
        gridPanel.getView().setEmptyText(getAttribute().isMulti() ? "none" : "undefined");

        
        RemoteCollectionPanel collectionPanel = createCollectionPanel(gridConfig, dataProxy);
        collectionPanel.setBorders(true);
        
        return collectionPanel;
    }

    protected Widget createButtonWidget() {
        ReposObjAttribute reposObjAttribute = getAttribute();
        if (!reposObjAttribute.isReadOnly()
                && (reposObjAttribute.getParent() == null
                        || reposObjAttribute.getParent().isEditAllowed())) {

            Image btnAdd = FieldWidgetHelper.createActionPoint(getAddIcon(),
                                                               "Add/Update a " + reposObjAttribute.getName(),
                                                               new ClickListener() {
                public void onClick(Widget sender) {
                    runAddCommand();
                }
            });
            return btnAdd;
        }
        return null;
    }

    protected RemoteCollectionPanel createCollectionPanel(SimpleGridConfigurator gridConfig,
                                                          RemoteRestDataProxy dataProxy) {
        return new RemoteCollectionPanel(gridConfig, dataProxy);
    }
    
    protected RemoteRestDataProxy createDataProxy() {
        return RemoteRestDataProxy.createProxyForCollection(getAttribute().getCollectionLink());
    }
    
    protected boolean showGridHeaders() {
        return false;
    }

    protected Listener<GridEvent> getEditorGridListener() {
        return null;
    }

    protected void runAddCommand() {
        
        AbstractReposObjCreationPanel cp = CreationPanelRegistry.getPanel(getAttribute().getRefEClass());
        
        if (cp != null) {
            AfterClosureCallBack callBack = new AfterClosureCallBack() {
                public void onDialogClosure() { /* nothing required */ }
            };
            AttributeCreationDialog cd =  new AttributeCreationDialog(getAttribute(), cp, callBack);
            cd.show();
            cd.center();
        
        } else {

            MainModule.getRemoteClient().createReposObj(getAttribute().getCollectionLink(),
                                                        new ReposObj(),
                                                        new AbstractUiObjCallBack() {
                    public void processReposObj(ReposObj anObj) { }
                });
        }
    }

    
    protected SimpleGridConfigurator getGridConfig() {
        return new SimpleGridConfigurator() {
            @Override
            protected void addVisibleColumns() {
                addNameColumn();
                if (!getAttribute().isReadOnly()) {
                    addDeleteColumn(true).setActionLabel("Remove").setHidden(false);
                }
            }
        };
    }
    
    /**
     * Best called before render
     */
    @Override
    public void setHeight(int heightInPixels) {
        // Re-direct height requirements to the gridPanel
        // TODO - attempt to find a height that allows a full row to be rendered at bottom of list  
        dataPanel.setHeight(heightInPixels); 
        dataPanel.setAutoHeight(false);
        
        if (dataPanel instanceof ScrollContainer) {
            ((ScrollContainer<?>)dataPanel).setScrollMode(Scroll.AUTO);
        }
    }


    // IFormPart implementation
    public void commit() {
        // do nothing - changes are sent to the server as soon as they are made
    }


    // IFormPart implementation
    public boolean isDirty() {
        return false;
    }


    // IFormPart implementation
    public void refresh() {
        // do nothing - dataPanel is always up to date.
        // *Could* get its store and reload it - if it has a store.
    }
    
    // IFormPart implementation
    public void setFormInput(Object formInput) {
        input = (ReposObj)formInput;
    }
}
