/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor.widgets;

import java.util.List;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.Style.HorizontalAlignment;
import com.extjs.gxt.ui.client.Style.VerticalAlignment;
import com.extjs.gxt.ui.client.data.ModelData;
import com.extjs.gxt.ui.client.dnd.ListViewDragSource;
import com.extjs.gxt.ui.client.dnd.ListViewDropTarget;
import com.extjs.gxt.ui.client.dnd.DND.Feedback;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.ListViewEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.VerticalPanel;
import com.extjs.gxt.ui.client.widget.button.IconButton;
import com.extjs.gxt.ui.client.widget.form.AdapterField;
import com.extjs.gxt.ui.client.widget.form.Field;
import com.extjs.gxt.ui.client.widget.form.ListField;
import com.extjs.gxt.ui.client.widget.form.FormPanel.LabelAlign;
import com.extjs.gxt.ui.client.widget.layout.ColumnData;
import com.extjs.gxt.ui.client.widget.layout.ColumnLayout;
import com.extjs.gxt.ui.client.widget.layout.FormData;
import com.extjs.gxt.ui.client.widget.layout.FormLayout;
import com.google.gwt.dom.client.NodeList;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;

/**
 * Combines two list fields and allows selections to be moved between fields either using buttons or by
 * dragging and dropping selections.
 * 
 * @param <D> the model type
 */
public class DualList<D extends ModelData> extends LayoutContainer {

    public enum Mode {
        APPEND, INSERT;
    }
    
    public enum Direction {
        LEFT, RIGHT;
    }

    protected ListField<D> rightField;
    protected ListField<D> leftField;
    protected VerticalPanel buttonBar;

    private final Mode mode;
    private final Direction direction;
    private final boolean dndEnabled;

    private String dndGroup;

    public DualList() {
        this(Mode.APPEND, Direction.LEFT, true);
    }
    
    public DualList(Mode aMode, Direction targetDirection, boolean enableDND) {        
        mode = aMode == null ? Mode.APPEND : aMode;
        direction = targetDirection == null ? Direction.LEFT : targetDirection;
        dndEnabled = enableDND;
        
        initListField();
        
        buttonBar = initButtons();
        
    }

    private void initListField() {
        leftField = new ListField<D>();
        leftField.getListView().addListener(Event.ONDBLCLICK, new Listener<ListViewEvent>() {
            public void handleEvent(ListViewEvent be) {
                moveSelectionToTheRight();
            }
        });
        
        rightField = new ListField<D>();
        rightField.getListView().addListener(Event.ONDBLCLICK, new Listener<ListViewEvent>() {
            public void handleEvent(ListViewEvent be) {
                moveSelectionToTheLeft();
            }
        });
    }

    @Override
    protected void onRender(Element parent, int index) {
        super.onRender(parent, index);
        
        ColumnLayout layout = new ColumnLayout();
        //layout.setAdjustForScroll(false);
        setLayout(layout);
        
        setStyleAttribute("padding", "5px");

        add(wrappField(leftField), new ColumnData(.5));
        
        AdapterField buttonAdapter = new AdapterField(buttonBar);
        buttonAdapter.setHideLabel(true);
        buttonAdapter.setHeight("100%");
        buttonAdapter.setStyleAttribute("padding", "0px");
        add(buttonAdapter, new ColumnData(30));
        
        add(wrappField(rightField), new ColumnData(.5));
        
        if (dndEnabled) {
            initDND();
        }
        
        DeferredCommand.addCommand(new Command() {
            public void execute() {
                fixREPO763();   // The TextFields appearing in the lists
            }
        });
    }

    private void fixREPO763() {
        NodeList<com.google.gwt.dom.client.Element> inputs = getElement().getElementsByTagName("input");
        if (inputs != null && inputs.getLength() > 0) {
            for (int i = 0; i < inputs.getLength(); i++) {
                com.google.gwt.dom.client.Element el = inputs.getItem(i);
                el.getStyle().setProperty("display", "none");
            }
        }
    }
    
    private LayoutContainer wrappField(Field<D> aField) {
        
        LayoutContainer fieldWrapper = new LayoutContainer();
        fieldWrapper.setLayout(new FormLayout(LabelAlign.TOP));
        
        fieldWrapper.setStyleAttribute("padding", "0px");
        
        fieldWrapper.add(aField, new FormData("100% 100%"));
        
        return fieldWrapper;
    }

    public Mode getMode() {
        return mode;
    }

    /**
     * Returns the from list field.
     * @return the field
     */
    public ListField<D> getFromList() {
        return Direction.LEFT == direction ? rightField : leftField;
    }

    /**
     * Returns the to list field.
     * @return the field
     */
    public ListField<D> getToList() {
        return Direction.LEFT == direction ? leftField : rightField;

    }

    protected VerticalPanel initButtons() {
        
        VerticalPanel theBar = new VerticalPanel();
        
        theBar.setBorders(false);
        
        theBar.setStyleAttribute("padding-top", "50px");
        //theBar.setStyleAttribute("margin", "15px");
        
        theBar.setSpacing(5);
        theBar.setHorizontalAlign(HorizontalAlignment.CENTER);
        theBar.setVerticalAlign(VerticalAlignment.MIDDLE);
        theBar.setTableHeight("100%");
        theBar.setTableWidth("100%");
        
        if (mode == Mode.INSERT) {
            //IconButton top = new IconButton("arrow-top");
            //theBar.add(top);
            
            IconButton up = new IconButton("arrow-up");
            up.setTitle("Move Up the selection");
            up.addListener(Events.Select, new Listener<ComponentEvent>() {
                public void handleEvent(ComponentEvent be) {
                    getToList().getListView().moveSelectedUp();
                }
            });
            theBar.add(up);
        }

        IconButton right = new IconButton("arrow-right");
        right.setTitle(Direction.RIGHT == direction ? "Add to Selection" : "Remove from Selection");
        right.addListener(Events.Select, new Listener<ComponentEvent>() {
            public void handleEvent(ComponentEvent be) {
                moveSelectionToTheRight();
            }
        });

        IconButton left = new IconButton("arrow-left");
        left.setTitle(Direction.LEFT == direction ? "Add to Selection" : "Remove from Selection");
        left.addListener(Events.Select, new Listener<ComponentEvent>() {
            public void handleEvent(ComponentEvent be) {
                moveSelectionToTheLeft();
            }
        });

        theBar.add(right);
        theBar.add(left);

        if (mode == Mode.INSERT) {
            IconButton down = new IconButton("arrow-down");
            down.setTitle("Move down the selection");
            down.addListener(Events.Select, new Listener<ComponentEvent>() {
                public void handleEvent(ComponentEvent be) {
                    getToList().getListView().moveSelectedDown();
                }
            });
            theBar.add(down);
            
            //IconButton bottom = new IconButton("arrow-bottom");
            //theBar.add(bottom);
        }
        
        return theBar;
    }

    /**
     * Sets the drag and drop grup name. A group name will be generated if none is specified.
     * @param group the group name
     */
    public void setDNDGroup(String group) {
        this.dndGroup = group;
    }

    public String getDNDGroup() {
        return dndGroup;
    }

    protected void initDND() {
        if (dndGroup == null) {
            dndGroup = getId() + "-group";
        }

        ListViewDragSource source1 = new ListViewDragSource(rightField.getListView());
        ListViewDragSource source2 = new ListViewDragSource(leftField.getListView());

        source1.setGroup(dndGroup);
        source2.setGroup(dndGroup);

        ListViewDropTarget target1 = new ListViewDropTarget(rightField.getListView());
        ListViewDropTarget target2 = new ListViewDropTarget(leftField.getListView());

        target1.setGroup(dndGroup);
        target2.setGroup(dndGroup);

        if (mode == Mode.INSERT) {
            target1.setFeedback(Feedback.INSERT);
            target2.setFeedback(Feedback.INSERT);
        }
    }

    private void moveSelectionToTheRight() {
        List<D> sel = leftField.getSelection();
        for (D model : sel) {
            leftField.getStore().remove(model);
            rightField.getStore().add(model);
        }
    }

    private void moveSelectionToTheLeft() {
        List<D> sel = rightField.getSelection();
        for (D model : sel) {
            rightField.getStore().remove(model);
            leftField.getStore().add(model);
        }
    }
}
