/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor.widgets;

import com.allen_sauer.gwt.log.client.Log;
import com.extjs.gxt.ui.client.data.ModelData;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.store.StoreEvent;
import com.extjs.gxt.ui.client.store.StoreListener;
import com.extjs.gxt.ui.client.widget.form.Field;
import com.extjs.gxt.ui.client.widget.form.FieldSet;
import com.extjs.gxt.ui.client.widget.form.ListField;
import com.extjs.gxt.ui.client.widget.form.NumberField;
import com.extjs.gxt.ui.client.widget.form.SimpleComboBox;
import com.extjs.gxt.ui.client.widget.form.TextArea;
import com.extjs.gxt.ui.client.widget.form.TextField;
import com.extjs.gxt.ui.client.widget.form.Validator;
import com.extjs.gxt.ui.client.widget.form.ComboBox.TriggerAction;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.extjs.gxt.ui.client.widget.layout.FormLayout;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Image;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.base.IFormPart;
import com.iona.soa.web.repository.main.client.base.ReposObjHelper;

public final class FieldWidgetHelper {

    public static final int EDITOR_BTNS_DEFAULT_WIDTH = 16;  // width of the "+" image
    
    //public static final int FORM_LABEL_WIDTH = 108;    
    public static final int FORM_LABEL_WIDTH = 100;
    
    public static final int TEXT_FIELD_HEIGHT = 18; // same as x-form-text height

    private FieldWidgetHelper() { }

    public static void configureField(Field simpleField,
                                      ReposObjAttribute reposObjAttribute) {
        simpleField.setName(reposObjAttribute.getName());
        
        String label = getLabel(reposObjAttribute);
        if (label == null) {
            simpleField.setFieldLabel(reposObjAttribute.getName());
        } else {
            simpleField.setFieldLabel(label);
        }
    
        if (simpleField instanceof TextField) {
            TextField textField = (TextField)simpleField;
            if (!reposObjAttribute.isHidden()) {
                // don't setAllowBlank for password fields
                textField.setAllowBlank(reposObjAttribute.isOptional());
            }
        }
        
        if (reposObjAttribute.isReadOnly()) {
            simpleField.setReadOnly(true);
            simpleField.setEnabled(false);
        }
    
        simpleField.setValue(reposObjAttribute.getValue());
    }

    public static IFormPart adapt(Field<?> field) {
        if (field instanceof IFormPart) {
            return (IFormPart)field;
        } else if (field instanceof TextField<?>) {
            return new TextFieldFormPart((TextField<?>)field);
        }
        // TODO
        Log.warn("Returning no-op implementation for " + field.getName());
        return new IFormPart() {
            public void commit() {
            }

            public boolean isDirty() {
                return false;
            }

            public void refresh() {
            }

            public void setFormInput(Object input) {
            }
        };
    }

    public static IFormPart adapt(Grid<? extends ModelData> grid) {
        return new ComponentWithListStoreFormPart(grid.getStore());
    }

    public static IFormPart adapt(ListField<? extends ModelData> list) {
        return new ComponentWithListStoreFormPart(list.getStore());
    }

    public static Image createActionPoint(String imgSrc, String toolTip, ClickListener listener) {
        Image actionPoint = new Image();
        
        actionPoint.setUrl(imgSrc);
        actionPoint.addStyleName("force-hand-pointer");
        actionPoint.addClickListener(listener);
        
        actionPoint.setTitle(toolTip);

//        actionPoint.setVisible(true);
//        actionPoint.setWidth(EDITOR_BTNS_DEFAULT_SIZE + "px");
//        actionPoint.setHeight(EDITOR_BTNS_DEFAULT_SIZE + "px");
        /*AdapterField adapterField = new AdapterField(actionPoint);
        
        ToolTip appDocTip = new ToolTip(adapterField);  
        appDocTip.setToolTip(toolTip);  
        appDocTip.setAutoHide(true);  */

        return actionPoint;
    }
    
    public static <T> Field<T> createField(ReposObjAttribute reposObjAttribute) {
        Field<?> theField;
        
        if (reposObjAttribute.isHidden()) {
            theField = createPasswordField();
        } else if (ReposObjAttribute.SIMPLE_TYPE_BOOLEAN.equals(reposObjAttribute.getPrimitiveType())) {
            theField = createBooleanField();
        } else if (ReposObjAttribute.SIMPLE_TYPE_INTEGER.equals(reposObjAttribute.getPrimitiveType())) {
            theField = createIntegerField();
        } else if (ReposObjAttribute.SIMPLE_TYPE_LONG_STRING.equals(reposObjAttribute.getPrimitiveType())) {
            theField = createLongStringField();
        } else if (ReposObjAttribute.SIMPLE_TYPE_TOKEN_STRING.equals(reposObjAttribute.getPrimitiveType())) {
            theField = createTokenStringField();
        } else {
            theField = createSimpleStringField();
        }

        configureField(theField, reposObjAttribute);

        return (Field<T>)theField;
    }
    
    public static String getLabel(ReposObjAttribute reposObjAttribute) {
        if (ReposObjHelper.isFolderType(reposObjAttribute.getRefEClass())) {
            return "Folder";
        } else if (reposObjAttribute.getName().equals("supportedPackageTypes")) {
            // hack - see REPO-232 for correct solution
            return "package types";
        } else {
            return reposObjAttribute.getName().substring(0, 1).toUpperCase()
                    + reposObjAttribute.getName().substring(1);
        }
    }
    
    public static FieldSet createFieldSet() {
        FieldSet fieldSet = new FieldSet();
        fieldSet.setCollapsible(false);
        fieldSet.setBorders(false);
        fieldSet.setCheckboxToggle(false);
        return fieldSet;
    }

    

    public static FieldSet createFieldSet(String title, int labelWidth, int defaultWidth) {
        FieldSet fieldSet = createFieldSet();
        FormLayout layout = new FormLayout();  
        layout.setLabelWidth(labelWidth); 
        layout.setDefaultWidth(defaultWidth);  
        fieldSet.setLayout(layout); 
        fieldSet.setTitle(title);
        fieldSet.setHeading(title);
        return fieldSet;
    }

    public static FieldSet createFieldSet(String title) {
        FieldSet fieldSet = createFieldSet();
        fieldSet.setTitle(title);
        fieldSet.setHeading(title);
        return fieldSet;
    }

    private static Field<?> createBooleanField() {
        SimpleComboBox<Boolean> field = new SimpleComboBox<Boolean>();
        
        field.add(true);
        field.add(false);
        
        field.setTypeAhead(false);  
        field.setTriggerAction(TriggerAction.ALL);  
        field.setEmptyText("Select a boolean...");  
        field.setSelectOnFocus(true); 
        
        return field;
    }
    
    private static Field<Number> createIntegerField() {
        NumberField field = new NumberField();
        
        field.setAllowDecimals(false);
        field.setAllowNegative(false);
        
        return field;
    }
    
    private static Field<String> createLongStringField() {
        TextArea field = new TextArea();
        return field;
    }
    
    private static Field<String> createTokenStringField() {
        TextField<String> field = new TextField<String>();
        
        field.setValidator(new Validator<String, Field<String>>() {
            public String validate(Field<String> field, String value) {
                boolean isInvalid =  !(value != null
                                            && (value.indexOf('\r') > -1
                                                || value.indexOf('\n') > -1
                                                || value.indexOf('\f') > -1
                                                || value.indexOf('\b') > -1
                                                || value.indexOf('\t') > -1
                                                || value.indexOf('\\') > -1));
                
                return isInvalid ? "Invalid text entry" : null ;
            }
        });
        
        field.setMaxLength(254);  
        return field;
    }
    
    private static Field<String> createSimpleStringField() {
        TextField<String> field = new TextField<String>();
        field.setMaxLength(254);  
        return field;
    }
    
    private static Field<String> createPasswordField() {
        TextField<String> field = new TextField<String>();

        field.setPassword(true);

//        field.setName(reposObjAttribute.getName());
//        if (label == null) {
//            field.setLabel(reposObjAttribute.getName());
//        } else {
//            field.setLabel(label);
//        }
//        field.setValue(reposObjAttribute.getValue());
        return field;
    }
    
    
    /**
     * Implementation of IFormPart that delegates to any Field widget.
     * Assumes that the name of the field is the id of the ReposObjAttribute.
     */
    private static final class TextFieldFormPart<T> implements IFormPart {
        private TextField<T> field;
        
        private ReposObj input;
        
        TextFieldFormPart(TextField<T> f) {
            field = f;
        }
        
        public void commit() {
            ReposObjAttribute attribute = input.getAttribute(field.getName());
            attribute.setSimpleValue(getFieldValue());
        }

        public boolean isDirty() {
            return !getModelValue().equals(getFieldValue());
        }

        public void refresh() {
            //field.setValue(getModelValue());
            field.setRawValue(getModelValue());
        }
        
        public void setFormInput(Object formInput) {
            input = (ReposObj)formInput;
        }
        
        private String getModelValue() {
            ReposObjAttribute attribute = input.getAttribute(field.getName());
            String value = attribute.getValue();
            return value == null ? "" : value;
        }
        
        private String getFieldValue() {
            String value = field.getRawValue();
            return value == null ? "" : value;
        }
    }

    private static final class ComponentWithListStoreFormPart<T extends ModelData> implements IFormPart {
        private ListStore<T> store;
        
        private boolean dirty;
        
        public ComponentWithListStoreFormPart(ListStore<T> s) {
            store = s;
            store.addStoreListener(new ComponentStoreListener());
        }
        
        public void commit() {
            // Currently grid panels stores all modify the attribute directly (I think)
            dirty = false;
        }

        public boolean isDirty() {
            return dirty;
        }

        public void refresh() {
            if((store != null) && (store.getLoader() != null)) {
                store.getLoader().load();
            }
        }
        
        public void setFormInput(Object formInput) {
            // do nothing.
            // Someday connect the repos obj to a store somehow.
            // Probably need the attribute also to do this.
        }
        
        private void markDirty() {
            Log.info("GridPanelFormPart::markAsDirty");            
            dirty = true;
        }
        
        private class ComponentStoreListener extends StoreListener<T> {
            @Override
            public void storeAdd(StoreEvent<T> se) {
                markDirty();
            }
            
            @Override
            public void storeClear(StoreEvent<T> se) {
                markDirty();
            }
            
            @Override
            public void storeRemove(StoreEvent<T> se) {
                markDirty();
            }

            @Override
            public void storeUpdate(StoreEvent<T> se) {
                markDirty();
            }
        }        
    }
}
