/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor.widgets;

import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.form.Field;
import com.extjs.gxt.ui.client.widget.form.TextField;
import com.extjs.gxt.ui.client.widget.layout.FormData;
import com.extjs.gxt.ui.client.widget.layout.FormLayout;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.base.IFormPart;
import com.iona.soa.web.repository.main.client.base.ui.MultiFieldPanel;

public class LinkField extends MultiFieldPanel implements IFormPart {

    private Field delegate;
    
    private ReposObj input;
    
    public LinkField(ReposObjAttribute reposObjAttribute) {
        // TODO find a better to check if the field is editable 
        boolean isExternal = "true".equals(reposObjAttribute.getParent().getAttributeValue("external"));
        setFormInput(reposObjAttribute.getParent());
        
        setBorders(false);
        setAutoHeight(false);
        setAutoWidth(false);

        setStyleAttribute("padding", "0");
        setStyleAttribute("margin", "0");
        
        LayoutContainer anchoringPanel = new LayoutContainer();
        anchoringPanel.setLayout(new FormLayout());
        anchoringPanel.setBorders(false);
        anchoringPanel.setStyleAttribute("padding", "0");
        anchoringPanel.setStyleAttribute("margin", "0");
        
        
        Widget theField;
        if (!isExternal || reposObjAttribute.isReadOnly()) {

            Label lblLink;
            String label = FieldWidgetHelper.getLabel(reposObjAttribute);
            if (label == null) {
                lblLink = new Label(reposObjAttribute.getName() + ":");
            } else {
                lblLink = new Label(label + ":");
            }
            
            lblLink.setStyleName("x-form-item");
            lblLink.addStyleName("label");
            addToRow(lblLink, FieldWidgetHelper.FORM_LABEL_WIDTH);

            theField = new HTML("<a href=\""
                                + reposObjAttribute.getValue()
                                + "\" target=\"_blank\">"
                                + reposObjAttribute.getValue()
                                + "</a>");
            
            theField.setStylePrimaryName("x-form-item");
            
            anchoringPanel.add(theField, new FormData("-5"));
        
        } else {
            TextField<String> basicStringField = new TextField<String>();
            basicStringField.setMaxLength(254);
            
            FieldWidgetHelper.configureField(basicStringField, reposObjAttribute);
            
            delegate = basicStringField;
            theField = basicStringField;
            
            anchoringPanel.add(theField, new FormData("-20"));
        }
        
        addToRow(anchoringPanel, null);
        
        if (!(!isExternal || reposObjAttribute.isReadOnly())) {
            Image btnView = FieldWidgetHelper.createActionPoint("images/download.gif",
                                                                "download/view the resource",
                                                                new ClickListener() {
                    public void onClick(Widget sender) {
                        Window.open(getModelValue(),
                                    "_blank",
                                    null);
                    }
                });

            addToRow(btnView, FieldWidgetHelper.EDITOR_BTNS_DEFAULT_WIDTH);
        }

    }
    
    // IFormPart implementation
    public void commit() {
        if (delegate != null) {
            getAttribute().setSimpleValue(getFieldValue());
        }
    }

    // IFormPart implementation
    public boolean isDirty() {
        return delegate == null ? false : !getModelValue().equals(getFieldValue());
    }

    // IFormPart implementation
    public void refresh() {
        if (delegate != null) {
            delegate.setValue(getModelValue());
        }
    }
    
    public void setFormInput(Object formInput) {
        input = (ReposObj)formInput;
    }

    protected final ReposObjAttribute getAttribute() {
        return input.getAttribute(delegate.getName());
    }
    
    private String getModelValue() {
        String value = getAttribute().getValue();
        return value == null ? "" : value;
    }
    
    private String getFieldValue() {
        String value = delegate.getRawValue();
        return value == null ? "" : value;
    }
}
