/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor.widgets;

import java.util.Arrays;
import java.util.Collection;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.Style;
import com.extjs.gxt.ui.client.event.DataListEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.MessageBoxEvent;
import com.extjs.gxt.ui.client.widget.DataList;
import com.extjs.gxt.ui.client.widget.DataListItem;
import com.extjs.gxt.ui.client.widget.Dialog;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.VerticalPanel;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.base.IFormPart;
import com.iona.soa.web.repository.main.client.base.ui.MultiFieldPanel;

public class MultiValueAttributeField extends MultiFieldPanel implements IFormPart {
    private DataList dataPanel;
    private Image btnDelete;
    private ReposObj input;
    private DataListItem emptyItem;
    private final String attributeName;

    public MultiValueAttributeField(ReposObjAttribute reposObjAttribute, String label) {
        attributeName = reposObjAttribute.getName();
        setFormInput(reposObjAttribute.getParent()); // allow subclasses access asap

        setBorders(false);
        setAutoHeight(false);
        setAutoWidth(false);
        setStyleName("x-form-item");
        
        Label lblAttr;
        if (label == null) {
            lblAttr = new Label(reposObjAttribute.getName() + ":");
        } else {
            lblAttr = new Label(label + ":");
        }
        lblAttr.setStyleName("x-form-item-label");
        addToRow(lblAttr, FieldWidgetHelper.FORM_LABEL_WIDTH);

        dataPanel = new DataList();
        for (String value : input.getAttribute(attributeName).getValues()) {
            dataPanel.add(value);
        }
        dataPanel.setBorders(false);
        dataPanel.setSelectionMode(Style.SelectionMode.MULTI);
        dataPanel.setFlatStyle(true);
        dataPanel.setBorders(true);
        dataPanel.disableTextSelection(true);
        addToRow(dataPanel, null);
        
        if (!reposObjAttribute.isReadOnly()
                && (reposObjAttribute.getParent() == null
                        || reposObjAttribute.getParent().isEditAllowed())) {

            Image btnAdd = FieldWidgetHelper.createActionPoint("images/create.gif",
                                                               "Add a value",
                                                               new ClickListener() {
                public void onClick(Widget sender) {
                    MessageBox box = MessageBox.prompt("New Value", "Please enter a new value:");
                    box.addCallback(new Listener<MessageBoxEvent>() {  
                        public void handleEvent(MessageBoxEvent be) {  
                            if (Dialog.OK.equals(be.buttonClicked.getItemId())) {
                                dataPanel.add(be.value);
                                if (removeEmptyItem()) {
                                    btnDelete.setVisible(true);
                                }
                            }
                        }  
                    });  
                }
            });
            btnDelete = FieldWidgetHelper.createActionPoint("images/delete_off.gif", 
                                                            "Remove selected value(s)", 
                                                            new ClickListener() {
                public void onClick(Widget sender) {
                    for (DataListItem item : dataPanel.getSelectedItems()) {
                        dataPanel.remove(item);
                    }
                    if (addEmptyItem()) {
                        btnDelete.setVisible(false);
                    }
                }
            });
            btnDelete.removeStyleName("force-hand-pointer");
            dataPanel.addListener(Events.SelectionChange, new Listener<DataListEvent>() {
                public void handleEvent(DataListEvent be) {
                    if (be.selected.size() > 0) {
                        btnDelete.setUrl("images/delete.gif");
                        btnDelete.addStyleName("force-hand-pointer");
                    } else {
                        btnDelete.setUrl("images/delete_off.gif");
                        btnDelete.removeStyleName("force-hand-pointer");
                    }
                    
                }
            });
            VerticalPanel panel = new VerticalPanel();
            panel.setBorders(false);
            panel.add(btnAdd);
            panel.add(btnDelete);
            addToRow(panel, FieldWidgetHelper.EDITOR_BTNS_DEFAULT_WIDTH);
        }
        if (addEmptyItem() && btnDelete != null) {
            btnDelete.setVisible(false);
        }
    }

    public void commit() {
        ReposObjAttribute attribute = input.getAttribute(attributeName);
        attribute.clearValues();
        if (isEmpty()) {
            return;
        }
        for (DataListItem item : dataPanel.getItems()) {
            attribute.addSimpleValue(item.getText());
        }
    }

    public boolean isDirty() {
        Collection<String> originalValues = Arrays.asList(input.getAttribute(attributeName).getValues());
        if (isEmpty()) {
            return originalValues.size() != 0;
        }
        if (originalValues.size() != dataPanel.getItems().size()) {
            return true;
        }
        for (DataListItem item : dataPanel.getItems()) {
            if (!originalValues.contains(item.getText())) {
                return true;
            }
        }
        addEmptyItem();
        return false;
    }

    public void refresh() {
        // TODO Auto-generated method stub

    }

    public void setFormInput(Object formInput) {
        input = (ReposObj)formInput;
    }
    
    private boolean isEmpty() {
        return dataPanel.getItemCount() == 0 || emptyItem != null;
    }
    
    private boolean removeEmptyItem() {
        boolean ret = false;
        if (emptyItem != null) {
            ret = dataPanel.remove(emptyItem);
            emptyItem = null;
        }
        return ret;
    }
    
    private boolean addEmptyItem() {
        if (dataPanel.getItemCount() == 0) {
            emptyItem = new DataListItem("Empty");
            emptyItem.setEnabled(false);
            dataPanel.add(emptyItem);
            return true;
        }
        return false;
    }
}
