/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor.widgets;

import com.extjs.gxt.ui.client.Style.HorizontalAlignment;
import com.extjs.gxt.ui.client.event.GridEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.form.TextField;
import com.extjs.gxt.ui.client.widget.grid.CellEditor;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.BaseCollectionPanel;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.AttributeCreationDialog;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.panels.PropertyPanel;
import com.iona.soa.web.repository.main.client.base.ui.editor.AfterClosureCallBack;

public class PropertyAttrField extends CollectionValAttrField {

    private static final String COLUMN_ID_PROP_VALUE = "value";
    
    protected String[] propertyKeySuggestions;
    
    public PropertyAttrField(ReposObjAttribute reposObjAttribute,
                             String label) {
        this(reposObjAttribute, label, new String[] {});
    }
    
    public PropertyAttrField(ReposObjAttribute anAttr,
                             String label,
                             String[] somePropertyKeySuggestions) {
        super(anAttr, label);
        propertyKeySuggestions = somePropertyKeySuggestions;
    }
    

    @Override
    protected Listener<GridEvent> getEditorGridListener() {
        return new Listener<GridEvent>() {
            
            public void handleEvent(GridEvent ge) {
                //if (Events.AfterEdit == ge.getEventType()Type()) {
                if (!ge.startValue.equals(ge.value)) {
                    ReposObj propObj = ((ReposObjDataModel)ge.record.getModel()).getPOJO();
                    propObj.getAttribute("value").setSimpleValue((String)ge.value);
                    
                    if (propObj.getSelfLink() != null) {
                        MainModule.getRemoteClient().updateReposObj(propObj,
                                                                    new AbstractUiObjCallBack() {
                                public void processReposObj(ReposObj anObj) { /* nothing required */ }
                            });
                    
                    } else {
                        String link = ReposHandle.getAbsoluteLink(getAttribute().getCollectionLink());
                        MainModule.getRemoteClient().createReposObj(link,
                                                                    propObj,
                                                                    new AbstractUiObjCallBack() {
                                public void processReposObj(ReposObj anObj) { /* nothing required */ }
                            });
                    }
                }
                //}
            };
        };
    }
    
    @Override
    protected boolean showGridHeaders() {
        return false;
    }

    @Override
    protected SimpleGridConfigurator getGridConfig() {
        return new PropsGridConfigurator(getAttribute().isReadOnly()
                                   || (getInput() != null && !getInput().isEditAllowed()));
    }
    
    @Override
    protected void runAddCommand() {
        ListStore<ReposObjDataModel> copy = new ListStore<ReposObjDataModel>();
        copy.add(((BaseCollectionPanel)dataPanel).getStore().getModels());

        final PropertyPanel inputPanel = new PropertyPanel(copy, propertyKeySuggestions);
        AfterClosureCallBack callBack = new AfterClosureCallBack() {
            public void onDialogClosure() { /* nothing required */ }
        };
        AttributeCreationDialog cd =  new AttributeCreationDialog(getAttribute(), inputPanel, callBack);
        cd.show();
        cd.center();
    }

    protected class PropsGridConfigurator extends SimpleGridConfigurator {
        
        private boolean isRreadOnly; 
        
        public PropsGridConfigurator(boolean readOnly) {
            isRreadOnly = readOnly;
        }
        
        @Override
        public String getExpandableColumnId() {
            return DefaultColumnIds.NAME;
        }

        @Override
        protected void addVisibleColumns() {
            
            SimpleColumnConfig nameCol = new SimpleColumnConfig(DefaultColumnIds.NAME, "Name") {
                public Object getCellData(ReposObj obj) {
                    return "<b>" + obj.getAttribute("name").getValue() + "</b>";
                }
            };
            nameCol.setAlignment(HorizontalAlignment.LEFT);
            columns.add(nameCol);
            
            SimpleColumnConfig valueCol = new SimpleColumnConfig(COLUMN_ID_PROP_VALUE, "Value") {
                public Object getCellData(ReposObj obj) {
                    return obj.getAttribute(COLUMN_ID_PROP_VALUE).getValue();
                }
            };
            valueCol.setResizable(true);
            if (!isRreadOnly) {
                valueCol.setEditor(new CellEditor(new TextField<String>()));  
            }
            columns.add(valueCol);
            
            if (!isRreadOnly) {
                addDeleteColumn(false).setActionLabel("Remove").setHidden(false);
            }
        }
    }


}
