/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor.widgets;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.extjs.gxt.ui.client.core.El;
import com.extjs.gxt.ui.client.data.BasePagingLoadResult;
import com.extjs.gxt.ui.client.data.DataReader;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.google.gwt.dom.client.Element;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.base.client.remote.ICollectionCallBack;
import com.iona.soa.web.repository.base.client.remote.RemoteClient;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiCollectionCallBack;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.ReposURL;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.ActionColumnConfig;
import com.iona.soa.web.repository.main.client.base.ui.BaseCollectionPanel;
import com.iona.soa.web.repository.main.client.base.ui.RemoteCollectionPanel;
import com.iona.soa.web.repository.main.client.base.ui.RemoteRestDataProxy;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator;
import com.iona.soa.web.repository.main.client.base.ui.RemoteCollectionPanel.StoreAndGridSynchronizer;

public class PropertyOverrideAttrField extends PropertyAttrField {

    private final String defaultsCollection;
    
    private final Map<String, ReposObj> defaultsMap = new HashMap<String, ReposObj>();
    
    public PropertyOverrideAttrField(ReposObjAttribute reposObjAttribute, 
                                     String label,
                                     String defaultsCollectionLink) {
        super(reposObjAttribute, label);
        defaultsCollection = defaultsCollectionLink;
    }
    
    public PropertyOverrideAttrField(final ReposObjAttribute anAttr, 
                                     final String label,
                                     String defaultsCollectionLink,
                                     final String[] somePropertyKeySuggestions) {
        
        super(anAttr, label, somePropertyKeySuggestions);
        defaultsCollection = defaultsCollectionLink;
    }

    @Override
    protected RemoteCollectionPanel createCollectionPanel(SimpleGridConfigurator gridConfig,
                                                          RemoteRestDataProxy dataProxy) {
        return new MergedDataSourceRemoteCollectionPanel(gridConfig, dataProxy);
    }
    
    @Override
    protected RemoteRestDataProxy createDataProxy() {
        MergedDataSourceRemoteRestDataProxy dataProxy 
            = new MergedDataSourceRemoteRestDataProxy(MainModule.getRemoteClient());
        dataProxy.setResourceURL(new ReposURL(getAttribute().getCollectionLink()));
        return dataProxy;
    }
    
    @Override
    protected SimpleGridConfigurator getGridConfig() {
        return new MergedDataSourcePropsGridConfig(getAttribute().isReadOnly()
                                                   || (getInput() != null && !getInput().isEditAllowed()));
    }
    
    private ReposObj getShallowCopy(ReposObj aPropObj) {
        ReposObj data = new ReposObj();
        
        ReposObjAttribute propNameAttribute = new ReposObjAttribute();
        propNameAttribute.setName("name");
        propNameAttribute.setSimpleValue(aPropObj.getAttributeValue("name"));
        data.addAttribute(propNameAttribute);

        ReposObjAttribute propValAttribute = new ReposObjAttribute();
        propValAttribute.setName("value");
        propValAttribute.setSimpleValue(aPropObj.getAttributeValue("value"));
        data.addAttribute(propValAttribute);
        return data;
    }
    
    private final class MergedDataSourceRemoteCollectionPanel extends RemoteCollectionPanel {

        public MergedDataSourceRemoteCollectionPanel(SimpleGridConfigurator gridConfig,
                                                     RemoteRestDataProxy dataProxy) {
            super(gridConfig, dataProxy);
        }
        
        @Override
        protected StoreAndGridSynchronizer createSynchronizer() {
            return new MergedDataSourceStoreAndGridSynchronizer(gridPanel, dataProxy);
        }
    }

    private final class MergedDataSourceRemoteRestDataProxy extends RemoteRestDataProxy {

        public MergedDataSourceRemoteRestDataProxy(RemoteClient client) {
            super(client);
        }
        
        @Override
        protected ICollectionCallBack 
        getLoadingCallBack(final DataReader<PagingLoadConfig, PagingLoadResult<ReposObjDataModel>> reader,
                           final AsyncCallback<PagingLoadResult<ReposObjDataModel>> callback) {
        
            return new AbstractUiCollectionCallBack() {
                public void processReposObjCollection(final ReposObjCollection overrides) {
                    try {
                        getRemoteClient().getReposObjList(ReposHandle.getAbsoluteLink(defaultsCollection), 
                                                     null, 
                                                     null, 
                                                     getMergedLoadingCallBack(callback, overrides));
                    } finally {
                        Utils.closeWaitDialog();
                    }
                }

            };
        }
        
        private ICollectionCallBack getMergedLoadingCallBack(
                                         final AsyncCallback<PagingLoadResult<ReposObjDataModel>> callback,
                                         final ReposObjCollection overrides) {
            
            return new AbstractUiCollectionCallBack() {
                    @Override
                    public void processReposObjCollection(ReposObjCollection defaults) {
                        for (ReposObj defaultObj : defaults.getTheReposObjs()) {
                            defaultsMap.put(defaultObj.getAttributeValue("name"), getShallowCopy(defaultObj));
                        }
                        
                        List<ReposObjDataModel> gridData = merge(defaults.getTheReposObjs(), 
                                                                 overrides.getTheReposObjs());
                        callback.onSuccess(new BasePagingLoadResult<ReposObjDataModel>(gridData));
/*                      
                        Map<String, ReposObj> dataMap = new HashMap<String, ReposObj>();
                        for (ReposObj defaultProp : defaultProps.getTheReposObjs()) {
                            
                            // clearing any ref to the resourtce it belongs too so there is no risk of
                            // commiting changes to it
                            defaultProp.setSelfLink(null);
                            defaultProp.setRepresentingNode(null);
                            defaultProp.setServerBaseLink(null);
                            defaultProp.setSelfLink(null);
                            defaultProp.setEditLink(null);
                            defaultProp.setFeedSubscriptionLink(null);
                            defaultProp.setId(null);
                            
                            dataMap.put(defaultProp.getAttributeValue("name"),
                                        getShallowCopy(defaultProp));
                            
                            defaultsMap.put(defaultProp.getAttributeValue("name"), defaultProp);
                        }
                        
                        for (ReposObj overrideObj : overrides.getTheReposObjs()) {
                            dataMap.put(overrideObj.getAttributeValue("name"), overrideObj);
                        }
        
                        // populate grid
                        ReposObj[] objs = dataMap.values().toArray(new ReposObj[dataMap.size()]);
                        List<ReposObjDataModel> gridData = new ArrayList<ReposObjDataModel>(objs.length);
                        for (ReposObj aReposObj : objs) {
                            gridData.add(new ReposObjDataModel(aReposObj));
                        }
                          
                        callback.onSuccess(new BasePagingLoadResult<ReposObjDataModel>(gridData));
*/                        
                    }
                };
        }
        
        private List<ReposObjDataModel> merge(List<ReposObj> defaults, List<ReposObj> overrides) {
            Map<String, ReposObj> mergeMap = new HashMap<String, ReposObj>();
            
            for (ReposObj overrideObj : overrides) {
                mergeMap.put(overrideObj.getAttributeValue("name"), overrideObj);
            }

            for (ReposObj defaultObj : defaults) {
                if (!mergeMap.containsKey(defaultObj.getAttributeValue("name"))) {
                    mergeMap.put(defaultObj.getAttributeValue("name"),
                                getShallowCopy(defaultObj));
                }
            }
            
            ReposObj[] objs = mergeMap.values().toArray(new ReposObj[mergeMap.size()]);
            List<ReposObjDataModel> merged = new ArrayList<ReposObjDataModel>(objs.length);
            for (ReposObj aReposObj : objs) {
                BaseCollectionPanel bc = (BaseCollectionPanel)dataPanel;
                Grid<ReposObjDataModel> grid = bc.getGrid();
                merged.add(RemoteRestDataReader.createModel(aReposObj, grid.getColumnModel()));
            }
            
            return merged;
        }
    }
    
    
    private final class MergedDataSourceStoreAndGridSynchronizer extends StoreAndGridSynchronizer {

        public MergedDataSourceStoreAndGridSynchronizer(Grid<ReposObjDataModel> gridPanel,
                                                        RemoteRestDataProxy dataProxy) {
            super(gridPanel, dataProxy);
        }
        
        @Override
        public void reposObjCreated(ReposObj createdObj, String parentCollection, int locationIndex) {

            String createdPropName = createdObj.getAttributeValue("name");
            // remove the default if there was any
            

            for (ReposObjDataModel rcd : store.getModels()) {
                String propName = rcd.getPOJO().getAttributeValue("name");
                if (createdPropName.equals(propName) && rcd.getPOJO().getSelfLink() == null) {
                    locationIndex = grid.getStore().indexOf(rcd);
                    store.remove(rcd);
                }
            }
            
            super.reposObjCreated(createdObj, parentCollection, locationIndex);
        }
        
        @Override
        public void reposObjDeleted(ReposObj deletedObj) {
            
            ReposObjDataModel deletedRcd = findRecord(deletedObj);
            if (deletedRcd == null) {
                return;
            }
            int index = grid.getStore().indexOf(deletedRcd);
            
            super.reposObjDeleted(deletedObj);
            
            // re-insert the default from the app if there was any
            ReposObj defaultProp = defaultsMap.get(deletedObj.getAttributeValue("name"));
            if (defaultProp != null) {
                ReposObjDataModel rcd = new ReposObjDataModel(defaultProp);
                store.insert(rcd, index);
                
                Element rowElement = grid.getView().getRow(grid.getStore().indexOf(rcd));
                if (rowElement != null) {
                    El el = new El((com.google.gwt.user.client.Element)rowElement);
                    //el.blink(FxConfig.NONE);
                    Utils.highlight(el);
                }
            }
        }

    }
    
    private final class MergedDataSourcePropsGridConfig extends PropsGridConfigurator {

        public MergedDataSourcePropsGridConfig(boolean readOnly) {
            super(readOnly);
        }
        
        @Override
        protected ActionColumnConfig<ReposObjDataModel> addDeleteColumn(final boolean askConfirmation) {
            ActionColumnConfig.ActionColumnCallback<ReposObjDataModel> callback = 
                new ActionColumnConfig.ActionColumnCallback<ReposObjDataModel>() {
                    public void actionClicked(ReposObjDataModel record, Element anchorElement) {
                        ReposObj obj = record.getPOJO();
                        MainModule.get().getWorkspaceControl().deleteObjects(new ReposObj[] {obj},
                                                                             null,
                                                                             askConfirmation);
                    }
                };  
            
            ActionColumnConfig<ReposObjDataModel> column =
                new ActionColumnConfig<ReposObjDataModel>(getGrid(), DefaultColumnIds.DELETE) {
                    @Override
                    protected String getActionLabel(ReposObjDataModel record) {
                        ReposObj obj = record.getPOJO();
                        boolean isDefaultPropAvailable =
                            defaultsMap.get(obj.getAttributeValue("name")) != null;
                        return obj.getSelfLink() == null
                                ? null
                                : isDefaultPropAvailable
                                      ? "Revert"
                                      : "Remove";
                    }
                };
            
            column.setActionLabel("Remove").setActionCallback(callback);
            
            columns.add(column);
            return column;
        }
        
    }

}
