/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor.widgets;

import java.util.ArrayList;
import java.util.List;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.Style.HorizontalAlignment;
import com.extjs.gxt.ui.client.core.El;
import com.extjs.gxt.ui.client.data.BaseModelData;
import com.extjs.gxt.ui.client.data.ModelData;
import com.extjs.gxt.ui.client.event.GridEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.store.StoreEvent;
import com.extjs.gxt.ui.client.store.StoreListener;
import com.extjs.gxt.ui.client.widget.BoxComponent;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.ColumnModel;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.base.client.remote.ICollectionCallBack;
import com.iona.soa.web.repository.main.client.HtmlSnippets;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiCollectionCallBack;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.ui.ActionColumnConfig;
import com.iona.soa.web.repository.main.client.base.ui.ActionColumnConfig.ActionColumnCallback;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator.DefaultColumnIds;
import com.iona.soa.web.repository.main.client.base.ui.editor.AfterClosureCallBack;
import com.iona.soa.web.repository.main.client.base.ui.editor.BaseSelectRefDialog;

public class RefValAttrField extends CollectionValAttrField {

    protected static final HtmlSnippets HTML_SNIPPETS = GWT.create(HtmlSnippets.class);
    private ListStore<ModelData> gridStore;
    private boolean dirty;
    private EditorGrid<ModelData> gridPanel;

    public RefValAttrField(final ReposObjAttribute anAttr, String label) {
        super(anAttr, label);
    }
    
    @Override
    protected BoxComponent createDataPanel() {
        
        
        gridPanel = new EditorGrid<ModelData>(null, null);

        //gridPanel.setAutoWidth(true);
        gridPanel.setAutoHeight(true);
        gridPanel.setHideHeaders(true);
        gridPanel.setStripeRows(true);
        gridPanel.setBorders(true);  
        gridPanel.disableTextSelection(true);

        gridPanel.setAutoExpandColumn(DefaultColumnIds.NAME); 
    
        //gridPanel.setSelectionModel(new CellSelectionModel<ModelData>());
        
        List<ColumnConfig> colCfgs = new ArrayList<ColumnConfig>();
        colCfgs.add(prepareCaptionColumn(gridPanel));
        if (!getAttribute().isReadOnly()
                && (getInput() == null || getInput().isEditAllowed())) {
            colCfgs.add(prepareRemoveColumn(gridPanel));
                        
            gridPanel.addListener(Events.RowClick, new Listener<GridEvent>() {
                public void handleEvent(GridEvent be) {
                    if (gridPanel.getStore() != null && gridPanel.getStore().getCount() == 0) {
                        runAddCommand();
                    }
                }
            });
        }
        
        gridStore = prepareStore();
        gridPanel.reconfigure(gridStore, new ColumnModel(colCfgs));

        // add view for handling empty state
        gridPanel.getView().setEmptyText(getAttribute().isMulti() ? "none" : "undefined");

        gridPanel.addStyleName("force-hand-pointer");

        return gridPanel;
    }
    
    public void markAsDirty() {
        dirty = true;
    }

    @Override
    protected void runAddCommand() {
        
        Utils.showWaitDialog();
        ICollectionCallBack collectionCallBack = new AbstractUiCollectionCallBack() {

            public void processReposObjCollection(ReposObjCollection list) {
                Utils.closeWaitDialog();
                AfterClosureCallBack callBack = new AfterClosureCallBack() {
                    public void onDialogClosure() {
                        refresh();
                    }
                };
                
                BaseSelectRefDialog dg = new BaseSelectRefDialog(getAttribute(), list, callBack);
                dg.show();
                dg.center();
            }
        };
        
        MainModule.getRemoteClient().getReposObjList(getAttribute().getCollectionLink(),
                                                     null,
                                                     null,
                                                     collectionCallBack);
    }
    
    private void addToStore(String caption, String href) {
        ModelData rcd = createGridRecord(caption, href);

        gridStore.add(rcd);
        
        int index = gridStore.indexOf(rcd);
        gridPanel.getView().refresh(false);
        Element rowElement = gridPanel.getView().getRow(index);
        if (rowElement != null) {
            El el = new El((com.google.gwt.user.client.Element)rowElement);
            //el.blink(FxConfig.NONE);
            Utils.highlight(el);
        }
    }

    private ModelData createGridRecord(String caption, String href) {
        ModelData rcd = new BaseModelData();
        rcd.set(DefaultColumnIds.NAME, caption);
        rcd.set(DefaultColumnIds.DELETE, href);
        rcd.set(DefaultColumnIds.SELF_LINK, href);
        return rcd;
    }
    
    private void updateGridContent() {
        
        // 1st scan for records to remove from the grid
        List<String> upToDateHrefs = new ArrayList<String>(getAttribute().getValues().length); 
        for (String href : getAttribute().getValues()) {
            upToDateHrefs.add(href);
        }
        List<ModelData> storeContent = new ArrayList<ModelData>();
        storeContent.addAll(gridStore.getModels());
        for (ModelData rcd : storeContent) {
            if (!upToDateHrefs.contains(rcd.get(DefaultColumnIds.SELF_LINK))) {
                gridStore.remove(rcd);
            }
        }
        
        // 2nd scan for records to add to the grid
        List<String> hrefsInGrid = new ArrayList<String>(gridStore.getModels().size()); 
        for (ModelData rcd : gridStore.getModels()) {
            hrefsInGrid.add((String)rcd.get(DefaultColumnIds.SELF_LINK));
        }
        for (String href : getAttribute().getValues()) {
            if (!hrefsInGrid.contains(href)) {
                addToStore(getAttribute().getValueCaption(href), href);
            }
        }
    }
    
    private ListStore<ModelData> prepareStore() {
        
        List<ModelData> data = new ArrayList<ModelData>();
        for (String href : getAttribute().getValues()) {
            data.add(createGridRecord(getAttribute().getValueCaption(href), href));
        }
        
        ListStore<ModelData> store = new ListStore<ModelData>();
        store.add(data);

        store.addStoreListener(new StoreListener<ModelData>() {
            
            @Override
            public void storeAdd(StoreEvent<ModelData> se) { markAsDirty(); }
            
            @Override
            public void storeRemove(StoreEvent<ModelData> se) { markAsDirty(); }
        });
        
        return store;
    }

    private ColumnConfig prepareCaptionColumn(final EditorGrid<ModelData> aGridPanel) {
        
        ActionColumnConfig<ModelData> nameColumn = new ActionColumnConfig<ModelData>(aGridPanel,
                                                                                     DefaultColumnIds.NAME) {
            @Override
            protected String getActionLabel(ModelData record) {
                return record.get(DefaultColumnIds.NAME);
            }
        };
        nameColumn.setActionCallback(new ActionColumnCallback<ModelData>() {
            public void actionClicked(ModelData record, Element anchorElement) {
                String absoluteLink =
                    ReposHandle.getAbsoluteLink((String)record.get(DefaultColumnIds.SELF_LINK));
                MainModule.get().getWorkspaceControl().openLink(absoluteLink, true);
            }
        });
        nameColumn.setSortable(false);
        nameColumn.setResizable(true);
        nameColumn.setAlignment(HorizontalAlignment.LEFT);
        nameColumn.setWidth(300);
        
        return nameColumn;
    }

    private ColumnConfig prepareRemoveColumn(final EditorGrid<ModelData> aGridPanel) {
        ActionColumnConfig<ModelData> removeColumn =
            new ActionColumnConfig<ModelData>(aGridPanel, DefaultColumnIds.DELETE);
        
        removeColumn.setActionLabel("Remove").setWidth(60);
        removeColumn.setActionCallback(new ActionColumnCallback<ModelData>() {
            public void actionClicked(ModelData record, Element anchorElement) {
                String href = (String)record.get(DefaultColumnIds.SELF_LINK);
                getAttribute().removeRefValue(href);
                aGridPanel.getStore().remove(record);
            }
        });
        removeColumn.setSortable(false);
        removeColumn.setResizable(false);
        return removeColumn;
    }

    @Override
    public void commit() {
        // Nothing to do as the RefDialog updates the attribute.
        // Just reset our isDirty state.
        dirty = false;
    }

    @Override
    public boolean isDirty() {
        return dirty;
    }

    @Override
    public void refresh() {
        updateGridContent();
        gridPanel.getView().refresh(false);        
    }
}
