/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.base.ui.editor.widgets;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.core.El;
import com.extjs.gxt.ui.client.data.BaseListLoader;
import com.extjs.gxt.ui.client.data.ListLoadConfig;
import com.extjs.gxt.ui.client.data.ListLoadResult;
import com.extjs.gxt.ui.client.data.LoadEvent;
import com.extjs.gxt.ui.client.data.Loader;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.store.StoreSorter;
import com.extjs.gxt.ui.client.widget.BoxComponent;
import com.extjs.gxt.ui.client.widget.form.ComboBox;
import com.google.gwt.user.client.ui.Widget;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.RemoteRestListDataProxy;

public class SingleRefValAttrField extends RefValAttrField {
    private MyCombo combo;
    
    public SingleRefValAttrField(final ReposObjAttribute anAttr, String label) {
        super(anAttr, label);
    }

    @Override
    public void commit() {
        if (isDirty()) {
            ReposObj obj = combo.getValue().getPOJO();
            if (obj == null) {
                getAttribute().clearValues();
            } else {
                getAttribute().setRefValue(obj);
            }
        }
    }

    @Override
    public boolean isDirty() {
        String attVal = getAttribute().getValue();
        ReposObjDataModel val = combo.getValue();
        ReposObj obj = val == null ? null : val.getPOJO();
        return !(val == null 
                || attVal == null && obj == null 
                || attVal != null && obj != null && attVal.endsWith(obj.getId()));
    }
    
    @Override
    public void refresh() {
        // do nothing?
    }

    @Override
    protected BoxComponent createDataPanel() {
        combo = new MyCombo();
        RemoteRestListDataProxy dataProxy = RemoteRestListDataProxy.createProxyForCollection(getAttribute().getCollectionLink());
        ListStore<ReposObjDataModel> store = new ListStore<ReposObjDataModel>(new BaseListLoader<ListLoadConfig, ListLoadResult<ReposObjDataModel>>(dataProxy) {
            @Override    
            protected void onLoadSuccess(ListLoadConfig loadConfig, ListLoadResult<ReposObjDataModel> data) {
                // Insert "None" record
                ReposObjDataModel noneRcd = new ReposObjDataModel(null);
                noneRcd.set(ReposObjDataModel.FIELD_CAPTION, "None");
                data.getData().add(noneRcd);
                super.onLoadSuccess(loadConfig, data);
            }
        });
        store.setStoreSorter(new StoreSorter<ReposObjDataModel>(new Comparator<ReposObjDataModel>() {
            public int compare(ReposObjDataModel o1, ReposObjDataModel o2) {
                if (o1.getPOJO() == null) {
                    return -1;
                } else if (o2.getPOJO() == null) {
                    return 1;
                }
                return o1.getPOJO().getCaption().compareToIgnoreCase(o2.getPOJO().getCaption());
            }
        }));
        combo.setStore(store);
        combo.getView().disableEvents(true);

        // Overrides for us
        combo.setAutoHeight(true);
        combo.addStyleName("force-hand-pointer");
        combo.setForceSelection(true);
        combo.setEditable(false);
        combo.setDisplayField(ReposObjDataModel.FIELD_CAPTION);
        combo.setTriggerAction(ComboBox.TriggerAction.ALL);
        combo.setLoadingText("Loading references...");
        combo.setTypeAhead(true);
        combo.setSelectOnFocus(true);
        combo.setBorders(false);  
        combo.disableTextSelection(true);
        combo.setTemplate(getTemplate());

        // trigger store data to be loaded on render
        combo.addListener(Events.Render, new Listener<ComponentEvent>() {
            public void handleEvent(ComponentEvent be) {
                combo.doQuery("", true);
            }
        });

        combo.addListener(Events.Change, new Listener<ComponentEvent>() {
            public void handleEvent(ComponentEvent be) {
                toggleSelectionEnable();
            }
        });

        store.getLoader().addListener(Loader.Load, new Listener<LoadEvent<?, ?>>() {
            public void handleEvent(LoadEvent<?, ?> be) {
                List<ReposObjDataModel> selection = new ArrayList<ReposObjDataModel>(1);
                String attVal = getAttribute().getValue();
                if (attVal == null) {
                    selection.add(combo.getStore().getAt(0));
                } else {
                    for (ReposObjDataModel obj : combo.getStore().getModels()) {
                        if (obj.getPOJO() != null && attVal.endsWith(obj.getPOJO().getId())) {
                            selection.add(obj);
                            break;
                        }
                    }
                }
                combo.setSelection(selection);
                toggleSelectionEnable();
            }
        });

        return combo;
    }
    
    @Override
    protected Widget createButtonWidget() {
        // do nothing. No Buttons required.
        return null;
    }
    
    // Toggle color of selected item to grey ("None" item) or black (all others) 
    private void toggleSelectionEnable() {
        if (combo.getValue() == null) {
            // do nothing
        } else if (combo.getValue().getPOJO() == null) {
            combo.getInputEl().addStyleName("x-item-disabled");
        } else {
            combo.getInputEl().removeStyleName("x-item-disabled");
        }
    }

    // List item display template: The "None" item is grey all others black
    private native String getTemplate()  /*-{ 
        return [ 
          '<tpl for=".">', 
          '<tpl if="values.POJO != null"><div class="x-combo-list-item">{[values.caption]}</div></tpl>', 
          '<tpl if="values.POJO == null"><div class="x-combo-list-item x-item-disabled">{[values.caption]}</div></tpl>', 
          '</tpl>' 
        ].join(""); 
    }-*/;  
    
    // Override to allow access to input El
    private static class MyCombo extends ComboBox<ReposObjDataModel> {
        @Override
        public El getInputEl() {
          return super.getInputEl();
        }
    }
}
