/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.categories;

import java.util.ArrayList;
import java.util.List;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.data.BaseListLoader;
import com.extjs.gxt.ui.client.data.BaseModelData;
import com.extjs.gxt.ui.client.data.DataProxy;
import com.extjs.gxt.ui.client.data.DataReader;
import com.extjs.gxt.ui.client.data.ListLoadConfig;
import com.extjs.gxt.ui.client.data.ListLoadResult;
import com.extjs.gxt.ui.client.data.ListLoader;
import com.extjs.gxt.ui.client.data.LoadEvent;
import com.extjs.gxt.ui.client.data.ModelData;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.GridEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.LoadListener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.Component;
import com.extjs.gxt.ui.client.widget.Container;
import com.extjs.gxt.ui.client.widget.Html;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.ColumnModel;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.extjs.gxt.ui.client.widget.grid.GridSelectionModel;
import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;
import com.iona.soa.web.repository.base.client.model.IReposObjChangeListener;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjChangeNotifier;
import com.iona.soa.web.repository.base.client.remote.IObjCallBack;
import com.iona.soa.web.repository.main.client.HtmlSnippets;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.IFilterable;
import com.iona.soa.web.repository.main.client.base.ReposURL;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.ActionColumnConfig;
import com.iona.soa.web.repository.main.client.base.ui.RemoteCollectionPanel;
import com.iona.soa.web.repository.main.client.base.ui.RemoteRestDataProxy;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator.DefaultColumnIds;

public class CategoriesPanel2 extends LayoutContainer {
    
    private static final String COL_ID_RENAME = "rename";
    
    private static final HtmlSnippets SNIPPETS = GWT.create(HtmlSnippets.class);
    
    private Container<Component> addCategoryPanel;
    private EditorGrid<BaseModelData> allPanel;
    private final RemoteRestDataProxy dataProxy;
    private RemoteCollectionPanel categoriesPanel;
    
    private ReposURL currentContextURL;
    private IFilterable activeCategoryRenderer;
    private boolean editing;

    
    public CategoriesPanel2() {
        setId("arr-categories-panel");
        setBorders(false);
        
        dataProxy = new RemoteRestDataProxy(MainModule.getRemoteClient());        
        createUI();

        final IReposObjChangeListener changeListener = new IReposObjChangeListener() {
            public void reposObjCreated(ReposObj createdObj, String containingCollectionUrl, int index) {
                update(createdObj);
            }

            public void reposObjDeleted(ReposObj deletedObj) {
                update(deletedObj);
            }

            public void reposObjUpdated(ReposObj updatedObj) {
                update(updatedObj);
            }
            
            private void update(ReposObj target) {
                // If we are showing categories, i..e the currentContext is categprizable
                // and the target eclass matches ours
                if (isVisible()
                    && (currentContextURL.getEClass().equals(target.getEClass())
                        || EClassHelper.CATEGORY_ECLASS.equals(target.getEClass()))) {
                    categoriesPanel.getStore().getLoader().load();
                }
            }
        };
        
        // Bind the remote client and the store synchronizer together
        ReposObjChangeNotifier.INSTANCE.addListener(changeListener);

        // Remove the binding when the panel is being destroyed
        Listener<ComponentEvent> listener = new Listener<ComponentEvent>() {
            public void handleEvent(ComponentEvent be) {
                ReposObjChangeNotifier.INSTANCE.removeListener(changeListener);
            }
        };
        super.addListener(Events.Detach, listener);
    }

    public void contextChanged(ReposURL contextURL) {
        currentContextURL = contextURL;
        if (EClassHelper.isCollectionURL(contextURL.getURL())
                && EClassHelper.isCategorizable(currentContextURL.getEClass())) {
            setVisible(true);
            
            String categoriesURL = EClassHelper.getCollectionLinkForClass(EClassHelper.CATEGORY_ECLASS)
                                    + "?emfql=eClassName="
                                    + currentContextURL.getEClass();
            dataProxy.setResourceURL(new ReposURL(categoriesURL));
            categoriesPanel.getStore().getLoader().load();

            // Refresh the All label by reloading the store and
            // select the one and only row - the "All" row
            allPanel.getStore().getLoader().load();
            allPanel.getSelectionModel().select(0);
            
        } else {
            setVisible(false);
            allPanel.getStore().removeAll();
            categoriesPanel.getStore().removeAll();
        }
    }
    
    public void contextChanged(ReposURL contextURL, IFilterable renderer) {
        activeCategoryRenderer = renderer;
        contextChanged(contextURL);
    }
    
    public ListStore<ReposObjDataModel> getStore() {
        return categoriesPanel.getStore(); // TOOD - perhaps get supplied the store later
    }
    
    private void createUI() {
        // We provide our own header here, one that can have stuff other than a "Tool" in it..
        Container<Component> headerPanel = createHeaderPanel();
        
        // Special single entry panel for "All" category
        allPanel = createAllPanel();
        
        categoriesPanel = createCategoriesPanel();
        categoriesPanel.getGrid().getColumnModel().getColumnById(COL_ID_RENAME).setHidden(true);
        categoriesPanel.getGrid().getColumnModel().getColumnById(DefaultColumnIds.DELETE).setHidden(true);
        
        // Add category link, referenced in editTag listener below.
        // Hidden initially, until edit is selected.
        addCategoryPanel = createAddPanel();
        
        add(headerPanel);
        add(allPanel);
        add(categoriesPanel);        
        add(addCategoryPanel);
        
        categoriesPanel.getGrid().addListener(Events.RowClick, new Listener<GridEvent>() {
            public void handleEvent(GridEvent ge) {
                
                if (ge.colIndex == 0) {
                    allPanel.getSelectionModel().deselect(0);
                    
                    ReposObj category =
                        ((ReposObjDataModel)ge.grid.getStore().getModels().get(ge.rowIndex)).getPOJO();
                    // TODO - pass category obj directly
                    activeCategoryRenderer.setFilterByCategory(category.getSelfLink(), category.getCaption());
                } else {
                    ge.doit = false;
                }
            }
        });
        
        allPanel.addListener(Events.RowClick, new Listener<GridEvent>() {
            public void handleEvent(GridEvent ge) {
                if (ge.colIndex == 0) {
                    categoriesPanel.getGrid().getSelectionModel().deselectAll();
                    
                    if (ge.grid.getSelectionModel().getSelectedItems().size() == 1) {
                        
                        String cn = ((ModelData)ge.grid.getStore().getModels().get(ge.rowIndex)).get("name");
                        activeCategoryRenderer.setFilterByCategory(null, cn);
                    }
                } else {
                    ge.doit = false;
                }
            }
        });
        
        // Selection preserver
        categoriesPanel.getStore().getLoader().addLoadListener(new LoadListener() {
            private String selectedUrl;
            
            @Override
            public void loaderBeforeLoad(LoadEvent le) {
                selectedUrl = null;
                GridSelectionModel<ReposObjDataModel> sm = categoriesPanel.getGrid().getSelectionModel();
                ReposObjDataModel selected = sm.getSelection().size() == 1 ? sm.getSelectedItem() : null;
                if (selected != null) {
                    selectedUrl = selected.get(ReposObjDataModel.FIELD_SELF_LINK);
                }
                le.doit =  true;
            }
            
            @Override
            public void loaderLoad(LoadEvent le) {
                restoreSelection();
            }
            
            @Override
            public void loaderLoadException(LoadEvent le) {
                restoreSelection();
            }
            
            private void restoreSelection() {
                if (selectedUrl != null) {
                    for (ReposObjDataModel model : categoriesPanel.getStore().getModels()) {
                        if (selectedUrl.equals(model.get(ReposObjDataModel.FIELD_SELF_LINK))) {
                            categoriesPanel.getGrid().getSelectionModel().select(model);
                        }
                    }
                }
            }
        });
    }
    
    private Container<Component> createAddPanel() {
        LayoutContainer acp = new LayoutContainer();
        acp.setBorders(false);
        acp.setId("arr-add-category-panel");
        acp.setVisible(false);
        
        HTML addCategoryLink = 
            new HTML(SNIPPETS.anchorTagWithCssCls("Add another category",
                                                  "arr-action-link arr-add-category-link"));
        
        addCategoryLink.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                // TODO - should use store listener instead of direct manipulation callback!
                // Causes selection to be lost after edit
                IObjCallBack categoriesUpdatedCallback = new AbstractUiObjCallBack() {
                    @Override
                    public void processReposObj(ReposObj anObj) {
                        categoriesPanel.getStore().getLoader().load();
                    }
                };
                
                CategoryContextMenuHelper.createNewCategory(currentContextURL.getEClass(), 
                                                            categoriesUpdatedCallback);
            }
        });
        acp.add(addCategoryLink);
        return acp;
    }
    
    private RemoteCollectionPanel createCategoriesPanel() {
        EditorGrid<ReposObjDataModel> gridPanel = new EditorGrid<ReposObjDataModel>(null, null);
        gridPanel.setId("arr-categories-grid");
        SimpleGridConfigurator gridConfig = new CategoriesGridConfig();
        gridConfig.configure(gridPanel, dataProxy);
        // Overrides for this grid
        gridPanel.setAutoHeight(true);
        gridPanel.setHideHeaders(true);
        gridPanel.setStripeRows(false);
        gridPanel.setBorders(false);
                
        // TODO - this is a hack - could make AtomClient handle null urls?
        String categoriesURL = EClassHelper.getCollectionLinkForClass(EClassHelper.CATEGORY_ECLASS)
                                                                      + "?emfql=eClassName=Unknown";
        dataProxy.setResourceURL(new ReposURL(categoriesURL));
        
        // store gets attached to gridPanel here
        RemoteCollectionPanel collectionPanel = new RemoteCollectionPanel(gridConfig, dataProxy);
        
        collectionPanel.addStyleName("force-hand-pointer");
        return collectionPanel;
    }
    
    private Container<Component> createHeaderPanel() {
        LayoutContainer headerPanel = new LayoutContainer();
        headerPanel.setId("arr-categories-header");
        headerPanel.setBorders(false);
        
        String editHtml = SNIPPETS.anchorTagWithCssCls("Edit", 
                "arr-action-link arr-edit-categories-link");
        final Html editLink = new Html(editHtml) {
            @Override
            protected void afterRender() {
                super.afterRender();
                el().addEventsSunk(Events.OnClick);
                el().addEventsSunk(Events.OnKeyUp);
            }
        };
        editLink.addStyleName("edit-category-link");
        Listener<ComponentEvent> listener = new Listener<ComponentEvent>() {
            public void handleEvent(ComponentEvent be) {
                editing = !editing;
                
                addCategoryPanel.setVisible(editing);
                categoriesPanel.getGrid().getColumnModel().getColumnById(COL_ID_RENAME).setHidden(!editing);
                categoriesPanel.getGrid().getColumnModel().getColumnById(DefaultColumnIds.DELETE)
                    .setHidden(!editing);
                categoriesPanel.getGrid().getView().refresh(false);
                
                if (editing) {
                    String anchorCssCls = "arr-action-link arr-done-editing-categories-link";
                    editLink.setHtml(SNIPPETS.anchorTagWithCssCls("Done Editing", anchorCssCls));
                    allPanel.removeStyleName("force-hand-pointer");
                    categoriesPanel.removeStyleName("force-hand-pointer");
                } else {
                    editLink.setHtml(SNIPPETS.anchorTagWithCssCls("Edit", "arr-edit-categories-link"));
                    allPanel.addStyleName("force-hand-pointer");
                    categoriesPanel.addStyleName("force-hand-pointer");
                }
            }
        }; 
        editLink.addListener(Events.OnClick, listener);
        editLink.addListener(Events.OnKeyUp, listener);
        
        // Edit tag floated right in css, must be added first.
        headerPanel.add(editLink); 
        headerPanel.add(new Label("Categories"));
        headerPanel.addStyleName("x-panel-header");
        headerPanel.addStyleName("x-panel-header-no-border");
        
        return headerPanel;
    }
    
    private EditorGrid<BaseModelData> createAllPanel() {
        
        
        List<ColumnConfig> columns = new ArrayList<ColumnConfig>(0);
        columns.add(new ColumnConfig("name", "Name", 255));
        ColumnModel columnModel = new ColumnModel(columns);
        
        DataProxy<ListLoadConfig, ListLoadResult<BaseModelData>> proxy = 
            new DataProxy<ListLoadConfig, ListLoadResult<BaseModelData>>() {

                public void load(DataReader<ListLoadConfig, ListLoadResult<BaseModelData>> reader,
                                 ListLoadConfig loadConfig,
                                 AsyncCallback<ListLoadResult<BaseModelData>> callback) {
                    // If we want to include the number then we'd have to be supplied the
                    // store of the ICollectionPanel from the caller of this method.  Then
                    // we'd could I suppose use that store directly with a custom renderer.
                    // Actually no - that store would change according to what category is selected
                    // so we'd could use an EMFQL query instead whenever that store changed.
                    // Too much work right now..
                    ListLoadResult<BaseModelData> result = new ListLoadResult<BaseModelData>() {

                        public List<BaseModelData> getData() {
                            String label =
                                "All " + EClassHelper.getPluralNameForClass(currentContextURL.getEClass());
                            
                            BaseModelData data = new BaseModelData();
                            data.set("name", label);
                            
                            List<BaseModelData> allData = new ArrayList<BaseModelData>(1);
                            allData.add(data);
                            
                            return allData;
                        }
                        
                    };
                    callback.onSuccess(result);
                }
                
            };
        
        ListLoader<ListLoadConfig> loader =
            new BaseListLoader<ListLoadConfig, ListLoadResult<BaseModelData>>(proxy);
        ListStore<BaseModelData> store = new ListStore<BaseModelData>(loader);

        EditorGrid<BaseModelData> ap = new EditorGrid<BaseModelData>(store, columnModel);
        ap.setId("arr-all-category-panel");
        ap.setAutoExpandColumn("name");
        ap.setHideHeaders(true);
        ap.setStripeRows(false);
        ap.setBorders(false);
        ap.addStyleName("force-hand-pointer");
        ap.setSelectionModel(new GridSelectionModel<BaseModelData>());
        
        return ap;
    }
    
    private final class CategoriesGridConfig extends SimpleGridConfigurator {
        

        @Override
        protected void addVisibleColumns() {
            addNameColumn();
            addRenameColumn();//.setHidden(true);
            addDeleteColumn(true);//.setHidden(true);
            
            columns.get(0).setWidth(100);
            columns.get(1).setWidth(70);
            columns.get(2).setWidth(70);
        }

        @Override
        protected ColumnConfig addNameColumn() {
            SimpleColumnConfig column = new SimpleColumnConfig(DefaultColumnIds.NAME, "Name") {
                @Override
                public Object getCellData(ReposObj category) {
                    int numEntries = category.getAttribute("content").getValues().length;
                    StringBuffer sb =
                        new StringBuffer(category.getCaption()).append(" (").append(numEntries).append(")");
                    return sb.toString();
                }
            };
            
            columns.add(column);
            return column;
        }
        
        protected ColumnConfig addRenameColumn() {
            ActionColumnConfig.ActionColumnCallback<ReposObjDataModel> callback =
                new ActionColumnConfig.ActionColumnCallback<ReposObjDataModel>() {
                
                    public void actionClicked(ReposObjDataModel record, Element anchorElement) {
                        CategoryContextMenuHelper.renameCategory(record.getPOJO());
                    }
                };  
            
            ActionColumnConfig<ReposObjDataModel> column =
                new ActionColumnConfig<ReposObjDataModel>(getGrid(), COL_ID_RENAME)
                    .setActionLabel("Rename")
                    .setActionCallback(callback);
            columns.add(column);
            return column;
        }
    }
}
