/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.provisioning.actions;

import java.util.List;

import com.extjs.gxt.ui.client.widget.Window;
import com.extjs.gxt.ui.client.widget.Window.CloseAction;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjCollection;
import com.iona.soa.web.repository.base.client.remote.ICollectionCallBack;
import com.iona.soa.web.repository.base.client.remote.RemoteClient;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiCollectionCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ICollectionPanel;
import com.iona.soa.web.repository.main.client.base.IContextAction;
import com.iona.soa.web.repository.main.client.base.ui.AbstractCreateObjectAction;
import com.iona.soa.web.repository.main.client.provisioning.panels.MavenRepoSearchAndResultsPanel;
import com.iona.soa.web.repository.main.client.provisioning.panels.MavenRepoSearchAndResultsPanel.AddArtifactsCallback;

public class AddMavenArtifactContextAction extends AbstractCreateObjectAction implements IContextAction {

    public AddMavenArtifactContextAction(ICollectionPanel contextPanel) {
        super(contextPanel);
    }

    public String getCaption() {
        return "Import Maven artifact";
    }

    public String getToolTip() {
        return "create a new entry for the 'Maven Repository' collection";
    }

    public String getIconCs() {
        return "create-icon";
    }

    public void doIt() {
        //MavenRepoSearchDialog dialog = new MavenRepoSearchDialog();
        //dialog.show();
        final Window w = new Window();
        w.setWidth(700);        
        w.setTitle("Find artifacts");
        w.setIconStyle("search-icon");
        w.setHeight(720);
        w.setCloseAction(CloseAction.CLOSE);
        w.setModal(true);
        
        final ICollectionCallBack importedCallback = new AbstractUiCollectionCallBack(w) {
            @Override
            public void processReposObjCollection(ReposObjCollection list) {
                // Collection screen will get notified independently and refresh itself
                // so nothing to do here except close the dialog
                w.close();
            }
        };
        
        AddArtifactsCallback addCallback = new AddArtifactsCallback() {
            public void add(List<ReposObj> artifacts) {
                if (artifacts.isEmpty()) {
                    cancel();
                } else {
                    w.el().mask("Importing artifacts...");
                    importArtifacts(artifacts, importedCallback);
                }
            }
            
            public void cancel() {
                w.close();
            }
            
            public boolean alreadyInUse(ReposObj artifact) {
                return isPersistedArtifact(artifact);
            }
        };
        
        w.add(new MavenRepoSearchAndResultsPanel(addCallback));
        //w.setAutoScroll(true);
        w.show();
    }
    
    private static boolean isPersistedArtifact(ReposObj artifact) {
        return artifact.getAttributeValue("guid") != null;            
    }
    
    private static void importArtifacts(final List<ReposObj> artifacts, 
                                       final ICollectionCallBack createdCallback) {
        final RemoteClient client = MainModule.getRemoteClient();
        final String artifactCollectionURL 
            = EClassHelper.getCollectionLinkForClass(EClassHelper.ARTIFACT_ECLASS);
        
        client.createReposObjs(artifactCollectionURL, 
                              artifacts,
                              createdCallback);
    }    
}
