/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.provisioning.actions;

import java.util.List;

import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.Component;
import com.google.gwt.core.client.GWT;
import com.iona.soa.web.repository.base.client.model.DeploymentProfileHelper;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.main.client.UIMessages;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ICollectionPanel;
import com.iona.soa.web.repository.main.client.base.IEditorPanel;
import com.iona.soa.web.repository.main.client.base.ScheduledTaskType;
import com.iona.soa.web.repository.main.client.base.ISelectionProvider.ISelectionListener;
import com.iona.soa.web.repository.main.client.base.ISelectionSensitiveContextAction;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.WorkspaceCollectionPanel;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.ObjectCreationDialog;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.panels.ScheduledTaskPanel
    .ScheduledTaskDefaults;

public class AddScheduleTaskPublishAction implements ISelectionSensitiveContextAction, ISelectionListener {

    ReposObj currentObj;
    Component renderedActionWidget;
    ICollectionPanel panel;
    
    public AddScheduleTaskPublishAction(ICollectionPanel panel) {
        super();
        this.panel = panel;
    }
    
    public AddScheduleTaskPublishAction() {
        super();
    }

    /**
     * @param applicationEditor
     */

    public String getCaption() {
        return "Publish later...";
    }

    public String getToolTip() {
        return "Schedule an publish task";
    }

    public String getIconCs() {
        return "schedule-index-icon";
    }

    public void doIt() {
        UIMessages messages = GWT.create(UIMessages.class);
        ScheduledTaskDefaults defaults = new ScheduledTaskDefaults();
        
        String[] parameters = new String[7];

        if (currentObj.getEClass().equals(EClassHelper.DEPLOYMENT_PROFILE_ECLASS)) {
            // probably not going to go in here 

            parameters[3] = currentObj.getAttributeValue("guid"); 
            parameters[6] = DeploymentProfileHelper.getCurrentVersionGuid(currentObj);        
        }
        
        if (currentObj.getEClass().equals(EClassHelper.VERSIONED_PROFILE_ECLASS)) {
            // work out how to get the guid of the parent deployment profile
            //currentObj.
            parameters[3] = currentObj.getAttributeValue("parentGuid");
            parameters[6] = currentObj.getAttributeValue("guid");        
            
        }

        parameters[0] = "SET";
        parameters[1] = "http://com.iona.soa.model";
        parameters[2] = "DeploymentProfile";
        parameters[4] = "REFERENCE";
        parameters[5] = "active";

        
        defaults.setTaskType(ScheduledTaskType.SPECIFIC_DATE_SELECTION.toString());
        defaults.setTaskParameters(parameters);
        defaults.setTaskName(messages.defaultSchedulePublishTaskName(currentObj.getAttributeValue("name")));
 
        ObjectCreationDialog dialog = new ObjectCreationDialog(EClassHelper.SCHEDULED_TASK_ECLASS,
                new ObjectCreationDialog.AfterCreationCallBack() {
                    public void onDialogClosure(ReposObj anObj) {
                        if (panel != null) {
                            getStoreFromContentPanel(panel).getLoader().load();    
                        }
                        Utils.closeWaitDialog();
                    }                
                }, defaults);
       
        dialog.show();
    }
    
    private ListStore<ReposObjDataModel> getStoreFromContentPanel(IEditorPanel contextPanel) {
        if (contextPanel instanceof WorkspaceCollectionPanel) {
            return ((WorkspaceCollectionPanel)contextPanel).getGrid().getStore();
        }
        return null;
    }
    
    public void setRenderedWidget(Component widget) {
        renderedActionWidget = widget;
        renderedActionWidget.disable();
    }
    
    public void onSelectionChange(List<ReposObj> selection) {
        if (selection.size() == 1 && selection.get(0).isEditAllowed()) {
            currentObj = selection.get(0);
            renderedActionWidget.enable();
        } else {
            currentObj = null;
            renderedActionWidget.disable();
        }
    }
}
