/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.provisioning.actions;

import java.util.List;

import com.extjs.gxt.ui.client.widget.Component;
import com.google.gwt.core.client.GWT;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.main.client.UIMessages;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ICollectionPanel;
import com.iona.soa.web.repository.main.client.base.ISelectionSensitiveContextAction;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.ISelectionProvider.ISelectionListener;
import com.iona.soa.web.repository.main.client.base.ui.DefaultContextAreaContributor;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.ObjectCreationDialog;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.panels.ScheduledTaskPanel.ScheduledTaskDefaults;

public class IndexLaterAction implements ISelectionSensitiveContextAction, ISelectionListener {

    ReposObj currentObj;
    Component renderedActionWidget;
    ICollectionPanel panel;
    
    public IndexLaterAction(ICollectionPanel contextPanel) {
        panel = contextPanel;
    }
    /**
     * @param applicationEditor
     */

    public String getCaption() {
        return "Index later...";
    }

    public String getToolTip() {
        return "Schedule an index task";
    }

    public String getIconCs() {
        return "schedule-index-icon";
    }

    public void doIt() {
        
        ScheduledTaskDefaults defaults = getDefaults();
        
        ObjectCreationDialog dialog = new ObjectCreationDialog(EClassHelper.SCHEDULED_TASK_ECLASS,
                new ObjectCreationDialog.AfterCreationCallBack() {
                    public void onDialogClosure(ReposObj anObj) {
                        // TODO - Only really need to reload one row - the selected row
                        DefaultContextAreaContributor.getStoreFromContentPanel(panel).getLoader().load();
                        Utils.closeWaitDialog();
                    }                
                }, defaults);
        dialog.setTitle(defaults.getTaskName());
        dialog.show();
    }

    protected ScheduledTaskDefaults getDefaults() {
        UIMessages messages = GWT.create(UIMessages.class);
        ScheduledTaskDefaults defaults = new ScheduledTaskDefaults();
        defaults.setTaskType("com.iona.repository.index.taskservice.SpiderTaskService");
        defaults.setTaskParameters(new String[] {currentObj.getAttributeValue("guid") });
        defaults.setTaskName(messages.defaultScheduleIndexTaskName(currentObj.getAttributeValue("name")));
        return defaults;
    }

    public void setRenderedWidget(Component widget) {
        renderedActionWidget = widget;
        renderedActionWidget.disable();
    }
    
    public void onSelectionChange(List<ReposObj> selection) {
        if (selection.size() == 1 && selection.get(0).isEditAllowed()) {
            currentObj = selection.get(0);
            renderedActionWidget.enable();
        } else {
            currentObj = null;
            renderedActionWidget.disable();
        }
    }    
}
