/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.provisioning.actions;

import java.util.List;

import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ICollectionPanel;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.panels.ScheduledTaskPanel
    .ScheduledTaskDefaults;

public class IndexNowAction extends IndexLaterAction {

    public IndexNowAction(ICollectionPanel contextPanel) {
        super(contextPanel);
    }
    
    public String getCaption() { return "Index now"; }
    public String getToolTip() { return "Index the selected repository"; }
    public String getIconCs() { return "refresh-remote-repo-icon"; }
    
    @Override
    public void doIt() {
        
        ReposObj onceOffTaskObj = buildRunTask();
        ScheduledTaskDefaults defaults = getDefaults();
        mapDefaultsToTaskObj(onceOffTaskObj, defaults);
        Utils.showWaitDialog();
        String collectionLinkForClass = EClassHelper.getCollectionLinkForClass(onceOffTaskObj.getEClass());
        MainModule.getRemoteClient().createReposObj(collectionLinkForClass,
                                                    onceOffTaskObj,
                                                    new AbstractUiObjCallBack() {
                public void processReposObj(ReposObj anObj) {
                //fillDependencyTree(queriedList);
                    String collLink = EClassHelper.getCollectionLinkForClass(EClassHelper.MAVENREPO_ECLASS);
                    MainModule.get().getWorkspaceControl().openLink(collLink, true);
                }
            });
    }


    private ReposObj buildRunTask() {
        ReposObj onceOffTaskObj = new ReposObj();
        
        onceOffTaskObj.setEClass(EClassHelper.SCHEDULED_ONCEOFF_TASK_ECLASS);
        ReposObjAttribute repoAttrAction = new ReposObjAttribute();
        
        repoAttrAction.setName("runNow");
        repoAttrAction.setSimpleValue("true");
        onceOffTaskObj.addAttribute(repoAttrAction);
        
        ReposObjAttribute repoAttrCleanup = new ReposObjAttribute();
        repoAttrCleanup.setName("cleanupOnCompletion");
        repoAttrCleanup.setSimpleValue("true");
        onceOffTaskObj.addAttribute(repoAttrCleanup);
        return onceOffTaskObj;
    }

    private void mapDefaultsToTaskObj(ReposObj onceOffTaskObj, ScheduledTaskDefaults defaults) {
        
        ReposObjAttribute attrName = new ReposObjAttribute();
        // name
        attrName.setName("name");
        attrName.setSimpleValue(defaults.getTaskName());
        onceOffTaskObj.addAttribute(attrName);
        
        ReposObjAttribute attrtaskType = new ReposObjAttribute();
        // taskType
        attrtaskType.setName("typeID");
        attrtaskType.setSimpleValue(defaults.getTaskType());
        onceOffTaskObj.addAttribute(attrtaskType);
        
        ReposObjAttribute attrParams = new ReposObjAttribute();
        //parameters
        attrParams.setName("parameters");
        attrParams.setMulti(true);
        String values[] = defaults.getTaskParameters();
        for (int i = 0; i < values.length; i++) {
            attrParams.addSimpleValue(values[i]);
        }
        onceOffTaskObj.addAttribute(attrParams);
    }


    
    public void onSelectionChange(List<ReposObj> selection) {
        if (selection.size() == 1
            && selection.get(0).isEditAllowed()
            && !selection.get(0).getAttributeValue("indexState").equals("INDEXING")) {
            currentObj = selection.get(0);
            renderedActionWidget.enable();
        } else {
            currentObj = null;
            renderedActionWidget.disable();
        }
    }
}
