/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.provisioning.actions;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.Style.HorizontalAlignment;
import com.extjs.gxt.ui.client.binding.FormBinding;
import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.FormEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.Window;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.form.FileUploadField;
import com.extjs.gxt.ui.client.widget.form.FormPanel;
import com.extjs.gxt.ui.client.widget.form.FormPanel.Encoding;
import com.extjs.gxt.ui.client.widget.form.FormPanel.Method;
import com.extjs.gxt.ui.client.widget.layout.FormLayout;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.ui.Hidden;

public class ResourceCreationDialog extends Window {
    
    private static final int WINDOW_HEIGHT = 60;
    private static final int WINDOW_WIDTH = 450;

    
    public ResourceCreationDialog(String collectionUrl, AfterClosureCallBack aCallBack) {
        setSize(WINDOW_WIDTH, WINDOW_HEIGHT);
        setHeading("Add an artifact");
        setCloseAction(CloseAction.CLOSE);

        setBorders(true);
        setBodyBorder(true);
        setFrame(true);
        setModal(true);
        setMaximizable(false);
        setCloseAction(CloseAction.HIDE); // TODO
        setPlain(true);

        final FileUploadPanel uploadPanel = new FileUploadPanel(collectionUrl, aCallBack);
        
        Button ok = new Button("OK");
        ok.addSelectionListener(new SelectionListener<ButtonEvent>() {
            @Override
            public void componentSelected(ButtonEvent ce) {
                uploadPanel.submit();
            }
        });
        addButton(ok);
        
        Button cancel = new Button("Cancel");
        cancel.addSelectionListener(new SelectionListener<ButtonEvent>() {
            @Override
            public void componentSelected(ButtonEvent ce) {
                close();
            }
        });
        addButton(cancel);    

        add(uploadPanel);
        
    }

    public interface AfterClosureCallBack {
        void onDialogClosure(String createdResourceObjLink);
    }
    
    private class FileUploadPanel extends LayoutContainer {
    	private final String postUrl;
    	
    	private final AfterClosureCallBack uploadedCallback;
    	
    	private FormPanel form;
    	
    	FileUploadPanel(String postUrl, AfterClosureCallBack uploadedCallback) {
    		this.postUrl = postUrl;
    		this.uploadedCallback = uploadedCallback;
    		
			super.setBorders(false);
			setStyleAttribute("margin", "10px");
    	}
    	
    	void submit() {
    	    if (form.isValid()) {
    	        form.submit();
    	    }
    	}
    	
		@Override
		protected void onRender(Element parent, int index) {
			super.onRender(parent, index);
			
			form = new FormPanel();
			FormLayout formLayout = new FormLayout();
			formLayout.setLabelWidth(100);
			form.setLayout(formLayout);
			
			form.setBorders(false);
			form.setHeaderVisible(false);
			form.setBodyBorder(false);
			form.setAction(postUrl);
			form.setEncoding(Encoding.MULTIPART);
			form.setMethod(Method.POST);
			form.setButtonAlign(HorizontalAlignment.CENTER);
			
			final FileUploadField fileField = new FileUploadField();
			fileField.setName("artifactFile");
			fileField.setAllowBlank(false);
			fileField.setFieldLabel("Artifact to upload");
			fileField.setAutoWidth(true);
			form.add(fileField);
			
			final Hidden simpleFileName = new Hidden(); 
			simpleFileName.setName("simpleFileName");
			form.add(simpleFileName);
			
            form.addListener(Events.BeforeSubmit, new Listener<FormEvent>() {
                public void handleEvent(FormEvent be) {
                    simpleFileName.setValue(fileField.getFileName());
                }
            });
			
			// TODO - figure out how failure is reported!!!
			form.addListener(Events.Submit, new Listener<FormEvent>() {
				public void handleEvent(FormEvent fe) {
					final String result = fe.resultHtml;
					final AfterClosureCallBack callback = uploadedCallback;
					DeferredCommand.addCommand(new Command() {
						public void execute() {
							callback.onDialogClosure(result);
						}
					});
					
                    close(); // the window
				}				
			});
						
		    FormBinding binding = new FormBinding(form);
		    binding.autoBind();
		    
			add(form);
		}

	}
}
