/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * 
 */
package com.iona.soa.web.repository.main.client.provisioning.panels;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.Style.SortDir;
import com.extjs.gxt.ui.client.data.BaseModelData;
import com.extjs.gxt.ui.client.data.ModelData;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.BoxComponent;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.ColumnData;
import com.extjs.gxt.ui.client.widget.grid.ColumnModel;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.extjs.gxt.ui.client.widget.grid.GridCellRenderer;
import com.extjs.gxt.ui.client.widget.grid.GridView;
import com.extjs.gxt.ui.client.widget.grid.GroupingView;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.ui.Label;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.HtmlSnippets;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.GridConfig;
import com.iona.soa.web.repository.main.client.base.ui.GroupByDateTimeGridConfig;
import com.iona.soa.web.repository.main.client.base.ui.RemoteCollectionPanel;
import com.iona.soa.web.repository.main.client.base.ui.RemoteRestDataProxy;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator;
import com.iona.soa.web.repository.main.client.base.ui.ToolTipActionColumnConfig;
import com.iona.soa.web.repository.main.client.base.ui.ToolTipActionColumnConfig.AbstractToolTipActionColumnProviderAdapter;

public final class ContainerGridConfig extends GridConfig {
    private static final HtmlSnippets SNIPPETS = GWT.create(HtmlSnippets.class);
    
    private static final class ColumnIds {
        private static final String PROFILE = "profile";
        private static final String STATUS = "status";
        private static final String HOST = "host";
        private static final String CONTAINER_TYPE = "type";
        private static final String MORE_INFO = "more_info";
        private static final String HISTORY = "history";
        
        private ColumnIds() { }
    }
    
    @Override
    protected void addVisibleColumns() {
        addCustomColumns();
        addRssColumn();
    }

    @Override
    protected void addCustomColumns() {
        columns.add(new SimpleColumnConfig(ColumnIds.PROFILE, "Profile") {
            public Object getCellData(ReposObj obj) {
                ReposObjAttribute attr = obj.getAttribute("profile");
                if (attr != null && attr.getValues().length > 0) {
                    return attr.getValueCaption(attr.getValues()[0]);
                }
                return "none";
            }
        });

        // Temporary, will remove
        addNameColumn().setWidth(200);
        
        SimpleColumnConfig statusColumn =
            new SimpleColumnConfig(ColumnIds.STATUS, "Status", 50) {
                public Object getCellData(ReposObj obj) {
                    return obj.getAttributeValue("status.liveness");
                }
            };
        statusColumn.setRenderer(new StatusColumnRenderer());
        columns.add(statusColumn);

        columns.add(new SimpleColumnConfig(ColumnIds.HOST, "Host", 100) {
            public Object getCellData(ReposObj obj) {
                return obj.getAttributeValue("property.hostname");
            }
        });

        columns.add(new SimpleColumnConfig(ColumnIds.CONTAINER_TYPE, "Type", 60) {
            public Object getCellData(ReposObj obj) {
                return obj.getAttributeValue("type");
            }
        });

        ToolTipActionColumnConfig infoColumn = new ToolTipActionColumnConfig(getGrid(), ColumnIds.MORE_INFO);
        infoColumn.setActionProvider(new AgentInfoTTACP());
        columns.add(infoColumn);
        
        ToolTipActionColumnConfig activityColumn = new ToolTipActionColumnConfig(getGrid(),
                                                                                 ColumnIds.HISTORY);
        activityColumn.setActionProvider(new AgentHistoryTTACP());
        columns.add(activityColumn);
    }
    
    static final class StatusColumnRenderer implements GridCellRenderer<ReposObjDataModel> {
        public String render(ReposObjDataModel record,
                             String property,
                             ColumnData config,
                             int rowIndex,
                             int colIndex,
                             ListStore<ReposObjDataModel> store) {

            if (record.get(property) != null) {
                boolean alive = "alive".equalsIgnoreCase(record.get(property, ""));
                if (alive) {
                    return SNIPPETS.imageTag("images/running.gif", "Alive");
                }
            }
            return SNIPPETS.imageTag("images/not_running.gif", "Down");
        }
    }

    
    static final class AgentInfoTTACP extends AbstractToolTipActionColumnProviderAdapter {

        public boolean actionRequired(ReposObjDataModel record) {
            return true;
        }
        
        public String getActionLabel() {
            return "Runtime data";
        }

        public int getToolTipContentsWidthHint() {
            return 300;
        }
        
        public BoxComponent createToolTipContents(ReposObjDataModel record) {
            ReposObj containerObj = record.getPOJO();
            
            Grid<ModelData> propsPanel = new ContainerPropertiesPanel(containerObj);
            
            GridView view = new GridView();
            view.setEmptyText("No applications deployed");
            
            ReposObjAttribute attr = containerObj.getAttribute("applications");
            RemoteRestDataProxy dataProxy =
                RemoteRestDataProxy.createProxyForCollection(attr.getCollectionLink());
            
            EditorGrid<ReposObjDataModel> grid = new EditorGrid<ReposObjDataModel>(null, null);
            SimpleGridConfigurator gridConfig = new SimpleGridConfigurator();
            gridConfig.configure(grid, dataProxy);
            grid.setHideHeaders(true);
            grid.setView(view);
            
            final RemoteCollectionPanel appsPanel = new RemoteCollectionPanel(gridConfig, dataProxy);
            
            // RemoteCollectionPanel instances load automatically when they are rendered.
            // Sp cater for multiple showings here - panel not destroyed when tooltip hidden
            appsPanel.addListener(Events.Show, new Listener<ComponentEvent>() {
                private boolean firstTime = true;
                public void handleEvent(ComponentEvent be) {
                    DeferredCommand.addCommand(new Command() {
                        public void execute() {
                            if (firstTime) {
                                firstTime = false;
                            } else {
                                appsPanel.getStore().getLoader().load();
                            }
                        }
                    });
                }
            });

            
            LayoutContainer holder = new LayoutContainer();
            holder.addStyleName("arr-container-runtime-data");
            holder.setBorders(false);
            holder.add(new Label("Deployed Applications"));
            holder.add(appsPanel);
            holder.add(new Label("Container Properties"));
            holder.add(propsPanel);
            
            // Parent tooltip doesn't like panels with margins (in CSS) so
            // we wrap it.
            LayoutContainer outer = new LayoutContainer();
            outer.setBorders(false);
            outer.add(holder);
            return outer;
        }
    }
    
    private static final class ContainerPropertiesPanel extends Grid<ModelData> {
        ContainerPropertiesPanel(ReposObj containerObj) {
            
            super(new ListStore<ModelData>(), getPropsColumnModel());
            
            setId("props-grid");
            getStore().setSortField("PropName");
            getStore().setSortDir(SortDir.ASC);
            
            setHideHeaders(true);
            
            GridView view = new GridView();
            view.setForceFit(true);
            view.setAutoFill(true);
            view.setForceFit(true); // the grid will never have scroll-bars
            setView(view);

            Map<String, ReposObjAttribute> attrs = containerObj.getAttributes();
            List<ModelData> values = new ArrayList<ModelData>();
            for (String key : attrs.keySet()) {
                if (key.startsWith("property.")) {
                    ModelData rcd = new BaseModelData();
                    rcd.set("PropName", key.substring(9));
                    rcd.set("PropValue", attrs.get(key).getValue());
                    values.add(rcd);
                }
            }

            getStore().add(values);

            setAutoExpandColumn("PropValue");
            
            setAutoHeight(true);
        }

        private static ColumnModel getPropsColumnModel() {
            
            List<ColumnConfig> colCfgs = new ArrayList<ColumnConfig>(2);
            colCfgs.add(new ColumnConfig("PropName", "PropName", 50));
            colCfgs.add(new ColumnConfig("PropValue", "PropValue", 50));
            return new ColumnModel(colCfgs);
        }
    }

    static final class AgentHistoryTTACP extends AbstractToolTipActionColumnProviderAdapter {

        public boolean actionRequired(ReposObjDataModel record) {
            return true;
        }
        
        public String getActionLabel() {
            return "History";
        }

        public int getToolTipContentsWidthHint() {
            return 400;
        }
        
        public BoxComponent createToolTipContents(ReposObjDataModel record) {
            ReposObj containerObj = record.getPOJO();
            
            String emfQuery = "targetGuid=" + containerObj.getAttributeValue("guid");
            RemoteRestDataProxy dataProxy =
                RemoteRestDataProxy.createProxyForEClass(EClassHelper.AUDIT_EVENT_ECLASS, emfQuery);
            
            EditorGrid<ReposObjDataModel> grid = new EditorGrid<ReposObjDataModel>(null, null);
            AuditEventsGridConfig gridConfig = new AuditEventsGridConfig();
            gridConfig.configure(grid, dataProxy);
            
            final RemoteCollectionPanel msgsPanel = new RemoteCollectionPanel(gridConfig, dataProxy);
            
            // RemoteCollectionPanel instances load automatically when they are rendered.
            // Sp cater for multiple showings here - panel not destroyed when tooltip hidden
            msgsPanel.addListener(Events.Show, new Listener<ComponentEvent>() {
                private boolean firstTime = true;
                public void handleEvent(ComponentEvent be) {
                    DeferredCommand.addCommand(new Command() {
                        public void execute() {
                            if (firstTime) {
                                firstTime = false;
                            } else {
                                msgsPanel.getStore().getLoader().load();
                            }
                        }
                    });    
                }
            });

            return msgsPanel;
        }
    }

    private static final class AuditEventsGridConfig extends GroupByDateTimeGridConfig {
        private static final class AuditEventColumnIds {
            private static final String LEVEL = "level";
            private static final String MESSAGE = "msg";
            
            private AuditEventColumnIds() { }
        }
        
        @Override
        protected void addVisibleColumns() {
            SimpleColumnConfig levelColumn = new SimpleColumnConfig(AuditEventColumnIds.LEVEL, "Level") {
                public Object getCellData(ReposObj auditEvent) {
                    String level = auditEvent.getAttributeValue("level");
                    if ("INFO".equals(level)) {
                        return SNIPPETS.imageTag("images/icon-info.gif", "Information");
                    } else if ("WARNING".equals(level)) {
                        return SNIPPETS.imageTag("images/icon-warning.gif", "Warning");
                    } else {
                        return SNIPPETS.imageTag("images/icon-error.gif", "Error");                        
                    }
                }
            };
            levelColumn.setWidth(20);
            columns.add(levelColumn);
            
            addGroupByDateColumn();
            
            addTimeColumn();
            
            SimpleColumnConfig messageColumn = new SimpleColumnConfig(AuditEventColumnIds.MESSAGE,
                                                                          "Message") {
                public Object getCellData(ReposObj auditEvent) {
                    return auditEvent.getAttributeValue("message");
                }
            };
            messageColumn.setWidth(300);
            columns.add(messageColumn);
        }

        @Override
        protected void configure(GroupingView view, final Grid<ReposObjDataModel> grid) {
            super.configure(view, grid);
            view.setEmptyText("No history available");
        }
    }

}