/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * 
 */
package com.iona.soa.web.repository.main.client.provisioning.panels;

import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.BoxComponent;
import com.extjs.gxt.ui.client.widget.grid.ColumnData;
import com.extjs.gxt.ui.client.widget.grid.GridCellRenderer;
import com.google.gwt.core.client.GWT;
import com.iona.soa.web.repository.base.client.model.DeploymentProfileHelper;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.model.ReposObjAttribute;
import com.iona.soa.web.repository.main.client.HtmlSnippets;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.GridConfig;
import com.iona.soa.web.repository.main.client.base.ui.ToolTipActionColumnConfig;
import com.iona.soa.web.repository.main.client.base.ui.ToolTipActionColumnConfig
    .AbstractToolTipActionColumnProviderAdapter;

public final class DeploymentProfileGridConfig extends GridConfig {
    
    private static interface ColumnIds {
        final String VERSIONS = "versions";
        final String PUBLISH_STATUS = "status";
    }

    @Override
    protected void addVisibleColumns() {
        addCustomColumns();
        addEditColumn();
        addDeleteColumn(true);
        addRssColumn();
    }
    
    @Override
    protected void addCustomColumns() {
        columns.add(new StatusColumn());
        columns.add(new ProfileNameColumn());
     
        ToolTipActionColumnConfig versionsColumn = new ToolTipActionColumnConfig(getGrid(), 
                                                                                 ColumnIds.VERSIONS);
        versionsColumn.setActionProvider(new VersionsToolTipActionColumnProvider());
        columns.add(versionsColumn);
    }
    
    private static final class ProfileNameColumn extends SimpleColumnConfig {
        ProfileNameColumn() {
            super(DefaultColumnIds.NAME, "Name");
            setRenderer(new ProfileNameColumnRenderer());
            setWidth(200);
        }
     
        @Override
        public Object getCellData(ReposObj obj) {
            return obj.getCaption();
        }
        
        private static final class ProfileNameColumnRenderer implements GridCellRenderer<ReposObjDataModel> {
            private HtmlSnippets snippets = GWT.create(HtmlSnippets.class);
            
            public String render(ReposObjDataModel record,
                                 String property,
                                 ColumnData config,
                                 int rowIndex,
                                 int colIndex,
                                 ListStore<ReposObjDataModel> store) {
                
                
                ReposObj deploymentProfile = record.getPOJO();
                String profileName = record.get(property);
                String publishStatus = getPublishStatus(deploymentProfile);
                String lastEditedBy = deploymentProfile.getAttributeValue("lastEditedBy");
                
                // Will become category later
                ReposObjAttribute folderAttrib = deploymentProfile.getAttribute("belongsTo");
                
                
                if (folderAttrib != null && folderAttrib.getValue() != null) {                
                    String category = folderAttrib.getValueCaption(folderAttrib.getValue());
                    return snippets.deploymentProfileNameColumnWithCategory(profileName, 
                                                                            publishStatus, 
                                                                            lastEditedBy, 
                                                                            category); 
                } else {
                    return snippets.deploymentProfileNameColumn(profileName, 
                                                                publishStatus, 
                                                                lastEditedBy); 
                }
            }
            
            private String getPublishStatus(ReposObj deploymentProfile) {
                String status = "Draft";
                String activeHref = deploymentProfile.getAttribute("active").getValue();
                if (activeHref != null) {
                    // This is the one that will be edited
                    String currentHref = deploymentProfile.getAttribute("current").getValue();
                    if (activeHref.equals(currentHref)) {
                        return "Published";
                    }
                }
                return status;
            }
        }
    }

    private static final class StatusColumn extends SimpleColumnConfig {

        public StatusColumn() {
            super(ColumnIds.PUBLISH_STATUS, "");
            setRenderer(new StatusColumnRenderer());
            setWidth(25);
        }

        @Override
        public Object getCellData(ReposObj obj) {
            return obj;
        }
        
        private static final class StatusColumnRenderer implements GridCellRenderer<ReposObjDataModel> {  
            private HtmlSnippets snippets = GWT.create(HtmlSnippets.class);
            
            public String render(ReposObjDataModel record,
                                 String property,
                                 ColumnData config,
                                 int rowIndex,
                                 int colIndex,
                                 ListStore<ReposObjDataModel> store) {
                ReposObj deploymentProfile = record.getPOJO();

                String html = "";
                if (DeploymentProfileHelper.isDeployed(deploymentProfile)) {
                    if (DeploymentProfileHelper.hasChangesForDeployment(deploymentProfile)) {
                        html = snippets.imageTag("images/published_draft.gif",
                        "Published (newer draft version exists)");
                    } else {
                        html = snippets.imageTag("images/published.gif", 
                        "Published");
                    }
                }
                
                return html;
            }
        }
    }
    
    private static final class 
    VersionsToolTipActionColumnProvider extends AbstractToolTipActionColumnProviderAdapter {
        
        public boolean actionRequired(ReposObjDataModel record) {
            ReposObj deploymentProfile = record.getPOJO();
            ReposObjAttribute attr = deploymentProfile.getAttribute("versions");
            if (attr != null && attr.isCollection()) {
                return attr.getCollectionSize() > 1;
            }
            return false;
        }

        public String getActionLabel() {
            return "Versions";
        }
        
        public int getToolTipContentsWidthHint() {
         // TODO - get from columnModel.  Will need to separate ColumnModel from panel somehow
            return 600;
        }
        
        public BoxComponent createToolTipContents(ReposObjDataModel record) {
            ReposObj deploymentProfile = record.getPOJO();
            DeploymentProfileVersionsPanel panel = new DeploymentProfileVersionsPanel(deploymentProfile);
            panel.setAutoHeight(true);
            panel.getStore().getLoader().load(); // TODO - do after panel shown?
            
            return panel;
        }
    }    
}