/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * 
 */
package com.iona.soa.web.repository.main.client.provisioning.panels;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.Style.SortDir;
import com.extjs.gxt.ui.client.data.LoadEvent;
import com.extjs.gxt.ui.client.event.GridEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.LoadListener;
import com.extjs.gxt.ui.client.event.WindowEvent;
import com.extjs.gxt.ui.client.store.Store;
import com.extjs.gxt.ui.client.store.StoreFilter;
import com.extjs.gxt.ui.client.widget.Dialog;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.google.gwt.dom.client.Element;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.ActionColumnConfig;
import com.iona.soa.web.repository.main.client.base.ui.GridConfig;
import com.iona.soa.web.repository.main.client.base.ui.RemoteRestDataProxy;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator.DefaultColumnIds;

/**
 * Panel that displays the list of VersionedProfile objects that
 * exist for a DeploymentProfile. 
 */
class DeploymentProfileVersionsPanel extends EditorGrid<ReposObjDataModel> {
    private ReposObj deploymentProfile;
    
    private final StoreFilter<ReposObjDataModel> filterCallback = new StoreFilter<ReposObjDataModel>() {
        public boolean select(Store store,
                              ReposObjDataModel parent,
                              ReposObjDataModel item,
                              String property) {
            String url = ReposHandle.getRelativeLink(item.getPOJO().getSelfLink());
            String currentProfileUrl = deploymentProfile.getAttributeValue("current");
            currentProfileUrl = ReposHandle.getRelativeLink(currentProfileUrl);
            return !currentProfileUrl.equals(url);
        }
    };
    
    private interface ColumnIds {
        final String REVERT = "revert";
    }
    
    /**
     * @param versionsUrl the url of the VersionedProfile collection
     * owned by a DeploymentProfile
     */
    DeploymentProfileVersionsPanel(ReposObj theDeploymentProfile) {
        super(null, null);
        
        deploymentProfile = theDeploymentProfile;
                
        String versionsHref = deploymentProfile.getAttribute("versions").getCollectionLink();
        RemoteRestDataProxy dataProxy = RemoteRestDataProxy.createProxyForCollection(versionsHref);
        
        GridConfig gridConfig = new VersionsGridConfig();
        gridConfig.configure(this, dataProxy);
        
        setBorders(false);
        setStripeRows(false);
        setAutoExpandColumn(DefaultColumnIds.NAME);
        
        // Make rows non-selectable - this table appears inside another one.
        addListener(Events.RowClick, new Listener<GridEvent>() {
            public void handleEvent(GridEvent be) {
                be.doit = false;
            }
        });

        // Order according to publish date - latest first.  This column is hidden
        getStore().setDefaultSort(DefaultColumnIds.TIMESTAMP, SortDir.DESC);

        getStore().getLoader().addLoadListener(new LoadListener() {
            @Override
            public void loaderLoad(LoadEvent le) {
                store.addFilter(filterCallback);
                store.applyFilters(DefaultColumnIds.NAME);
            }
        });
        
        setWidth(getColumnModel().getTotalWidth(false));
    }
    
    private class VersionsGridConfig extends GridConfig {
        
        @Override
        protected void addVisibleColumns() {
            addNameColumn(false).setWidth(200);
            
            ColumnConfig lastModifiedColumn = addLastModifiedColumn();
            lastModifiedColumn.setHeader("Published");
            
            ColumnConfig lastModifiedByColumn = addLastModifiedByColumn();
            lastModifiedByColumn.setHeader("Published by");
            
            addRevertColumn();
            
            // Editing of earlier versions is implemented server side but doesn't appear to work.
            // I see an error logged
            // SEVERE: Message=java.sql.SQLException: The statement was aborted because it would 
            // have caused a duplicate key value in a unique or primary key constraint or unique 
            // index identified by 'PK_DEPLOYMENTPROFILE_VERSION' defined on 
            // 'DEPLOYMENTPROFILE_VERSION'., Location=null, Line=0, Column=0
            // addEditColumn();
            
            addDeleteColumn(true);
            
            // Hidden, use for sorting
            addTimestampColumn(true);
            
            for (SimpleColumnConfig column : columns) {
                if (column instanceof ColumnConfig) {
                    ((ColumnConfig)column).setSortable(false);
                }
            }
        }
        
        private ColumnConfig addRevertColumn() {
            ActionColumnConfig.ActionColumnCallback<ReposObjDataModel> callback =
                new ActionColumnConfig.ActionColumnCallback<ReposObjDataModel>() {
                    public void actionClicked(ReposObjDataModel record, Element anchorElement) {
                        ReposObj versionedProfile = record.getPOJO();
                        revertTo(versionedProfile);
                    }
                };            
            
            ActionColumnConfig<ReposObjDataModel> column =
                new ActionColumnConfig<ReposObjDataModel>(getGrid(), ColumnIds.REVERT)
                    .setActionLabel("Revert")
                    .setActionCallback(callback);
            
            columns.add(column);
            return column;
        }
        
        private void revertTo(final ReposObj versionedProfile) {
            MessageBox.confirm("Revert Deployment Profile",
                               "Reverting to a previously published profile will result in the removal "
                               + "of all newer profiles.  Do you want to continue?",
                               new Listener<WindowEvent>() {
                    public void handleEvent(WindowEvent we) {
                        
                        if (we.window.getButtonBar().getButtonById(Dialog.YES)
                                != we.window.getButtonBar().getButtonPressed()) {
                            return;
                        }
                        
                        deploymentProfile.getAttribute("active").setRefValue(versionedProfile);
                        MainModule.getRemoteClient().updateReposObj(deploymentProfile, 
                                                                    new AbstractUiObjCallBack() {
                                public void processReposObj(ReposObj anObj) {
                                    
                                    getStore().getLoader().load();
                                    // TODO
                                    // Collection panel (deployment profiles) needs to reload 
                                    // BaseReposObjCollectionPanel should automatically respond 
                                    // to an update?
                                    //  
                                    // Popup needs to reload - similarly 
                                    // BaseReposObjCollectionPanel listener 
                                }
                            });
                    }
                });
        }
    }
}