/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.root;


import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.admin.panels.ScheduledTaskGridConfig;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.IEditorPanel;
import com.iona.soa.web.repository.main.client.base.ReposURL;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.GridConfig;
import com.iona.soa.web.repository.main.client.base.ui.RemoteRestDataProxy;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator;
import com.iona.soa.web.repository.main.client.base.ui.WorkspaceCollectionPanel;
import com.iona.soa.web.repository.main.client.provisioning.panels.ContainerGridConfig;
import com.iona.soa.web.repository.main.client.provisioning.panels.DeploymentProfileGridConfig;
import com.iona.soa.web.repository.main.client.provisioning.panels.MavenRepoGridConfig;

final class CollectionPanelFactory {
    
    private CollectionPanelFactory() {
    }
    
    static IEditorPanel getCollectionPanel(ReposURL collectionURL, String filterQuery) {
        RemoteRestDataProxy dataProxy = new RemoteRestDataProxy(MainModule.getRemoteClient());
        dataProxy.setResourceURL(collectionURL);
        dataProxy.setFilterQuery(filterQuery);
        
        SimpleGridConfigurator gridConfig = getGridConfig(collectionURL);
        gridConfig.configure(new EditorGrid<ReposObjDataModel>(null, null), dataProxy);
        WorkspaceCollectionPanel panel = new WorkspaceCollectionPanel(gridConfig, dataProxy);
        return panel;
    }    

    static EditorGrid<ReposObjDataModel> getGridPanel(ReposURL collectionURL, 
                                                      String filterQuery) {
        RemoteRestDataProxy dataProxy = new RemoteRestDataProxy(MainModule.getRemoteClient());
        dataProxy.setResourceURL(collectionURL);
        dataProxy.setFilterQuery(filterQuery);
        
        SimpleGridConfigurator gridConfig = getGridConfig(collectionURL);
        gridConfig.configure(new EditorGrid<ReposObjDataModel>(null, null), dataProxy);
        
        return gridConfig.getGrid();
    }

    static EditorGrid<ReposObjDataModel> getSimpleGridPanel(ReposURL collectionURL, 
                                                            String filterQuery) {
        RemoteRestDataProxy dataProxy = new RemoteRestDataProxy(MainModule.getRemoteClient());
        dataProxy.setResourceURL(collectionURL);
        dataProxy.setFilterQuery(filterQuery);
        
        SimpleGridConfigurator gridConfig = new SimpleGridConfigurator();
        gridConfig.configure(new EditorGrid<ReposObjDataModel>(null, null), dataProxy);
        
        return gridConfig.getGrid();
    }
    
    private static SimpleGridConfigurator getGridConfig(ReposURL collectionURL) {
        SimpleGridConfigurator gridConfig = null;
        
        if (collectionURL.represents(EClassHelper.CONTAINER_ECLASS)) {
            gridConfig = new ContainerGridConfig();
        } else if (collectionURL.represents(EClassHelper.DEPLOYMENT_PROFILE_ECLASS)) {
            gridConfig = new DeploymentProfileGridConfig();
        } else if (collectionURL.represents(EClassHelper.MAVENREPO_ECLASS)) {
            gridConfig = new MavenRepoGridConfig();
        } else if (collectionURL.represents(EClassHelper.SCHEDULED_TASK_ECLASS)) {
            gridConfig = new ScheduledTaskGridConfig();
        } else {
            gridConfig = new GridConfig();
        }
        return gridConfig;
    }
}
