/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.root;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.Style.SelectionMode;
import com.extjs.gxt.ui.client.event.DataListEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.DataList;
import com.extjs.gxt.ui.client.widget.DataListItem;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ICollectionPanel;
import com.iona.soa.web.repository.main.client.base.IContextAction;
import com.iona.soa.web.repository.main.client.base.IContextActionWithId;
import com.iona.soa.web.repository.main.client.base.IContextAreaContributor;
import com.iona.soa.web.repository.main.client.base.IEditorPanel;
import com.iona.soa.web.repository.main.client.base.IFilterable;
import com.iona.soa.web.repository.main.client.base.ISelectionSensitiveContextAction;
import com.iona.soa.web.repository.main.client.base.LinkHistoryTracker;
import com.iona.soa.web.repository.main.client.base.ReposURL;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.DefaultContextAreaContributor;
import com.iona.soa.web.repository.main.client.base.ui.DefaultContextAreaContributor.
    ArtifactContextAreaContributor;
import com.iona.soa.web.repository.main.client.base.ui.DefaultContextAreaContributor.
    DeploymentProfileContextAreaContributor;
import com.iona.soa.web.repository.main.client.base.ui.DefaultContextAreaContributor.
    MavenRepoContextAreaContributor;
import com.iona.soa.web.repository.main.client.base.ui.DefaultContextAreaContributor.
    NullContextAreaContributor;
import com.iona.soa.web.repository.main.client.base.ui.DefaultContextAreaContributor.
    ScheduledTaskContextAreaContributor;
import com.iona.soa.web.repository.main.client.categories.CategoriesPanel2;
import com.iona.soa.web.repository.main.client.search.SearchComboBox;
import com.iona.soa.web.repository.main.client.search.SearchComboBox.SearchHandler;

public final class ContextBar extends LayoutContainer {
    
//    private static final HtmlSnippets SNIPPETS = GWT.create(HtmlSnippets.class);
    
    private static final Map<String, IContextAreaContributor> CONTEXT_CONTRIBUTORS 
        = new HashMap<String, IContextAreaContributor>();

    private ReposURL currentContextURL;
    
    private DataList actionsList;

    private CategoriesPanel2 categoriesPanel;
    
    private SearchComboBox filterComboBox;
    
    private IFilterable filterBoxWatcher;

    private ContextHelpPanel contextHelpPanel;

    static {
        CONTEXT_CONTRIBUTORS.put(EClassHelper.ARTIFACT_ECLASS, 
                                 new ArtifactContextAreaContributor());
        CONTEXT_CONTRIBUTORS.put(EClassHelper.DEPLOYMENT_PROFILE_ECLASS, 
                                 new DeploymentProfileContextAreaContributor());
        CONTEXT_CONTRIBUTORS.put(EClassHelper.MAVENREPO_ECLASS, 
                                 new MavenRepoContextAreaContributor());
        CONTEXT_CONTRIBUTORS.put(EClassHelper.SCHEDULED_TASK_ECLASS,
                                 new ScheduledTaskContextAreaContributor());
        CONTEXT_CONTRIBUTORS.put(EClassHelper.SCHEDULED_ONCEOFF_TASK_ECLASS,
                                 new ScheduledTaskContextAreaContributor());
        CONTEXT_CONTRIBUTORS.put(EClassHelper.SCHEDULED_INTERVAL_REPEATED_TASK_ECLASS,
                                 new ScheduledTaskContextAreaContributor());
        CONTEXT_CONTRIBUTORS.put(EClassHelper.SCHEDULED_DAILY_TASK_ECLASS,
                                 new ScheduledTaskContextAreaContributor());
        CONTEXT_CONTRIBUTORS.put(EClassHelper.SCHEDULED_WEEKLY_TASK_ECLASS,
                                 new ScheduledTaskContextAreaContributor());
        CONTEXT_CONTRIBUTORS.put(EClassHelper.SCHEDULED_MONTHLY_TASK_ECLASS,
                                 new ScheduledTaskContextAreaContributor());
        CONTEXT_CONTRIBUTORS.put(null, new NullContextAreaContributor());
    }

    ContextBar() {
        setId("arr-context-panel");
        createWidgets();
    }

    void clearContext() {
        disableFiltering();
        clearActions();
        showContextHelp(null);
        doLayout();
    }
    
    void contextChanged(ReposURL contextURL, IEditorPanel content) {
        currentContextURL = contextURL;
        IContextAreaContributor contributor = getContextContributor(contextURL);
        
        filterComboBox.clear();
        if (content instanceof IFilterable) {
            enableFiltering((IFilterable)content); // TODO
        } else {
            disableFiltering();
        }
        
        clearActions();
        
        List<IContextActionWithId> actionsWithIds = new ArrayList<IContextActionWithId>();
        List<IContextAction> otherActions = null;
        if (content instanceof ICollectionPanel) {
            IContextAction action = contributor.getCreateAction(contextURL, (ICollectionPanel)content);
            if (action != null) {
                addAction(action);
                if (action instanceof IContextActionWithId) {
                    actionsWithIds.add((IContextActionWithId)action);
                }
            }
            otherActions = contributor.getCollectionActions(contextURL, (ICollectionPanel)content);
        } else if (content instanceof IEditorPanel) {
            otherActions = contributor.getEditorActions(contextURL, (IEditorPanel)content);
        }
        
        for (IContextAction action : otherActions) {
            addAction(action);
            if (action instanceof IContextActionWithId) {
                actionsWithIds.add((IContextActionWithId)action);
            }
        }

        if (content instanceof IFilterable) {
            categoriesPanel.contextChanged(contextURL, (IFilterable)content); // TODO
        } else {
            categoriesPanel.contextChanged(contextURL);            
        }
        
        showContextHelp(contributor.getHelpId(contextURL));
        
        doLayout();
        
        if (contextURL.hasParameter("action")) {
            String id = contextURL.getParameterValue("action");
            for (IContextActionWithId actionWithId : actionsWithIds) {
                if (actionWithId.getId().equals(id)) {
                    final IContextAction toRun = actionWithId;
                    DeferredCommand.addCommand(new Command() {
                        public void execute() {
                            // Do this later, allowing UI to refresh and 
                            // history token from previous context to get registered etc..
                            toRun.doIt();
                        }
                    });
                    break;
                }
            }
        }
    }
    
    ListStore<ReposObjDataModel> getCategoriesStore() {
        return categoriesPanel.getStore();
    }

    private void addAction(final IContextAction anAction) {
        DataListItem item = new DataListItem(anAction.getCaption());
        item.setIconStyle(anAction.getIconCs());
        item.setToolTip(anAction.getToolTip());
        item.setData("actionDefinition", anAction);

        actionsList.add(item);

        if (anAction instanceof ISelectionSensitiveContextAction) {
            ((ISelectionSensitiveContextAction)anAction).setRenderedWidget(item);
        }
    }
    
    private void clearActions() {
        actionsList.removeAll();
    }

    private void showContextHelp(String contextId) {
        if (contextId == null || contextId.trim().length() == 0) {
            contextHelpPanel.setVisible(false);
        } else {
            contextHelpPanel.setVisible(true);
            contextHelpPanel.showContextHelp(contextId);
        }
    }
    
    private void createWidgets() {
        setBorders(false);
        
        filterComboBox = new SearchComboBox(new SearchHandler() {
            public void runSearch(String query) {
                filterBoxWatcher.setFilterByQuery(query);
                LinkHistoryTracker.INSTANCE.reportLink(currentContextURL.getURL(), 
                        query);            
            }
            
            public void reset() {
                filterBoxWatcher.setFilterByQuery(null);
            }
        });
        filterComboBox.setVisible(false);
        add(filterComboBox);
        
        actionsList = new DataList();
        actionsList.setBorders(false);
        actionsList.setId("arr-other-actions-panel");
        actionsList.setSelectionMode(SelectionMode.SIMPLE);
        Listener<DataListEvent> listener = new Listener<DataListEvent>() {
            
            private long lastEventTimeStamp = -1;
            public void handleEvent(DataListEvent be) {
                long pts = lastEventTimeStamp;
                long timestamp = (new Date()).getTime();
                
                if ((pts < 0 || (timestamp - 1000 > pts))
                        && be.selected.size() == 1) {
                    
                    lastEventTimeStamp = timestamp;
                    
                    DataListItem selected =  be.selected.get(0);
                    if (selected.isEnabled()) {
                        final IContextAction action = selected.getData("actionDefinition");
                        DeferredCommand.addCommand(new Command() {
                                public void execute() {
                                    actionsList.getSelectionModel().deselectAll();
                                    action.doIt();
                                }
                            });
                    }
                }
            }
        };

        actionsList.addListener(Events.SelectionChange, listener);
        add(actionsList);

        categoriesPanel = new CategoriesPanel2();
        categoriesPanel.setVisible(false);
        add(categoriesPanel);
        
        contextHelpPanel = new ContextHelpPanel();
        contextHelpPanel.setVisible(false);
        add(contextHelpPanel);
    }
    
    private void enableFiltering(IFilterable aFilterable) {
        if (!filterComboBox.isVisible()) {
            filterComboBox.setVisible(true);
        }
        filterBoxWatcher = aFilterable;
    }
    
    private void disableFiltering() {
        if (filterComboBox.isVisible()) {
            filterComboBox.setVisible(false);
        }
        filterBoxWatcher = null;
    }

    private IContextAreaContributor getContextContributor(ReposURL contextURL) {
        IContextAreaContributor contributor = CONTEXT_CONTRIBUTORS.get(contextURL.getEClass());
        return contributor == null ? new DefaultContextAreaContributor() : contributor;
    }
}
