/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.root;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.allen_sauer.gwt.log.client.Log;
import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.Style.SelectionMode;
import com.extjs.gxt.ui.client.event.DataListEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.DataList;
import com.extjs.gxt.ui.client.widget.DataListItem;
import com.extjs.gxt.ui.client.widget.Html;
import com.google.gwt.core.client.GWT;
import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.RequestException;
import com.google.gwt.http.client.Response;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.Window;
import com.google.gwt.xml.client.Document;
import com.google.gwt.xml.client.Element;
import com.google.gwt.xml.client.NodeList;
import com.google.gwt.xml.client.XMLParser;
import com.iona.soa.web.repository.main.client.MainModule;

public class ContextHelpPanel extends ContentPanel {
    
    protected static final String BASE_TOPIC_URL = MainModule.DOCS_SERVER_URL + "topic/guide/";

    private static final int TIME_OUT_MILLIS = 0;

    private static Map<String, ContextHelp> contexts;

    private DataList topicsList;
    
    private Html desc;
    
    public ContextHelpPanel() {
        createWidgets();
    }

    private void createWidgets() {
        setId("arr-context-help-panel");
        setHeading("Help");
        setCollapsible(true);
        setTitleCollapse(true);
        setTitle("Help");
        setIconStyle("help-icon");
        setBorders(false);
        setBodyBorder(false);

        desc = new Html();
        desc.setId("arr-context-help-desc");
        add(desc);
        
        topicsList = new DataList();
        topicsList.setId("arr-context-help-topics");
        topicsList.setBorders(false);
        topicsList.setSelectionMode(SelectionMode.SINGLE);
        Listener<DataListEvent> listener = new Listener<DataListEvent>() {
			public void handleEvent(DataListEvent be) {
				if (be.selected.size() == 1) {
					DataListItem selected =  be.selected.get(0);
					final String href = selected.getData("href");
                    DeferredCommand.addCommand(new Command() {
                        public void execute() {
                            Window.open(href, "_contextHelp", "");
                        }
                    });
                	topicsList.getSelectionModel().deselectAll();
				}
			}
        };
        topicsList.addListener(Events.SelectionChange, listener);
        add(topicsList);
    }
    
    
    private void clearContextHelp() {
        desc.setHtml("");
        topicsList.removeAll();
    }
    
    public void showContextHelp(String contextId) {
        if (contexts == null) {
            loadContextsAndShowHelp(contextId);
        }
        
        doShowContextHelp(contextId);
    }

    private void doShowContextHelp(String contextId) {
        ContextHelp theHelp = contexts.get(contextId);
        
        clearContextHelp();
        
        if (theHelp == null) {
            return;
        }
        
        desc.setHtml(theHelp.getDesc());
        
        for (Iterator<String> iter = theHelp.getTopicCaptions().iterator(); iter.hasNext();) {
            String caption = iter.next();
            addTopicLink(caption, theHelp.getTopicLinkHref(caption));
        }
    }
    
    private void addTopicLink(String caption, String href) {
        DataListItem topicItem = new DataListItem(caption);
        topicItem.setIconStyle("help-icon");
        topicItem.setData("href", href);
        if (href == null) {
            topicItem.disable();
        }
        
        topicsList.add(topicItem);
    }

    private void loadContextsAndShowHelp(final String contextId) {
        if (contexts != null) {
            Log.debug("\n\nhelp contexts already loaded");
            return;
        }
        
        Log.debug("\n\nloading help contexts ... ");
        
        desc.setHtml("Initializing Context Help...");
        doLayout();
        
        contexts = new HashMap<String, ContextHelp>();
        try {
            RequestBuilder rb = new RequestBuilder(RequestBuilder.GET,
                                                   BASE_TOPIC_URL + "context.xml");
            rb.setHeader("Cache-Control", "max-age=0, no-store");
            rb.setTimeoutMillis(TIME_OUT_MILLIS);

            
            Log.debug("\nQuerying: " + rb.getUrl());
            Log.debug("\nHostPageBase: " + GWT.getHostPageBaseURL());
            Log.debug("\nModuleBaseURL: " + GWT.getModuleBaseURL());

            rb.sendRequest(null, new RequestCallback() {

                public void onError(Request request, Throwable exception) {
                    Log.debug("an error occured while loading help contexts: ", exception);
                    //callBack.layoutLoaded(null);
                }

                public void onResponseReceived(Request request, Response response) {
                    Log.debug("context help init GET (STATUS CODE) ==>" + response.getStatusCode());
                    Log.debug("context help init GET (STATUS TEXT) ==>" + response.getStatusText());
                    Log.debug("context help init GET (RESPONSE) ==>" + response.getText() + "<==");
                    
                    parseContexts(response.getText(), contexts);
                    
                    doShowContextHelp(contextId);
                }
            });
        } catch (RequestException e) {
            Log.debug("GET (ERROR) ==> " + e.getMessage(), e);
            //callBack.layoutLoaded(null);
        }
    }

    protected static void parseContexts(String xmlContexts,
                                        Map<String, ContextHelp> contextsBuffer) {
        
        Document document = XMLParser.parse(xmlContexts);

        NodeList sectionElts = document.getElementsByTagName("context");
        for (int sectionIndex = 0; sectionIndex < sectionElts.getLength(); sectionIndex++) {
            
            Element contextElt = (Element)sectionElts.item(sectionIndex);
            
            String contextId = contextElt.getAttribute("id");
            
            NodeList descElts = contextElt.getElementsByTagName("description");
            String desc = null;
            if (descElts.getLength() == 1) { 
                desc = descElts.item(0).getChildNodes().item(0).getNodeValue();
            }
            
            ContextHelp contextObj = new ContextHelp(desc);
            
            NodeList topicElts = contextElt.getElementsByTagName("topic");
            for (int columnIndex = 0; columnIndex < topicElts.getLength(); columnIndex++) {
                
                Element topicElt = (Element)topicElts.item(columnIndex);
                
                String lbl = topicElt.getAttribute("label");
                if (lbl != null) {
                    String href = topicElt.getAttribute("href");
                    contextObj.addTopicLink(lbl, href != null ? BASE_TOPIC_URL + href : null);
                }
            }

            contextsBuffer.put(contextId, contextObj);
        }
    }   
    
    public static class ContextHelp {
        
        private String desc;
        private Map<String, String> topicLinks;
        
        public ContextHelp(String aDesc) {
            desc = aDesc;
            topicLinks = new HashMap<String, String>();
        }
        
        public void addTopicLink(String caption, String href) {
            topicLinks.put(caption, href);
        }
        
        public String getDesc() {
            return desc;
        }
        
        public Collection<String> getTopicCaptions() {
            return topicLinks.keySet();
        }
        
        public String getTopicLinkHref(String topicCaption) {
            return topicLinks.get(topicCaption);
        }
    }
}
