/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.root;

import java.util.HashMap;
import java.util.Map;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.Html;
import com.extjs.gxt.ui.client.widget.layout.FlowLayout;
import com.google.gwt.user.client.ui.Widget;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ISubModule;
import com.iona.soa.web.repository.main.client.base.ReposHandle;

public class FlowNavBar extends ContentPanel {

    private Widget selectedLinkWidget;
    
    private Map<String, Widget> urlToWidgetMap;
    
    public FlowNavBar() {
        setId("arr-flowbar-panel");
        setHeaderVisible(false);
        setBorders(false);
        setBodyBorder(false);                             
        setLayout(new FlowLayout());
        urlToWidgetMap = new HashMap<String, Widget>();        
    }

    public void resetFlow(ISubModule aSuModule, final String[] someOrderedTypes) {
        removeAll();
        urlToWidgetMap.clear();

        Html appNameLabel = new Html("<h1>" + aSuModule.getTitle() + "</h1>");
        appNameLabel.addStyleName("arr-flowbar-title");
        add(appNameLabel);
        
        Widget homeLink = createNavigationLink(aSuModule.getApplicationIntroPageUrl(),
                                             "Home",
                                             aSuModule);
        add(homeLink);
        urlToWidgetMap.put(ReposHandle.getAbsoluteLink(aSuModule.getApplicationIntroPageUrl()), 
                           homeLink);
        
        for (int i = 0; i < someOrderedTypes.length; i++) {
            final String eClass = someOrderedTypes[i];
            String caption = EClassHelper.getPluralNameForClass(eClass);
            String url = EClassHelper.getCollectionLinkForClass(eClass);
            
            final Widget flowLink = createNavigationLink(url, caption, aSuModule);
            urlToWidgetMap.put(ReposHandle.getAbsoluteLink(url), flowLink);
            add(flowLink);
        }
        
        doLayout();
    }

    void setActive(String url) {
        setActive(urlToWidgetMap.get(ReposHandle.getAbsoluteLink(url)));
    }
    
    private void setActive(Widget link) {
        if (link == null) {
            return;
        }
        
        if (selectedLinkWidget != null) {
            selectedLinkWidget.removeStyleDependentName("selected");
        }
        
        link.addStyleDependentName("selected");
        selectedLinkWidget = link;
        doLayout();
    }
    
    private Widget createNavigationLink(final String selfLink,
            String caption, 
            final ISubModule context) {
        Html lnkRefAtrr = new Html("<a href='javascript:;'>" + caption + "</a>") {
            @Override
            protected void afterRender() {
                super.afterRender();
                el().addEventsSunk(Events.OnClick);
                el().addEventsSunk(Events.OnKeyUp);
            }
        };
        lnkRefAtrr.addStyleName("arr-flowbar-link");
        
        Listener<ComponentEvent> listener = new Listener<ComponentEvent>() {
            public void handleEvent(ComponentEvent be) {
                MainModule.get().getWorkspaceControl().openLink(selfLink, context, true);
            }
        }; 
        lnkRefAtrr.addListener(Events.OnClick, listener);
        lnkRefAtrr.addListener(Events.OnKeyUp, listener);
        
        return lnkRefAtrr;
    }
}
