/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.root;

import java.util.Map;
import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.Html;
import com.extjs.gxt.ui.client.widget.VerticalPanel;
import com.extjs.gxt.ui.client.widget.Window;
import com.extjs.gxt.ui.client.widget.Window.CloseAction;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.layout.FitLayout;
import com.extjs.gxt.ui.client.widget.menu.Menu;
import com.extjs.gxt.ui.client.widget.menu.MenuItem;
import com.extjs.gxt.ui.client.widget.menu.SeparatorMenuItem;
import com.extjs.gxt.ui.client.widget.toolbar.SplitToolItem;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.ui.Image;
import com.iona.soa.web.repository.base.client.util.PropertyCallBack;
import com.iona.soa.web.repository.base.client.util.PropertyHelper;
import com.iona.soa.web.repository.main.client.MainModule;

public class HelpMenu extends SplitToolItem {
    
    // TODO: Product name etc should be put somewhere central
    private static final String PRODUCT_NAME = "FUSE Depot";
    private static final String PRODUCT_DESC = "Provisioning Server Console";
    private static final String COMPANY_NAME = "Progress Software Corporation";
    private static final String COMPANY_URL = "http://www.progress.com";
    private static final String COPYRIGHT_YEARS = "2009";
    
    private static final String VESION_KEY = "depot.build.version";
    private static final String TIMESTAMP_KEY = "depot.build.timestamp";
    
    private static final String ABOUT_TITLE = "About " + PRODUCT_NAME;
    private static String aboutHtml = null;
    
    public HelpMenu() {
        super("Help");
        
        setIconStyle("helpLogo-headerSmall");
        setToolTip("Help...");
        
        SelectionListener<ComponentEvent> helpListener = new SelectionListener<ComponentEvent>() {
            @Override
            public void componentSelected(ComponentEvent ce) {
                com.google.gwt.user.client.Window.open(MainModule.DOCS_SERVER_URL, "_blank", "");
            };
        };
                
        addSelectionListener(helpListener);
        
        
        MenuItem helpItem = new MenuItem("Help Contents");
        helpItem.setIconStyle("helpLogo-headerSmall");
        helpItem.addSelectionListener(helpListener);       
        
        final MenuItem aboutItem = new MenuItem(ABOUT_TITLE);
        aboutItem.setIconStyle("aboutLogo-headerSmall");
        aboutItem.addSelectionListener(new SelectionListener<ComponentEvent>() {
            @Override
            public void componentSelected(ComponentEvent ce) {
                PropertyHelper.loadProperties("buildinfo/buildinfo.properties", new PropertyCallBack() {
                    public void propertiesLoaded(Map<String, String> properties) {
                        AboutWindow about = new AboutWindow(properties);
                        about.show(aboutItem.getElement());
                    }
                });

            }
        });
        
        
        Menu menu = new Menu();
        menu.add(helpItem);

        menu.add(new SeparatorMenuItem());
        menu.add(aboutItem);
        
        setMenu(menu);
    }

    class AboutWindow {
        
        
        private Button buttonOK;
        private Window window;
        private String version;
        private String buildTimestamp;

        public AboutWindow(Map<String, String> properties) {

            version = properties.get(VESION_KEY);
            buildTimestamp = properties.get(TIMESTAMP_KEY);

            window = new Window();
            window.setAutoHide(false);
            window.setClosable(false);
            window.setFrame(true);
            window.setModal(false);
            window.setCloseAction(CloseAction.CLOSE);
            window.setHeading(ABOUT_TITLE);
            window.setHeaderVisible(true);
            window.setResizable(true);

            window.setLayout(new FitLayout());

            final VerticalPanel panel = new VerticalPanel();
            panel.setId("arr-about-panel");
            panel.setBorders(false);
            panel.setAutoHeight(true);
            panel.setAutoWidth(true);
            
            buttonOK = new Button("Ok", new SelectionListener<ButtonEvent>() {
                @Override
                public void componentSelected(ButtonEvent ce) {
                    window.close();
                }
            });

            buttonOK.setIconStyle("ok-icon");
            buttonOK.setId("ok-icon");    

            panel.add(new Image("images/arr_logo.gif"));
            panel.add(new Html(getAboutHtml()));
            panel.add(buttonOK);
            
            window.add(panel);
        }
              
        public void show(Element el) {
            /* TODO get this anchoring working
             * 
            window.alignTo((com.google.gwt.user.client.Element)el,
                    "br",
                    new int[] {0, 0});
             */
            window.show();
        }
        
        
        private synchronized String getAboutHtml() {
            
            if (aboutHtml != null) {
                return aboutHtml;
            }
           
            if (version == null) {
                version = "?";
            }
            if (buildTimestamp == null) {
                buildTimestamp = "?";
            }
            
            StringBuilder sb = new StringBuilder()                 
                .append("<br><p>")
                .append(PRODUCT_NAME).append(" - ")
                .append(PRODUCT_DESC)
                .append("</p><br><p>Version: ")
                .append(version)
                .append(" (")
                .append(buildTimestamp)
                .append(")</p><br><p>Copyright \u00A9 ")
                .append(COPYRIGHT_YEARS)
                .append(" ").append(COMPANY_NAME)
                .append(". All rights reserved.</p><br><p>") 
                .append("Contact us at <a target=\"_blank\" href=\"") 
                .append(COMPANY_URL)
                .append("\">")
                .append(COMPANY_URL)
                .append("</a></p><br>");
            return sb.toString();
        }
    }    
}   
