/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.root;

import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.Html;
import com.extjs.gxt.ui.client.widget.menu.Menu;
import com.extjs.gxt.ui.client.widget.menu.MenuItem;
import com.extjs.gxt.ui.client.widget.tips.ToolTipConfig;
import com.extjs.gxt.ui.client.widget.toolbar.AdapterToolItem;
import com.extjs.gxt.ui.client.widget.toolbar.FillToolItem;
import com.extjs.gxt.ui.client.widget.toolbar.SeparatorToolItem;
import com.extjs.gxt.ui.client.widget.toolbar.SplitToolItem;
import com.extjs.gxt.ui.client.widget.toolbar.TextToolItem;
import com.extjs.gxt.ui.client.widget.toolbar.ToolBar;
import com.extjs.gxt.ui.client.widget.toolbar.ToolItem;
import com.google.gwt.user.client.Cookies;
import com.google.gwt.user.client.Window;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.ISubModule;

public class TopToolBar extends ToolBar {

    public TopToolBar(final ISubModule[] modules) {
        for (int i = 0; i < modules.length; i++) {
            add(getMenu(modules[i]));  
            add(new SeparatorToolItem());
        }

        add(new HelpMenu());                
        add(new FillToolItem());
        
        String html = "<div id=\"arr-header-login-info\">"
            + "Logged in as <b>" + Cookies.getCookie("userId") + "</b> "
            + "<a href=\"Session?action=logout\">(Log out)</a>"
            + "</div>";
        Html loginDetails = new Html(html);
        add(new AdapterToolItem(loginDetails));

    }

    private ToolItem getMenu(final ISubModule module) {
        SplitToolItem menuItem = new SplitToolItem(module.getTitle());
        menuItem.setIconStyle(module.getImageCs());
        menuItem.setToolTip(new ToolTipConfig(module.getTitle() + "?... What's that for?", 
                module.getApplicationToolTip()));
        menuItem.addSelectionListener(new SelectionListener<ComponentEvent>() {
            @Override
            public void componentSelected(ComponentEvent ce) {
                MainModule.get().getWorkspaceControl().reloadSubModule(module);
            }
        });
        
        Menu menu = new Menu();
        String[] types = module.getTypes();
        for (int i = 0; i < types.length; i++) {
            final String eclass = types[i];
            MenuItem item = new MenuItem(EClassHelper.getPluralNameForClass(eclass));
            item.setIconStyle(EClassHelper.getIconClsForClass(eclass));
            
            item.addSelectionListener(new SelectionListener<ComponentEvent>() {
                @Override
                public void componentSelected(ComponentEvent ce) {
                    String feedUrl = EClassHelper.getCollectionLinkForClass(eclass);
                    MainModule.get().getWorkspaceControl().openLink(feedUrl, module, true);
                }
            });
            
            menu.add(item);
        }
        
        menuItem.setMenu(menu);

        return menuItem;
    }
}
