/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.root;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.data.ListLoader;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.WindowEvent;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.Dialog;
import com.extjs.gxt.ui.client.widget.Html;
import com.extjs.gxt.ui.client.widget.HtmlContainer;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.grid.EditorGrid;
import com.google.gwt.core.client.GWT;
import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.RequestException;
import com.google.gwt.http.client.Response;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.WindowCloseListener;
import com.google.gwt.user.client.ui.Widget;
import com.iona.soa.web.repository.base.client.model.ReposObj;
import com.iona.soa.web.repository.base.client.remote.IDeletionCallBack;
import com.iona.soa.web.repository.main.client.HtmlSnippets;
import com.iona.soa.web.repository.main.client.MainModule;
import com.iona.soa.web.repository.main.client.UIMessages;
import com.iona.soa.web.repository.main.client.base.AbstractUiObjCallBack;
import com.iona.soa.web.repository.main.client.base.EClassHelper;
import com.iona.soa.web.repository.main.client.base.IEditorPanel;
import com.iona.soa.web.repository.main.client.base.ISubModule;
import com.iona.soa.web.repository.main.client.base.LinkHistoryTracker;
import com.iona.soa.web.repository.main.client.base.ReposHandle;
import com.iona.soa.web.repository.main.client.base.ReposObjHelper;
import com.iona.soa.web.repository.main.client.base.ReposURL;
import com.iona.soa.web.repository.main.client.base.Utils;
import com.iona.soa.web.repository.main.client.base.gxtdatamodels.ReposObjDataModel;
import com.iona.soa.web.repository.main.client.base.ui.CollectionContextMenuHelper;
import com.iona.soa.web.repository.main.client.base.ui.GridConfig;
import com.iona.soa.web.repository.main.client.base.ui.RemoteRestDataProxy;
import com.iona.soa.web.repository.main.client.base.ui.SimpleGridConfigurator;
import com.iona.soa.web.repository.main.client.base.ui.WorkspaceCollectionPanel;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.CreationPanelRegistry;
import com.iona.soa.web.repository.main.client.base.ui.creationpanels.ObjectCreationDialog;

/**
 * the unique entry point of the ARR web tooling
 */
public final class WorkspacePanel extends LayoutContainer implements WindowCloseListener {

    //private static final HtmlSnippets SNIPPETS = GWT.create(HtmlSnippets.class);

    private static final UIMessages MESSAGES = GWT.create(UIMessages.class);
    
    private static final HtmlSnippets SNIPPETS = GWT.create(HtmlSnippets.class);
    
    private String currentLink;
    
    private ISubModule activeSubModule;

    private FlowNavBar navigationBar;

    private LayoutContainer contentHolderPanel;
    
    private ContextBar contextPanel;

    private final Map<String, ISubModule> eClassToModuleMap = new HashMap<String, ISubModule>();
    
    private boolean unsavedMsgBoxActive;
    
    /**
     * This method constructs the application user interface by instantiating controls and hooking up event
     * listeners.
     */
    public WorkspacePanel(ISubModule[] modules, FlowNavBar navBar) {
        setId("arr-workspace-panel");
        setBorders(false);
        
        navigationBar = navBar; 
        
        createUI();
        
        for (int i = 0; i < modules.length; i++) {
            String[] types = modules[i].getTypes();
            for (int j = 0; j < types.length; j++) {
                eClassToModuleMap.put(types[j], modules[i]);
            }
        }
    }

    /**
     * Used to prompt user about unsaved changes if any exist.
     * @see com.google.gwt.user.client.WindowCloseListener 
     */
    public void onWindowClosed() {
        // do nothing - anything that could be done is in onWindowClosing
    }

    /**
     * Used to prompt user about unsaved changes if any exist.
     * @see com.google.gwt.user.client.WindowCloseListener 
     */
    public String onWindowClosing() {
        if (currentPageDirty() && !unsavedMsgBoxActive) {
            return MESSAGES.navigateAwayWhenUnsavedMsg();
        }
        return null;
    }
    
    public void activateSubModule(ISubModule aSubModule) {
        if (activeSubModule != aSubModule) {
            activeSubModule = aSubModule;
            updateUI(aSubModule);
            DeferredCommand.addCommand(new Command() {
                public void execute() {
                    if (activeSubModule != null && currentLink == null) {
                        // load intro page
                        openLink(activeSubModule.getApplicationIntroPageUrl(), 
                                 true);
                    }
                }
            });
        }
    }
    
    void reloadSubModule(ISubModule aSubModule) {
        currentLink = null;
        activeSubModule = null;
        activateSubModule(aSubModule);
    }
    
    public String getCurrentLink() {
        return currentLink;
    }

    public void openLink(String link, ISubModule context, boolean force) {
        if (context != null) {
            activateSubModule(context);
        }
        openLink(link, force);
    }

    public void openCollection(String link, String filterQuery, ISubModule context, boolean force) {
        if (context != null) {
            activateSubModule(context);
        }
        
        doOpenLink(new ReposURL(link), true, filterQuery, force);
    }

    public void openLink(final String selfLink, boolean force) {
        if (isSearchQuery(selfLink)) {
            runSearch(selfLink, force);
        } else {
            boolean isCollectionLink = EClassHelper.isCollectionURL(selfLink);
            doOpenLink(new ReposURL(selfLink), isCollectionLink, null, force);
        }
    }
    
    public void openObj(ReposObj anObj) {
        boolean force = false;
        boolean isCollectionLink = false;
        String filterQuery = null;
        
        doOpenLink(anObj == null ? null : new ReposURL(anObj.getSelfLink()), 
                isCollectionLink, 
                filterQuery, 
                force);
    }

    public void runSearch(final String query, boolean force) {
        if (query == null) {
            return;
        }

        SimpleGridConfigurator gridConfig = new GridConfig();
        ReposURL queryURL = getQueryReposURL(query); 
        RemoteRestDataProxy dataProxy = new RemoteRestDataProxy(MainModule.getRemoteClient());
        dataProxy.setResourceURL(queryURL);
        dataProxy.setFilterQuery(null);
        
        gridConfig.configure(new EditorGrid<ReposObjDataModel>(null, null), dataProxy);        
        
        WorkspaceCollectionPanel collectionPanel 
            = new WorkspaceCollectionPanel(gridConfig, dataProxy);
        collectionPanel.addStyleName("arr-content-panel");
        
        contentHolderPanel.removeAll();
        contentHolderPanel.add(collectionPanel);
        contextPanel.contextChanged(queryURL, collectionPanel);            
        contentHolderPanel.layout();
        
        LinkHistoryTracker.INSTANCE.reportLink(queryURL.getURL(), query);            
    }
    
    public void displayUnsavedChangesDialog(final Listener<WindowEvent> callback) {
        Listener<WindowEvent> secondCallBack = new Listener<WindowEvent>() {
            public void handleEvent(WindowEvent we) {
                unsavedMsgBoxActive = false;
                callback.handleEvent(we);
            }
        };
        
        MessageBox msgBox = MessageBox.confirm(MESSAGES.discardChangesTitle(), 
                MESSAGES.discardChangesMsg(), secondCallBack);
        msgBox.getDialog().getButtonById(Dialog.YES).setText(MESSAGES.discardChangesBtnText());
        msgBox.getDialog().getButtonById(Dialog.NO).setText(MESSAGES.cancelBtnText());
        unsavedMsgBoxActive = true;
    }

    private void doOpenLink(final ReposURL url, 
                            final boolean isCollectionLink, 
                            final String filterQuery, 
                            boolean force) {
        String link = url.getURL();
        if (!(link.toLowerCase().endsWith(".html")
                || link.toLowerCase().endsWith(".htm")
                || link.toLowerCase().endsWith(".jsp"))) {
            link = ReposHandle.getAbsoluteLink(link);
        }
        
        if (!force
                && ((currentLink == link && currentLink == null) 
                    || (currentLink != null && currentLink.equals(link)))) {
            return;
        }
        
        if (currentPageDirty()) {
            displayUnsavedChangesDialog(new Listener<WindowEvent>() {
                public void handleEvent(WindowEvent we) {
                    if (Dialog.YES.equals(we.buttonClicked.getItemId())) {
                        definitelyOpenLink(url, isCollectionLink, filterQuery);
                    } else {
                        // Typically user has attempted to move away from the link
                        navigationBar.setActive(currentLink);
                    }
                }
            });
        } else {
            definitelyOpenLink(url, isCollectionLink, filterQuery);
        }
        
    }

    private void definitelyOpenLink(ReposURL url, 
                                    boolean isCollectionLink, 
                                    String filterQuery) {
        currentLink = url.getURL();
        contentHolderPanel.removeAll();

        if (isCollectionLink) {
            String eClass = EClassHelper.getClassForCollectionLink(ReposHandle.getAbsoluteLink(currentLink));
            // only done if a change is needed
            activateSubModule(eClassToModuleMap.get(eClass));            
        }
        
        navigationBar.setActive(currentLink);
        
        IEditorPanel newContent = null;
        
        if (currentLink.endsWith(".html")) {
            newContent = new HTMLContentPanel(url);
        } else if (isCollectionLink) {
            newContent = CollectionPanelFactory.getCollectionPanel(url, filterQuery);
            
            if (EClassHelper.isCategorizable(url.getEClass())) {
                WorkspaceCollectionPanel collectionPanel = (WorkspaceCollectionPanel)newContent;
                CollectionContextMenuHelper.addCategoriesContextMenu(collectionPanel, 
                                                                     contextPanel.getCategoriesStore());
            }            
        } else {
            newContent = EditorFactory.getEditorPanel(url); 
        }
        
        ((Widget)newContent).addStyleName("arr-content-panel");
        
        contentHolderPanel.add((Widget)newContent);
        contentHolderPanel.layout();
        contextPanel.contextChanged(url, newContent);
        
        // TODO
        // If Routed here via Welcome page stuff implemented in ModuleHistoryListener
        // then history is message up right now - 11/11/08.
        // That url is already added in the history?
        LinkHistoryTracker.INSTANCE.reportLink(currentLink, filterQuery);
    }
    
    public void createReposObj(ReposURL contextURL, ListStore<ReposObjDataModel> store) {
        if (CreationPanelRegistry.getPanel(contextURL.getEClass()) != null) {
            ObjectCreationDialog ocd =
                    new ObjectCreationDialog(contextURL.getEClass(),
                                             new ObjectCreationDialog.AfterCreationCallBack() {
                        public void onDialogClosure(ReposObj anObj) {
                            Utils.closeWaitDialog();
                            MainModule.get().getWorkspaceControl().openObj(anObj);
                        }                
                    });
            ocd.show();
        } else {
            Utils.showWaitDialog();
            String collectionName = EClassHelper.getContextForClass(contextURL.getEClass());
            String objectName = WorkspacePanel.getUniqueNameInCollection(store, contextURL.getEClass());
            ReposObjHelper.createObject(collectionName, objectName, new AbstractUiObjCallBack() {
                    public void processReposObj(ReposObj anObj) {
                        Utils.closeWaitDialog();
                        MainModule.get().getWorkspaceControl().openObj(anObj);
                    }
                });
        }
    }
    
    public void deleteObjects(final ReposObj[] objs,
                              final IDeletionCallBack callBack,
                              boolean askConfirmation) {
        if (askConfirmation) {
            String items = objs.length > 1 ? "objects?" : "object?";
            String title = "Delete " + items;
            String msg = "Are you sure you want to delete the selected " + items;
            Listener<WindowEvent> callback = new Listener<WindowEvent>() {
                public void handleEvent(WindowEvent we) {
                    if (Dialog.YES.equals(we.buttonClicked.getItemId())) {
                        doDeleteObj(objs, callBack);
                    }
                }
            };
            MessageBox.confirm(title, msg, callback);
        } else {
            doDeleteObj(objs, callBack);
        }
    }

    private void doDeleteObj(final ReposObj[] objs, final IDeletionCallBack callBack) {
        Utils.showWaitDialog();
        
        List<ReposObj> objList = new ArrayList<ReposObj>(objs.length);
        for (int i = 0; i < objs.length; i++) {
            objList.add(objs[i]);
        }
        
        
        MainModule.getRemoteClient().deleteReposObjs(objList, new IDeletionCallBack() {
            public void processSuccess() {
                Utils.closeWaitDialog();
                if (callBack != null) {
                    callBack.processSuccess();
                }
            }

            public void processConnectionError(Throwable e) {
                Utils.closeWaitDialog();
                if (callBack != null) {
                    callBack.processConnectionError(e);
                } else {
                    Utils.handleErrorMsg("unexpected error while connecting to the server: "
                                         + e.getMessage(), false);
                }
            }

            public void processResponseError(String anErrorCode, String anErrorMsg) {
                Utils.closeWaitDialog();
                if (callBack != null) {
                    callBack.processResponseError(anErrorCode, anErrorMsg);
                } else {
                    Utils.handleErrorMsg("unexpected response from the server (" + anErrorCode
                                         + "): " + anErrorMsg, "401".equals(anErrorCode));
                }
            }
        });
    }

    public ISubModule getActiveSubModule() {
        return activeSubModule;
    }
    
    private boolean isSearchQuery(String selfLink) {
        // REVISIT ...
        return selfLink != null 
            && selfLink.indexOf(".html") == -1
            && selfLink.indexOf(ReposHandle.INSTANCE.getServerBaseUrl()) == -1;
    }
    
    private void updateUI(final ISubModule aSubModule) {
        contentHolderPanel.removeAll();
        contextPanel.clearContext();
        
        if (aSubModule != null) {
            navigationBar.resetFlow(aSubModule, aSubModule.getTypes());
        }

        doLayout();        
    }
    
    private void createUI() {
        LayoutContainer rightPanelWrapper = new LayoutContainer();
        rightPanelWrapper.setBorders(false);
        rightPanelWrapper.setId("arr-context-wrapper");
        
        contextPanel = new ContextBar();
        rightPanelWrapper.add(contextPanel);
        
        // Add before mainarea as it is floated right in CSS
        add(rightPanelWrapper);

        contentHolderPanel = new LayoutContainer();
        contentHolderPanel.setId("arr-content-holder");
        contentHolderPanel.setBorders(false);
        
        add(contentHolderPanel);
    }
    
    private boolean currentPageDirty() {
        if (currentLink != null 
                && contentHolderPanel != null
                && contentHolderPanel.getItemCount() > 0
                && contentHolderPanel.getItem(0) instanceof IEditorPanel) {
            IEditorPanel cp = (IEditorPanel)contentHolderPanel.getItem(0);
            return cp.isDirty();
        }
        return false;
    }
    
    public static String getUniqueNameInCollection(ListStore<ReposObjDataModel> store, String eClass) {
        store.clearFilters();
        List<ReposObjDataModel> records = store.getModels();
        Collection<String> existingNames = new ArrayList<String>();
        for (ReposObjDataModel record : records) {
            existingNames.add(record.getPOJO().getCaption());
        }
        String baseName = "New " + EClassHelper.getNameForClass(eClass);
        String suggestedName;
        int count = 0;
        do {
            suggestedName  = baseName;
            if (count > 0) {
                suggestedName += " (" + count + ")";
            }
            count++;
        } while (existingNames.contains(suggestedName));
        return suggestedName;
    }

    private static ReposURL getQueryReposURL(String query) {
        String baseUrl = ReposHandle.INSTANCE.getServerBaseUrl();
        String searchUrl = baseUrl.endsWith("/") ? baseUrl + "search" : baseUrl + "/" + "search";
        return new ReposURL(searchUrl);
    }
    
    // TODO - rename to Overview page or HomePage as the implementation assumes
    // a particular kind of content is supplied.
    //
    private final class HTMLContentPanel extends LayoutContainer implements IEditorPanel {
        public HTMLContentPanel(ReposURL contentFileURL) {
            setBorders(false);
            addStyleName("arr-overview-panel");
            
            String url = GWT.getModuleBaseURL() + "/" + contentFileURL.getURL();
            RequestBuilder rb = new RequestBuilder(RequestBuilder.GET, url);
            try {
                rb.sendRequest(null, new RequestCallback() {
                    public void onError(Request request, Throwable t) {
                        add(new Html(MESSAGES.failedToLoadContent(t.getMessage())));
                        layout();
                    }
    
                    public void onResponseReceived(Request request, Response response) {
                        final String content = response.getText(); 
                        DeferredCommand.addCommand(new Command() {
                            public void execute() {
                                if (HTMLContentPanel.this.isAttached()) {
                                    HtmlContainer container = new HtmlContainer();
                                    fill(container, content);
                                    add(container);
                                    layout();
                                }
                            }
                        });
                    }
                });
            } catch (RequestException ex) {
                add(new Html(MESSAGES.failedToLoadContent(ex.getMessage())));
            }            
        }
        
        /**
         * @see com.iona.soa.web.repository.main.client.base.IEditorPanel
         */
        public boolean isDirty() {
            return false;
        }  

        private void fill(HtmlContainer container, String content) {
            container.setHtml(content);
            
            insertCollectionLinks(container);
            insertCreateLinks(container);
            
            if (content.indexOf("id=\"intro-maven-repositories\"") != -1) {
                insertMavenRepositoriesPanel(container);
            }
        }
        
        private void insertCollectionLinks(HtmlContainer container) {
            String[] eClasses = new String[] {
                EClassHelper.APPLICATION_ECLASS,
                EClassHelper.MAVENREPO_ECLASS,
                EClassHelper.CONTAINER_ECLASS,
                EClassHelper.DEPLOYMENT_PROFILE_ECLASS
            };
            
            for (int i = 0; i < eClasses.length; i++) {
                
                final String url = EClassHelper.getCollectionLinkForClass(eClasses[i]);
                String plural = EClassHelper.getPluralNameForClass(eClasses[i]);
                Runnable clickListener = new Runnable() {
                    public void run() {
                        doOpenLink(new ReposURL(url), true, null, true);
                    }
                };
                Html html = createHtmlPanel(SNIPPETS.anchorTagWithCssCls(plural, "arr-action-link"),
                        clickListener);
                
                // The targetId must be present in the html already in the container.
                String targetId = EClassHelper.getContextForClass(eClasses[i]);
                container.add(html, "#" + targetId);
            }
        }
        
        private void insertCreateLinks(HtmlContainer container) {
            String[] eClasses = new String[] {
                EClassHelper.APPLICATION_ECLASS,
                EClassHelper.MAVENREPO_ECLASS,
                EClassHelper.DEPLOYMENT_PROFILE_ECLASS
            };
            
            String[] labels = new String[] {
                "Create an application",
                "Register another Maven Repository with Depot",
                "Create a profile"
            };
            
            for (int i = 0; i < eClasses.length; i++) {                
                final String url = EClassHelper.getCollectionLinkForClass(eClasses[i]);
                Runnable clickListener = new Runnable() {
                    public void run() {
                        Map<String, String> params = new HashMap<String, String>();
                        params.put("action", "create");
                        doOpenLink(new ReposURL(url, params), 
                                   true, 
                                   null, 
                                   true);
                    }
                };
                Html html = createHtmlPanel(SNIPPETS.anchorTagWithCssCls(labels[i], "arr-action-link"),
                        clickListener);
                
                // The targetId must be present in the html already in the container.
                String targetId = EClassHelper.getContextForClass(eClasses[i]) + "-create";
                container.add(html, "#" + targetId);
            }
        }
        
        private void insertMavenRepositoriesPanel(HtmlContainer container) {
            ReposURL mavenURL =
                new ReposURL(EClassHelper.getCollectionLinkForClass(EClassHelper.MAVENREPO_ECLASS)); 
            EditorGrid<ReposObjDataModel> gp = CollectionPanelFactory.getSimpleGridPanel(mavenURL, null);
            gp.setBorders(false);
            gp.setStripeRows(false);
            gp.setHideHeaders(true);
            gp.setTrackMouseOver(false);
            
            container.add(gp, "#intro-maven-repositories");
            
            final ListLoader<?> loader = gp.getStore().getLoader(); 
            DeferredCommand.addCommand(new Command() {
                public void execute() {
                    if (HTMLContentPanel.this.isAttached()) {
                        loader.load();
                    }
                }
            });
        }
        
        // TODO - put in Utils class?
        private Html createHtmlPanel(String content, final Runnable clickListener) {
            Html html = new Html(content) {
                @Override
                protected void afterRender() {
                    super.afterRender();
                    super.el().addEventsSunk(Events.OnClick);
                    super.el().addEventsSunk(Events.OnKeyUp);
                }
            };
            
            Listener<ComponentEvent> listener = new Listener<ComponentEvent>() {
                public void handleEvent(ComponentEvent be) {
                    clickListener.run();
                }
            }; 
            html.addListener(Events.OnClick, listener);
            html.addListener(Events.OnKeyUp, listener);
            return html;
        }
    }
    
    /**
     * Knows how to process tokens of form
     * <moduleName>?context=<contextId>&action=<actionId>
     * 
     * Initially created to implement REPO-354.
     * 
     * Ideally all tokens would be routed through this listener, and application navigation
     * "requests" would be issued via History.newItem() or via GWT Hyperlink 
     * or NoHistoryLink widgets.  This would solve REPO-308 also, however the LinkHistoryTracker
     * and WorkspacePanel collaboration makes this a hefty task.
     */
    /*
    private final class ModuleHistoryListener implements HistoryListener {
        private final Map<String, ISubModule> idToModuleMap = new HashMap<String, ISubModule>();
        
        private String lastTokenHandled = null;
        
        ModuleHistoryListener(ISubModule[] modules) {
            for (int i = 0; i < modules.length; i++) {
                idToModuleMap.put(modules[i].getURLParamId(), modules[i]);
            }
        }
        public void onHistoryChanged(String historyToken) {
            if (historyToken.equals(lastTokenHandled)) {
                lastTokenHandled = null;
                History.back();
                return;
            }
            
            String[] parts = historyToken.split("\\?");
            if (parts.length > 0) {
                String moduleId = parts[0];
                ISubModule module = idToModuleMap.get(moduleId);
                if (module != null) {
                    lastTokenHandled = historyToken;
                    activateSubModule(module);
                    
                    if (parts.length == 2) {
                        Map<String, String> params = parseParameters(parts[1]);
                        if (params.containsKey("context")) {
                            String eClass = EClassHelper.getEClassForContext(params.get("context"));
                            if (eClass != null) {
                                String collectionURL = EClassHelper.getCollectionLinkForClass(eClass);
                                ReposURL url = new ReposURL(collectionURL, params);
                                doOpenLink(url, true, null, true);
                            }
                        } 
                    }
                }
            }
        }
        
        
        private Map<String, String> parseParameters(String params) {
            String[] nvPairs = params.split("&");

            Map<String, String> map = new HashMap<String, String>();

            for (int i = 0; i < nvPairs.length; i++) {
                String[] nvPair = nvPairs[i].split("=");
                map.put(nvPair[0], nvPair[1]);
            }
            return map;
        }
        
    }
    */
}
