/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.main.client.search;

import com.extjs.gxt.ui.client.Events;
import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.FieldEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.form.SimpleComboBox;

public class SearchComboBox extends LayoutContainer {
	private final SearchHandler searchHandler;
	
    protected final SimpleComboBox<String> searchField;

    public interface SearchHandler {
    	void runSearch(String query);
    	
    	void reset();
    }
    
    public SearchComboBox(SearchHandler handler) {
    	searchHandler = handler;
    	
        addStyleName("arr-search-combo");
        setBorders(false);
        
        searchField = createSearchCombo();
        add(searchField);
        add(createSearchButton());        
    }

    public void clear() {
    	searchField.setRawValue(null);
    }
    
    private SimpleComboBox<String> createSearchCombo() {
    	SimpleComboBox<String> searchField = new SimpleComboBox<String>();
        searchField.setMinChars(1);
        searchField.setEditable(true);
        searchField.setEmptyText("Enter query");
        searchField.setLoadingText("Searching...");
        searchField.setTypeAhead(true);
        searchField.setSelectOnFocus(true);
        searchField.setHideTrigger(true);

        Listener<FieldEvent> enterKeyListener = new Listener<FieldEvent>() {
			public void handleEvent(FieldEvent fe) {
				if (fe.getKeyCode() == 13) {
					fe.stopEvent();
					runSearch();
				}
			}
        };
        searchField.addListener(Events.SpecialKey, enterKeyListener);
        return searchField;
    }

    private Button createSearchButton() {
    	Button button = new Button("Search");
    	button.addSelectionListener(new SelectionListener<ButtonEvent>() {
			@Override
			public void componentSelected(ButtonEvent ce) {
				runSearch();
			}
    	});
    	return button;
    }
    
    protected String getToolTipText() {
        return "Search";
    }

    private void runSearch() {
		String term = searchField.getRawValue();
    	
    	if (!isEmpty(term)) {
	        searchHandler.runSearch(term.trim());
	        handleSearchHistories(term);
        } else {
        	searchHandler.reset();
        }
    }
    
//    protected void doRunSearch(String term) {
//        MainModule.get().getWorkspaceControl().runSearch(term, true);
//    	System.out.println(term);
//    	Info.display("Search", term.toString());
//    }

    private void handleSearchHistories(String term) {
    	searchField.remove(term);
    	searchField.add(term);
    }
    
    private boolean isEmpty(String term) {
    	return term == null || term.trim().length() == 0;
    }
}
