/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.security.filters;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.iona.soa.security.AuthenticationException;
import com.iona.soa.security.SecurityContext;
import com.iona.soa.security.SecurityService;
import com.iona.soa.web.repository.security.servlets.SessionServlet;

public class LoginFilter implements Filter {
    private static final String[] EXCLUDES = {"/css", "/images"};

    private FilterConfig filterConfig;

    public void destroy() {
        // TODO Auto-generated method stub

    }

    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
        throws IOException, ServletException {

        HttpServletRequest httpRequest = (HttpServletRequest)request;
        HttpServletResponse httpResponse = (HttpServletResponse)response;

        Cookie ssoCookie = SessionServlet.findCookie(httpRequest, SecurityService.TOKEN_TYPE);

        if ((requestAllowed(httpRequest.getRequestURI())) || (isValidSession(ssoCookie))) {
            chain.doFilter(request, response);
        } else {
            if (ssoCookie != null) {
                ssoCookie.setMaxAge(0);
                httpResponse.addCookie(ssoCookie);
            }
            filterConfig.getServletContext().getRequestDispatcher("/Session").forward(request, response);
        }
    }

    private boolean isValidSession(Cookie ssoCookie) {
        if (ssoCookie == null) {
            return false;
        }

        SecurityService securityService = RepositoryProtectorFactory.getInstance().getSecurityService();

        if (securityService != null) {
            try {
                SecurityContext context = securityService.authenticate(SecurityService.TOKEN_TYPE + " "
                                                                       + ssoCookie.getValue());
                return context.getSSOToken().equals(ssoCookie.getValue());
            } catch (AuthenticationException ae) {
                return false;
            }
        } else {
            return true;
        }
    }

    private boolean requestAllowed(String uri) {
        for (String exclude : EXCLUDES) {
            if (uri.indexOf(exclude) > -1) {
                return true;
            }
        }
        return false;
    }

    public void init(FilterConfig config) throws ServletException {
        this.filterConfig = config;
    }

}
