/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.web.repository.security.servlets;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.iona.soa.security.AuthenticationException;
import com.iona.soa.security.SecurityContext;
import com.iona.soa.security.SecurityService;
import com.iona.soa.web.repository.security.filters.RepositoryProtectorFactory;

public class SessionServlet extends HttpServlet {
    public static String USERID_COOKIE = "userId";
    private static final long serialVersionUID = 0L;

    public void init(ServletConfig config) throws ServletException {
        super.init(config);
    }

    public void doGet(HttpServletRequest request, HttpServletResponse response) {
        doPost(request, response);
    }

    public void doPost(HttpServletRequest request, HttpServletResponse response) {
        String action = request.getParameter("action");

        if ("login".equals(action)) {
            SecurityService securityService = RepositoryProtectorFactory.getInstance().getSecurityService();
            if (securityService != null) {

                String username = request.getParameter("username");
                String password = request.getParameter("pass");
                
                HttpSession session = request.getSession(true);
                try {
                    SecurityContext secCtx = securityService.authenticate(username, password);
                    session.setAttribute("loginError", null);
                    
                    // Create a cookie for authentication using REST API
                    Cookie ssoCookie = new Cookie(SecurityService.TOKEN_TYPE, 
                                                  secCtx.getSSOToken());
                    ssoCookie.setPath("/");
                    response.addCookie(ssoCookie);
                    
                    Cookie userIdCookie = new Cookie(USERID_COOKIE, username);
                    userIdCookie.setPath("/");
                    response.addCookie(userIdCookie);

                    redirect("display.jsp", request, response, false);
                    return;
                } catch (AuthenticationException e) {
                    session.setAttribute("loginError", "Invalid username or password!");
                    Cookie cookie = findCookie(request, SecurityService.TOKEN_TYPE);
                    if (cookie != null) {
                        cookie.setMaxAge(0);
                        response.addCookie(cookie);
                    }
                    cookie = findCookie(request, USERID_COOKIE);
                    if (cookie != null) {
                        cookie.setMaxAge(0);
                        response.addCookie(cookie);
                    }
                }
                
            }
        } else if ("logout".equals(action)) {
            HttpSession session = request.getSession(false);
            if (session != null) {
                session.setAttribute("loginError", null);
            }
            Cookie ssoCookie = findCookie(request, SecurityService.TOKEN_TYPE);
            if (ssoCookie != null) {
                SecurityService securityService = RepositoryProtectorFactory.getInstance().getSecurityService();
                if (securityService != null) {
                    securityService.logoutUser(ssoCookie.getValue());    
                }
                ssoCookie.setMaxAge(0);
                ssoCookie.setPath("/");
                response.addCookie(ssoCookie);
            }
        }
        redirect("/login.jsp", request, response, true);
    }


    public void redirect(String location, HttpServletRequest request, HttpServletResponse response,
                         boolean forward) {
        try {
            if (forward) {
                request.getRequestDispatcher(location).forward(request, response);
            } else {
                response.sendRedirect(location);
            }
        } catch (Exception ioe) {
            // ingore  
        }
    }
    
    public static Cookie findCookie(HttpServletRequest request, String cookieName) {
        Cookie[] cookies = request.getCookies();
        if (cookies == null) {
            return null;
        }        
        for (Cookie cookie : cookies) {
            if (cookieName.equals(cookie.getName())) {
                return cookie;
            }
        }
        return null;
    }

}
