/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.sample.jbi;

import java.text.DateFormat;
import java.text.MessageFormat;
import java.util.Date;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;

public class SampleBean {
    private static final String LOG_NAME = "FUSE Depot demo [JBI Service Unit]";
    
    private static final String LOG_SEPARATOR = " - ";
    
    private static final DateFormat DATE_FORMAT = DateFormat.getTimeInstance();
    
    
    public SampleBean() {
    }

    @PostConstruct
    public void installed() {
        log("");
        log(">>> Installed >>>");
        log("");

        printProperty("prop1");
        printProperty("prop2");
        printProperty("prop3");
        log("");

    }

    @PreDestroy()
    public void uninstalled() {
        log("");
        log(">>> Uninstalled >>>");
        log("");
    }
    
    private void printProperty(String prop) {
        String result = System.getProperty(prop);
        if (result == null) {
            log("Property name=\"{0}\" has not been set", prop);
        } else {
            log("Property name=\"{0}\" set to value \"{1}\"", 
                new Object[] {prop, result});
        }
    }
    

    private void log(String message, Object... args) {
        StringBuffer sb = new StringBuffer();
        sb.append(DATE_FORMAT.format(new Date()));
        sb.append(LOG_SEPARATOR);
        sb.append(LOG_NAME);
        sb.append(LOG_SEPARATOR);
        sb.append(MessageFormat.format(message, args));
        System.out.println(sb.toString());
    }    
}
