/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.model.helpers;

import javax.xml.namespace.QName;

import com.iona.soa.model.userattributes.IAttributes;
import com.iona.soa.model.userattributes.ITagged;
import com.iona.soa.model.userattributes.UserAttributeDefinition;
import com.iona.soa.model.userattributes.UserAttributeDictionary;
import com.iona.soa.model.userattributes.UserAttributeValue;
import com.iona.soa.model.userattributes.UserAttributesFactory;

public final class AttributesHelper {
    private AttributesHelper() {
        // Intentionally left blank.
    }
    
    public static UserAttributeDictionary createDictionary(QName name, String description) {
        UserAttributeDictionary uad =
                UserAttributesFactory.eINSTANCE.createUserAttributeDictionary();
        
        uad.setName(name.getLocalPart());
        uad.setNamespace(name.getNamespaceURI());
        uad.setDescription(description);
        return uad;
    }
    
    @SuppressWarnings("unchecked")
    public static UserAttributeDefinition defineAttribute(
            UserAttributeDictionary dict,
            QName name,
            String description) {
        
        UserAttributeDefinition uad =
                UserAttributesFactory.eINSTANCE.createUserAttributeDefinition();
        uad.setName(name.getLocalPart());
        uad.setNamespace(name.getNamespaceURI());
        uad.setDescription(description);
        
        dict.getAttributeDefs().add(uad);
        return uad;        
    }
    
    @SuppressWarnings("unchecked")
    public static UserAttributeValue addAttributeValue(IAttributes target, 
            UserAttributeDefinition uad, String value) {
        UserAttributeValue ua = UserAttributesFactory.eINSTANCE.createUserAttributeValue();
        ua.setValue(value);
        ua.setDefinition(uad);        
        target.getAttributes().add(ua);
        return ua;
    }
    
    @SuppressWarnings("unchecked")
    public static void addTag(ITagged target, UserAttributeDefinition uad) {   
        target.getTags().add(uad);
    }
}
