/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.soa.model.helpers;

import com.iona.soa.model.events.DataChangeEvent;
import com.iona.soa.model.events.EventsPackage;
import com.iona.soa.model.notification.NotificationDefinition;
import com.iona.soa.model.notification.NotificationPackage;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.ocl.OCL;
import org.eclipse.ocl.ParserException;
import org.eclipse.ocl.ecore.CallOperationAction;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.ecore.SendSignalAction;
import org.eclipse.ocl.helper.OCLHelper;

/**
 * Package access helper for use within generated code. 
 */
public final class NotificationDefinitionHelper {
    //OCL 1.1.0
    private static final OCL<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, 
        EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> OCLI;
    
    private static final OCLHelper<EClassifier, ?, ?, Constraint> HELPER;
    
    static {
        OCLI = OCL.newInstance(EcoreEnvironmentFactory.INSTANCE);
                
        HELPER = OCLI.createOCLHelper();
        HELPER.setContext(EventsPackage.Literals.DATA_CHANGE_EVENT);
    }
    
    private NotificationDefinitionHelper() {
    }
    
    public static synchronized boolean evaluate(NotificationDefinition definition, DataChangeEvent event) {
        if (!definition.eIsSet(NotificationPackage.Literals.NOTIFICATION_DEFINITION__RULE)) {
            throw new UnsupportedOperationException(
                "NotificationDefinition :: Operation not supported as no rule specified");
        }
                
        try {
            Constraint expr = HELPER.createInvariant(definition.getRule());
            org.eclipse.ocl.Query<EClassifier, EClass, EObject> eval = OCLI.createQuery(expr);
            
            return eval.check(event);
        } catch (ParserException ex) {
            throw new RuntimeException(ex);
        }
    }
    /*
    // OCL 1.0.1
    public static synchronized boolean evaluate(NotificationDefinition definition, DataChangeEvent event) {
        Query query = QueryFactory.eINSTANCE.createQuery(
                                                         definition.getRule(),
                                                         EventsPackage.eINSTANCE.getDataChangeEvent());
        return query.check(event);
    }
    */
}
