/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.util.i18n;

import java.util.ResourceBundle;
import java.util.logging.Logger;

/**
 * Internationalization utilities.
 */
public final class I18nUtils {

    private static final String BUNDLE_NAME = "Messages";

    private I18nUtils() {
        // Utility class.
    }

    /**
     * Returns the default resource bundle name for the specified class.
     *
     * @param cls  the class requesting the resource bundle.
     *
     * @return the default resource bundle name.
     */
    public static String getResourceBundleName(Class<?> cls) {

        String pkg = cls.getPackage() == null ? "" : cls.getPackage().getName();
        if (pkg.length() == 0) {
            return BUNDLE_NAME;
        } else {
            return pkg + "." + BUNDLE_NAME;
        }
    }

    /**
     * Returns the default resource bundle for the specified class.
     *
     * @param cls  the class requesting the resource bundle.
     *
     * @return the default resource bundle.
     */
    public static ResourceBundle getResourceBundle(Class<?> cls) {

        return ResourceBundle.getBundle(getResourceBundleName(cls));
    }

    /**
     * Returns the logger for the specified class using the default resource bundle.
     *
     * @param cls  the class requesting the logger.
     *
     * @return the logger.
     */
    public static Logger getLogger(Class<?> cls) {

        return Logger.getLogger(cls.getName(), getResourceBundleName(cls));
    }
}
