/**
 * Copyright (C) [2009]  Progress Software Corporation.
 * FUSE DEPOT 1.0
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.iona.repository.util.i18n;

import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * Provides language-neutral messages.
 */
public class Message {

    protected ResourceBundle bundle;
    protected String key;
    protected Object[] arguments;

    /**
     * Creates an instance of <code>Message</code>.
     *
     * @param resourceBundle  the resource bundle.
     * @param key             the message catalog key (or message string).
     * @param arguments       the message format arguments.
     */
    public Message(ResourceBundle bundle, String key, Object... arguments) {
        assert bundle != null;
        assert key != null;

        this.bundle = bundle;
        this.key = key;
        this.arguments = arguments;
    }

    /**
     * Returns the string representation of this message.
     *
     * @return the string representation of this message.
     */
    public String toString() {
        String pattern = null;
        try {
            pattern = bundle.getString(key);
        } catch (MissingResourceException ex) {
            return key;
        }
        return MessageFormat.format(pattern, arguments);
    }

    /**
     * Returns the resource bundle of this message.
     *
     * @return the resource bundle of this message.
     */
    public ResourceBundle getResourceBundle() {
        return bundle;
    }

    /**
     * Returns the catalog key of this message.
     *
     * @return the catalog key of this message.
     */
    public String getKey() {
        return key;
    }

    /**
     * Returns the format arguments of this message.
     *
     * @return the format arguments of this message.
     */
    public Object[] getArguments() {
        return arguments;
    }
}
